@@ -4,11 +4,11 @@ chmod a+x ./autogen.sh
 if [ "$TRAVIS_OS_NAME" = "linux" ]; then
 	export LIBOPTS=""
 	export EXTRA_TOOLS="--enable-gimp-plugin --enable-gnome-shp-thumbnailer --enable-compiler --enable-mods --with-usecode-debugger=yes"
-	export NPROC=`nproc`
+	export NPROC=$(nproc)
 else
-	export LIBOPTS="--enable-static-libraries --disable-alsa --disable-fluidsynth --disable-timidity-midi --with-macosx-static-lib-path=/opt/local/lib"
+	export LIBOPTS="--enable-static-libraries --disable-alsa --disable-fluidsynth --disable-timidity-midi"
 	export EXTRA_TOOLS=""
-	export NPROC=`sysctl -n hw.ncpu`
+	export NPROC=$(sysctl -n hw.ncpu)
 	export SDK=' -w -mmacosx-version-min=10.7 '
 	export MACOSX_DEPLOYMENT_TARGET=10.7
 	export CPPFLAGS='-I/opt/local/include '$SDK
@@ -20,10 +20,10 @@ else
 	export LIBTOOLFLAGS="--silent"
 fi
 
-./autogen.sh && ./configure --with-cxx=`which $CXX` $LIBOPTS $EXTRA_OPT	\
+./autogen.sh && ./configure --with-cxx=$(which $CXX) $LIBOPTS $EXTRA_OPT	\
                             --disable-oggtest --disable-vorbistest	\
                             --enable-exult-studio --enable-exult-studio-support	\
                             --enable-mt32emu --enable-zip-support	\
-                            --enable-shared --enable-opengl --enable-midi-sfx	\
+                            --enable-shared --enable-midi-sfx	\
                             $EXTRA_TOOLS && make -j $NPROC
 
