@@ -1,5 +1,5 @@
 /*
-*  Copyright (C) 2000-2013  The Exult Team
+*  Copyright (C) 2000-2022  The Exult Team
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
@@ -75,157 +75,139 @@ struct	Chunk
 };
 
 
-Audio *Audio::self = 0;
-int const *Audio::bg2si_sfxs = 0;
-int const *Audio::bg2si_songs = 0;
+Audio *Audio::self = nullptr;
+int const *Audio::bg2si_sfxs = nullptr;
+int const *Audio::bg2si_songs = nullptr;
 
 //----- Utilities ----------------------------------------------------
 
 //----- SFX ----------------------------------------------------------
 
-
-/*
-*	This is a resource-management class for SFX. Maybe make it a
-*	template class and use for other resources also?
-*	Based on code by Sam Lantinga et al on:
-*	http://www.ibm.com/developerworks/library/l-pirates2/
-*/
-class SFX_cache_manager
+// Tries to locate a sfx in the cache based on sfx num.
+SFX_cache_manager::SFX_cached *SFX_cache_manager::find_sfx(int id)
 {
-protected:
-	typedef std::pair<int,AudioSample*> SFX_cached;
-	typedef std::map<int, SFX_cached>::iterator cache_iterator;
+	auto found = cache.find(id);
+	if (found  == cache.end()) return nullptr;
+	return &(found->second);
+}
 
-	std::map<int, SFX_cached> cache;
+SFX_cache_manager::~SFX_cache_manager() {
+	flush();
+}
 
-	// Tries to locate a sfx in the cache based on sfx num.
-	SFX_cached *find_sfx(int id)
-	{
-		cache_iterator found = cache.find(id);
-		if (found  == cache.end()) return 0;
-		return &(found->second);
+// For SFX played through 'play_wave_sfx'. Searched cache for
+// the sfx first, then loads from the sfx file if needed.
+AudioSample *SFX_cache_manager::request(Flex *sfx_file, int id)
+{
+	SFX_cached *loaded = find_sfx(id);
+	if (!loaded) {
+		SFX_cached new_sfx;
+		new_sfx.first = 0;
+		new_sfx.second = nullptr;
+		loaded = &(cache[id] = new_sfx);
 	}
 
-public:
-	SFX_cache_manager()
-	{ }
-	~SFX_cache_manager()
-	{ flush(); }
-
-	// For SFX played through 'play_wave_sfx'. Searched cache for
-	// the sfx first, then loads from the sfx file if needed.
-	AudioSample *request(Flex *sfx_file, int id)
+	if (!loaded->second)
 	{
-		SFX_cached *loaded = find_sfx(id);
-		if (!loaded) {
-			SFX_cached new_sfx;
-			new_sfx.first = 0;
-			new_sfx.second = 0;
-			loaded = &(cache[id] = new_sfx);
-		}
+		garbage_collect();
 
-		if (!loaded->second)
-		{
-			garbage_collect();
-
-			size_t wavlen;			// Read .wav file.
-			uint8 *wavbuf = reinterpret_cast<uint8*>(sfx_file->retrieve(id, wavlen));
-			loaded->second = AudioSample::createAudioSample(wavbuf,wavlen);
-		}
-
-		if (!loaded->second) return 0;
-
-		// Increment counter
-		++loaded->first;
-
-		return loaded->second;
+		size_t wavlen;			// Read .wav file.
+		auto wavbuf = sfx_file->retrieve(id, wavlen);
+		loaded->second = AudioSample::createAudioSample(std::move(wavbuf), wavlen);
 	}
 
-	// Empties the cache.
-	void flush(AudioMixer *mixer = 0)
+	if (!loaded->second) return nullptr;
+
+	// Increment counter
+	++loaded->first;
+
+	return loaded->second;
+}
+
+// Empties the cache.
+void SFX_cache_manager::flush(AudioMixer *mixer)
+{
+	for (auto it = cache.begin() ; it != cache.end(); it = cache.begin())
 	{
-		for (cache_iterator it = cache.begin() ; it != cache.end(); it = cache.begin())
+		if (it->second.second) 
 		{
-			if (it->second.second) 
-			{
-				if (it->second.second->getRefCount() != 1 && mixer)
-					mixer->stopSample(it->second.second);
-				it->second.second->Release();
-			}
-			it->second.second = 0;
-			cache.erase(it);
+			if (it->second.second->getRefCount() != 1 && mixer)
+				mixer->stopSample(it->second.second);
+			it->second.second->Release();
+		}
+		it->second.second = nullptr;
+		cache.erase(it);
+	}
+}
+
+// Remove unused sounds from the cache.
+void SFX_cache_manager::garbage_collect()
+{
+	// Maximum 'stable' number of sounds we will cache (actual
+	// count may be higher if all of the cached sounds are
+	// being played).
+	const int max_fixed = 6;
+
+	std::multiset <int> sorted;
+
+	for (auto& it : cache)
+	{
+		if (it.second.second && it.second.second->getRefCount() == 1)
+			sorted.insert(it.second.first);
+	}
+
+	if (sorted.empty()) return;
+
+	int threshold = INT_MAX;
+	int count = 0;
+
+	for ( auto it = sorted.rbegin( ) ; it != sorted.rend( ); ++it )
+	{
+		if (count < max_fixed)
+		{
+			threshold = *it;
+			count++;
+		}
+		else if (*it == threshold)
+		{
+			count++;
+		}
+		else
+		{
+			break;
 		}
 	}
 
-	// Remove unused sounds from the cache.
-	void garbage_collect()
+	if (count <= max_fixed) 
+		return;
+
+	for (auto& it : cache)
 	{
-		// Maximum 'stable' number of sounds we will cache (actual
-		// count may be higher if all of the cached sounds are
-		// being played).
-		const int max_fixed = 6;
-
-		std::multiset <int> sorted;
-
-		for (cache_iterator it = cache.begin(); it != cache.end(); ++it)
+		if (it.second.second)
 		{
-			if (it->second.second && it->second.second->getRefCount() == 1) 
-				sorted.insert(it->second.first); 
-		}
-
-		if (sorted.empty()) return;
-
-		int threshold = INT_MAX;
-		int count = 0;
-
-		for ( std::multiset <int>::reverse_iterator it = sorted.rbegin( ) ; it != sorted.rend( ); ++it )
-		{
-			if (count < max_fixed)
+			if (it.second.first < threshold)
 			{
-				threshold = *it;
-				count++;
+				it.second.second->Release();
+				it.second.second = nullptr;
 			}
-			else if (*it == threshold)
+			else if (it.second.first == threshold && count > max_fixed)
 			{
-				count++;
-			}
-			else
-			{
-				break;
+				it.second.second->Release();
+				it.second.second = nullptr;
+				count--;
 			}
 		}
-
-		if (count <= max_fixed) 
-			return;
-
-		for (cache_iterator it = cache.begin(); it != cache.end(); ++it)
-		{
-			if (it->second.second)
-			{
-				if (it->second.first < threshold) 
-				{
-					it->second.second->Release();
-					it->second.second = 0;
-				}
-				else if (it->second.first == threshold && count > max_fixed) 
-				{
-					it->second.second->Release();
-					it->second.second = 0;
-					count--;
-				}
-			}
-		}
-
 	}
-};
+
+}
 
 //---- Audio ---------------------------------------------------------
-void Audio::Init(void)
+void Audio::Init()
 {
 	// Crate the Audio singleton object
 	if (!self)
 	{
-		int sample_rate = 22050;
+		int sample_rate = 44100;
 		bool stereo = true;
 
 		config->value("config/audio/sample_rate", sample_rate, sample_rate);
@@ -236,27 +218,25 @@ void Audio::Init(void)
 	}
 }
 
-void Audio::Destroy(void)
+void Audio::Destroy()
 {
 	delete self;
-	self = 0;
+	self = nullptr;
 }
 
-Audio	*Audio::get_ptr(void)
+Audio	*Audio::get_ptr()
 {
 	// The following assert here might be too harsh, maybe we should leave
 	// it to the caller to handle non-inited audio-system?
-	assert(self != NULL);
+	assert(self != nullptr);
 
 	return self;
 }
 
 
-Audio::Audio() :
-truthful_(false),speech_enabled(true), speech_with_subs(false), music_enabled(true),
-effects_enabled(true), initialized(false), mixer(0), sfx_file(0)
+Audio::Audio()
 {
-	assert(self == NULL);
+	assert(self == nullptr);
 
 	string s;
 
@@ -275,17 +255,15 @@ effects_enabled(true), initialized(false), mixer(0), sfx_file(0)
 	config->value("config/audio/midi/looping",s,"yes");
 	allow_music_looping = (s!="no");
 
-	mixer = 0;
-	sfxs = new SFX_cache_manager();
+	mixer.reset();
+	sfxs = std::make_unique<SFX_cache_manager>();
 }
 
 void Audio::Init(int _samplerate,int _channels)	
 {
 	if (!audio_enabled) return;
 
-	FORGET_OBJECT(mixer);
-
-	mixer = new AudioMixer(_samplerate,_channels==2,MIXER_CHANNELS);
+	mixer = std::make_unique<AudioMixer>(_samplerate,_channels==2,MIXER_CHANNELS);
 
 	COUT("Audio initialisation OK");
 
@@ -297,35 +275,25 @@ bool	Audio::can_sfx(const std::string &file, std::string *out)
 {
 	if (file.empty())
 		return false;
-	string d = file;
-	// Full path?
-	if (U7exists(d.c_str()))
-		{
-		if (out)
-			*out = d;
-		return true;
+	string options[] = {"", "<BUNDLE>", "<DATA>"};
+	for (auto& d : options) {
+		string f;
+		if (!d.empty()) {
+			if (!is_system_path_defined(d)) {
+				continue;
+			}
+			f = d;
+			f += '/';
+			f += file;
+		} else {
+			f = file;
 		}
-
-#ifdef MACOSX
-	// Check in the app bundle:
-	d = "<BUNDLE>/" + file;
-	if (is_system_path_defined("<BUNDLE>") && U7exists(d.c_str()))
-		{
-		if (out)
-			*out = d;
-		return true;
+		if (U7exists(f.c_str())) {
+			if (out)
+				*out = f;
+			return true;
 		}
-#endif
-
-	// Also just check in the actual data dir
-	d = "<DATA>/" + file;
-	if (U7exists(d.c_str()))
-		{
-		if (out)
-			*out = d;
-		return true;
-		}
-
+	}
 	return false;
 }
 
@@ -337,7 +305,7 @@ bool Audio::have_roland_sfx(Exult_Game game, std::string *out)
 		return can_sfx(SFX_ROLAND_SI, out);
 	return false;
 	}
-	
+
 bool Audio::have_sblaster_sfx(Exult_Game game, std::string *out)
 	{
 	if (game == BLACK_GATE)
@@ -346,7 +314,7 @@ bool Audio::have_sblaster_sfx(Exult_Game game, std::string *out)
 		return can_sfx(SFX_BLASTER_SI, out);
 	return false;
 	}
-	
+
 bool Audio::have_midi_sfx(std::string *out)
 	{
 #ifdef ENABLE_MIDISFX
@@ -356,7 +324,7 @@ bool Audio::have_midi_sfx(std::string *out)
 	return false;
 #endif
 	}
-	
+
 bool Audio::have_config_sfx(const std::string &game, std::string *out)
 	{
 	string s;
@@ -364,14 +332,14 @@ bool Audio::have_config_sfx(const std::string &game, std::string *out)
 	config->value(d.c_str(), s, "---");
 	return (s != "---") && can_sfx(s, out);
 	}
-	
+
 void	Audio::Init_sfx()
 {
-	FORGET_OBJECT(sfx_file);
+	sfx_file.reset();
 
 	if (sfxs)
 		{
-		sfxs->flush(mixer);
+		sfxs->flush(mixer.get());
 		sfxs->garbage_collect();
 		}
 
@@ -383,8 +351,8 @@ void	Audio::Init_sfx()
 	}
 	else
 	{
-		bg2si_sfxs = 0;
-		bg2si_songs = 0;
+		bg2si_sfxs = nullptr;
+		bg2si_songs = nullptr;
 	}
 	// Collection of .wav's?
 	string flex;
@@ -394,7 +362,7 @@ void	Audio::Init_sfx()
 	if (have_midi_sfx(&flex) && v != "no")
 		{
 		cout << "Opening midi SFX's file: \"" << flex << "\"" << endl;
-		sfx_file = new FlexFile(flex.c_str());
+		sfx_file = std::make_unique<FlexFile>(std::move(flex));
 		return;
 		}
 	else if (!have_midi_sfx(&flex))
@@ -424,16 +392,13 @@ void	Audio::Init_sfx()
 			}
 		}
 	cout << "Opening digital SFX's file: \"" << flex << "\"" << endl;
-	sfx_file = new FlexFile(flex.c_str());
+	sfx_file = std::make_unique<FlexFile>(std::move(flex));
 }
 
 Audio::~Audio()
 { 
 	if (!initialized)
 	{
-		delete sfxs;
-		delete sfx_file;
-		self = 0;
 		//SDL_open = false;
 		return;
 	}
@@ -441,31 +406,24 @@ Audio::~Audio()
 	CERR("~Audio:  about to stop_music()");
 	stop_music();
 
-	FORGET_OBJECT(mixer);
-
-	delete sfxs;
-	delete sfx_file;
-
 	CERR("~Audio:  about to quit subsystem");
-	self = 0;
 }
 
 void Audio::copy_and_play(const uint8 *sound_data, uint32 len, bool wait)
 {
-	uint8 *new_sound_data = new uint8[len];
-	std::memcpy(new_sound_data, sound_data, len);
-	play(new_sound_data, len, wait);
+	auto new_sound_data = std::make_unique<uint8[]>(len);
+	std::memcpy(new_sound_data.get(), sound_data, len);
+	play(std::move(new_sound_data), len, wait);
 }
 
-void Audio::play(uint8 *sound_data, uint32 len, bool wait)
+void Audio::play(std::unique_ptr<uint8[]> sound_data, uint32 len, bool wait)
 {
 	ignore_unused_variable_warning(wait);
 	if (!audio_enabled || !speech_enabled || !len) {
-		delete [] sound_data;
 		return;
 	}
 
-	AudioSample *audio_sample = AudioSample::createAudioSample(sound_data, len);
+	AudioSample *audio_sample = AudioSample::createAudioSample(std::move(sound_data), len);
 
 	if (audio_sample) {
 		mixer->playSample(audio_sample,0,128);
@@ -474,7 +432,7 @@ void Audio::play(uint8 *sound_data, uint32 len, bool wait)
 
 }
 
-void Audio::cancel_streams(void)
+void Audio::cancel_streams()
 {
 	if (!audio_enabled)
 		return;
@@ -484,7 +442,7 @@ void Audio::cancel_streams(void)
 
 }
 
-void Audio::pause_audio(void)
+void Audio::pause_audio()
 {
 	if (!audio_enabled)
 		return;
@@ -492,7 +450,7 @@ void Audio::pause_audio(void)
 	mixer->setPausedAll(true);
 }
 
-void 	Audio::resume_audio(void)
+void 	Audio::resume_audio()
 {
 	if (!audio_enabled)
 		return;
@@ -506,22 +464,21 @@ void Audio::playfile(const char *fname, const char *fpatch, bool wait)
 	if (!audio_enabled)
 		return;
 
-	U7multiobject sample(fname, fpatch, 1);
+	U7multiobject sample(fname, fpatch, 0);
 
 	size_t len;
-	uint8 *buf = reinterpret_cast<uint8*>(sample.retrieve(len));
+	auto buf = sample.retrieve(len);
 	if (!buf || len == 0) {
 		// Failed to find file in patch or static dirs.
 		CERR("Audio::playfile: Error reading file '" << fname << "'");
-		delete [] buf;
 		return;
 	}
 
-	play(buf, len, wait);
+	play(std::move(buf), len, wait);
 }
 
 
-bool	Audio::playing(void)
+bool	Audio::playing()
 {
 	return false;
 }
@@ -589,7 +546,7 @@ void	Audio::stop_music()
 	if (!audio_enabled) return;
 
 	if(mixer && mixer->getMidiPlayer())
-		mixer->getMidiPlayer()->stop_music();
+		mixer->getMidiPlayer()->stop_music(true);
 }
 
 bool Audio::start_speech(int num, bool wait)
@@ -614,14 +571,12 @@ bool Audio::start_speech(int num, bool wait)
 	U7multiobject sample(filename, patchfile, num);
 
 	size_t len;
-	uint8 *buf = reinterpret_cast<uint8*>(sample.retrieve(len));
-	if (!buf || len == 0)
-	{
-		delete [] buf;
+	auto buf = sample.retrieve(len);
+	if (!buf || len == 0) {
 		return false;
 	}
 
-	play(buf,len,wait);
+	play(std::move(buf), len, wait);
 	return true;
 }
 
@@ -643,18 +598,19 @@ int	Audio::play_sound_effect (int num, int volume, int balance, int repeat, int
 #ifdef ENABLE_MIDISFX
 	string v; // TODO: should make this check faster
 	config->value("config/audio/effects/midi", v, "no");
-	if (v != "no" && mixer && mixer->getMidiPlayer())
+	if (v != "no" && mixer && mixer->getMidiPlayer()) {
 		mixer->getMidiPlayer()->start_sound_effect(num);
-	else
+		return -1;
+	}
 #endif
 	// Where sort of sfx are we using????
-	if (sfx_file != 0)		// Digital .wav's?
+	if (sfx_file != nullptr)		// Digital .wav's?
 		return play_wave_sfx(num, volume, balance, repeat, distance);
 	return -1;
 }
 
 /*
-*	Play a .wav format sound effect, 
+*	Play a .wav format sound effect,
 *  return the channel number playing on or -1 if not playing, (0 is a valid channel in SDL_Mixer!)
 */
 int Audio::play_wave_sfx
@@ -666,20 +622,15 @@ int Audio::play_wave_sfx
 	int distance
 )
 {
-	if (!effects_enabled || !sfx_file || !mixer) 
+	if (!effects_enabled || !sfx_file || !mixer)
 		return -1;  // no .wav sfx available
 
-#if 0
-	if (Game::get_game_type() == BLACK_GATE)
-		num = bgconv[num];
-	CERR("; after bgconv:  " << num);
-#endif
 	if (num < 0 || static_cast<unsigned>(num) >= sfx_file->number_of_objects())
 	{
 		cerr << "SFX " << num << " is out of range" << endl;
 		return -1;
 	}
-	AudioSample *wave = sfxs->request(sfx_file, num);
+	AudioSample *wave = sfxs->request(sfx_file.get(), num);
 	if (!wave)
 	{
 		cerr << "Couldn't play sfx '" << num << "'" << endl;
@@ -694,13 +645,69 @@ int Audio::play_wave_sfx
 	}
 
 	CERR("Playing SFX: " << num);
-	
+
 	mixer->set2DPosition(instance_id,distance,balance);
 	mixer->setPaused(instance_id,false);
 
 	return instance_id;
 }
 
+/*
+*	This returns a 'unique' ID, but only for .wav SFX's (for now).
+*/
+int	Audio::play_sound_effect (const File_spec& sfxfile, int num, int volume, int balance, int repeat, int distance)
+{
+	if (!audio_enabled || !effects_enabled) return -1;
+	// TODO: No support for MIDI SFX at this time here.
+	return play_wave_sfx(sfxfile, num, volume, balance, repeat, distance);
+}
+
+/*
+*	Play a .wav format sound effect,
+*  return the channel number playing on or -1 if not playing, (0 is a valid channel in SDL_Mixer!)
+*/
+int Audio::play_wave_sfx
+(
+	const File_spec& sfxfile,
+	int num,
+	int volume,		// 0-256.
+	int balance,		// balance, -256 (left) - +256 (right)
+	int repeat,		// Keep playing.
+	int distance
+)
+{
+	if (!effects_enabled || !mixer || !U7exists(sfxfile.name)) {
+		return -1;  // no .wav sfx available
+	}
+	IExultDataSource ds(sfxfile, num);
+	if (!ds.good()) {
+		cerr << "SFX " << num << " from {" << sfxfile.name << ", " << sfxfile.index << "} is out of range" << endl;
+		return -1;
+	}
+
+	size_t wavlen;			// Read .wav file.
+	auto wavbuf = ds.steal_data(wavlen);
+	auto *wave = AudioSample::createAudioSample(std::move(wavbuf), wavlen);
+
+	int instance_id = mixer->playSample(wave, repeat, 0, true, AUDIO_DEF_PITCH, volume,volume);
+	// Either AudioMixer::playSample called IncRef through AudioChannel::playSample and the sample
+	// will be played, of the sample was not queued for playback.
+	// In either case we need to Release the sample to avoid a memory leak.
+	wave->Release();
+	if (instance_id < 0)
+	{
+		CERR("No channel was available to play sfx '" << num << "' from {" << sfxfile.name << ", " << sfxfile.index << "}");
+		return -1;
+	}
+
+	CERR("Playing SFX: " << num << " from {" << sfxfile.name << ", " << sfxfile.index << "}");
+
+	mixer->set2DPosition(instance_id, distance, balance);
+	mixer->setPaused(instance_id, false);
+
+	return instance_id;
+}
+
 /*
 static int slow_sqrt(int i)
 {
@@ -719,7 +726,7 @@ void Audio::get_2d_position_for_tile(const Tile_coord &tile, int &distance, int
 	balance = 0;
 
 	Game_window *gwin = Game_window::get_instance();
-	Rectangle size = gwin->get_win_tile_rect();
+	TileRect size = gwin->get_win_tile_rect();
 	Tile_coord apos(size.x+size.w/2,size.y+size.h/2,gwin->get_camera_actor()->get_lift());
 
 	int sqr_dist = apos.square_distance_screen_space(tile);
@@ -786,7 +793,7 @@ void Audio::stop_sound_effect(int chan)
 
 void Audio::stop_sound_effects()
 {
-	if (sfxs) sfxs->flush(mixer);
+	if (sfxs) sfxs->flush(mixer.get());
 
 #ifdef ENABLE_MIDISFX
 	if (mixer && mixer->getMidiPlayer())
@@ -819,7 +826,7 @@ void Audio::set_audio_enabled(bool ena)
 	{
 		audio_enabled = true;
 
-		int sample_rate = 22050;
+		int sample_rate = 44100;
 		bool stereo = true;
 
 		config->value("config/audio/sample_rate", sample_rate, sample_rate);
@@ -833,13 +840,18 @@ void Audio::set_audio_enabled(bool ena)
 	}
 }
 
-bool Audio::is_track_playing(int num)
+bool Audio::is_track_playing(int num) const
 {
-	MyMidiPlayer *midi = mixer?mixer->getMidiPlayer():0;
+	MyMidiPlayer *midi = get_midi();
 	return midi && midi->is_track_playing(num);
 }
 
-MyMidiPlayer *Audio::get_midi()
+bool Audio::is_voice_playing() const
 {
-	return mixer?mixer->getMidiPlayer():0;
+	return mixer && mixer->isPlayingVoice();
+}
+
+MyMidiPlayer *Audio::get_midi() const
+{
+	return mixer ? mixer->getMidiPlayer() : nullptr;
 }
