@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2000-2013  The Exult Team
+Copyright (C) 2000-2022  The Exult Team
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -25,6 +25,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <fstream>
 #include <climits>
 
+#include <vorbis/codec.h>
+
 #include "fnames.h"
 #include "exult.h"
 #include "game.h"
@@ -33,7 +35,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "../files/U7file.h"
 #include "../files/utils.h"
 #include "Midi.h"
-#include "XMidiFile.h"
 #include "MidiDriver.h"
 #include "LowLevelMidiDriver.h"
 #include "conv.h"
@@ -53,7 +54,6 @@ using std::cerr;
 using std::cout;
 using std::endl;
 using std::string;
-using std::strcpy;
 
 //
 // Midi devices types and conversions
@@ -90,6 +90,29 @@ using std::strcpy;
 #define SEQ_NUM_MUSIC	0
 #define SEQ_NUM_SFX		1
 
+static std::unique_ptr<IDataSource> open_music_flex(const std::string& flex, int num) {
+	// Try in patch dir first.
+	string pflex("<PATCH>/");
+	size_t prefix_len = 0;
+	if (flex[0] == '<')
+	{
+		prefix_len = flex.find(">/");
+		if (prefix_len != string::npos)
+			prefix_len += 2;
+		else
+			prefix_len = 0;
+	}
+
+	pflex += flex.c_str() + prefix_len;
+	if (is_system_path_defined("<BUNDLE>")) {
+		string bflex("<BUNDLE>/");
+		bflex += flex.c_str() + prefix_len;
+		return std::make_unique<IExultDataSource>(flex, bflex, pflex, num);
+	} else {
+		return std::make_unique<IExultDataSource>(flex, pflex, num);
+	}
+}
+
 void	MyMidiPlayer::start_music(int num,bool repeat,std::string flex)
 {
 	// No output device
@@ -127,7 +150,6 @@ void	MyMidiPlayer::start_music(int num,bool repeat,std::string flex)
 
 	// OGG Handling
 	if (ogg_enabled) {
-
 		// Play ogg for this track
 		if (ogg_play_track(flex,num,repeat))
 			return;
@@ -140,9 +162,11 @@ void	MyMidiPlayer::start_music(int num,bool repeat,std::string flex)
 		if (!midi_driver || (!is_mt32() && Game_window::get_instance()->is_bg_track(num) && flex == MAINMUS)) return;
 	}
 
+	if (!midi_driver)
+		return;
+
 	// Handle FM Synth
 	if (midi_driver->isFMSynth())  {
-
 		// use the fmsynth music, which is bank 3
 		if (flex == MAINMUS) flex = MAINMUS_AD;
 		// Bank 1 is BG menu/intro. We need Bank 4
@@ -151,42 +175,14 @@ void	MyMidiPlayer::start_music(int num,bool repeat,std::string flex)
 		else if (flex == MAINSHP_FLX) num--;
 	}
 
-	IDataSource *mid_data = 0;
-
-	// Try in patch dir first.
-	string pflex("<PATCH>/");
-	size_t prefix_len = 0;
-	if (flex[0] == '<')
-	{
-		prefix_len = flex.find(">/");
-		if (prefix_len != string::npos)
-			prefix_len+=2;
-		else
-			prefix_len = 0;
-
-	}
-	pflex += flex.c_str() + prefix_len;
-#ifdef MACOSX
-	string bflex("<BUNDLE>/");
-	bflex += flex.c_str() + prefix_len;
-#endif
-	mid_data =
-#ifdef MACOSX
-		is_system_path_defined("<BUNDLE>") ?
-			new IExultDataSource(flex, bflex, pflex, num):
-#endif
-			new IExultDataSource(flex, pflex, num);
-
+	std::unique_ptr<IDataSource> mid_data = open_music_flex(flex, num);
 	// Extra safety.
 	if (!mid_data->getSize())
-		{
-		delete mid_data;
+	{
 		return;
-		}
+	}
 
-	XMidiFile midfile(mid_data, setup_timbre_for_track(flex));
-
-	delete mid_data;
+	XMidiFile midfile(mid_data.get(), setup_timbre_for_track(flex));
 
 	// Now give the xmidi object to the midi device
 
@@ -214,7 +210,6 @@ void	MyMidiPlayer::start_music(std::string fname,int num,bool repeat)
 
 	// OGG Handling
 	if (ogg_enabled) {
-
 		// Play ogg for this track
 		if (ogg_play_track(fname,num,repeat))
 			return;
@@ -222,11 +217,11 @@ void	MyMidiPlayer::start_music(std::string fname,int num,bool repeat)
 		// If we failed to play the track, call stop to clean up and put us back into
 		// midi synth mode
 		ogg_stop_track();
-
-		// No midi driver so don't fall through
-		if (!midi_driver) return;
 	}
 
+	if (!midi_driver)
+		return;
+
 	// Handle FMSynth Stuff here
 	if (midi_driver->isFMSynth()) {
 		if (fname == ENDSCORE_XMI)
@@ -238,27 +233,21 @@ void	MyMidiPlayer::start_music(std::string fname,int num,bool repeat)
 	}
 
 	// Read the data into the XMIDI class
+	IFileDataSource mid_data(fname.c_str());
+	if (!mid_data.good()) {
+		return;
+	}
 
-	std::ifstream mid_file;
-	U7open(mid_file, fname.c_str());
-	if (!mid_file.good()) return;
-	IDataSource *mid_data = new IStreamDataSource(&mid_file);
-
-	XMidiFile midfile(mid_data, setup_timbre_for_track(fname));
-
-	delete mid_data;
-	mid_file.close();
+	XMidiFile midfile(&mid_data, setup_timbre_for_track(fname));
 
 	// Now give the xmidi object to the midi device
-
 	XMidiEventList *eventlist = midfile.GetEventList(num);
 	if (eventlist) {
 		midi_driver->startSequence(SEQ_NUM_MUSIC, eventlist, repeat, 255);
 	}
-
 }
 
-void MyMidiPlayer::set_timbre_lib(TimberLibrary lib)
+void MyMidiPlayer::set_timbre_lib(TimbreLibrary lib)
 {
 	// Fixme?? - This can be VERY SLOW
 	if (lib != timbre_lib)
@@ -271,7 +260,7 @@ void MyMidiPlayer::set_timbre_lib(TimberLibrary lib)
 int MyMidiPlayer::setup_timbre_for_track(std::string &str)
 {
 	// Default to GM
-	TimberLibrary lib = TIMBRE_LIB_GM;
+	TimbreLibrary lib = TIMBRE_LIB_GM;
 
 	// Both Games
 	if (str == MAINMUS) lib = TIMBRE_LIB_GAME;
@@ -284,6 +273,7 @@ int MyMidiPlayer::setup_timbre_for_track(std::string &str)
 
 	// SI
 	else if (str == MAINSHP_FLX) lib = TIMBRE_LIB_MAINMENU;
+	else if (str == EXULT_SI_FLX) lib = TIMBRE_LIB_INTRO;
 	else if (str == R_SINTRO) lib = TIMBRE_LIB_INTRO;
 	else if (str == A_SINTRO) lib = TIMBRE_LIB_INTRO;
 	else if (str == R_SEND) lib = TIMBRE_LIB_ENDGAME;
@@ -334,7 +324,7 @@ void MyMidiPlayer::load_timbres()
 		music_conversion != XMIDIFILE_CONVERT_NOCONVERSION)
 		return;
 
-	const char *u7voice = 0;
+	const char *u7voice = nullptr;
 
 	// Black Gate Settings
 	if (GAME_BG && timbre_lib == TIMBRE_LIB_INTRO)
@@ -385,10 +375,8 @@ void MyMidiPlayer::load_timbres()
 		}
 	}
 
-
 	if (timbre_lib_filename == filename && timbre_lib_index == index &&
-		timbre_lib_game == Game::get_game_type())
-	{
+		timbre_lib_game == Game::get_game_type()) {
 		return;
 	}
 
@@ -396,29 +384,25 @@ void MyMidiPlayer::load_timbres()
 	timbre_lib_index = index;
 	timbre_lib_game = Game::get_game_type();
 
-	std::ifstream file;
-	IDataSource *ds = 0;
+	std::unique_ptr<IDataSource> ds;
 
-	if (index == -1)
-	{
-		U7open(file, filename);
-		if (!file.good()) return;
-		ds = new IStreamDataSource(&file);
-	}
-	else if (index >= 0)
-	{
-		ds = new IExultDataSource(filename,index);
+	if (index == -1) {
+		ds = std::make_unique<IFileDataSource>(filename);
+		if (!ds->good()) {
+			return;
+		}
+	} else if (index >= 0) {
+		ds = std::make_unique<IExultDataSource>(filename, index);
 	}
 
-	midi_driver->loadTimbreLibrary(ds,type);
-
-	file.close();
-	delete ds;
+	// Note: ds can be null here if inde == -2. In this case, the pointer
+	// will never be used inside loadTimbreLibrary.
+	midi_driver->loadTimbreLibrary(ds.get(), type);
 }
 
-void	MyMidiPlayer::stop_music()
+void MyMidiPlayer::stop_music(bool quitting)
 {
-	if(!ogg_enabled && !midi_driver && !init_device(false))
+	if(!ogg_enabled && !midi_driver && !quitting && !init_device(false))
 		return;
 
 	current_track = -1;
@@ -438,7 +422,7 @@ bool MyMidiPlayer::is_track_playing(int num)
 	return false;
 }
 
-int MyMidiPlayer::get_current_track()
+int MyMidiPlayer::get_current_track() const
 {
 	if (current_track == -1) return -1;
 
@@ -503,7 +487,7 @@ void MyMidiPlayer::set_midi_driver(const std::string& desired_driver, bool use_o
 		stop_music();
 		if (midi_driver) midi_driver->destroyMidiDriver();
 		delete midi_driver;
-		midi_driver = 0;
+		midi_driver = nullptr;
 		initialized = false;
 	}
 
@@ -520,7 +504,7 @@ void MyMidiPlayer::set_midi_driver(const std::string& desired_driver, bool use_o
 bool MyMidiPlayer::init_device(bool timbre_load)
 {
 	// already initialized? Do this first
-	if (initialized) return (midi_driver != 0) || ogg_enabled;
+	if (initialized) return (midi_driver != nullptr) || ogg_enabled;
 
 	string	s;
 	string	driver_default = "default";
@@ -607,7 +591,7 @@ bool MyMidiPlayer::init_device(bool timbre_load)
 	// no need for a MIDI device (for now)
 	if (!sfx && !music)
 	{
-		midi_driver = 0;
+		midi_driver = nullptr;
 		return false;
 	}
 
@@ -655,7 +639,7 @@ bool MyMidiPlayer::init_device(bool timbre_load)
 // Check for true mt32, mt32emu or fakemt32
 // primarily for the mt32 background music tracks
 
-bool MyMidiPlayer::is_mt32()
+bool MyMidiPlayer::is_mt32() const
 {
 	return midi_driver && (midi_driver->isMT32() ||
 		get_music_conversion() == XMIDIFILE_CONVERT_NOCONVERSION ||
@@ -663,19 +647,12 @@ bool MyMidiPlayer::is_mt32()
 		!midi_driver->isFMSynth();
 }
 
-bool MyMidiPlayer::is_adlib()
+bool MyMidiPlayer::is_adlib() const
 {
 	return midi_driver && (midi_driver->isFMSynth());
 }
 
-MyMidiPlayer::MyMidiPlayer()	: repeating(false),current_track(-1),
-				  midi_driver_name("default"), midi_driver(0), initialized(false),
-				  timbre_lib(TIMBRE_LIB_GM),
-				  timbre_lib_index(0), timbre_lib_game(NONE),
-				  music_conversion(XMIDIFILE_CONVERT_MT32_TO_GM),
-				  effects_conversion(XMIDIFILE_CONVERT_GS127_TO_GS),
-				  ogg_enabled(false), ogg_instance_id(-1)
-
+MyMidiPlayer::MyMidiPlayer()
 {
 	init_device(false);
 }
@@ -687,7 +664,7 @@ MyMidiPlayer::~MyMidiPlayer()
 	{
 		midi_driver->destroyMidiDriver();
 		delete midi_driver;
-		midi_driver = 0;
+		midi_driver = nullptr;
 	}
 }
 
@@ -724,39 +701,25 @@ void    MyMidiPlayer::start_sound_effect(int num)
 	// Only support SFX on devices with 2 or more sequences
 	if (midi_driver->maxSequences() < 2) return;
 
-	char		*buffer;
-	size_t		size;
-	IDataSource *mid_data;
+	std::unique_ptr<IExultDataSource> mid_data;
+	if (is_system_path_defined("<BUNDLE>")) {
+		mid_data = std::make_unique<IExultDataSource>("<DATA>/midisfx.flx",
+		                                "<BUNDLE>/midisfx.flx", real_num);
+	} else {
+		mid_data = std::make_unique<IExultDataSource>("<DATA>/midisfx.flx", real_num);
+	}
 
-	U7object	*track =
-#ifdef MACOSX
-		is_system_path_defined("<BUNDLE>") ?
-			new U7multiobject("<DATA>/midisfx.flx",
-			                  "<BUNDLE>/midisfx.flx", real_num) :
-#endif
-			new U7object("<DATA>/midisfx.flx", real_num);
-
-	buffer = track->retrieve(size);
-	delete track;
-	if (!buffer || size <= 0)
-		{
-		delete [] buffer;
+	if (!mid_data->good()) {
 		return;
-		}
+	}
 
 	// Read the data into the XMIDI class
-	mid_data = new IBufferDataSource(buffer, size);
-
 	// It's already GM, so dont convert
-	XMidiFile		midfile(mid_data, effects_conversion);
-
-	delete mid_data;
-	delete [] buffer;
+	XMidiFile midfile(mid_data.get(), effects_conversion);
 
 	// Now give the xmidi object to the midi device
 	XMidiEventList *eventlist = midfile.GetEventList(0);
 	if (eventlist) midi_driver->startSequence(1,eventlist,false,255);
-
 }
 
 void    MyMidiPlayer::stop_sound_effects()
@@ -806,6 +769,10 @@ bool MyMidiPlayer::ogg_play_track(const std::string& filename, int num, bool rep
 			snprintf(outputstr, 255, "%02dbg.ogg", num);
 			ogg_name = outputstr;
 			}
+		else if (filename == EXULT_BG_FLX)
+			{
+			ogg_name = filename;
+			}
 		}
 	else if (Game::get_game_type() == SERPENT_ISLE)
 		{
@@ -833,6 +800,10 @@ bool MyMidiPlayer::ogg_play_track(const std::string& filename, int num, bool rep
 				ogg_name = outputstr;
 				}
 			}
+		else if (filename == EXULT_SI_FLX)
+			{
+			ogg_name = filename;
+			}
 		}
 	else
 		{
@@ -842,43 +813,36 @@ bool MyMidiPlayer::ogg_play_track(const std::string& filename, int num, bool rep
 		basepath = "<STATIC>/music/";
 		}
 
-	if (ogg_name.empty()) return false;
+	if (ogg_name.empty()) {
+		return false;
+	}
 
-	if (U7exists("<PATCH>/music/" + ogg_name))
-		ogg_name = get_system_path("<PATCH>/music/" + ogg_name);
-#ifdef MACOSX
-	else if (is_system_path_defined("<BUNDLE>") &&
-	         U7exists("<BUNDLE>/music/" + ogg_name))
-		ogg_name = get_system_path("<BUNDLE>/music/" + ogg_name);
-#endif
-	else
-		ogg_name = get_system_path(basepath + ogg_name);
+	const bool flex_source = ogg_name == filename;
+	auto ds = [&ogg_name, &basepath, num, flex_source]() -> std::unique_ptr<IDataSource> {
+		if (flex_source) {
+			return open_music_flex(ogg_name, num);
+		}
+		if (U7exists("<PATCH>/music/" + ogg_name)) {
+			ogg_name = get_system_path("<PATCH>/music/" + ogg_name);
+		} else if (is_system_path_defined("<BUNDLE>") &&
+				U7exists("<BUNDLE>/music/" + ogg_name)) {
+			ogg_name = get_system_path("<BUNDLE>/music/" + ogg_name);
+		} else {
+			ogg_name = get_system_path(basepath + ogg_name);
+		}
+		return std::make_unique<IFileDataSource>(ogg_name);
+	}();
 
 #ifdef DEBUG
 	cout << "OGG audio: Music track " << ogg_name << endl;
 #endif
 
-	std::ifstream *stream = 0;
-	try {
-		stream = new std::ifstream(ogg_name.c_str(), std::ios::in | std::ios::binary);
-	}
-	catch (std::exception &) {
-		delete stream;
+	if (!ds->good()) {
 		return false;
 	}
 
-	if (!stream->good()) {
-		delete stream;
-		return false;
-	}
-
-	IDataSource *ds = new IFileDataSource(stream);
-
-	if (!Pentagram::OggAudioSample::isThis(ds))
-	{
+	if (!Pentagram::OggAudioSample::isThis(ds.get())) {
 		std::cerr << "Failed to play OGG Music Track " << ogg_name << ". Reason: " << "Unknown" << std::endl;
-
-		delete ds;
 		return false;
 	}
 
@@ -890,7 +854,7 @@ bool MyMidiPlayer::ogg_play_track(const std::string& filename, int num, bool rep
 	}
 
 	ds->seek(0);
-	Pentagram::AudioSample *ogg_sample = new Pentagram::OggAudioSample(ds);
+	Pentagram::AudioSample *ogg_sample = new Pentagram::OggAudioSample(std::move(ds));
 
 	ogg_instance_id = mixer->playSample(ogg_sample, repeat?-1:0, INT_MAX);
 
@@ -899,7 +863,7 @@ bool MyMidiPlayer::ogg_play_track(const std::string& filename, int num, bool rep
 	return  true;
 }
 
-void MyMidiPlayer::ogg_stop_track(void)
+void MyMidiPlayer::ogg_stop_track()
 {
 	if (ogg_instance_id != -1) {
 		Pentagram::AudioMixer *mixer = Pentagram::AudioMixer::get_instance();
@@ -908,7 +872,7 @@ void MyMidiPlayer::ogg_stop_track(void)
 	}
 }
 
-bool MyMidiPlayer::ogg_is_playing(void)
+bool MyMidiPlayer::ogg_is_playing() const
 {
 	if (ogg_instance_id != -1) {
 		Pentagram::AudioMixer *mixer = Pentagram::AudioMixer::get_instance();
