@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2005 The Pentagram team
-Copyright (C) 2010 The Exult team
+Copyright (C) 2010-2022 The Exult team
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -26,13 +26,11 @@ namespace Pentagram {
 class RawAudioSample : public AudioSample
 {
 public:
-	RawAudioSample(uint8* buffer, uint32 size,
+	RawAudioSample(std::unique_ptr<uint8[]> buffer, uint32 size,
 				   uint32 rate, bool signeddata, bool stereo);
-	virtual ~RawAudioSample();
-
-	virtual void initDecompressor(void *DecompData) const;
-	virtual uint32 decompressFrame(void *DecompData, void *samples) const;
-	virtual void rewind(void *DecompData) const;
+	void initDecompressor(void *DecompData) const override;
+	uint32 decompressFrame(void *DecompData, void *samples) const override;
+	void freeDecompressor(void *DecompData) const override;
 
 protected:
 
