@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2010 The Exult team
+Copyright (C) 2010-2022 The Exult team
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -26,7 +26,7 @@ namespace Pentagram {
 	class WavAudioSample : public RawAudioSample
 	{
 	public:
-		WavAudioSample(uint8* buffer, uint32 size);
+		WavAudioSample(std::unique_ptr<uint8[]> buffer, uint32 size);
 
 		static bool isThis(IDataSource *ds);
 	};
