@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2003-2005  The Pentagram Team
-Copyright (C) 2010 The Exult team
+Copyright (C) 2010-2022  The Exult team
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -20,10 +20,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "pent_include.h"
 #include <cstring>
 
-#ifndef PENTAGRAM_IN_EXULT
-#include "IDataSource.h"
-#endif
-
 #include "LowLevelMidiDriver.h"
 #include "XMidiSequence.h"
 #include "XMidiFile.h"
@@ -132,12 +128,6 @@ static inline int ConvBaseToActual(uint32 address_base)
 using std::string;
 using std::endl;
 
-LowLevelMidiDriver::LowLevelMidiDriver() :
-    mutex(0), cbmutex(0),cond(0),
-    global_volume(255), thread(0)
-{
-}
-
 LowLevelMidiDriver::~LowLevelMidiDriver()
 {
 	// Just kill it
@@ -147,16 +137,12 @@ LowLevelMidiDriver::~LowLevelMidiDriver()
 		//destroyMidiDriver();
 		if (thread)
 		{
-#if SDL_VERSION_ATLEAST(2, 0, 0)
 			int status_thread;
 			quit_thread = true; // The thread should stop based upon this flag
 			SDL_WaitThread(thread, &status_thread);
-#else
-			SDL_KillThread(thread);
-#endif
 		}
 	}
-	thread = 0;
+	thread = nullptr;
 }
 
 //
@@ -171,9 +157,10 @@ int LowLevelMidiDriver::initMidiDriver(uint32 samp_rate, bool is_stereo)
 	string s;
 
 	// Reset the current stream states
-	std::memset(sequences, 0, sizeof (XMidiSequence*) * LLMD_NUM_SEQ);
-	std::memset(chan_locks, -1, sizeof (sint32) * 16);
-	std::memset(chan_map, -1, sizeof (sint32) * LLMD_NUM_SEQ * 16);
+	std::fill(std::begin(sequences), std::end(sequences), nullptr);
+	std::fill(std::begin(chan_locks), std::end(chan_locks), -1);
+	for (auto& chan : chan_map)
+		std::fill(std::begin(chan), std::end(chan), -1);
 	for (int i = 0; i < LLMD_NUM_SEQ; i++) {
 		playing[i] = false;
 		callback_data[i] = -1;
@@ -182,19 +169,22 @@ int LowLevelMidiDriver::initMidiDriver(uint32 samp_rate, bool is_stereo)
 	mutex = SDL_CreateMutex();
 	cbmutex = SDL_CreateMutex();
 	cond = SDL_CreateCond();
-	thread = 0;
+	thread = nullptr;
 	sample_rate = samp_rate;
 	stereo = is_stereo;
 	uploading_timbres = false;
 	next_sysex = 0;
 
 	// Zero the memory
-	std::memset(mt32_patch_banks,0,sizeof(mt32_patch_banks[0])*128);
-	std::memset(mt32_timbre_banks,0,sizeof(mt32_timbre_banks[0])*128);
-	std::memset(mt32_timbre_used,-1,sizeof(mt32_timbre_used[0])*64);
-	std::memset(mt32_bank_sel,0,sizeof(mt32_bank_sel[0])*LLMD_NUM_SEQ);
-	std::memset(mt32_patch_bank_sel,0,sizeof(mt32_patch_bank_sel[0])*128);
-	std::memset(mt32_rhythm_bank,0,sizeof(mt32_rhythm_bank[0])*128);
+	std::fill(std::begin(mt32_patch_banks), std::end(mt32_patch_banks), nullptr);
+	std::fill(std::begin(mt32_timbre_banks), std::end(mt32_timbre_banks), nullptr);
+	for (auto& timbre : mt32_timbre_used)
+		std::fill(std::begin(timbre), std::end(timbre), -1);
+	for (auto& bank : mt32_bank_sel)
+		std::fill(std::begin(bank), std::end(bank), 0);
+	// Note: This is inconsistent with the one from LowLevelMidiDriver::loadTimbreLibrary
+	std::fill(std::begin(mt32_patch_bank_sel), std::end(mt32_patch_bank_sel), 0);
+	std::fill(std::begin(mt32_rhythm_bank), std::end(mt32_rhythm_bank), nullptr);
 
 	int code = 0;
 
@@ -207,10 +197,10 @@ int LowLevelMidiDriver::initMidiDriver(uint32 samp_rate, bool is_stereo)
 		SDL_DestroyMutex(mutex);
 		SDL_DestroyMutex(cbmutex);
 		SDL_DestroyCond(cond);
-		thread = 0;
-		mutex = 0;
-		cbmutex = 0;
-		cond = 0;
+		thread = nullptr;
+		mutex = nullptr;
+		cbmutex = nullptr;
+		cond = nullptr;
 	}
 	else
 		initialized = true;
@@ -232,10 +222,10 @@ void LowLevelMidiDriver::destroyMidiDriver()
 	SDL_DestroyMutex(mutex);
 	SDL_DestroyMutex(cbmutex);
 	SDL_DestroyCond(cond);
-	cbmutex = 0;
-	mutex = 0;
-	thread = 0;
-	cond = 0;
+	cbmutex = nullptr;
+	mutex = nullptr;
+	thread = nullptr;
+	cond = nullptr;
 
 	giveinfo();
 }
@@ -430,11 +420,7 @@ int LowLevelMidiDriver::initThreadedSynth()
 	sendComMessage(message);
 
 	quit_thread = false;
-#if SDL_VERSION_ATLEAST(2, 0, 0)
 	thread = SDL_CreateThread (threadMain_Static, "LowLevelMidiDriver", this);
-#else
-	thread = SDL_CreateThread (threadMain_Static, this);
-#endif
 
 	while (peekComMessageType() == LLMD_MSG_THREAD_INIT)
 		yield ();
@@ -481,13 +467,9 @@ void LowLevelMidiDriver::destroyThreadedSynth()
 	// We waited a while and it still didn't terminate
 	if (count == 400 && peekComMessageType() != 0) {
 		perr << "MidiPlayer Thread failed to stop in time. Killing it." << std::endl;
-#if SDL_VERSION_ATLEAST(2, 0, 0)
 		int status_thread;
 		quit_thread = true; // The thread should stop based upon this flag
 		SDL_WaitThread(thread, &status_thread);
-#else
-		SDL_KillThread (thread);
-#endif
 	}
 
 	lockComMessage();
@@ -502,7 +484,7 @@ void LowLevelMidiDriver::destroyThreadedSynth()
 int LowLevelMidiDriver::threadMain_Static(void *data)
 {
 	giveinfo();
-	LowLevelMidiDriver *ptr=static_cast<LowLevelMidiDriver *>(data);
+	auto *ptr=static_cast<LowLevelMidiDriver *>(data);
 	giveinfo();
 	return ptr->threadMain();
 }
@@ -731,7 +713,7 @@ bool LowLevelMidiDriver::playSequences ()
 			else if (pending_events == -1)
 			{
 				delete sequences[seq];
-				sequences[seq] = 0;
+				sequences[seq] = nullptr;
 				lockComMessage();
 				playing[seq] = false;
 				unlockComMessage();
@@ -755,7 +737,7 @@ bool LowLevelMidiDriver::playSequences ()
 			case LLMD_MSG_FINISH:
 				{
 					delete sequences[message.sequence];
-					sequences[message.sequence] = 0;
+					sequences[message.sequence] = nullptr;
 					playing[message.sequence] = false;
 					callback_data[message.sequence] = -1;
 					unlockAndUnprotectChannel(message.sequence);
@@ -767,7 +749,7 @@ bool LowLevelMidiDriver::playSequences ()
 					for (i = 0; i < LLMD_NUM_SEQ; i++)
 					{
 						delete sequences[i];
-						sequences[i] = 0;
+						sequences[i] = nullptr;
 						playing[i] = false;
 						callback_data[i] = -1;
 						unlockAndUnprotectChannel(i);
@@ -812,7 +794,7 @@ bool LowLevelMidiDriver::playSequences ()
 				{
 					// Kill the previous stream
 					delete sequences[message.sequence];
-					sequences[message.sequence] = 0;
+					sequences[message.sequence] = nullptr;
 					playing[message.sequence] = false;
 					callback_data[message.sequence] = -1;
 					unlockAndUnprotectChannel(message.sequence);
@@ -859,7 +841,7 @@ bool LowLevelMidiDriver::playSequences ()
 						{
 							int bank_sel[16] = {0};
 
-							for (XMidiEvent *e = message.data.play.list->x_patch_bank; e != 0; e=e->next_patch_bank)
+							for (XMidiEvent *e = message.data.play.list->x_patch_bank; e != nullptr; e=e->next_patch_bank)
 							{
 								if ((e->status >> 4) == MIDI_STATUS_CONTROLLER)
 								{
@@ -1130,7 +1112,9 @@ int LowLevelMidiDriver::lockChannel(uint16 sequence_id, int chan, bool lock)
 	if (lock)
 	{
 		int notes_on[16] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
-		int s, c, phys;
+		int s;
+		int c;
+		int phys;
 
 		for (s = 0; s < LLMD_NUM_SEQ; s++)
 		{
@@ -1243,7 +1227,8 @@ int LowLevelMidiDriver::unlockAndUnprotectChannel(uint16 sequence_id)
 void LowLevelMidiDriver::loadTimbreLibrary(IDataSource *ds, TimbreLibraryType type)
 {
 	// Ensure all sequences are stopped
-	uint32 i,j;
+	uint32 i;
+	uint32 j;
 	for (i = 0 ; i < LLMD_NUM_SEQ; i++) {
 		ComMessage message(LLMD_MSG_FINISH, i);
 		sendComMessage(message);
@@ -1271,22 +1256,24 @@ void LowLevelMidiDriver::loadTimbreLibrary(IDataSource *ds, TimbreLibraryType ty
 	}
 
 	// Zero the memory
-	std::memset(mt32_patch_banks,0,sizeof(mt32_patch_banks[0])*128);
+	std::fill(std::begin(mt32_patch_banks), std::end(mt32_patch_banks), nullptr);
 
 	// Zero the memory
-	std::memset(mt32_timbre_banks,0,sizeof(mt32_timbre_banks[0])*128);
+	std::fill(std::begin(mt32_timbre_banks), std::end(mt32_timbre_banks), nullptr);
 
 	// Mask it out
-	std::memset(mt32_timbre_used,-1,sizeof(mt32_timbre_used[0])*64);
+	for (auto& timbre : mt32_timbre_used)
+		std::fill(std::begin(timbre), std::end(timbre), -1);
 
 	// Zero the memory
-	std::memset(mt32_bank_sel,0,sizeof(mt32_bank_sel[0])*LLMD_NUM_SEQ);
+	for (auto& bank : mt32_bank_sel)
+		std::fill(std::begin(bank), std::end(bank), 0);
 
-	// Zero the memory
-	std::memset(mt32_patch_bank_sel,-1,sizeof(mt32_patch_bank_sel[0])*128);
+	// Zero the memory; note: this is inconsistent with LowLevelMidiDriver::initMidiDriver
+	std::fill(std::begin(mt32_patch_bank_sel), std::end(mt32_patch_bank_sel), -1);
 
 	// Zero
-	std::memset(mt32_rhythm_bank,0,sizeof(mt32_rhythm_bank[0])*128);
+	std::fill(std::begin(mt32_rhythm_bank), std::end(mt32_rhythm_bank), nullptr);
 
 	// Setup Default Patch library
 	mt32_patch_banks[0] = new MT32Patch*[128];
@@ -1321,11 +1308,11 @@ void LowLevelMidiDriver::loadTimbreLibrary(IDataSource *ds, TimbreLibraryType ty
 		{87, {0x04, 0x64, 0x08, 0x01}}
 	};
 
-	for (unsigned ii = 0; ii < array_size(default_rhythms); ii++) {
-		loadRhythm(default_rhythms[ii].rhythm, default_rhythms[ii].note);
+	for (const auto& default_rhythm : default_rhythms) {
+		loadRhythm(default_rhythm.rhythm, default_rhythm.note);
 	}
 
-	XMidiFile *xmidi = 0;
+	XMidiFile *xmidi = nullptr;
 
 	if (type == TIMBRE_LIBRARY_U7VOICE_MT)
 		xmidi = new XMidiFile(ds,XMIDIFILE_HINT_U7VOICE_MT_FILE);
@@ -1393,7 +1380,7 @@ void LowLevelMidiDriver::loadTimbreLibrary(IDataSource *ds, TimbreLibraryType ty
 		uploading_timbres = false;
 
 		ComMessage precache(LLMD_MSG_PRECACHE_TIMBRES, -1);
-		precache.data.precache.list = 0;
+		precache.data.precache.list = nullptr;
 		sendComMessage(precache);
 	}
 	delete xmidi;
@@ -1415,7 +1402,7 @@ void LowLevelMidiDriver::extractTimbreLibrary(XMidiEventList *eventlist)
 	const uint32 timbre_unk_add_end = timbre_unk_add_start + timbre_mem_offset(128);
 
 	int i = 0;
-	XMidiEvent **next = 0;
+	XMidiEvent **next = nullptr;
 	for (XMidiEvent **event = &eventlist->events; *event; event = next)
 	{
 		next = &((*event)->next);
@@ -1441,20 +1428,19 @@ void LowLevelMidiDriver::extractTimbreLibrary(XMidiEventList *eventlist)
 			if ((actual_address+sysex_size) >= timbre_add_start && actual_address < timbre_add_end)
 			{
 				uint32 start = actual_address;
-				uint32 size = sysex_size;
+				// uint32 size = sysex_size;
 				if (actual_address < timbre_add_start)
 				{
 					sysex_size -= timbre_add_start-actual_address;
 					actual_address = timbre_add_start;
 				}
-				if ((start+size) > timbre_add_end) size = timbre_add_end-start;
+				// if ((start+size) > timbre_add_end) size = timbre_add_end-start;
 				start -= timbre_add_start;
 				start /= 256;
 
 				// Allocate memory
 				if (!mt32_timbre_banks[2]) {
-					mt32_timbre_banks[2] = new MT32Timbre*[128];
-					std::memset(mt32_timbre_banks[2],0,sizeof(mt32_timbre_banks[2][0])*128);
+					mt32_timbre_banks[2] = new MT32Timbre*[128]{};
 				}
 				if (!mt32_timbre_banks[2][start]) mt32_timbre_banks[2][start] = new MT32Timbre;
 
@@ -1507,7 +1493,7 @@ void LowLevelMidiDriver::extractTimbreLibrary(XMidiEventList *eventlist)
 				uint32 size = sysex_size;
 				if (actual_address < rhythm_add_start)
 				{
-					sysex_size -= rhythm_add_start-actual_address;
+					// sysex_size -= rhythm_add_start-actual_address;
 					actual_address = rhythm_add_start;
 				}
 				if ((start+size) > rhythm_add_end) size = rhythm_add_end-start;
@@ -1541,7 +1527,7 @@ void LowLevelMidiDriver::extractTimbreLibrary(XMidiEventList *eventlist)
 				XMidiEvent *e = *event;
 				*event = *next;
 				next = event;
-				e->next = 0;
+				e->next = nullptr;
 				e->FreeThis();
 			}
 			else {
@@ -1552,7 +1538,7 @@ void LowLevelMidiDriver::extractTimbreLibrary(XMidiEventList *eventlist)
 	}
 }
 
-// If data is NULL, then it is assumed that sysex_buffer already contains the data
+// If data is nullptr, then it is assumed that sysex_buffer already contains the data
 // address_base is 7-bit, while address_offset is 8 bit!
 void LowLevelMidiDriver::sendMT32SystemMessage(uint32 address_base, uint16 address_offset, uint32 len, const void *data)
 {
@@ -1713,7 +1699,7 @@ void LowLevelMidiDriver::loadRhythmTemp(int temp)
 	sendMT32SystemMessage(rhythm_base,rhythm_mem_offset(temp),rhythm_mem_size, mt32_rhythm_bank[temp] );
 
 	delete mt32_rhythm_bank[temp];
-	mt32_rhythm_bank[temp] = 0;
+	mt32_rhythm_bank[temp] = nullptr;
 }
 
 void LowLevelMidiDriver::uploadTimbre(int bank, int patch)
@@ -1793,7 +1779,7 @@ void LowLevelMidiDriver::loadXMidiTimbreLibrary(IDataSource *ds)
 	uint32 i;
 
 	// Read all the timbres
-	for (i = 0; ds->getPos() < ds->getSize(); i++) {
+	for (i = 0; ds->getAvail() > 0; i++) {
 		// Seek to the entry
 		ds->seek(i*6);
 
@@ -1824,14 +1810,12 @@ void LowLevelMidiDriver::loadXMidiTimbreLibrary(IDataSource *ds)
 
 		// Allocate memory
 		if (!mt32_timbre_banks[bank]) {
-			mt32_timbre_banks[bank] = new MT32Timbre*[128];
-			std::memset(mt32_timbre_banks[bank],0,sizeof(mt32_timbre_banks[bank][0])*128);
+			mt32_timbre_banks[bank] = new MT32Timbre*[128]{};
 		}
 		if (!mt32_timbre_banks[bank][patch]) mt32_timbre_banks[bank][patch] = new MT32Timbre;
 
 		if (!mt32_patch_banks[bank]) {
-			mt32_patch_banks[bank] = new MT32Patch*[128];
-			std::memset(mt32_patch_banks[bank],0,sizeof(mt32_patch_banks[bank][0])*128);
+			mt32_patch_banks[bank] = new MT32Patch*[128]{};
 		}
 		if (!mt32_patch_banks[bank][patch]) mt32_patch_banks[bank][patch] = new MT32Patch;
 
