@@ -1,5 +1,6 @@
 /*
 Copyright (C) 2004-2005  The Pentagram Team
+Copyright (C) 2019-2022  The Exult Team
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -37,12 +38,12 @@ public:
 	UnixSeqMidiDriver();
 
 protected:
-	virtual int			open();
-	virtual void		close();
-	virtual void		send(uint32 message);
-//	virtual void		yield();
-	virtual void		send_sysex(uint8 status, const uint8 *msg,
-								   uint16 length);
+	int			open() override;
+	void		close() override;
+	void		send(uint32 b) override;
+//	void		yield() override;
+	void		send_sysex(uint8 status, const uint8 *msg,
+								   uint16 length) override;
 
 	std::string devname;
 	bool isOpen;
