@@ -1,4 +1,6 @@
 /*
+ *  Copyright (C) 2000-2022 The Exult Team
+ *  
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -19,14 +21,12 @@
 
 class SoundTester
 	{
-	int song;
-	int sfx;
-	int voice;
-	int active;
-	bool repeat;
+	int song = 0;
+	int sfx = 0;
+	int voice = 0;
+	int active = 0;
+	bool repeat = true;
 public:
-		SoundTester();
-		~SoundTester();
 		void test_sound();
 	};
 	
