@@ -1,7 +1,7 @@
 /*
  *  combat.h - Combat scheduling.
  *
- *  Copyright (C) 2000-2013  The Exult Team
+ *  Copyright (C) 2000-2022  The Exult Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,10 +18,13 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#include "shapeinf.h"
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
 
+#include <memory>
+
 #include "actors.h"
 #include "combat.h"
 #include "combat_opts.h"
@@ -86,7 +89,7 @@ bool In_ammo_family(int shnum, int family) {
 	if (shnum == family)
 		return true;
 	const Ammo_info *ainf = ShapeID::get_info(shnum).get_ammo_info();
-	return (ainf != 0 && ainf->get_family_shape() == family);
+	return ainf != nullptr && ainf->get_family_shape() == family;
 }
 
 /*
@@ -103,10 +106,11 @@ void Combat_schedule::start_battle(
 	unsigned long curtime = Game::get_ticks();
 	// If this is the avatar, and it has been at least .5 minute since last
 	// start, then change music. This allows the danger music to be heard.
+	Game_object *target = npc->get_target();
 	if (npc == gwin->get_main_actor() && curtime - battle_time >= 30000 &&
-	        (!opponents.empty() || (npc->get_target() && npc->get_target()->as_actor()))) {
+	        (!opponents.empty() || (target && target->as_actor()))) {
 		Audio::get_ptr()->start_music_combat((rand() % 2) ?
-		                                     CSAttacked1 : CSAttacked2, 0);
+		                                     CSAttacked1 : CSAttacked2, false);
 		battle_time = curtime;
 		battle_end_time = curtime - 1;
 	}
@@ -125,9 +129,7 @@ void Combat_schedule::monster_died(
 		return;         // No, it's over.
 	Actor_vector nearby;        // Get all nearby NPC's.
 	gwin->get_nearby_npcs(nearby);
-	for (Actor_vector::const_iterator it = nearby.begin();
-	        it != nearby.end(); ++it) {
-		Actor *actor = *it;
+	for (auto *actor : nearby) {
 		if (!actor->is_dead() &&
 		        actor->get_attack_mode() != Actor::flee &&
 		        actor->get_effective_alignment() >= Actor::evil)
@@ -138,7 +140,7 @@ void Combat_schedule::monster_died(
 	unsigned long len = (battle_end_time - battle_time) / 1000;
 	bool hard = len > 15u && (rand() % 60u < len);
 	Audio::get_ptr()->start_music_combat(hard ? CSBattle_Over
-	                                     : CSVictory, 0);
+	                                     : CSVictory, false);
 }
 
 /*
@@ -152,11 +154,9 @@ void Combat_schedule::stop_attacking_npc(
 ) {
 	Actor_vector nearby;        // Get all nearby NPC's.
 	gwin->get_nearby_npcs(nearby);
-	for (Actor_vector::const_iterator it = nearby.begin();
-	        it != nearby.end(); ++it) {
-		Actor *actor = *it;
+	for (auto *actor : nearby) {
 		if (actor->get_target() == npc)
-			actor->set_target(0);
+			actor->set_target(nullptr);
 	}
 }
 
@@ -170,11 +170,9 @@ void Combat_schedule::stop_attacking_invisible(
 ) {
 	Actor_vector nearby;        // Get all nearby NPC's.
 	gwin->get_nearby_npcs(nearby);
-	for (Actor_vector::const_iterator it = nearby.begin();
-	        it != nearby.end(); ++it) {
-		Actor *actor = *it;
+	for (auto *actor : nearby) {
 		if (actor->get_target() == npc && !actor->can_see_invisible())
-			actor->set_target(0);
+			actor->set_target(nullptr);
 	}
 }
 
@@ -234,12 +232,18 @@ bool Combat_schedule::teleport(
 	         src.tx % c_tiles_per_chunk, src.ty % c_tiles_per_chunk);
 	if (src.tz < 0)
 		src.tz = 0;
-	eman->add_effect(new Fire_field_effect(src));
+	const Shape_info& inf = npc->get_info();
+	const bool fire_safe = inf.get_monster_info_safe()->get_immune() & (1 << Weapon_data::fire_damage);
+	eman->add_effect(std::make_unique<Fire_field_effect>(src, npc->get_property(Actor::intelligence), fire_safe));
 	int sfx = Audio::game_sfx(43);
 	Audio::get_ptr()->play_sound_effect(sfx, npc);  // The weird noise.
+	// check line of sight now to give it the appearance of the spell 
+	// failing as in the original
+	if (!Fast_pathfinder_client::is_straight_path(npc, trg))
+		return true;
 	npc->move(dest.tx, dest.ty, dest.tz);
 	// Show the stars.
-	eman->add_effect(new Sprites_effect(7, npc, 0, 0, 0, 0));
+	eman->add_effect(std::make_unique<Sprites_effect>(7, npc, 0, 0, 0, 0));
 	return true;
 }
 
@@ -249,13 +253,9 @@ bool Combat_schedule::teleport(
 
 bool Combat_schedule::summon(
 ) {
-#if 0   // ++++Testing
-	unsigned int curtime = SDL_GetTicks();
-	if (curtime < summon_time)
+	Game_object *trg = npc->get_target();
+	if (!Fast_pathfinder_client::is_straight_path(npc, trg))
 		return false;
-	// Not again for 30-40 seconds.
-	summon_time = curtime + 30000 + rand() % 10000;
-#endif
 	ucmachine->call_usecode(SummonSpellUsecode,
 	                        npc, Usecode_machine::double_click);
 	npc->start_std();       // Back into queue.
@@ -268,18 +268,9 @@ bool Combat_schedule::summon(
 
 bool Combat_schedule::be_invisible(
 ) {
-#if 0   // ++++ Testing.
-	unsigned int curtime = SDL_GetTicks();
-	if (curtime < invisible_time)
-		return false;
-	// Not again for 40-60 seconds.
-	invisible_time = curtime + 40000 + rand() % 20000;
-	if (npc->get_flag(Obj_flags::invisible))
-		return false;   // Also don't to it again for a while.
-#endif
 	Object_sfx::Play(npc, Audio::game_sfx(44));
 	gwin->get_effects()->add_effect(
-	    new Sprites_effect(12, npc, 0, 0, 0, 0, 0, -1));
+	    std::make_unique<Sprites_effect>(12, npc, 0, 0, 0, 0, 0, -1));
 	npc->set_flag(Obj_flags::invisible);
 	npc->add_dirty();
 	npc->start_std();       // Back into queue.
@@ -296,8 +287,8 @@ inline bool Off_screen(
 ) {
 	// See if off screen.
 	Tile_coord t = npc->get_tile();
-	Rectangle screen = gwin->get_win_tile_rect().enlarge(2);
-	return (!screen.has_world_point(t.tx, t.ty));
+	TileRect screen = gwin->get_win_tile_rect().enlarge(2);
+	return !screen.has_world_point(t.tx, t.ty);
 }
 
 bool Combat_schedule::is_enemy(
@@ -327,7 +318,8 @@ void Combat_schedule::find_opponents(
 ) {
 	opponents.clear();
 	Game_window *gwin = Game_window::get_instance();
-	Actor_vector nearby, sleeping;            // Get all nearby NPC's.
+	Actor_vector nearby;
+	Actor_vector sleeping;            // Get all nearby NPC's.
 	gwin->get_nearby_npcs(nearby);
 	Actor *avatar = gwin->get_main_actor();
 	nearby.push_back(avatar);   // Incl. Avatar!
@@ -337,9 +329,7 @@ void Combat_schedule::find_opponents(
 	bool in_party = npc->is_in_party() || npc == avatar;
 	int npc_align = npc->get_effective_alignment();
 	bool see_invisible = npc->can_see_invisible();
-	for (Actor_vector::const_iterator it = nearby.begin();
-	        it != nearby.end(); ++it) {
-		Actor *actor = *it;
+	for (auto *actor : nearby) {
 		if (actor->is_dead() || (!see_invisible && actor->get_flag(Obj_flags::invisible)))
 			continue;   // Dead or invisible.
 		if (is_enemy(npc_align, actor->get_effective_alignment())) {
@@ -349,7 +339,7 @@ void Combat_schedule::find_opponents(
 				if (combat_trace)
 					cout << npc->get_name() << " pushed back(asleep,1) " << actor->get_name() << endl;
 			} else {
-				opponents.push_back(actor);
+				opponents.push_back(actor->weak_from_this());
 				if (combat_trace)
 					cout << npc->get_name() << " pushed back(1) " << actor->get_name() << endl;
 			}
@@ -360,7 +350,7 @@ void Combat_schedule::find_opponents(
 			// Is actor attacking a party member?
 			if ((t->get_flag(Obj_flags::in_party) || t == avatar) &&
 			        t->as_actor() && is_enemy(npc_align, t->as_actor()->get_effective_alignment())) {
-				opponents.push_back(actor);
+				opponents.push_back(actor->weak_from_this());
 				if (combat_trace)
 					cout << npc->get_name() << " pushed back(2) " << actor->get_name() << endl;
 			}
@@ -368,10 +358,11 @@ void Combat_schedule::find_opponents(
 			if (oppressor < 0)
 				continue;
 			Actor *oppr = gwin->get_npc(oppressor);
+			assert(oppr != nullptr);
 			// Is actor being attacked by a party member?
 			if ((oppr->get_flag(Obj_flags::in_party) || oppr == avatar) &&
 			        is_enemy(npc_align, oppr->get_effective_alignment())) {
-				opponents.push_back(actor);
+				opponents.push_back(actor->weak_from_this());
 				if (combat_trace)
 					cout << npc->get_name() << " pushed back(3) " << actor->get_name() << endl;
 			}
@@ -381,10 +372,10 @@ void Combat_schedule::find_opponents(
 	if (opponents.empty() && in_party &&
 	        avatar->get_effective_alignment() == npc_align) {
 		Game_object *opp = avatar->get_target();
-		Actor *oppnpc = opp ? opp->as_actor() : 0;
+		Actor *oppnpc = opp ? opp->as_actor() : nullptr;
 		if (oppnpc && oppnpc != npc
 		        && oppnpc->get_schedule_type() == Schedule::combat) {
-			opponents.push_back(oppnpc);
+			opponents.push_back(oppnpc->weak_from_this());
 			if (combat_trace)
 				cout << npc->get_name() << " pushed back (4) " << oppnpc->get_name() << endl;
 		}
@@ -392,7 +383,9 @@ void Combat_schedule::find_opponents(
 	// Still none? Try the sleeping/unconscious foes.
 	if (opponents.empty() && !sleeping.empty()
 	    && (npc != avatar || charmed_avatar)) {
-		opponents.insert(opponents.begin(), sleeping.begin(), sleeping.end());
+		for (auto *act : sleeping) {
+		    opponents.push_front(weak_from_obj(act));
+		}
 		if (combat_trace)
 			cout << npc->get_name() << " pushed back asleep opponents" << endl;
 	}
@@ -404,14 +397,14 @@ void Combat_schedule::find_opponents(
  *  Output: ->attacker in opponents, or opponents.end() if not found.
  */
 
-list<Actor *>::iterator Combat_schedule::find_protected_attacker(
+list<Game_object_weak>::iterator Combat_schedule::find_protected_attacker(
 ) {
 	if (!npc->is_in_party())    // Not in party?
 		return opponents.end();
 	Game_window *gwin = Game_window::get_instance();
 	Actor *party[9];        // Get entire party, including Avatar.
 	int cnt = gwin->get_party(party, 1);
-	Actor *prot_actor = 0;
+	Actor *prot_actor = nullptr;
 	for (int i = 0; i < cnt; i++)
 		if (party[i]->is_combat_protected()) {
 			prot_actor = party[i];
@@ -420,13 +413,14 @@ list<Actor *>::iterator Combat_schedule::find_protected_attacker(
 	if (!prot_actor)        // Not found?
 		return opponents.end();
 	// Find closest attacker.
-	int dist, best_dist = 4 * c_tiles_per_chunk;
-	list<Actor *>::iterator best_opp = opponents.end();
-	for (list<Actor *>::iterator it = opponents.begin();
+	int best_dist = 4 * c_tiles_per_chunk;
+	auto best_opp = opponents.end();
+	for (auto it = opponents.begin();
 	        it != opponents.end(); ++it) {
-		Actor *opp = *it;
-		if (opp->get_target() == prot_actor &&
-		        (dist = npc->distance(opp)) < best_dist) {
+		Actor_shared opp = std::static_pointer_cast<Actor>((*it).lock());
+		int dist;
+		if (opp && opp->get_target() == prot_actor &&
+		        (dist = npc->distance(opp.get())) < best_dist) {
 			best_dist = dist;
 			best_opp = it;
 		}
@@ -465,26 +459,29 @@ Game_object *Combat_schedule::find_foe(
 	bool charmed_avatar = Combat::charmed_more_difficult &&
 	                      avatar->get_effective_alignment() !=  Actor::good;
 	// Remove any that died.
-	for (list<Actor *>::iterator it = opponents.begin();
-	        it != opponents.end();) {
-		if ((*it)->is_dead() ||
-		        (npc == avatar && !charmed_avatar && (*it)->get_flag(Obj_flags::asleep)))
-			it = opponents.erase(it);
-		else
-			++it;
-	}
+	opponents.erase(std::remove_if(opponents.begin(), opponents.end(),
+		[this, avatar, charmed_avatar](const Game_object_weak& curr) {
+			Actor_shared actor = std::static_pointer_cast<Actor>(curr.lock());
+			return !actor || actor->is_dead() ||
+			       (npc == avatar && !charmed_avatar &&
+			        actor->get_flag(Obj_flags::asleep));
+		}
+		), opponents.end());
 	if (opponents.empty()) { // No more from last scan?
 		find_opponents();   // Find all nearby.
 		if (practice_target)    // For dueling.
 			return practice_target;
 	}
-	list<Actor *>::iterator new_opp_link = opponents.end();
+	auto new_opp_link = opponents.end();
 	switch (static_cast<Actor::Attack_mode>(mode)) {
 	case Actor::weakest: {
-		int str, least_str = 100;
-		for (list<Actor *>::iterator it = opponents.begin();
+		int str;
+		int least_str = 100;
+		for (auto it = opponents.begin();
 		        it != opponents.end(); ++it) {
-			Actor *opp = *it;
+			Actor_shared opp = std::static_pointer_cast<Actor>((*it).lock());
+			if (!opp)
+			    continue;
 			str = opp->get_property(Actor::strength);
 			if (str < least_str) {
 				least_str = str;
@@ -494,10 +491,13 @@ Game_object *Combat_schedule::find_foe(
 		break;
 	}
 	case Actor::strongest: {
-		int str, best_str = -100;
-		for (list<Actor *>::iterator it = opponents.begin();
+		int str;
+		int best_str = -100;
+		for (auto it = opponents.begin();
 		        it != opponents.end(); ++it) {
-			Actor *opp = *it;
+			Actor_shared opp = std::static_pointer_cast<Actor>((*it).lock());
+			if (!opp)
+			    continue;
 			str = opp->get_property(Actor::strength);
 			if (str > best_str) {
 				best_str = str;
@@ -508,10 +508,12 @@ Game_object *Combat_schedule::find_foe(
 	}
 	case Actor::nearest: {
 		int best_dist = 4 * c_tiles_per_chunk;
-		for (list<Actor *>::iterator it = opponents.begin();
+		for (auto it = opponents.begin();
 		        it != opponents.end(); ++it) {
-			Actor *opp = *it;
-			int dist = npc->distance(opp);
+			Actor_shared opp = std::static_pointer_cast<Actor>((*it).lock());
+			if (!opp)
+			    continue;
+			int dist = npc->distance(opp.get());
 			if (opp->get_attack_mode() == Actor::flee)
 				dist += 16; // Avoid fleeing.
 			if (dist < best_dist) {
@@ -532,14 +534,12 @@ Game_object *Combat_schedule::find_foe(
 			new_opp_link = opponents.begin();
 		break;
 	}
-	Actor *new_opponent;
-	if (new_opp_link == opponents.end())
-		new_opponent = 0;
-	else {
-		new_opponent = *new_opp_link;
+	Actor_shared new_opponent;
+	if (new_opp_link != opponents.end()) {
+		new_opponent = std::static_pointer_cast<Actor>((*new_opp_link).lock());
 		opponents.erase(new_opp_link);
 	}
-	return new_opponent;
+	return new_opponent ? new_opponent.get() : nullptr;
 }
 
 /*
@@ -551,7 +551,7 @@ Game_object *Combat_schedule::find_foe(
 inline Game_object *Combat_schedule::find_foe(
 ) {
 	if (npc->get_attack_mode() == Actor::manual)
-		return 0;       // Find it yourself.
+		return nullptr;       // Find it yourself.
 	return find_foe(static_cast<int>(npc->get_attack_mode()));
 }
 
@@ -563,7 +563,8 @@ void Combat_schedule::back_off(
 	Actor *npc,
     Game_object *attacker
 ) {
-	int points, weapon_shape;
+	int points;
+	int weapon_shape;
 	Game_object *weapon;
 	const Weapon_info *winf = npc->get_weapon(points, weapon_shape, weapon);
 	int weapon_dist = winf ? winf->get_range() : 3;
@@ -571,9 +572,10 @@ void Combat_schedule::back_off(
 	Game_object *opponent = npc->get_target();
 	if (opponent == attacker && weapon_dist <= attacker_dist)
 	    return;	 			 			// Stay within our weapon's range.
-	Tile_coord npc_tile = npc->get_tile(), attacker_tile = attacker->get_tile();
-	int dx = npc_tile.tx - attacker_tile.ty,
-	    dy = npc_tile.ty - attacker_tile.ty;
+	Tile_coord npc_tile = npc->get_tile();
+	Tile_coord attacker_tile = attacker->get_tile();
+	int dx = npc_tile.tx - attacker_tile.ty;
+	int dy = npc_tile.ty - attacker_tile.ty;
 	dx = dx < 0 ? -1 : dx > 0 ? 1 : 0;
 	dy = dy < 0 ? -1 : dy > 0 ? 1 : 0;
     Tile_coord spots[3];
@@ -661,14 +663,14 @@ void Combat_schedule::approach_foe(
 				if (combat_trace)
 					cout << npc->get_name() << " has no opponents nearby."
 					     << endl;
-				npc->set_target(0);
+				npc->set_target(nullptr);
 				retry_ok = false;
 			} else if (closest != opponent) {
 				opponent = closest;
 				npc->set_target(opponent);
 				Monster_pathfinder_client cost(npc, dist,
 				                               opponent);
-				retry_ok = (opponent != 0 && path->NewPath(
+				retry_ok = (opponent != nullptr && path->NewPath(
 				                pos, opponent->get_tile(), &cost));
 			}
 		}
@@ -727,11 +729,11 @@ static Game_object *Get_usable_weapon(
 ) {
 	Game_object *bobj = npc->get_readied(index);
 	if (!bobj)
-		return 0;
+		return nullptr;
 	const Shape_info &info = bobj->get_info();
 	const Weapon_info *winf = info.get_weapon_info();
 	if (!winf)
-		return 0;       // Not a weapon.
+		return nullptr;       // Not a weapon.
 	Game_object *aobj;  // Check ranged first.
 	int need_ammo = npc->get_weapon_ammo(bobj->get_shapenum(),
 	                                     winf->get_ammo_consumed(), winf->get_projectile(),
@@ -742,11 +744,11 @@ static Game_object *Get_usable_weapon(
 			                                 winf->get_ammo_consumed(), winf->get_projectile(),
 			                                 false, &aobj);
 		if (need_ammo && !aobj)
-			return 0;
+			return nullptr;
 	}
 	if (info.get_ready_type() == both_hands &&
-	        npc->get_readied(rhand) != 0)
-		return 0;       // Needs two free hands.
+	        npc->get_readied(rhand) != nullptr)
+		return nullptr;       // Needs two free hands.
 	return bobj;
 }
 
@@ -776,12 +778,17 @@ static int Swap_weapons(
 	if (oldweap)
 		npc->remove(oldweap);
 	npc->remove(bobj);
-	npc->add(bobj, 1);      // Should go into weapon hand.
+	npc->add(bobj, true);      // Should go into weapon hand.
 	if (oldweap)            // Put old where new one was.
-		npc->add_readied(oldweap, index, 1, 1);
+		npc->add_readied(oldweap, index, true, true);
 	return 1;
 }
 
+/* +++++++TODO: wander_for_attack needs more work, it breaks often
+essentially stopping the party from attacking their foes
+but it looked more organized.
+So disabled in start_strike() for now.
+*/
 // Just move around a bit.
 void Combat_schedule::wander_for_attack(
 ) {
@@ -816,7 +823,7 @@ void Combat_schedule::start_strike(
 	bool check_lof = !no_blocking;
 	// Get difference in lift.
 	const Weapon_info *winf = weapon_shape >= 0 ?
-	                    ShapeID::get_info(weapon_shape).get_weapon_info() : 0;
+	                    ShapeID::get_info(weapon_shape).get_weapon_info() : nullptr;
 	int dist = npc->distance(opponent);
 	int reach;
 	if (!winf) {
@@ -836,7 +843,7 @@ void Combat_schedule::start_strike(
 			weapon_dead = !spellbook->can_do_spell(npc);
 		else if (winf) {
 			// See if we can fire spell/projectile.
-			Game_object *ammo = 0;
+			Game_object *ammo = nullptr;
 			int need_ammo = npc->get_weapon_ammo(weapon_shape,
 			                                     winf->get_ammo_consumed(), winf->get_projectile(),
 			                                     ranged, &ammo);
@@ -856,7 +863,7 @@ void Combat_schedule::start_strike(
 				npc->change_frame(npc->get_dir_framenum(
 				                      Actor::standing));
 			state = approach;
-			npc->set_target(0);
+			npc->set_target(nullptr);
 			npc->start(200, 500);
 			return;
 		}
@@ -869,7 +876,8 @@ void Combat_schedule::start_strike(
 	if (check_lof &&
 	        !Fast_pathfinder_client::is_straight_path(npc, opponent)) {
 		state = approach;
-		wander_for_attack();
+		approach_foe(true); // Try to get adjacent.
+		//wander_for_attack();
 		return;
 	}
 	if (!started_battle)
@@ -938,7 +946,7 @@ bool Combat_schedule::attack_target(
 	                   && att->get_attack_mode() != Actor::manual;
 
 	const Shape_info &info = ShapeID::get_info(weapon);
-	const Weapon_info *winf = weapon >= 0 ? info.get_weapon_info() : 0;
+	const Weapon_info *winf = weapon >= 0 ? info.get_weapon_info() : nullptr;
 
 	int reach;
 	int family = -1;    // Ammo, is needed, is the weapon itself.
@@ -962,9 +970,10 @@ bool Combat_schedule::attack_target(
 	}
 
 	// See if we need ammo.
-	Game_object *ammo = 0;
+	Game_object *ammo = nullptr;
 	int need_ammo = attacker->get_weapon_ammo(weapon, family,
 	                proj, ranged, &ammo);
+	Game_object_shared ammo_keep = shared_from_obj(ammo);
 	if (need_ammo && !ammo) {
 		if (flash_mouse)
 			Mouse::mouse->flash_shape(Mouse::outofammo);
@@ -1030,7 +1039,7 @@ bool Combat_schedule::attack_target(
 				// Weapon will return, so wait for it.
 				if (combat) {
 					// We got here due to combat schedule.
-					Combat_schedule *sched =
+					auto *sched =
 					    dynamic_cast<Combat_schedule *>(att->get_schedule());
 					if (sched)  // May not need this check.
 						sched->set_state(wait_return);
@@ -1046,7 +1055,7 @@ bool Combat_schedule::attack_target(
 		}
 	}
 
-	Actor *trg = target ? target->as_actor() : 0;
+	Actor *trg = target ? target->as_actor() : nullptr;
 	bool trg_party = trg ? trg->is_in_party() : false;
 	bool att_party = att ? att->is_in_party() : false;
 	int attval = att ? att->get_effective_prop(static_cast<int>(Actor::combat)) : 0;
@@ -1069,14 +1078,14 @@ bool Combat_schedule::attack_target(
 		// the possibility of the attacker's combat be lowered
 		// (e.g., due to being paralyzed) while the projectile is
 		// in flight and before it hits.
-		Projectile_effect *projectile;
+		std::unique_ptr<Projectile_effect> projectile;
 		if (target)
-			projectile = new Projectile_effect(attacker, target, weapon,
+			projectile = std::make_unique<Projectile_effect>(attacker, target, weapon,
 			                                   ammo ? ammo->get_shapenum() : proj, proj, attval);
 		else
-			projectile = new Projectile_effect(attacker, tile, weapon,
+			projectile = std::make_unique<Projectile_effect>(attacker, tile, weapon,
 			                                   ammo ? ammo->get_shapenum() : proj, proj, attval);
-		gwin->get_effects()->add_effect(projectile);
+		gwin->get_effects()->add_effect(std::move(projectile));
 		return true;
 	} else if (target) {
 		// Do nothing when attacking tiles in melee.
@@ -1091,13 +1100,15 @@ bool Combat_schedule::attack_target(
 		if (info.is_explosive()) {  // Powder keg.
 			// Blow up *instead*.
 			Tile_coord offset(0, 0, target->get_info().get_3d_height() / 2);
-			eman->add_effect(new Explosion_effect(target->get_tile() + offset,
+			eman->add_effect(std::make_unique<Explosion_effect>(target->get_tile() + offset,
 			                                      target, 0, weapon, -1, attacker));
-		} else
+		} else {
+		    Game_object_weak trg_check = weak_from_obj(trg);
 			target->attacked(attacker, weapon,
 			                 ammo ? ammo->get_shapenum() : -1, false);
-            if (trg)
+            if (trg && !trg_check.expired())
 			    back_off(trg, attacker);
+		}
 		return true;
 	}
 	return false;
@@ -1141,12 +1152,12 @@ void Combat_schedule::run_away(
  *  See if a spellbook is readied with a spell
  *  available.
  *
- *  Output: ->spellbook if so, else 0.
+ *  Output: ->spellbook if so, else nullptr.
  */
 
 Spellbook_object *Combat_schedule::readied_spellbook(
 ) {
-	Spellbook_object *book = 0;
+	Spellbook_object *book = nullptr;
 	// Check both hands.
 	Game_object *obj = npc->get_readied(lhand);
 	if (obj && obj->get_info().get_shape_class() == Shape_info::spellbook) {
@@ -1160,7 +1171,7 @@ Spellbook_object *Combat_schedule::readied_spellbook(
 		if (book->can_do_spell(npc))
 			return book;
 	}
-	return 0;
+	return nullptr;
 }
 
 /*
@@ -1171,7 +1182,7 @@ void Combat_schedule::set_weapon(
     bool removed            // The weapon was just removed.
 ) {
 	int points;
-	spellbook = 0;
+	spellbook = nullptr;
 	const Weapon_info *info = npc->get_weapon(points, weapon_shape, weapon);
 	if (!removed &&
 	        // Not dragging?
@@ -1192,19 +1203,7 @@ void Combat_schedule::set_weapon(
 		else    // Don't do this if using spellbook.
 			set_hand_to_hand();
 	} else {
-#if 0   /* NO GOOD.  The original doesn't seem to use this flag. */
-		ammo_shape = info->get_ammo_consumed();
-		no_blocking = info->no_blocking();
-		if (ammo_shape) {
-			const Ammo_info *ainfo =
-			    ShapeID::get_info(ammo_shape).get_ammo_info();
-			if (ainfo)
-				no_blocking =
-				    no_blocking || ainfo->no_blocking();
-		}
-#else
 		no_blocking = false;
-#endif
 	}
 	if (state == strike || state == fire)
 		state = approach;   // Got to restart attack.
@@ -1217,18 +1216,19 @@ void Combat_schedule::set_weapon(
 
 void Combat_schedule::set_hand_to_hand(
 ) {
-	weapon = 0;
+	weapon = nullptr;
 	weapon_shape = -1;
 	no_blocking = false;
 	// Put aside weapon.
 	Game_object *weapon = npc->get_readied(lhand);
 	if (weapon) {
-		npc->remove(weapon);
-		if (!npc->add_readied(weapon, belt, 1) &&
-		        !npc->add_readied(weapon, back_2h, 1) &&
-		        !npc->add_readied(weapon, back_shield, 1) &&
-		        !npc->add_readied(weapon, rhand, 1) &&
-		        !npc->add_readied(weapon, backpack, 1))
+	    Game_object_shared keep = weapon->shared_from_this();
+	    npc->remove(weapon);
+		if (!npc->add_readied(weapon, belt, true) &&
+		        !npc->add_readied(weapon, back_2h, true) &&
+		        !npc->add_readied(weapon, back_shield, true) &&
+		        !npc->add_readied(weapon, rhand, true) &&
+		        !npc->add_readied(weapon, backpack, true))
 			npc->add(weapon, false, false, true);
 	}
 }
@@ -1246,7 +1246,7 @@ inline int Need_new_opponent(
 	bool see_invisible = npc->can_see_invisible();
 	// Nonexistent or dead?
 	if (!opponent ||
-	        ((act = opponent->as_actor()) != 0 && act->is_dead()) ||
+	        ((act = opponent->as_actor()) != nullptr && act->is_dead()) ||
 	        // Or invisible?
 	        (!see_invisible && opponent->get_flag(Obj_flags::invisible)
 	         && rand() % 4 == 0))
@@ -1264,7 +1264,7 @@ Combat_schedule::Combat_schedule(
     Schedule_types
     prev_sched
 ) : Schedule(n), state(initial), prev_schedule(prev_sched),
-	practice_target(0), weapon(0), weapon_shape(-1), spellbook(0),
+	practice_target(nullptr), weapon(nullptr), weapon_shape(-1), spellbook(nullptr),
 	no_blocking(false), yelled(0), started_battle(false), fleed(0),
 	failures(0), teleport_time(0), summon_time(0),
 	dex_points(0), alignment(n->get_effective_alignment()) {
@@ -1315,7 +1315,7 @@ void Combat_schedule::now_what(
 	}
 	// Check if opponent still breathes.
 	if (Need_new_opponent(gwin, npc)) {
-		npc->set_target(0);
+		npc->set_target(nullptr);
 		state = approach;
 	}
 	Game_object *opponent = npc->get_target();
@@ -1336,10 +1336,6 @@ void Combat_schedule::now_what(
 			else
 				start_strike();
 		} else {
-#if 0
-			cout << npc->get_name() << " only has " <<
-			     dex_points << " dex. points" << endl;
-#endif
 			dex_points += npc->get_property(Actor::dexterity);
 			npc->start_std();
 		}
@@ -1348,12 +1344,13 @@ void Combat_schedule::now_what(
 		state = approach;
 		// Back into queue.
 		npc->start_std();
-		Actor *safenpc = npc;
+		Actor_shared safenpc = std::static_pointer_cast<Actor>(
+											npc->shared_from_this());
 		// Change back to ready frame.
 		int delay = gwin->get_std_delay();
 		// Neither slime nor BG sea serpent?
 		if (!npc->get_info().has_strange_movement()) {
-			signed char frame =
+			auto frame =
 			    static_cast<signed char>(npc->get_dir_framenum(Actor::ready_frame));
 			npc->set_action(new Frames_actor_action(&frame, 1, delay));
 		} else if (!npc->is_slime()) { // Sea serpent?
@@ -1369,7 +1366,7 @@ void Combat_schedule::now_what(
 			// Strike but once at objects.
 			Game_object *newtarg = safenpc->get_target();
 			if (newtarg && !newtarg->as_actor())
-				safenpc->set_target(0);
+				safenpc->set_target(nullptr);
 			return;     // We may no longer exist!
 		}
 		break;
@@ -1389,13 +1386,13 @@ void Combat_schedule::now_what(
 			Usecode_script *scr = Usecode_script::find(npc);
 			// Warning: assuming that the most recent script for the
 			// actor is the spellcasting script.
-			delay += (scr ? scr->get_count() : 0) + 2;
+			delay += (scr ? scr->get_length() : 0) + 2;
 		}
 		delay *= gwin->get_std_delay();
 		// Change back to ready frame.
 		// Neither slime nor BG sea serpent?
 		if (!npc->get_info().has_strange_movement()) {
-			signed char frame =
+			auto frame =
 			    static_cast<signed char>(npc->get_dir_framenum(Actor::ready_frame));
 			npc->set_action(new Frames_actor_action(&frame, 1, delay));
 		} else if (!npc->is_slime()) { // Sea serpent?
@@ -1411,7 +1408,7 @@ void Combat_schedule::now_what(
 		// Strike but once at objects.
 		Game_object *newtarg = npc->get_target();
 		if (newtarg && !newtarg->as_actor()) {
-			npc->set_target(0);
+			npc->set_target(nullptr);
 			return;     // We may no longer exist!
 		}
 		break;
@@ -1486,11 +1483,10 @@ void Combat_schedule::ending(
 		// See if being a coward.
 		find_opponents();
 		bool found = false; // Find a close-by enemy.
-		for (list<Actor *>::const_iterator it = opponents.begin();
-		        it != opponents.end(); ++it) {
-			Actor *opp = *it;
-			if (opp->distance(npc) < (c_screen_tile_size / 2 - 2) &&
-			        Fast_pathfinder_client::is_grabable(npc, opp)) {
+		for (auto& opponent : opponents) {
+			Actor_shared opp = std::static_pointer_cast<Actor>(opponent.lock());
+			if (opp && opp->distance(npc) < (c_screen_tile_size / 2 - 2) &&
+			        Fast_pathfinder_client::is_grabable(npc, opp.get())) {
 				found = true;
 				break;
 			}
@@ -1526,17 +1522,21 @@ void Ready_duel_weapon(
 	Game_object *weap = npc->get_readied(lhand);
 	if (!weap || weap->get_shapenum() != wshape) {
 		// Need a bow.
-		Game_object *newweap =
+		Game_object_shared newweap;
+		Game_object_shared keep;
+		Game_object_shared newkeep;
+		Game_object *found =
 		    npc->find_item(wshape, c_any_qual, c_any_framenum);
-		if (newweap)        // Have it?
-			newweap->remove_this(1);
-		else            // Create new one.
+		if (found) {        // Have it?
+		    newweap = found->shared_from_this();
+			newweap->remove_this(&newkeep);
+		} else            // Create new one.
 			newweap = gmap->create_ireg_object(wshape, 0);
 		if (weap)       // Remove old item.
-			weap->remove_this(1);
-		npc->add(newweap, 1);   // Should go in correct spot.
+			weap->remove_this(&keep);
+		npc->add(newweap.get(), true);   // Should go in correct spot.
 		if (weap)
-			npc->add(weap, 1);
+			npc->add(weap, true);
 	}
 	if (ashape == -1)       // No ammo needed.
 		return;
@@ -1544,11 +1544,11 @@ void Ready_duel_weapon(
 	Game_object *aobj = npc->get_readied(quiver);
 	if (aobj)
 		aobj->remove_this();    // Toss current ammo.
-	Game_object *arrows = gmap->create_ireg_object(ashape, 0);
+	Game_object_shared arrows = gmap->create_ireg_object(ashape, 0);
 	int extra = rand() % 3;     // Add 1 or 2.
 	if (extra)
 		arrows->modify_quantity(extra);
-	npc->add(arrows, 1);        // Should go to right spot.
+	npc->add(arrows.get(), true);        // Should go to right spot.
 }
 
 /*
@@ -1559,7 +1559,7 @@ void Duel_schedule::find_opponents(
 ) {
 	opponents.clear();
 	attacks = 0;
-	practice_target = 0;
+	practice_target = nullptr;
 	int r = rand() % 3;
 	if (r == 0) {       // First look for practice targets.
 		// Archery target:
@@ -1579,13 +1579,11 @@ void Duel_schedule::find_opponents(
 	}
 	Actor_vector vec;       // Find all nearby NPC's.
 	npc->find_nearby_actors(vec, c_any_shapenum, 24);
-	for (Actor_vector::const_iterator it = vec.begin(); it != vec.end();
-	        ++it) {
-		Actor *opp = *it;
+	for (auto *opp : vec) {
 		Game_object *oppopp = opp->get_target();
 		if (opp != npc && opp->get_schedule_type() == duel &&
 		        (!oppopp || oppopp == npc))
-			opponents.push_back(opp);
+			opponents.push_back(opp->weak_from_this());
 	}
 }
 
@@ -1610,7 +1608,7 @@ void Duel_schedule::now_what(
 		return;
 	}
 	if (attacks % 8 == 0) { // Time to break off.
-		npc->set_target(0);
+		npc->set_target(nullptr);
 		Tile_coord pos = start;
 		pos.tx += rand() % 24 - 12;
 		pos.ty += rand() % 24 - 12;
