@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2013  The Exult Team
+ *  Copyright (C) 2000-2022  The Exult Team
  *
  *  Original file by Dancer A.L Vesperman
  *
@@ -31,27 +31,6 @@
 using std::ifstream;
 using std::size_t;
 
-/**
- *  Retrieves the contents of a FLAT "file".
- *  @param objnum   Ignored.
- *  @param len  Length of the data buffer or zero in any failure.
- *  @return A buffer created with new[] containing the file data,
- *  or null in any failure.
- */
-char *Flat::retrieve(uint32 objnum, size_t &len) {
-	ignore_unused_variable_warning(objnum);
-	if (!data || !data->good()) {
-		len = 0;
-		return 0;
-	}
-	data->seek(0);
-	len = data->getSize();
-	char *buffer = new char[len];
-	data->read(buffer, len);
-
-	return buffer;
-}
-
 /**
  *  Verifies if a datasource is a FLAT file.
  *  @param in   DataSource containing the data we wish to investigate.
