@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2001 The Exult Team
+Copyright (C) 2001-2022 The Exult Team
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -19,7 +19,10 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #ifndef LISTFILES_H
 #define LISTFILES_H
 
-typedef std::vector<std::string> FileList;
+#include <string>
+#include <vector>
+
+using FileList = std::vector<std::string>;
 
 int U7ListFiles(const std::string &mask, FileList &files);
 
