@@ -2,7 +2,7 @@
  *  utils.cc - Common utility routines.
  *
  *  Copyright (C) 1998-1999  Jeffrey S. Freedman
- *  Copyright (C) 2000-2013  The Exult Team
+ *  Copyright (C) 2000-2022  The Exult Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -30,11 +30,16 @@
 #include <string>
 #include <fstream>
 #include <map>
+#include <vector>
 #include <list>
 #include <sys/stat.h>
 #include <unistd.h>
 
-#ifdef WIN32
+#ifdef __IPHONEOS__
+#  include "ios_utils.h"
+#endif
+
+#ifdef _WIN32
 #include <windows.h>
 #include <shlobj.h>
 #include <direct.h> // For mkdir and chdir
@@ -47,9 +52,9 @@
 #include "fnames.h"
 #include "ignore_unused_variable_warning.h"
 
-#ifdef MACOSX
-#include <CoreFoundation/CoreFoundation.h>
-#include <sys/param.h> // for MAXPATHLEN
+#if defined(MACOSX) || defined(__IPHONEOS__)
+#  include <CoreFoundation/CoreFoundation.h>
+#  include <sys/param.h> // for MAXPATHLEN
 #endif
 
 using std::cerr;
@@ -80,13 +85,17 @@ void reset_system_paths() {
 	path_map = stored_path_map;
 }
 
-static const string remove_trainling_slash(const string &value) {
-	string new_path = value;
-	if (*(new_path.end() - 1) == '/'
-#ifdef WIN32
-	        || *(new_path.end() - 1) == '\\'
+static bool is_path_separator(char cc) {
+#ifdef _WIN32
+	return cc == '/' || cc == '\\';
+#else
+	return cc == '/';
 #endif
-	   ) {
+}
+
+static string remove_trailing_slash(const string &value) {
+	string new_path = value;
+	if (is_path_separator(new_path.back())) {
 		std::cerr << "Warning, trailing slash in path: \"" << new_path << "\"" << std::endl;
 		new_path.resize(new_path.size() - 1);
 	}
@@ -102,7 +111,7 @@ void add_system_path(const string &key, const string &value) {
 			          << value << "'." << std::endl;
 			exit(1);
 		} else
-			path_map[key] = remove_trainling_slash(value);
+			path_map[key] = remove_trailing_slash(value);
 	} else {
 		clear_system_path(key);
 	}
@@ -117,7 +126,7 @@ void clone_system_path(const string &new_key, const string &old_key) {
 }
 
 void clear_system_path(const string &key) {
-	std::map<string, string>::iterator iter = path_map.find(key);
+	auto iter = path_map.find(key);
 	if (iter != path_map.end())
 		path_map.erase(iter);
 }
@@ -126,7 +135,7 @@ void clear_system_path(const string &key) {
  *  Has a path been entered?
  */
 bool is_system_path_defined(const string &path) {
-	return (path_map.find(path) != path_map.end());
+	return path_map.find(path) != path_map.end();
 }
 
 /*
@@ -148,7 +157,8 @@ string get_system_path(const string &path) {
 		string syspath = new_path.substr(0, pos);
 		if (is_system_path_defined(syspath)) {
 			string new_prefix = path_map[syspath];
-			new_path = new_prefix + new_path.substr(pos);
+			new_prefix += new_path.substr(pos);
+			new_path.swap(new_prefix);
 			pos = new_path.find('>');
 			pos2 = new_path.find('<');
 		} else {
@@ -167,8 +177,8 @@ string get_system_path(const string &path) {
 	}
 
 	switch_slashes(new_path);
-#ifdef WIN32
-	if (*(new_path.end() - 1) == '/' || *(new_path.end() - 1) == '\\') {
+#ifdef _WIN32
+	if (new_path.back() == '/' || new_path.back() == '\\') {
 		//std::cerr << "Trailing slash in path: \"" << new_path << "\"" << std::endl << "...compensating, but go complain to Colourless anyway" << std::endl;
 		std::cerr << "Warning, trailing slash in path: \"" << new_path << "\"" << std::endl;
 		new_path += '.';
@@ -181,12 +191,11 @@ string get_system_path(const string &path) {
 	// and we'll only convert to short if a space is found
 	// really, we don't need to do this, but hey, you never know
 	if (pos == pos2 && pos3 != string::npos) {
-		int num_chars = GetShortPathName(new_path.c_str(), NULL, 0);
+		int num_chars = GetShortPathName(new_path.c_str(), nullptr, 0);
 		if (num_chars > 0) {
-			char *short_path = new char [num_chars + 1];
-			GetShortPathName(new_path.c_str(), short_path, num_chars + 1);
-			new_path = short_path;
-			delete [] short_path;
+			std::string short_path(num_chars + 1);
+			GetShortPathName(new_path.c_str(), &short_path[0], num_chars + 1);
+			new_path = std::move(short_path);
 		}
 		//else std::cerr << "Warning unable to get short path for: " << new_path << std::endl;
 	}
@@ -203,12 +212,8 @@ string get_system_path(const string &path) {
  */
 
 void to_uppercase(string &str) {
-	for (string::iterator X = str.begin(); X != str.end(); ++X) {
-#if (defined(OPENBSD) || defined(CYGWIN))
-		if ((*X >= 'a') && (*X <= 'z')) *X -= 32;
-#else
-		*X = static_cast<char>(std::toupper(*X));
-#endif
+	for (auto& chr : str) {
+		chr = static_cast<char>(std::toupper(static_cast<unsigned char>(chr)));
 	}
 }
 
@@ -236,17 +241,13 @@ static bool base_to_uppercase(string &str, int count) {
 		if (todo <= 0)
 			break;
 
-#if (defined(OPENBSD) || defined(CYGWIN))
-		if ((*X >= 'a') && (*X <= 'z')) *X -= 32;
-#else
-		*X = static_cast<char>(std::toupper(*X));
-#endif
+		*X = static_cast<char>(std::toupper(static_cast<unsigned char>(*X)));
 	}
 	if (X == str.rend())
 		todo--; // start of pathname counts as separator too
 
 	// false if it didn't reach 'count' parts
-	return (todo <= 0);
+	return todo <= 0;
 }
 
 
@@ -254,10 +255,10 @@ static bool base_to_uppercase(string &str, int count) {
 static void switch_slashes(
     string &name
 ) {
-#ifdef WIN32
-	for (string::iterator X = name.begin(); X != name.end(); ++X) {
-		if (*X == '/')
-			*X = '\\';
+#ifdef _WIN32
+	for (char& X : name) {
+		if (X == '/')
+			X = '\\';
 	}
 #else
 	ignore_unused_variable_warning(name);
@@ -278,15 +279,8 @@ bool U7open(
     const char *fname,          // May be converted to upper-case.
     bool is_text                // Should the file be opened in text mode
 ) {
-#if defined(MACOS) || (__GNUG__ > 2)
 	std::ios_base::openmode mode = std::ios::in;
 	if (!is_text) mode |= std::ios::binary;
-#elif defined(XWIN)
-	int mode = std::ios::in;
-#else
-	int mode = std::ios::in;
-	if (!is_text) mode |= std::ios::binary;
-#endif
 	string name = get_system_path(fname);
 	int uppercasecount = 0;
 	do {
@@ -305,7 +299,7 @@ bool U7open(
 	} while (base_to_uppercase(name, ++uppercasecount));
 
 	// file not found.
-	throw(file_open_exception(get_system_path(fname)));
+	throw file_open_exception(get_system_path(fname));
 	return false;
 }
 
@@ -322,15 +316,8 @@ bool U7open(
     const char *fname,          // May be converted to upper-case.
     bool is_text                // Should the file be opened in text mode
 ) {
-#if defined(MACOS) || (__GNUG__ > 2)
 	std::ios_base::openmode mode = std::ios::out | std::ios::trunc;
 	if (!is_text) mode |= std::ios::binary;
-#elif defined(XWIN)
-	int mode = std::ios::out | std::ios::trunc;
-#else
-	int mode = std::ios::out | std::ios::trunc;
-	if (!is_text) mode |= std::ios::binary;
-#endif
 	string name = get_system_path(fname);
 
 	// We first "clear" the stream object. This is done to prevent
@@ -346,49 +333,22 @@ bool U7open(
 	} while (base_to_uppercase(name, ++uppercasecount));
 
 	// file not found.
-	throw(file_open_exception(get_system_path(fname)));
+	throw file_open_exception(get_system_path(fname));
 	return false;
 }
 
-/*
- *  Open a file with the access rights specified in mode,
- *  works just like fopen but in a system independant fashion.
- *
- *  Output: A pointer to a FILE
- */
-
-std::FILE *U7open(
-    const char *fname,         // May be converted to upper-case.
-    const char *mode           // File access mode.
-) {
-	std::FILE *f;
-	string name = get_system_path(fname);
-
-	int uppercasecount = 0;
-	do {
-		f = std::fopen(name.c_str(), mode); // Try to open
-		if (f)
-			return f; // found it!
-	} while (base_to_uppercase(name, ++uppercasecount));
-
-	// file not found.
-	throw(file_open_exception(get_system_path(fname)));
-	return 0;
-}
-
 DIR *U7opendir(
     const char *fname			// May be converted to upper-case.
 ) {
-	DIR *dir;
 	string name = get_system_path(fname);
 	int uppercasecount = 0;
 
 	do {
-		dir = opendir(name.c_str()); // Try to open
+		DIR *dir = opendir(name.c_str()); // Try to open
 		if (dir)
 			return dir; // found it!
 	} while (base_to_uppercase(name, ++uppercasecount));
-	return 0;
+	return nullptr;
 }
 
 /*
@@ -401,7 +361,7 @@ void U7remove(
 ) {
 	string name = get_system_path(fname);
 
-#if defined(WIN32) && defined(UNICODE)
+#if defined(_WIN32) && defined(UNICODE)
 	const char *n = name.c_str();
 	int nLen = std::strlen(n) + 1;
 	LPTSTR lpszT = (LPTSTR) alloca(nLen * 2);
@@ -409,12 +369,11 @@ void U7remove(
 	DeleteFile(lpszT);
 #else
 
-	bool exists;
 	struct stat sbuf;
 
 	int uppercasecount = 0;
 	do {
-		exists = (stat(name, &sbuf) == 0);
+		bool exists = (stat(name, &sbuf) == 0);
 		if (exists) {
 			std::remove(name.c_str());
 		}
@@ -459,12 +418,11 @@ bool U7exists(
     const char *fname         // May be converted to upper-case.
 ) {
 	string name = get_system_path(fname);
-	bool    exists;
 	struct stat sbuf;
 
 	int uppercasecount = 0;
 	do {
-		exists = (stat(name, &sbuf) == 0);
+		bool exists = (stat(name, &sbuf) == 0);
 		if (exists)
 			return true; // found it!
 	} while (base_to_uppercase(name, ++uppercasecount));
@@ -482,20 +440,18 @@ int U7mkdir(
     int mode
 ) {
 	string name = get_system_path(dirname);
-#if defined(MACOSX)
 	// remove any trailing slashes
 	string::size_type pos = name.find_last_not_of('/');
 	if (pos != string::npos)
 		name.resize(pos + 1);
-#endif
-#if defined(WIN32) && defined(UNICODE)
+#if defined(_WIN32) && defined(UNICODE)
 	const char *n = name.c_str();
 	int nLen = std::strlen(n) + 1;
 	LPTSTR lpszT = (LPTSTR) alloca(nLen * 2);
 	MultiByteToWideChar(CP_ACP, 0, n, -1, lpszT, nLen);
 	ignore_unused_variable_warning(mode);
-	return CreateDirectory(lpszT, NULL);
-#elif defined(WIN32)
+	return CreateDirectory(lpszT, nullptr);
+#elif defined(_WIN32)
 	ignore_unused_variable_warning(mode);
 	return mkdir(name.c_str());
 #else
@@ -503,11 +459,11 @@ int U7mkdir(
 #endif
 }
 
-#ifdef WIN32
+#ifdef _WIN32
 class shell32_wrapper {
 protected:
 	HMODULE hLib;
-	typedef HRESULT(WINAPI *SHGetFolderPathFunc) (
+	using SHGetFolderPathFunc = HRESULT (WINAPI*)(
 	    HWND hwndOwner,
 	    int nFolder,
 	    HANDLE hToken,
@@ -517,7 +473,7 @@ protected:
 	SHGetFolderPathFunc      SHGetFolderPath;
 	/*
 	// Will leave this for someone with Vista/W7 to implement.
-	typedef HRESULT (*SHGetKnownFolderPathFunc) (
+	using SHGetKnownFolderPathFunc = HRESULT (WINAPI*) (
 	    REFKNOWNFOLDERID rfid,
 	    DWORD dwFlags,
 	    HANDLE hToken,
@@ -525,43 +481,51 @@ protected:
     );
 	SHGetKnownFolderPathFunc SHGetKnownFolderPath;
 	*/
+
+	template <typename Dest>
+	Dest get_function(const char *func) {
+		static_assert(sizeof(Dest) == sizeof(decltype(GetProcAddress(hLib, func))), "sizeof(FARPROC) is not equal to sizeof(Dest)!");
+		Dest fptr;
+		const FARPROC optr = GetProcAddress(hLib, func);
+		std::memcpy(&fptr, &optr, sizeof(optr));
+		return fptr;
+	}
+
 public:
 	shell32_wrapper() {
 		hLib = LoadLibrary("shell32.dll");
-		if (hLib != NULL) {
-			SHGetFolderPath      = reinterpret_cast<SHGetFolderPathFunc>(
-			                          GetProcAddress(hLib, "SHGetFolderPathA"));
+		if (hLib != nullptr) {
+			SHGetFolderPath      = get_function<SHGetFolderPathFunc>("SHGetFolderPathA");
 			/*
-			SHGetKnownFolderPath = reinterpret_cast<SHGetKnownFolderPathFunc>(
-			                GetProcAddress(hLib, "SHGetKnownFolderPath"));
+			SHGetKnownFolderPath = get_function<SHGetKnownFolderPathFunc>("SHGetKnownFolderPath");
 			*/
 		} else {
-			SHGetFolderPath      = NULL;
-			//SHGetKnownFolderPath = NULL;
+			SHGetFolderPath      = nullptr;
+			//SHGetKnownFolderPath = nullptr;
 		}
 	}
 	~shell32_wrapper() {
 		FreeLibrary(hLib);
 	}
-	const string Get_local_appdata() {
+	string Get_local_appdata() {
 		/*  Not yet.
-		if (SHGetKnownFolderPath != NULL)
+		if (SHGetKnownFolderPath != nullptr)
 		    {
 		    }
 		else */
-		if (SHGetFolderPath != NULL) {
+		if (SHGetFolderPath != nullptr) {
 			CHAR szPath[MAX_PATH];
-			HRESULT code = SHGetFolderPath(NULL, CSIDL_LOCAL_APPDATA,
-			                               NULL, 0, szPath);
+			HRESULT code = SHGetFolderPath(nullptr, CSIDL_LOCAL_APPDATA,
+			                               nullptr, 0, szPath);
 			if (code == E_INVALIDARG)
-				return string("");
+				return string();
 			else if (code == S_FALSE)   // E_FAIL for Unicode version.
 				// Lets try creating it through the API flag:
-				code = SHGetFolderPath(NULL,
+				code = SHGetFolderPath(nullptr,
 				                       CSIDL_LOCAL_APPDATA | CSIDL_FLAG_CREATE,
-				                       NULL, 0, szPath);
+				                       nullptr, 0, szPath);
 			if (code == E_INVALIDARG)
-				return string("");
+				return string();
 			else if (code == S_OK)
 				return string(reinterpret_cast<const char *>(szPath));
 			// We don't have a folder yet at this point. This means we have
@@ -569,22 +533,59 @@ public:
 			// Just to be sure, we fall back to the old behaviour.
 			// Is anyone still needing this?
 		}
-		return string("");
+		return string();
 	}
 };
 
 #ifdef USE_CONSOLE
-// ++++ TODO: Need to check if this actually works.
 void redirect_output(const char *prefix) {
+	ignore_unused_variable_warning(prefix);
 }
 void cleanup_output(const char *prefix) {
+	ignore_unused_variable_warning(prefix);
 }
 #else
-#include <cstdio>
-const std::string Get_home();
+std::string Get_home();
 
 // Pulled from exult_studio.cc.
 void redirect_output(const char *prefix) {
+	if (GetFileType(GetStdHandle(STD_OUTPUT_HANDLE)) == FILE_TYPE_PIPE) {
+		// If we are at a msys/msys2 shell running in MinTTY, do not redirect the output,
+		// and print it to console instead.
+		return;
+	}
+	if (AttachConsole(ATTACH_PARENT_PROCESS)) {
+		// If we are in something like Windows Terminal, we need to connect to
+		// the parent terminal and manually redirect stdout/stderr to the
+		// console. Note: in powershell or cmd.exe, this will give the
+		// impression of being stuck when exult/ES exits because they return to
+		// prompt immediately after exult/ES start, and this gets lost in the
+		// stdout/stderr output.
+		// Another possibility is to compile as a console application. We will
+		// always have an attached terminal in this case, even when starting
+		// from Windows Explorer. This console can be destroyed, but it will
+		// cause it to flash into view, then disappear right away.
+		// TODO: Figure out a way to make Exult/ES "return" the terminal.
+		HANDLE stdout_handle = GetStdHandle(STD_OUTPUT_HANDLE);
+		HANDLE stderr_handle = GetStdHandle(STD_ERROR_HANDLE);
+		if (stdout_handle != INVALID_HANDLE_VALUE
+			&& stderr_handle != INVALID_HANDLE_VALUE) {
+			// We will only redirect outputs if both calls succeed.
+			FILE* newstdout = freopen("CONOUT$", "w", stdout);
+			FILE* newstderr = freopen("CONOUT$", "w", stderr);
+			// If either freopen fails, lets just go on through to write to
+			// files instead.
+			if (newstdout != nullptr && newstderr != nullptr) {
+				// Set correct buffer mode: line-buffered for stdout, unbuffered
+				// for stderr.
+				setvbuf(stdout, nullptr, _IOLBF, 0);
+				setvbuf(stderr, nullptr, _IONBF, 0);
+				return;
+			}
+		}
+	}
+	// Starting from GUI, or from cmd.exe, we will need to redirect the output.
+
 	// Flush the output in case anything is queued
 	fclose(stdout);
 	fclose(stderr);
@@ -596,7 +597,7 @@ void redirect_output(const char *prefix) {
 
 	// Redirect standard input and standard output
 	FILE *newfp = freopen(stdoutfile, "w", stdout);
-	if (newfp == NULL) {
+	if (newfp == nullptr) {
 		// This happens on NT
 #if !defined(stdout)
 		stdout = fopen(stdoutfile, "w");
@@ -611,7 +612,7 @@ void redirect_output(const char *prefix) {
 	const char *stderrfile = stderrPath.c_str();
 
 	newfp = freopen(stderrfile, "w", stderr);
-	if (newfp == NULL) {
+	if (newfp == nullptr) {
 		// This happens on NT
 #if !defined(stderr)
 		stderr = fopen(stderrfile, "w");
@@ -621,8 +622,8 @@ void redirect_output(const char *prefix) {
 			*stderr = *newfp;
 #endif
 	}
-	setvbuf(stdout, NULL, _IOLBF, BUFSIZ);  // Line buffered
-	setbuf(stderr, NULL);           // No buffering
+	setvbuf(stdout, nullptr, _IOLBF, BUFSIZ);  // Line buffered
+	setbuf(stderr, nullptr);                   // No buffering
 }
 
 void cleanup_output(const char *prefix) {
@@ -639,11 +640,11 @@ void cleanup_output(const char *prefix) {
 	}
 }
 #endif // USE_CONSOLE
-#endif  // WIN32
+#endif  // _WIN32
 
-const string Get_home() {
+string Get_home() {
 	std::string home_dir;
-#ifdef WIN32
+#ifdef _WIN32
 #ifdef PORTABLE_EXULT_WIN32
 	home_dir = ".";
 #else
@@ -652,38 +653,58 @@ const string Get_home() {
 	else {
 		shell32_wrapper shell32;
 		home_dir = shell32.Get_local_appdata();
-		if (home_dir != "")
+		if (!home_dir.empty())
 			home_dir += "\\Exult";
 		else
 			home_dir = ".";
 	}
 #endif // PORTABLE_WIN32_EXULT
-#elif !defined(MACOS)
-	const char *home = 0;
-	if ((home = getenv("HOME")) != 0)
+#else
+	const char *home = nullptr;
+	if ((home = getenv("HOME")) != nullptr)
 		home_dir = home;
 #endif
 	return home_dir;
 }
 
+#if defined(MACOSX) || defined(__IPHONEOS__)
+struct CFDeleter
+{
+	void operator()(CFTypeRef ptr) const
+	{
+		if (ptr) CFRelease(ptr);
+	}
+};
+#endif
+
 void setup_data_dir(
     const std::string &data_path,
     const char *runpath
 ) {
-#ifdef MACOSX
+#if defined(MACOSX) || defined(__IPHONEOS__)
 	// Can we try from the bundle?
-	CFURLRef fileUrl = CFBundleCopyResourcesDirectoryURL(CFBundleGetMainBundle());
+	std::unique_ptr<std::remove_pointer<CFURLRef>::type, CFDeleter> fileUrl {std::move(CFBundleCopyResourcesDirectoryURL(CFBundleGetMainBundle()))};
 	if (fileUrl) {
 		unsigned char buf[MAXPATHLEN];
-		if (CFURLGetFileSystemRepresentation(fileUrl, true, buf, sizeof(buf))) {
-			string path((const char *)buf);
+		if (CFURLGetFileSystemRepresentation(fileUrl.get(), true, buf, sizeof(buf))) {
+			string path(reinterpret_cast<const char *>(buf));
 			path += "/data";
-			add_system_path("<BUNDLE>", path.c_str());
+			add_system_path("<BUNDLE>", path);
 			if (!U7exists(BUNDLE_EXULT_FLX))
 				clear_system_path("<BUNDLE>");
 		}
 	}
 #endif
+#ifdef __IPHONEOS__
+	if (is_system_path_defined("<BUNDLE>")) {
+		// We have the flxfiles in the bundle, so lets use it.
+		// But lets use <DATA> in the iTunes file sharing.
+		string path(ios_get_documents_dir());
+		path += "/data";
+		add_system_path("<DATA>", path);
+		return;
+	}
+#endif
 
 	// First, try the cfg value:
 	add_system_path("<DATA>", data_path);
@@ -707,25 +728,20 @@ void setup_data_dir(
 	if (!sep) sep = std::strrchr(runpath, '\\');
 	if (sep) {
 		int plen = sep - runpath;
-		char *dpath = new char[plen + 10];
-		std::strncpy(dpath, runpath, plen + 1);
-		dpath[plen + 1] = 0;
-		std::strcat(dpath, "data");
+		std::string dpath(runpath, plen + 1);
+		dpath += "data";
 		add_system_path("<DATA>", dpath);
-		delete [] dpath;
 	} else
 		add_system_path("<DATA>", "data");
 	if (U7exists(EXULT_FLX))
 		return;
 
-#ifdef MACOSX
 	if (is_system_path_defined("<BUNDLE>")) {
 		// We have the bundle, so lets use it. But lets also leave <DATA>
 		// with a sensible default.
 		add_system_path("<DATA>", data_path);
 		return;
 	}
-#endif
 
 	// We've tried them all...
 	std::cerr << "Could not find 'exult.flx' anywhere." << std::endl;
@@ -735,25 +751,66 @@ void setup_data_dir(
 	exit(-1);
 }
 
-void setup_program_paths() {
-	string home_dir(Get_home()), config_dir(home_dir),
-	       savehome_dir(home_dir), gamehome_dir(".");
-
-#if defined(__IPHONEOS__)
-	config_dir = "../Library/Preferences";
-	savehome_dir = "../Documents/save";
-	gamehome_dir = "game";
+static string Get_config_dir(const string& home_dir) {
+#ifdef __IPHONEOS__
+	ignore_unused_variable_warning(home_dir);
+	return ios_get_documents_dir();
 #elif defined(MACOSX)
+	string config_dir(home_dir);
 	config_dir += "/Library/Preferences";
+	return config_dir;
+#else
+	return home_dir;
+#endif
+}
+
+static string Get_savehome_dir(const string& home_dir, const string& config_dir) {
+#ifdef __IPHONEOS__
+	ignore_unused_variable_warning(home_dir);
+	string savehome_dir(config_dir);
+	savehome_dir += "/save";
+	return savehome_dir;
+#elif defined(MACOSX)
+	ignore_unused_variable_warning(config_dir);
+	string savehome_dir(home_dir);
 	savehome_dir += "/Library/Application Support/Exult";
-	gamehome_dir = "/Library/Application Support/Exult";
+	return savehome_dir;
 #elif defined(XWIN)
+	ignore_unused_variable_warning(config_dir);
+	string savehome_dir(home_dir);
 	savehome_dir += "/.exult";
-	gamehome_dir = EXULT_DATADIR;
+	return savehome_dir;
+#else
+	ignore_unused_variable_warning(config_dir);
+	return home_dir;
 #endif
-#ifdef WIN32
+}
+
+static string Get_gamehome_dir(const string& home_dir, const string& config_dir) {
+#ifdef __IPHONEOS__
+	ignore_unused_variable_warning(home_dir);
+	string gamehome_dir(config_dir);
+	gamehome_dir += "/game";
+	return gamehome_dir;
+#elif defined(MACOSX)
+	ignore_unused_variable_warning(home_dir, config_dir);
+	return "/Library/Application Support/Exult";
+#elif defined(XWIN)
+	ignore_unused_variable_warning(home_dir, config_dir);
+	return EXULT_DATADIR;
+#else
+	ignore_unused_variable_warning(home_dir, config_dir);
+	return ".";
+#endif
+}
+
+void setup_program_paths() {
+	string home_dir(Get_home());
+	string config_dir(Get_config_dir(home_dir));
+	string savehome_dir(Get_savehome_dir(home_dir, config_dir));
+	string gamehome_dir(Get_gamehome_dir(home_dir, config_dir));
+
 	if (get_system_path("<HOME>") != ".")
-#endif
 		add_system_path("<HOME>", home_dir);
 	add_system_path("<CONFIG>", config_dir);
 	add_system_path("<SAVEHOME>", savehome_dir);
@@ -761,30 +818,35 @@ void setup_program_paths() {
 	U7mkdir("<HOME>", 0755);
 	U7mkdir("<CONFIG>", 0755);
 	U7mkdir("<SAVEHOME>", 0755);
+#ifdef MACOSX
+	// setup APPBUNDLE needed to launch Exult Studio from one
+	std::unique_ptr<std::remove_pointer<CFURLRef>::type, CFDeleter> fileUrl {std::move(CFBundleCopyBundleURL(CFBundleGetMainBundle()))};
+	if (fileUrl) {
+		unsigned char buf[MAXPATHLEN];
+		if (CFURLGetFileSystemRepresentation(fileUrl.get(), true, buf, sizeof(buf))) {
+			string path(reinterpret_cast<const char *>(buf));
+			add_system_path("<APPBUNDLE>", path);
+			// test whether it is actually an app bundle
+			path += "/Contents/Info.plist";
+			if (!U7exists(path))
+				clear_system_path("<APPBUNDLE>");
+		}
+	}
+#endif
 }
 
 /*
  *  Change the current directory
- *
- *  TODO: Make this work in WinCE - Colourless
  */
 
 int U7chdir(
     const char *dirname // May be converted to upper-case.
 ) {
-#ifdef MACOS
-	string name(dirname);
-	switch_slashes(name);
-	return chdir(name.c_str());
-#else
 	return chdir(dirname);
-#endif
 }
 
 /*
  *  Copy a file. May throw an exception.
- *
- *  TODO: Make this work in WinCE - Colourless
  */
 void U7copy(
     const char *src,
@@ -798,28 +860,18 @@ void U7copy(
 	} catch (exult_exception &e) {
 		in.close();
 		out.close();
-		throw(e);
-	}
-	size_t bufsize = 0x8000;
-	char *buf = new char[0x8000];
-	in.seekg(0, ios::end);      // Get filesize.
-	size_t filesize = in.tellg();
-	in.seekg(0, ios::beg);
-	while (filesize > 0) {      // Copy.
-		size_t toread = bufsize < filesize ? bufsize : filesize;
-		in.read(buf, toread);
-		out.write(buf, toread);
-		filesize -= toread;
+		throw;
 	}
+	out << in.rdbuf();
 	out.flush();
-	delete [] buf;
-	bool inok = in.good(), outok = out.good();
+	bool inok = in.good();
+	bool outok = out.good();
 	in.close();
 	out.close();
 	if (!inok)
-		throw(file_read_exception(src));
+		throw file_read_exception(src);
 	if (!outok)
-		throw(file_write_exception(dest));
+		throw file_write_exception(dest);
 }
 
 /*
@@ -850,7 +902,7 @@ uint32 msb32(uint32 x) {
 	x |= (x >>  4);
 	x |= (x >>  8);
 	x |= (x >> 16);
-	return (x & ~(x >> 1));
+	return x & ~(x >> 1);
 }
 
 /*
@@ -870,7 +922,7 @@ int fgepow2(uint32 n) {
 
 char *newstrdup(const char *s) {
 	if (!s)
-		throw(std::invalid_argument("NULL pointer passed to newstrdup"));
+		throw std::invalid_argument("nullptr pointer passed to newstrdup");
 	char *ret = new char[std::strlen(s) + 1];
 	std::strcpy(ret, s);
 	return ret;
@@ -891,14 +943,14 @@ char *Get_mapped_name(
 		strcpy(to, from);   // Default map.
 	else {
 		const char *sep = strrchr(from, '/');
-		assert(sep != 0);
+		assert(sep != nullptr);
 		size_t len = sep - from;
 		memcpy(to, from, len);  // Copy dir.
 		strcpy(to + len, MULTIMAP_DIR);
 		len = strlen(to);
-		to[len] = static_cast<char>('0' + num / 16);
-		int lb = num % 16;
-		to[len + 1] = static_cast<char>(lb < 10 ? ('0' + lb) : ('a' + (lb - 10)));
+		constexpr static const char hexLUT[] = "0123456789abcdef";
+		to[len] = hexLUT[num / 16];
+		to[len + 1] = hexLUT[num % 16];
 		strcpy(to + len + 2, sep);
 	}
 	return to;
