@@ -2,7 +2,7 @@
    Version 0.15 beta, Mar 19th, 1998,
 
    Modified by Ryan Nunn. Nov 9th 2001
-
+   Modified by the Exult Team. 2003-2022
    Read zip.h for more info
 */
 
@@ -26,7 +26,6 @@ extern int errno;
 #endif
 using namespace std;
 
-#include "zlib.h"
 #include "zip.h"
 
 /* Added by Ryan Nunn to overcome DEF_MEM_LEVEL being undeclared */
@@ -82,7 +81,7 @@ using namespace std;
 #define SIZECENTRALHEADER (0x2e) /* 46 */
 
 struct linkedlist_datablock_internal {
-	struct linkedlist_datablock_internal *next_datablock;
+	linkedlist_datablock_internal *next_datablock;
 	uLong  avail_in_this_block;
 	uLong  filled_in_this_block;
 	uLong  unused; /* for future use and alignement */
@@ -124,8 +123,8 @@ struct zip_internal {
 static linkedlist_datablock_internal *allocate_new_datablock() {
 	linkedlist_datablock_internal *ldi;
 	ldi = new linkedlist_datablock_internal();
-	if (ldi != NULL) {
-		ldi->next_datablock = NULL ;
+	if (ldi != nullptr) {
+		ldi->next_datablock = nullptr ;
 		ldi->filled_in_this_block = 0 ;
 		ldi->avail_in_this_block = SIZEDATA_INDATABLOCK ;
 	}
@@ -133,7 +132,7 @@ static linkedlist_datablock_internal *allocate_new_datablock() {
 }
 
 static void free_datablock(linkedlist_datablock_internal *ldi) {
-	while (ldi != NULL) {
+	while (ldi != nullptr) {
 		linkedlist_datablock_internal *ldinext = ldi->next_datablock;
 		delete ldi;
 		ldi = ldinext;
@@ -141,27 +140,19 @@ static void free_datablock(linkedlist_datablock_internal *ldi) {
 }
 
 static void init_linkedlist(linkedlist_data *ll) {
-	ll->first_block = ll->last_block = NULL;
+	ll->first_block = ll->last_block = nullptr;
 }
 
-/* Currently not being used */
-#if 0
-static void free_linkedlist(linkedlist_data *ll) {
-	free_datablock(ll->first_block);
-	ll->first_block = ll->last_block = NULL;
-}
-#endif
-
 static int add_data_in_datablock(linkedlist_data *ll, const void *buf, uLong len) {
 	linkedlist_datablock_internal *ldi;
 	const unsigned char *from_copy;
 
-	if (ll == NULL)
+	if (ll == nullptr)
 		return ZIP_INTERNALERROR;
 
-	if (ll->last_block == NULL) {
+	if (ll->last_block == nullptr) {
 		ll->first_block = ll->last_block = allocate_new_datablock();
-		if (ll->first_block == NULL)
+		if (ll->first_block == nullptr)
 			return ZIP_INTERNALERROR;
 	}
 
@@ -175,7 +166,7 @@ static int add_data_in_datablock(linkedlist_data *ll, const void *buf, uLong len
 
 		if (ldi->avail_in_this_block == 0) {
 			ldi->next_datablock = allocate_new_datablock();
-			if (ldi->next_datablock == NULL)
+			if (ldi->next_datablock == nullptr)
 				return ZIP_INTERNALERROR;
 			ldi = ldi->next_datablock ;
 			ll->last_block = ldi;
@@ -199,22 +190,6 @@ static int add_data_in_datablock(linkedlist_data *ll, const void *buf, uLong len
 	return ZIP_OK;
 }
 
-
-/* Currently not being used */
-#if 0
-static int write_datablock(FILE *fout, linkedlist_data *ll) {
-	linkedlist_datablock_internal *ldi;
-	ldi = ll->first_block;
-	while (ldi != NULL) {
-		if (ldi->filled_in_this_block > 0)
-			if (fwrite(ldi->data, ldi->filled_in_this_block, 1, fout) != 1)
-				return ZIP_ERRNO;
-		ldi = ldi->next_datablock;
-	}
-	return ZIP_OK;
-}
-#endif
-
 /****************************************************************************/
 
 /* ===========================================================================
@@ -238,7 +213,7 @@ static int ziplocal_putValue(FILE *file, uLong x, int nbByte) {
 
 static void ziplocal_putValue_inmemory(void *dest, uLong x, int nbByte);
 static void ziplocal_putValue_inmemory(void *dest, uLong x, int nbByte) {
-	unsigned char *buf = static_cast<unsigned char *>(dest);
+	auto *buf = static_cast<unsigned char *>(dest);
 	int n;
 	for (n = 0; n < nbByte; n++) {
 		buf[n] = static_cast<unsigned char>(x & 0xff);
@@ -267,7 +242,7 @@ extern zipFile ZEXPORT zipOpen(const char *pathname, int append) {
 	zip_internal ziinit;
 	zip_internal *zi;
 
-	ziinit.filezip = NULL;
+	ziinit.filezip = nullptr;
 
 	/* Start changes by Ryan Nunn to fix append mode bug */
 
@@ -276,12 +251,12 @@ extern zipFile ZEXPORT zipOpen(const char *pathname, int append) {
 		ziinit.filezip = fopen(pathname, "r+b");
 
 	/* If not append, or failed, use wb */
-	if (ziinit.filezip == NULL)
+	if (ziinit.filezip == nullptr)
 		ziinit.filezip = fopen(pathname, "wb");
 
 	/* Still doesn't exist, means can't create */
-	if (ziinit.filezip == NULL)
-		return NULL;
+	if (ziinit.filezip == nullptr)
+		return nullptr;
 
 	/* Make sure we are at the end of the file */
 	fseek(ziinit.filezip, 0, SEEK_END);
@@ -314,7 +289,7 @@ extern int ZEXPORT zipOpenNewFileInZip(zipFile file,
 	uInt i;
 	int err = ZIP_OK;
 
-	if (file == NULL)
+	if (file == nullptr)
 		return ZIP_PARAMERROR;
 	if ((method != 0) && (method != Z_DEFLATED))
 		return ZIP_PARAMERROR;
@@ -326,17 +301,17 @@ extern int ZEXPORT zipOpenNewFileInZip(zipFile file,
 	}
 
 
-	if (filename == NULL)
+	if (filename == nullptr)
 		filename = "-";
 
-	if (comment == NULL)
+	if (comment == nullptr)
 		size_comment = 0;
 	else
 		size_comment = strlen(comment);
 
 	size_filename = strlen(filename);
 
-	if (zipfi == NULL)
+	if (zipfi == nullptr)
 		file->ci.dosDate = 0;
 	else {
 		if (zipfi->dosDate != 0)
@@ -376,12 +351,12 @@ extern int ZEXPORT zipOpenNewFileInZip(zipFile file,
 	ziplocal_putValue_inmemory(file->ci.central_header + 32, size_comment, 2);
 	ziplocal_putValue_inmemory(file->ci.central_header + 34, 0, 2); /*disk nm start*/
 
-	if (zipfi == NULL)
+	if (zipfi == nullptr)
 		ziplocal_putValue_inmemory(file->ci.central_header + 36, 0, 2);
 	else
 		ziplocal_putValue_inmemory(file->ci.central_header + 36, zipfi->internal_fa, 2);
 
-	if (zipfi == NULL)
+	if (zipfi == nullptr)
 		ziplocal_putValue_inmemory(file->ci.central_header + 38, 0, 4);
 	else
 		ziplocal_putValue_inmemory(file->ci.central_header + 38, zipfi->external_fa, 4);
@@ -398,7 +373,7 @@ extern int ZEXPORT zipOpenNewFileInZip(zipFile file,
 	for (i = 0; i < size_comment; i++)
 		*(file->ci.central_header + SIZECENTRALHEADER + size_filename +
 		  size_extrafield_global + i) = *(filename + i);
-	if (file->ci.central_header == NULL)
+	if (file->ci.central_header == nullptr)
 		return ZIP_INTERNALERROR;
 
 	/* write the local header */
@@ -444,9 +419,9 @@ extern int ZEXPORT zipOpenNewFileInZip(zipFile file,
 	file->ci.stream.total_out = 0;
 
 	if ((err == ZIP_OK) && (file->ci.method == Z_DEFLATED)) {
-		file->ci.stream.zalloc = static_cast<alloc_func>(0);
-		file->ci.stream.zfree = static_cast<free_func>(0);
-		file->ci.stream.opaque = 0;
+		file->ci.stream.zalloc = nullptr;
+		file->ci.stream.zfree = nullptr;
+		file->ci.stream.opaque = nullptr;
 
 		err = deflateInit2(&file->ci.stream, level,
 		                   Z_DEFLATED, -MAX_WBITS, DEF_MEM_LEVEL, 0);
@@ -461,18 +436,18 @@ extern int ZEXPORT zipOpenNewFileInZip(zipFile file,
 	return err;
 }
 
-extern int ZEXPORT zipWriteInFileInZip(zipFile file, const voidp buf, unsigned len) {
+extern int ZEXPORT zipWriteInFileInZip(zipFile file, voidpc buf, unsigned len) {
 	int err = ZIP_OK;
 
-	if (file == NULL)
+	if (file == nullptr)
 		return ZIP_PARAMERROR;
 
 	if (file->in_opened_file_inzip == 0)
 		return ZIP_PARAMERROR;
 
-	file->ci.stream.next_in = static_cast<Bytef *>(buf);
+	file->ci.stream.next_in = static_cast<const Bytef *>(buf);
 	file->ci.stream.avail_in = len;
-	file->ci.crc32 = crc32(file->ci.crc32, static_cast<Bytef *>(buf), len);
+	file->ci.crc32 = crc32(file->ci.crc32, static_cast<const Bytef *>(buf), len);
 
 	while ((err == ZIP_OK) && (file->ci.stream.avail_in > 0)) {
 		if (file->ci.stream.avail_out == 0) {
@@ -490,23 +465,20 @@ extern int ZEXPORT zipWriteInFileInZip(zipFile file, const voidp buf, unsigned l
 			file->ci.pos_in_buffered_data += (file->ci.stream.total_out - uTotalOutBefore) ;
 
 		} else {
-			uInt copy_this, i;
+			uInt copy_this;
 			if (file->ci.stream.avail_in < file->ci.stream.avail_out)
 				copy_this = file->ci.stream.avail_in;
 			else
 				copy_this = file->ci.stream.avail_out;
-			for (i = 0; i < copy_this; i++)
-				*(reinterpret_cast<char *>(file->ci.stream.next_out) + i) =
-				    *(reinterpret_cast<const char *>(file->ci.stream.next_in) + i);
-			{
-				file->ci.stream.avail_in -= copy_this;
-				file->ci.stream.avail_out -= copy_this;
-				file->ci.stream.next_in += copy_this;
-				file->ci.stream.next_out += copy_this;
-				file->ci.stream.total_in += copy_this;
-				file->ci.stream.total_out += copy_this;
-				file->ci.pos_in_buffered_data += copy_this;
-			}
+			for (uInt i = 0; i < copy_this; i++)
+				file->ci.stream.next_out[i] = file->ci.stream.next_in[i];
+			file->ci.stream.avail_in -= copy_this;
+			file->ci.stream.avail_out -= copy_this;
+			file->ci.stream.next_in += copy_this;
+			file->ci.stream.next_out += copy_this;
+			file->ci.stream.total_in += copy_this;
+			file->ci.stream.total_out += copy_this;
+			file->ci.pos_in_buffered_data += copy_this;
 		}
 	}
 
@@ -516,7 +488,7 @@ extern int ZEXPORT zipWriteInFileInZip(zipFile file, const voidp buf, unsigned l
 extern int ZEXPORT zipCloseFileInZip(zipFile file) {
 	int err = ZIP_OK;
 
-	if (file == NULL)
+	if (file == nullptr)
 		return ZIP_PARAMERROR;
 
 	if (file->in_opened_file_inzip == 0)
@@ -594,14 +566,14 @@ extern int ZEXPORT zipClose(zipFile file, const char *global_comment) {
 	uLong size_centraldir = 0;
 	uLong centraldir_pos_inzip ;
 	uInt size_global_comment;
-	if (file == NULL)
+	if (file == nullptr)
 		return ZIP_PARAMERROR;
 
 	if (file->in_opened_file_inzip == 1) {
 		err = zipCloseFileInZip(file);
 	}
 
-	if (global_comment == NULL)
+	if (global_comment == nullptr)
 		size_global_comment = 0;
 	else
 		size_global_comment = strlen(global_comment);
@@ -610,7 +582,7 @@ extern int ZEXPORT zipClose(zipFile file, const char *global_comment) {
 	centraldir_pos_inzip = ftell(file->filezip);
 	if (err == ZIP_OK) {
 		linkedlist_datablock_internal *ldi = file->central_dir.first_block ;
-		while (ldi != NULL) {
+		while (ldi != nullptr) {
 			if ((err == ZIP_OK) && (ldi->filled_in_this_block > 0))
 				if (fwrite(ldi->data, ldi->filled_in_this_block,
 				           1, file->filezip) != 1)
