@@ -1,7 +1,7 @@
 /*
  *  gamedat.cc - Create gamedat files from a savegame.
  *
- *  Copyright (C) 2000-2013  The Exult Team
+ *  Copyright (C) 2000-2022  The Exult Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
 #  include <config.h>
 #endif
 
-#ifdef WIN32
+#ifdef _WIN32
 #include <io.h>
 #endif
 
@@ -54,6 +54,12 @@
 #include "version.h"
 #include "array_size.h"
 
+// Zip file support
+#ifdef HAVE_ZIP_SUPPORT
+#include "files/zip/unzip.h"
+#include "files/zip/zip.h"
+#endif
+
 #include <cstddef>
 #ifndef offsetof    // Broken <cstddef>? Just in case...
 #   define offsetof(type, field) reinterpret_cast<long>(&(static_cast<type *>(0)->field))
@@ -79,7 +85,6 @@ using std::strlen;
 using std::strncpy;
 using std::time_t;
 using std::tm;
-using std::time_t;
 using std::localtime;
 using std::time;
 
@@ -107,14 +112,15 @@ void Game_window::restore_flex_files(
 	int baselen = strlen(basepath);
 	for (i = 0; i < numfiles; i++) { // Now read each file.
 		// Get file length.
-		int len = finfo[2 * i + 1] - 13;
-		if (len <= 0)
+		size_t len = finfo[2 * i + 1];
+		if (len <= 13)
 			continue;
+		len -= 13;
 		in.seek(finfo[2 * i]);  // Get to it.
 		char fname[50];     // Set up name.
 		strcpy(fname, basepath);
 		in.read(&fname[baselen], 13);
-		int namelen = strlen(fname);
+		size_t namelen = strlen(fname);
 		// Watch for names ending in '.'.
 		if (fname[namelen - 1] == '.')
 			fname[namelen - 1] = 0;
@@ -129,12 +135,12 @@ void Game_window::restore_flex_files(
 			fname[namelen] = 0;
 
 			IBufferDataSource ds(buf, len);
-			if (!Flex::is_flex(&ds))
+			if (!Flex::is_flex(&ds)) {
 				// Save is most likely corrupted. Ignore the file but keep
 				// reading the savegame.
-				std::cerr << "Error reading flex: file '" <<
-				          fname << "' is not a valid flex file. This probably means a corrupt save game." << endl;
-			else {
+				std::cerr << "Error reading flex: file '" << fname
+				          << "' is not a valid flex file. This probably means a corrupt save game." << endl;
+			} else {
 				// fname should be a path hare.
 				U7mkdir(fname, 0755);
 				// Append trailing slash:
@@ -142,7 +148,6 @@ void Game_window::restore_flex_files(
 				fname[namelen + 1] = 0;
 				restore_flex_files(ds, fname);
 			}
-			delete [] buf;
 			continue;
 		}
 		ofstream out;
@@ -152,7 +157,7 @@ void Game_window::restore_flex_files(
 		if (!out.good())
 			abort("Error writing '%s'.", fname);
 		out.close();
-		CYCLE_RED_PLASMA();
+		cycle_load_palette();
 	}
 	delete [] finfo;
 }
@@ -183,32 +188,27 @@ void Game_window::restore_gamedat(
 	}
 	// Check for a ZIP file first
 #ifdef HAVE_ZIP_SUPPORT
-	if (restore_gamedat_zip(fname) != false)
+	if (restore_gamedat_zip(fname))
 		return;
 #endif
 
 	ifstream in_stream;
 
-#ifdef RED_PLASMA
 	// Display red plasma during load...
 	setup_load_palette();
-#endif
 
 	U7mkdir("<GAMEDAT>", 0755);     // Create dir. if not already there. Don't
 	// use GAMEDAT define cause that's got a
 	// trailing slash
-	try {
-		U7open(in_stream, fname);   // Open file; throws an exception
-	} catch (const file_exception &f) {
+	IFileDataSource in(fname);
+	if (!in.good()) {
 		if (!Game::is_editing())    // Ok if map-editing.
-			throw;
+			throw file_read_exception(fname);
 		std::cerr << "Warning (map-editing): Couldn't open '" <<
 		          fname << "'" << endl;
 		return;
 	}
 
-	IStreamDataSource in(&in_stream);
-
 	U7remove(USEDAT);
 	U7remove(USEVARS);
 	U7remove(U7NBUF_DAT);
@@ -232,9 +232,7 @@ void Game_window::restore_gamedat(
 
 	cout.flush();
 
-#ifdef RED_PLASMA
 	load_palette_timer = 0;
-#endif
 }
 
 /*
@@ -279,85 +277,43 @@ static const char *sisavefiles[] = {
 };
 static const int sinumsavefiles = array_size(sisavefiles);
 
-/*
- *  Save a single file into an IFF repository.
- *
- *  Output: Length of data saved.
- *      Errors reported.
- */
-
-static long Savefile(
-    ostream &out,           // Write here.
-    const char *fname           // Name of file to save.
-) {
-	ifstream in_stream;
-	try {
-		U7open(in_stream, fname);
-	} catch (exult_exception &e) {
-		if (Game::is_editing())
-			return 0;   // Newly developed game.
-		throw;
-	}
-	IStreamDataSource in(&in_stream);
-	long len = in.getSize();
-	in.seek(0);
-	char namebuf[13];       // First write 13-byte name.
-	memset(namebuf, 0, sizeof(namebuf));
-	const char *base = strrchr(fname, '/');// Want the base name.
-	if (!base)
-		base = strrchr(fname, '\\');
-	if (base)
-		base++;
-	else
-		base = fname;
-	strncpy(namebuf, base, sizeof(namebuf));
-	out.write(namebuf, sizeof(namebuf));
-	char *buf = new char[len];  // Get it all at once.
-	in.read(buf, len);
-	out.write(buf, len);
-	delete [] buf;
-	if (!in_stream.good())
-		throw file_read_exception(fname);
-	return len + 13;        // Include filename.
-}
-
-static long SavefileFromDataSource(
-    ostream &out,       // write here
+static void SavefileFromDataSource(
+    Flex_writer &flex,
     IDataSource &source, // read from here
     const char *fname   // store data using this filename
 ) {
-	long len = source.getSize();
-	char namebuf[13];
-	memset(namebuf, 0, sizeof(namebuf));
-	const char *base = strrchr(fname, '/');// Want the base name.
-	if (!base)
-		base = strrchr(fname, '\\');
-	if (base)
-		base++;
-	else
-		base = fname;
-	strncpy(namebuf, base, sizeof(namebuf));
-	out.write(namebuf, sizeof(namebuf));
-	char *buf = new char[len];
-	source.read(buf, len);
-	out.write(buf, len);
-	delete [] buf;
-	return len + 13;
+	flex.write_file(fname, source);
+}
+
+/*
+ *  Save a single file into an IFF repository.
+ */
+
+static void Savefile(
+    Flex_writer &flex,
+    const char *fname           // Name of file to save.
+) {
+	IFileDataSource source(fname);
+	if (!source.good()) {
+		if (Game::is_editing())
+			return;   // Newly developed game.
+		throw file_read_exception(fname);
+	}
+	SavefileFromDataSource(flex, source, fname);
 }
 
 inline static void save_gamedat_chunks(
     Game_map *map,
-    ostream &out,
     Flex_writer &flex) {
 	for (int schunk = 0; schunk < 12 * 12; schunk++) {
 		char iname[128];
 		//Check to see if the ireg exists before trying to
 		//save it; prevents crash when creating new maps
 		//for existing games
-		if (U7exists(map->get_schunk_file_name(U7IREG,
-		                                       schunk, iname)))
-			Savefile(out, iname);
-		flex.mark_section_done();
+		if (U7exists(map->get_schunk_file_name(U7IREG, schunk, iname)))
+			Savefile(flex, iname);
+		else
+			flex.empty_object();	// TODO: Get rid of this by making it redundant.
 	}
 }
 
@@ -371,10 +327,9 @@ void Game_window::save_gamedat(
     const char *fname,          // File to create.
     const char *savename            // User's savegame name.
 ) {
-
 	// First check for compressed save game
 #ifdef HAVE_ZIP_SUPPORT
-	if (save_compression > 0 && save_gamedat_zip(fname, savename) != false)
+	if (save_compression > 0 && save_gamedat_zip(fname, savename))
 		return;
 #endif
 
@@ -384,54 +339,45 @@ void Game_window::save_gamedat(
 	const char **savefiles = (Game::get_game_type() == BLACK_GATE) ?
 	                         bgsavefiles : sisavefiles;
 
-	ofstream out;
-	U7open(out, fname);
-	vector<Game_map *>::iterator it;
+	OFileDataSource out(fname);
 	int count = numsavefiles;   // Count up #files to write.
 	count += 12 * 12 - 1; // First map outputs IREG's directly to
 	// gamedat flex, while all others have a flex
 	// of their own contained in gamedat flex.
-	for (it = maps.begin(); it != maps.end(); ++it)
-		if (*it)
+	for (auto *map : maps)
+		if (map)
 			count++;
 	// Use samename for title.
 	Flex_writer flex(out, savename, count);
 	int i;              // Start with listed files.
 	for (i = 0; i < numsavefiles; i++) {
-		Savefile(out, savefiles[i]);
-		flex.mark_section_done();
+		Savefile(flex, savefiles[i]);
 	}
 	// Now the Ireg's.
-	for (it = maps.begin(); it != maps.end(); ++it) {
-		if (!*it)
+	for (auto *map : maps) {
+		if (!map)
 			continue;
-		if (!(*it)->get_num())
+		if (!map->get_num())
 			// Map 0 is a special case.
-			save_gamedat_chunks(*it, out, flex);
+			save_gamedat_chunks(map, flex);
 		else {
 			// Multimap directory entries. Each map is stored in their
 			// own flex file contained inside the general gamedat flex.
 			char dname[128];
 			// Need to have read/write access here.
 			std::stringstream outbuf(std::ios::in | std::ios::out | std::ios::binary);
-			OStreamDataSource outds(dynamic_cast<std::ostream *>(&outbuf));
-			Flex_writer flexbuf(&outds,
-			                    (*it)->get_mapped_name(GAMEDAT, dname), 12 * 12);
-			// Save chunks to memory flex...
-			save_gamedat_chunks(*it, outbuf, flexbuf);
-			// ... and then close it.
-			flexbuf.close();
-			IStreamDataSource inds(dynamic_cast<std::istream *>(&outbuf));
-			int len = strlen(dname);
-			if (dname[len - 1] == '/' || dname[len - 1] == '\\')
-				dname[len - 1] = 0; // Should always be the case.
-			SavefileFromDataSource(out, inds, dname);
-			flex.mark_section_done();
+			OStreamDataSource outds(&outbuf);
+			{
+				Flex_writer flexbuf(outds,
+			                        map->get_mapped_name(GAMEDAT, dname), 12 * 12);
+				// Save chunks to memory flex
+				save_gamedat_chunks(map, flexbuf);
+			}
+			outbuf.seekg(0);
+			IStreamDataSource inds(&outbuf);
+			SavefileFromDataSource(flex, inds, dname);
 		}
 	}
-	bool result = flex.close(); // Write it all out.
-	if (!result)            // ++++Better error system needed??
-		throw file_write_exception(fname);
 }
 
 /*
@@ -483,112 +429,105 @@ void Game_window::read_save_names(
 
 
 void Game_window::write_saveinfo() {
-	ofstream out_stream;
-	int i;
-
 	int save_count = 1;
 
-	try {
-		ifstream in;
-		U7open(in, GSAVEINFO);      // Open file; throws an exception
-
-		IStreamDataSource ds(&in);
-		ds.skip(10);    // Skip 10 bytes.
-		save_count += ds.read2();
-
-		in.close();
-	} catch (const file_exception & /*f*/) {
+	{
+		IFileDataSource ds(GSAVEINFO);
+		if (ds.good()) {
+			ds.skip(10);    // Skip 10 bytes.
+			save_count += ds.read2();
+		}
 	}
 
 	int party_size = party_man->get_count() + 1;
 
-	time_t t = time(0);
-	struct tm *timeinfo = localtime(&t);
+	{
+		OFileDataSource out(GSAVEINFO);   // Open file; throws an exception - Don't care
 
-	U7open(out_stream, GSAVEINFO);      // Open file; throws an exception - Don't care
-	OStreamDataSource out(&out_stream);
+		time_t t = time(nullptr);
+		tm *timeinfo = localtime(&t);
 
-	// This order must match struct SaveGame_Details
+		// This order must match struct SaveGame_Details
 
-	// Time that the game was saved
-	out.write1(timeinfo->tm_min);
-	out.write1(timeinfo->tm_hour);
-	out.write1(timeinfo->tm_mday);
-	out.write1(timeinfo->tm_mon + 1);
-	out.write2(timeinfo->tm_year + 1900);
+		// Time that the game was saved
+		out.write1(timeinfo->tm_min);
+		out.write1(timeinfo->tm_hour);
+		out.write1(timeinfo->tm_mday);
+		out.write1(timeinfo->tm_mon + 1);
+		out.write2(timeinfo->tm_year + 1900);
 
-	// The Game Time that the save was done at
-	out.write1(clock->get_minute());
-	out.write1(clock->get_hour());
-	out.write2(clock->get_day());
+		// The Game Time that the save was done at
+		out.write1(clock->get_minute());
+		out.write1(clock->get_hour());
+		out.write2(clock->get_day());
 
-	out.write2(save_count);
-	out.write1(party_size);
+		out.write2(save_count);
+		out.write1(party_size);
 
-	out.write1(0);          // Unused
+		out.write1(0);          // Unused
 
-	out.write1(timeinfo->tm_sec);   // 15
-
-	// Packing for the rest of the structure
-	for (size_t j = offsetof(SaveGame_Details, reserved0);
-	        j < sizeof(SaveGame_Details); j++)
-		out.write1(0);
-
-	for (i = 0; i < party_size ; i++) {
-		Actor *npc;
-		if (i == 0)
-			npc = main_actor;
-		else
-			npc = get_npc(party_man->get_member(i - 1));
-
-		char name[18];
-		std::string namestr = npc->get_npc_name();
-		strncpy(name, namestr.c_str(), 18);
-		out.write(name, 18);
-		out.write2(npc->get_shapenum());
-
-		out.write4(npc->get_property(Actor::exp));
-		out.write4(npc->get_flags());
-		out.write4(npc->get_flags2());
-
-		out.write1(npc->get_property(Actor::food_level));
-		out.write1(npc->get_property(Actor::strength));
-		out.write1(npc->get_property(Actor::combat));
-		out.write1(npc->get_property(Actor::dexterity));
-		out.write1(npc->get_property(Actor::intelligence));
-		out.write1(npc->get_property(Actor::magic));
-		out.write1(npc->get_property(Actor::mana));
-		out.write1(npc->get_property(Actor::training));
-
-		out.write2(npc->get_property(Actor::health));
-		out.write2(npc->get_shapefile());
+		out.write1(timeinfo->tm_sec);   // 15
 
 		// Packing for the rest of the structure
-		for (size_t j = offsetof(SaveGame_Party, reserved1);
-		        j < sizeof(SaveGame_Party); j++)
+		for (size_t j = offsetof(SaveGame_Details, reserved0);
+		        j < sizeof(SaveGame_Details); j++)
 			out.write1(0);
+
+		for (int i = 0; i < party_size ; i++) {
+			Actor *npc;
+			if (i == 0)
+				npc = main_actor;
+			else
+				npc = get_npc(party_man->get_member(i - 1));
+
+			char name[18];
+			std::string namestr = npc->get_npc_name();
+			strncpy(name, namestr.c_str(), 18);
+			out.write(name, 18);
+			out.write2(npc->get_shapenum());
+
+			out.write4(npc->get_property(Actor::exp));
+			out.write4(npc->get_flags());
+			out.write4(npc->get_flags2());
+
+			out.write1(npc->get_property(Actor::food_level));
+			out.write1(npc->get_property(Actor::strength));
+			out.write1(npc->get_property(Actor::combat));
+			out.write1(npc->get_property(Actor::dexterity));
+			out.write1(npc->get_property(Actor::intelligence));
+			out.write1(npc->get_property(Actor::magic));
+			out.write1(npc->get_property(Actor::mana));
+			out.write1(npc->get_property(Actor::training));
+
+			out.write2(npc->get_property(Actor::health));
+			out.write2(npc->get_shapefile());
+
+			// Packing for the rest of the structure
+			for (size_t j = offsetof(SaveGame_Party, reserved1);
+			        j < sizeof(SaveGame_Party); j++)
+				out.write1(0);
+		}
 	}
 
-	out_stream.close();
+	{
+		// Save Shape
+		std::unique_ptr<Shape_file> map = create_mini_screenshot();
+		OFileDataSource out(GSCRNSHOT);     // Open file; throws an exception - Don't care
+		map->save(&out);
+	}
 
-	// Save Shape
-	Shape_file *map = create_mini_screenshot();
-	U7open(out_stream, GSCRNSHOT);      // Open file; throws an exception - Don't care
-	map->save(&out);
-	out_stream.close();
-	delete map;
-
-	// Current Exult version
-
-	U7open(out_stream, GEXULTVER);
-	getVersionInfo(out_stream);
-	out_stream.close();
+	{
+		// Current Exult version
+		ofstream out_stream;
+		U7open(out_stream, GEXULTVER);     // Open file; throws an exception - Don't care
+		getVersionInfo(out_stream);
+	}
 
 	// Exult version that started this game
 	if (!U7exists(GNEWGAMEVER)) {
-		U7open(out_stream, GNEWGAMEVER);
-		out_stream << "Unknown" << endl;
-		out_stream.close();
+		OFileDataSource out(GNEWGAMEVER);
+		const string unkver("Unknown");
+		out.write(unkver);
 	}
 }
 
@@ -649,7 +588,7 @@ void Game_window::read_saveinfo(IDataSource *in,
 	}
 }
 
-bool Game_window::get_saveinfo(int num, char *&name, Shape_file *&map, SaveGame_Details *&details, SaveGame_Party  *&party) {
+bool Game_window::get_saveinfo(int num, char *&name, std::unique_ptr<Shape_file> &map, SaveGame_Details *&details, SaveGame_Party  *&party) {
 	char fname[50];         // Set up name.
 	snprintf(fname, 50, SAVENAME, num,
 	         Game::get_game_type() == BLACK_GATE ? "bg" :
@@ -657,13 +596,14 @@ bool Game_window::get_saveinfo(int num, char *&name, Shape_file *&map, SaveGame_
 
 	// First check for compressed save game
 #ifdef HAVE_ZIP_SUPPORT
-	if (get_saveinfo_zip(fname, name, map, details, party) != false)
+	if (get_saveinfo_zip(fname, name, map, details, party))
 		return true;
 #endif
 
-	ifstream in_stream;
-	U7open(in_stream, fname);       // Open file; throws an exception
-	IStreamDataSource in(&in_stream);
+	IFileDataSource in(fname);
+	if (!in.good()) {
+		throw file_read_exception(fname);
+	}
 	// in case of an error.
 	// Always try to Read Name
 	char buf[0x50];
@@ -677,81 +617,66 @@ bool Game_window::get_saveinfo(int num, char *&name, Shape_file *&map, SaveGame_
 
 	// Now get dir info
 	in.seek(0x54);          // Get to where file count sits.
-	int numfiles = in.read4();
+	size_t numfiles = in.read4();
 	in.seek(0x80);          // Get to file info.
 	// Read pos., length of each file.
-	long *finfo = new long[2 * numfiles];
-	int i;
-	for (i = 0; i < numfiles; i++) {
+	auto finfo = std::make_unique<uint32[]>(2 * numfiles);
+	for (size_t i = 0; i < numfiles; i++) {
 		finfo[2 * i] = in.read4();  // The position, then the length.
 		finfo[2 * i + 1] = in.read4();
 	}
 
 	// Always first two entires
-	for (i = 0; i < 2; i++) { // Now read each file.
+	for (size_t i = 0; i < 2; i++) { // Now read each file.
 		// Get file length.
-		int len = finfo[2 * i + 1] - 13;
-		if (len <= 0)
+		size_t len = finfo[2 * i + 1];
+		if (len <= 13)
 			continue;
+		len -= 13;
 		in.seek(finfo[2 * i]);  // Get to it.
 		char fname[50];     // Set up name.
 		strcpy(fname, GAMEDAT);
 		in.read(&fname[sizeof(GAMEDAT) - 1], 13);
-		int namelen = strlen(fname);
+		size_t namelen = strlen(fname);
 		// Watch for names ending in '.'.
 		if (fname[namelen - 1] == '.')
 			fname[namelen - 1] = 0;
 
 		if (!strcmp(fname, GSCRNSHOT)) {
-			char *buf = new char[len];
-			in.read(buf, len);
-			IBufferDataSource ds(buf, len);
-			map = new Shape_file(&ds);
-			delete [] buf;
+			auto ds = in.makeSource(len);
+			map = std::make_unique<Shape_file>(ds.get());
 		} else if (!strcmp(fname, GSAVEINFO)) {
 			read_saveinfo(&in, details, party);
 		}
 
 	}
-	in_stream.close();
-
-	delete [] finfo;
-
 	return true;
 }
 
-void Game_window::get_saveinfo(Shape_file *&map, SaveGame_Details *&details, SaveGame_Party  *&party) {
-	try {
-		ifstream in;
-		U7open(in, GSAVEINFO);      // Open file; throws an exception
-		IStreamDataSource ds(&in);
-		read_saveinfo(&ds, details, party);
-		in.close();
-	} catch (const file_exception & /*f*/) {
-		details = NULL;
-		party = NULL;
+void Game_window::get_saveinfo(std::unique_ptr<Shape_file> &map, SaveGame_Details *&details, SaveGame_Party  *&party) {
+	{
+		IFileDataSource ds(GSAVEINFO);
+		if (ds.good()) {
+			read_saveinfo(&ds, details, party);
+		} else {
+			details = nullptr;
+			party = nullptr;
+		}
 	}
 
-	try {
-		ifstream in;
-		U7open(in, GSCRNSHOT);      // Open file; throws an exception
-		IStreamDataSource ds(&in);
-		map = new Shape_file(&ds);
-		in.close();
-	} catch (const file_exception & /*f*/) {
-		// yes, this is weird, but seems to work-around a compiler
-		// problem... (gcc-2.95.2-12mdk)    -wjp
-		map = 0;
-		map = 0;
+	{
+		IFileDataSource ds(GSCRNSHOT);
+		if (ds.good()) {
+			map = std::make_unique<Shape_file>(&ds);
+		} else {
+			map.reset();
+		}
 	}
 }
 
 // Zip file support
 #ifdef HAVE_ZIP_SUPPORT
 
-#include "files/zip/unzip.h"
-#include "files/zip/zip.h"
-
 static const char *remove_dir(const char *fname) {
 	const char *base = strchr(fname, '/');// Want the base name.
 	if (!base)
@@ -763,7 +688,7 @@ static const char *remove_dir(const char *fname) {
 }
 
 
-bool Game_window::get_saveinfo_zip(const char *fname, char *&name, Shape_file *&map, SaveGame_Details *&details, SaveGame_Party  *&party) {
+bool Game_window::get_saveinfo_zip(const char *fname, char *&name, std::unique_ptr<Shape_file> &map, SaveGame_Details *&details, SaveGame_Party  *&party) {
 	// If a flex, so can't read it
 	if (Flex::is_flex(fname)) return false;
 
@@ -780,26 +705,26 @@ bool Game_window::get_saveinfo_zip(const char *fname, char *&name, Shape_file *&
 
 	// Things we need
 	unz_file_info file_info;
-	char *buf = 0;
+	char *buf = nullptr;
 
 	// Get the screenshot first
 	if (unzLocateFile(unzipfile, remove_dir(GSCRNSHOT), 2) == UNZ_OK) {
-		unzGetCurrentFileInfo(unzipfile, &file_info, NULL, 0, NULL, 0, NULL, 0);
+		unzGetCurrentFileInfo(unzipfile, &file_info, nullptr, 0, nullptr, 0, nullptr, 0);
 		buf = new char[file_info.uncompressed_size];
 
 		unzOpenCurrentFile(unzipfile);
 		unzReadCurrentFile(unzipfile, buf, file_info.uncompressed_size);
 		if (unzCloseCurrentFile(unzipfile) == UNZ_OK) {
 			IBufferDataSource ds(buf, file_info.uncompressed_size);
-			map = new Shape_file(&ds);
+			map = std::make_unique<Shape_file>(&ds);
+		} else {
+			delete [] buf;
 		}
-
-		delete [] buf;
 	}
 
 	// Now saveinfo
 	if (unzLocateFile(unzipfile, remove_dir(GSAVEINFO), 2) == UNZ_OK) {
-		unzGetCurrentFileInfo(unzipfile, &file_info, NULL, 0, NULL, 0, NULL, 0);
+		unzGetCurrentFileInfo(unzipfile, &file_info, nullptr, 0, nullptr, 0, nullptr, 0);
 		buf = new char[file_info.uncompressed_size];
 
 		unzOpenCurrentFile(unzipfile);
@@ -807,9 +732,9 @@ bool Game_window::get_saveinfo_zip(const char *fname, char *&name, Shape_file *&
 		if (unzCloseCurrentFile(unzipfile) == UNZ_OK) {
 			IBufferDataSource ds(buf, file_info.uncompressed_size);
 			read_saveinfo(&ds, details, party);
+		} else {
+			delete [] buf;
 		}
-
-		delete [] buf;
 	}
 
 	unzClose(unzipfile);
@@ -822,12 +747,10 @@ bool Game_window::get_saveinfo_zip(const char *fname, char *&name, Shape_file *&
 bool Game_window::Restore_level2(
     void *uzf, const char *dirname, int dirlen
 ) {
-	unzFile unzipfile = static_cast<unzFile>(uzf);
+	auto *unzipfile = static_cast<unzFile>(uzf);
 
 	char oname[50];     // Set up name.
 	char *oname2 = oname + sizeof(GAMEDAT) + dirlen - 1;
-	char size_buffer[4];
-	int size;
 	strcpy(oname, dirname);
 	oname2[0] = '/';
 	oname2++;
@@ -849,12 +772,13 @@ bool Game_window::Restore_level2(
 		if (*oname2 == 0) break;
 
 		// Get file length.
+		unsigned char size_buffer[4];
 		if (unzReadCurrentFile(unzipfile, size_buffer, 4) != 4) {
 			std::cerr << "Couldn't read for size" << std::endl;
 			return false;
 		}
-		IBufferDataSource ds(size_buffer, 4);
-		size = ds.read4();
+		const unsigned char *ptr = size_buffer;
+		int size = Read4(ptr);
 
 		if (size) {
 			// Watch for names ending in '.'.
@@ -882,7 +806,7 @@ bool Game_window::Restore_level2(
 				return false;
 			}
 			out.close();
-			CYCLE_RED_PLASMA();
+			cycle_load_palette();
 		}
 	}
 
@@ -905,10 +829,8 @@ bool Game_window::restore_gamedat_zip(
 	} catch (const file_exception & /*f*/) {
 		return false;       // Ignore if not found.
 	}
-#ifdef RED_PLASMA
 	// Display red plasma during load...
 	setup_load_palette();
-#endif
 	std::string filestr = get_system_path(fname);
 	unzFile unzipfile = unzOpen(filestr.c_str());
 	if (!unzipfile) return false;
@@ -949,15 +871,15 @@ bool Game_window::restore_gamedat_zip(
 
 		// For safer handling, better do it in two steps.
 		unzGetCurrentFileInfo(unzipfile, &file_info,
-		                      NULL, 0,
-		                      NULL, 0,
-		                      NULL, 0);
+		                      nullptr, 0,
+		                      nullptr, 0,
+		                      nullptr, 0);
 		// Get the needed buffer size.
 		int filenamelen = file_info.size_filename;
-		unzGetCurrentFileInfo(unzipfile, 0,
+		unzGetCurrentFileInfo(unzipfile, nullptr,
 		                      oname2, filenamelen,
-		                      NULL, 0,
-		                      NULL, 0);
+		                      nullptr, 0,
+		                      nullptr, 0);
 		oname2[filenamelen] = 0;
 
 		// Get file length.
@@ -970,12 +892,12 @@ bool Game_window::restore_gamedat_zip(
 			// Files for map # > 0; create dir first.
 			U7mkdir(oname, 0755);
 			// Put a final marker in the dir name.
-			if (Restore_level2(unzipfile, oname, filenamelen) == false)
+			if (!Restore_level2(unzipfile, oname, filenamelen))
 				abort("Error reading level2 from zip '%s'.", fname);
 			continue;
 		} else if (!std::strcmp("GAMEDAT", oname2)) {
 			// Put a final marker in the dir name.
-			if (Restore_level2(unzipfile, oname, 0) == false)
+			if (!Restore_level2(unzipfile, oname, 0))
 				abort("Error reading level2 from zip '%s'.", fname);
 			// Flag that this is a level 2 save.
 			level2zip = true;
@@ -1018,16 +940,14 @@ bool Game_window::restore_gamedat_zip(
 			abort("Error closing current in zip '%s'.", fname);
 		delete [] buf;
 
-		CYCLE_RED_PLASMA();
+		cycle_load_palette();
 	} while (unzGoToNextFile(unzipfile) == UNZ_OK);
 
 	unzClose(unzipfile);
 
 	cout.flush();
 
-#ifdef RED_PLASMA
 	load_palette_timer = 0;
-#endif
 
 	return true;
 }
@@ -1035,28 +955,20 @@ bool Game_window::restore_gamedat_zip(
 
 // Level 1 Compression
 static bool Save_level1(zipFile zipfile, const char *fname) {
-	ifstream in;
-	try {
-		U7open(in, fname);
-	} catch (exult_exception &e) {
+	IFileDataSource ds(fname);
+	if (!ds.good()) {
 		if (Game::is_editing())
 			return false;   // Newly developed game.
-		throw;
+		throw file_read_exception(fname);
 	}
 
+	const size_t size = ds.getSize();
+	const auto buf = ds.readN(size);
 
-	IStreamDataSource ds(&in);
+	zipOpenNewFileInZip(zipfile, remove_dir(fname), nullptr, nullptr, 0,
+	                    nullptr, 0, nullptr, Z_DEFLATED, Z_BEST_COMPRESSION);
 
-	unsigned int size = ds.getSize();
-	char *buf = new char[size];
-	ds.read(buf, size);
-
-
-	zipOpenNewFileInZip(zipfile, remove_dir(fname), NULL, NULL, 0,
-	                    NULL, 0, NULL, Z_DEFLATED, Z_BEST_COMPRESSION);
-
-	zipWriteInFileInZip(zipfile, buf, size);
-	delete [] buf;
+	zipWriteInFileInZip(zipfile, buf.get(), size);
 
 	return zipCloseFileInZip(zipfile) == ZIP_OK;
 }
@@ -1069,43 +981,45 @@ static bool Begin_level2(zipFile zipfile, int mapnum) {
 		oname[7] = 0;
 	} else {
 		strcpy(oname, "map");
-		oname[3] = '0' + mapnum / 16;
-		int lb = mapnum % 16;
-		oname[4] = lb < 10 ? ('0' + lb) : ('a' + (lb - 10));
+		constexpr static const char hexLUT[] = "0123456789abcdef";
+		oname[3] = hexLUT[mapnum / 16];
+		oname[4] = hexLUT[mapnum % 16];
 		oname[5] = 0;
 	}
-	return zipOpenNewFileInZip(zipfile, oname, NULL, NULL, 0,
-	                           NULL, 0, NULL, Z_DEFLATED, Z_BEST_COMPRESSION) == ZIP_OK;
+	return zipOpenNewFileInZip(zipfile, oname, nullptr, nullptr, 0,
+	                           nullptr, 0, nullptr, Z_DEFLATED, Z_BEST_COMPRESSION) == ZIP_OK;
 }
 
 static bool Save_level2(zipFile zipfile, const char *fname) {
-	ifstream in;
-	try {
-		U7open(in, fname);
-	} catch (exult_exception &e) {
+	IFileDataSource ds(fname);
+	if (!ds.good()) {
 		if (Game::is_editing())
 			return false;   // Newly developed game.
-		throw;
+		throw file_read_exception(fname);
 	}
 
-	IStreamDataSource ds(&in);
-
-	uint32 size = ds.getSize();
-	char *buf = new char[size < 13 ? 13 : size]; // We want at least 13 bytes
+	const size_t size = ds.getSize();
+	char * const buf = new char[size < 13 ? 13 : size]; // We want at least 13 bytes
 
 	// Filename first
 	memset(buf, 0, 13);
-	const char *fname2 = strrchr(fname, '/') + 1;
-	if (!fname2)
-		fname2 = strchr(fname, '\\') + 1;
-	strncpy(buf, fname2 ? fname2 : fname, 13);
+	const char *fname2 = strrchr(fname, '/');
+	if (!fname2) {
+		fname2 = strchr(fname, '\\');
+	}
+	if (fname2) {
+		fname2++;
+	} else {
+		fname2 = fname;
+	}
+	strncpy(buf, fname2, 13);
 	int err = zipWriteInFileInZip(zipfile, buf, 12);
 
 	// Size of the file
 	if (err == ZIP_OK) {
 		// Must be platform independant
-		OBufferDataSource bds(buf, 4);
-		bds.write4(size);
+		auto *ptr = reinterpret_cast<unsigned char*>(buf);
+		Write4(ptr, size);
 		err = zipWriteInFileInZip(zipfile, buf, 4);
 	}
 
@@ -1128,7 +1042,7 @@ static bool End_level2(zipFile zipfile) {
 	if (err == ZIP_OK) err = zipWriteInFileInZip(zipfile, &zeros, 4);
 	if (err == ZIP_OK) err = zipWriteInFileInZip(zipfile, &zeros, 4);
 
-	return zipCloseFileInZip(zipfile) == ZIP_OK;
+	return zipCloseFileInZip(zipfile) == ZIP_OK && err == ZIP_OK;
 }
 
 
@@ -1145,7 +1059,6 @@ bool Game_window::save_gamedat_zip(
 	                   bgnumsavefiles : sinumsavefiles;
 	const char **savefiles = (Game::get_game_type() == BLACK_GATE) ?
 	                         bgsavefiles : sisavefiles;
-	vector<Game_map *>::iterator it;
 
 	// Name
 	{
@@ -1167,14 +1080,14 @@ bool Game_window::save_gamedat_zip(
 			Save_level1(zipfile, savefiles[i]);
 
 		// Now the Ireg's.
-		for (it = maps.begin(); it != maps.end(); ++it) {
-			if (!*it)
+		for (auto *map : maps) {
+			if (!map)
 				continue;
 			for (int schunk = 0; schunk < 12 * 12; schunk++) {
 				//Check to see if the ireg exists before trying to
 				//save it; prevents crash when creating new maps
 				//for existing games
-				if (U7exists((*it)->get_schunk_file_name(U7IREG,
+				if (U7exists(map->get_schunk_file_name(U7IREG,
 				             schunk, iname)))
 					Save_level1(zipfile, iname);
 			}
@@ -1195,20 +1108,20 @@ bool Game_window::save_gamedat_zip(
 			Save_level2(zipfile, savefiles[i]);
 
 		// Now the Ireg's.
-		for (it = maps.begin(); it != maps.end(); ++it) {
-			if (!*it)
+		for (auto *map : maps) {
+			if (!map)
 				continue;
-			if ((*it)->get_num() != 0) {
+			if (map->get_num() != 0) {
 				// Finish the open file (GAMEDAT or mapXX).
 				End_level2(zipfile);
 				// Start the next file (mapXX).
-				Begin_level2(zipfile, (*it)->get_num());
+				Begin_level2(zipfile, map->get_num());
 			}
 			for (int schunk = 0; schunk < 12 * 12; schunk++)
 				//Check to see if the ireg exists before trying to
 				//save it; prevents crash when creating new maps
 				//for existing games
-				if (U7exists((*it)->get_schunk_file_name(U7IREG,
+				if (U7exists(map->get_schunk_file_name(U7IREG,
 				             schunk, iname)))
 					Save_level2(zipfile, iname);
 		}
