@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2013  The Exult Team
+ *  Copyright (C) 2000-2022  The Exult Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,26 +21,27 @@
 
 #include "game.h"
 
+struct File_spec;
 
 class BG_Game: public Game {
 public:
 	BG_Game();
-	~BG_Game();
 
-	virtual void play_intro();
-	virtual void end_game(bool success);
-	virtual void top_menu();
-	virtual void show_quotes();
-	virtual void show_credits();
-	virtual bool new_game(Vga_file &shapes);
-	virtual int  get_start_tile_x() {
-		return (64 * c_tiles_per_chunk);
+	void play_intro() override;
+	void end_game(bool success) override;
+	void top_menu() override;
+	void show_quotes() override;
+	void show_credits() override;
+	bool new_game(Vga_file &shapes) override;
+	int  get_start_tile_x() override {
+		return 64 * c_tiles_per_chunk;
 	}
-	virtual int  get_start_tile_y() {
-		return (136 * c_tiles_per_chunk);
+	int  get_start_tile_y() override {
+		return 136 * c_tiles_per_chunk;
 	}
-	virtual void show_journey_failed();
-	virtual Shape_frame *get_menu_shape();
+	void show_journey_failed() override;
+	Shape_frame *get_menu_shape() override;
+	static File_spec get_sfx_subflex();
 
 private:
 	Vga_file shapes;
