@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2001-2013  The Exult Team
+ *  Copyright (C) 2001-2022  The Exult Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -26,24 +26,6 @@
  */
 class DEV_Game: public SI_Game {
 public:
-	DEV_Game() : SI_Game()
-	{  }
-//	~DEV_Game();
-#if 0
-	virtual void play_intro();
-	virtual void end_game(bool success);
-	virtual void top_menu();
-	virtual void show_quotes();
-	virtual void show_credits();
-	virtual bool new_game(Vga_file &shapes);
-	virtual int  get_start_tile_x() {
-		return (25 * c_tiles_per_chunk);
-	}
-	virtual int  get_start_tile_y() {
-		return (155 * c_tiles_per_chunk);
-	}
-	virtual void show_journey_failed();
-#endif
 };
 
 
