@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2013  The Exult Team
+ *  Copyright (C) 2000-2022  The Exult Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,22 +25,21 @@
 class SI_Game: public Game {
 public:
 	SI_Game();
-	~SI_Game();
 
-	virtual void play_intro();
-	virtual void end_game(bool success);
-	virtual void top_menu();
-	virtual void show_quotes();
-	virtual void show_credits();
-	virtual bool new_game(Vga_file &shapes);
-	virtual int  get_start_tile_x() {
-		return (25 * c_tiles_per_chunk);
+	void play_intro() override;
+	void end_game(bool success) override;
+	void top_menu() override;
+	void show_quotes() override;
+	void show_credits() override;
+	bool new_game(Vga_file &shapes) override;
+	int  get_start_tile_x() override {
+		return 25 * c_tiles_per_chunk;
 	}
-	virtual int  get_start_tile_y() {
-		return (155 * c_tiles_per_chunk);
+	int  get_start_tile_y() override {
+		return 155 * c_tiles_per_chunk;
 	}
-	virtual void show_journey_failed();
-	virtual Shape_frame *get_menu_shape();
+	void show_journey_failed() override;
+	Shape_frame *get_menu_shape() override;
 };
 
 
