@@ -2,7 +2,7 @@
  *  gamerend.h - Rendering methods.
  *
  *  Copyright (C) 1998-1999  Jeffrey S. Freedman
- *  Copyright (C) 2000-2013  The Exult Team
+ *  Copyright (C) 2000-2022  The Exult Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,16 +22,16 @@
 #ifndef GAMEREND_H
 #define GAMEREND_H 1
 
+class Game_object;
+
 /*
  *  A helper-class for rendering.
  */
 class Game_render {
-	unsigned long render_seq;   // For marking rendered objects.
-	int skip;           // Set for each render.  We skip
+	unsigned long render_seq = 0;   // For marking rendered objects.
+	int skip = 31;                  // Set for each render.  We skip
 	//   painting at or above this.
 public:
-	Game_render() : render_seq(0), skip(31)
-	{  }
 	void paint_terrain_only(int start_chunkx, int start_chunky,
 	                        int stop_chunkx, int stop_chunky);
 	// Render the map & objects.
@@ -47,8 +47,9 @@ public:
 	// Paint an obj. after dependencies.
 	void paint_object(Game_object *obj);
 	// Render dungeon blackness
-	void paint_blackness(int cx, int cy, int stop_chunkx, int stop_chunky,
+	void paint_blackness(int start_chunkx, int start_chunky, int stop_chunkx, int stop_chunky,
 	                     int index = 0);
+	int get_light_strength(const Game_object *obj, const Game_object *av) const;
 };
 
 #endif
