@@ -2,7 +2,7 @@
  *  gamewin.h - X-windows Ultima7 map browser.
  *
  *  Copyright (C) 1998-1999  Jeffrey S. Freedman
- *  Copyright (C) 2000-2013  The Exult Team
+ *  Copyright (C) 2000-2022  The Exult Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,21 +29,10 @@
 #include "vgafile.h"
 #include "shapeid.h"
 
+#include <memory>
 #include <string>   // STL string
 #include <vector>
 
-#define RED_PLASMA  1
-
-#ifdef RED_PLASMA
-#define CYCLE_RED_PLASMA()  cycle_load_palette()
-#else
-#define CYCLE_RED_PLASMA()
-#endif
-
-#ifdef __IPHONEOS__
-#include "objs/objs.h"
-#endif
-
 #ifndef ATTR_PRINTF
 #ifdef __GNUC__
 #define ATTR_PRINTF(x,y) __attribute__((format(printf, (x), (y))))
@@ -85,8 +74,13 @@ class ShapeID;
 class Shape_info;
 class Game_render;
 class Effects_manager;
+using Actor_shared = std::shared_ptr<Actor>;
 
-using std::vector;
+struct Position2d {
+	int x;
+	int y;
+};
+using Game_object_map_xy = std::map<Game_object*, Position2d>;
 
 /*
  *  The main game window:
@@ -97,7 +91,7 @@ class Game_window {
 	Dragging_info *dragging;    // Dragging info:
 	Effects_manager *effects;   // Manages special effects.
 	Game_clock *clock;      // Keeps track of time.
-	vector<Game_map *> maps; // Hold all terrain.
+	std::vector<Game_map *> maps; // Hold all terrain.
 	Game_map *map;          // The current map.
 	Game_render *render;        // Helps with rendering.
 	Gump_manager *gump_man;     // Open containers on screen.
@@ -129,13 +123,12 @@ class Game_window {
 	Barge_object *moving_barge; // ->cart/ship that's moving, or 0.
 	Main_actor *main_actor;     // Main sprite to move around.
 	Actor *camera_actor;        // What to center view around.
-	vector<Actor *> npcs;       // Array of NPC's + the Avatar.
-	vector<Dead_body *> bodies; // Corresponding Dead_body's.
-	Deleted_objects *removed;   // List of 'removed' objects.
+	std::vector<Actor_shared> npcs;  // Array of NPC's + the Avatar.
+	std::vector<Dead_body *> bodies; // Corresponding Dead_body's.
 	// Rendering info:
 	int scrolltx, scrollty;     // Top-left tile of screen.
-	Rectangle scroll_bounds;    // Walking outside this scrolls.
-	Rectangle dirty;        // Dirty rectangle.
+	TileRect scroll_bounds;    // Walking outside this scrolls.
+	TileRect dirty;        // Dirty rectangle.
 	// Savegames:
 	char *save_names[10];       // Names of saved games.
 	// Options:
@@ -152,17 +145,22 @@ class Game_window {
 	uint8 use_shortcutbar; // 0 = no, 1 = trans, 2 = yes
 	Pixel_colors outline_color;
 	bool sb_hide_missing;
+	bool extended_intro;
+
+	// Touch Options
+	bool item_menu;
+	int dpad_location;
+	bool touch_pathfind;
+
 	// Private methods:
 	void set_scrolls(Tile_coord cent);
 	void clear_world(bool restoremapedit);      // Clear out world's contents.
 	void read_save_names();     // Read in saved-game names.
 	long check_time_stopped();
 
-#ifdef RED_PLASMA
 	// Red plasma animation during game load
 	uint32 load_palette_timer;
 	int plasma_start_color, plasma_cycle_range;
-#endif
 
 public:
 	friend class Game_render;
@@ -215,27 +213,26 @@ public:
 	inline int get_scrollty() const {
 		return scrollty;
 	}
-	inline Rectangle get_game_rect() const  // Get window's rectangle.
-//		{ return Rectangle(win->get_start_x(), win->get_start_y(), win->get_full_width(), win->get_full_height());}
+	inline TileRect get_game_rect() const  // Get window's rectangle.
 	{
-		return Rectangle(0, 0, win->get_game_width(), win->get_game_height());
+		return TileRect(0, 0, win->get_game_width(), win->get_game_height());
 	}
-	inline Rectangle get_full_rect() const { // Get window's rectangle.
-		return Rectangle(win->get_start_x(), win->get_start_y(), win->get_full_width(), win->get_full_height());
+	inline TileRect get_full_rect() const { // Get window's rectangle.
+		return TileRect(win->get_start_x(), win->get_start_y(), win->get_full_width(), win->get_full_height());
 	}
-	Rectangle get_win_tile_rect() const { // Get it in tiles, rounding up.
-		return Rectangle(get_scrolltx(), get_scrollty(),
-		                 (win->get_game_width() + c_tilesize - 1) / c_tilesize,
-		                 (win->get_game_height() + c_tilesize - 1) / c_tilesize);
+	TileRect get_win_tile_rect() const { // Get it in tiles, rounding up.
+		return TileRect(get_scrolltx(), get_scrollty(),
+		                (win->get_game_width() + c_tilesize - 1) / c_tilesize,
+		                (win->get_game_height() + c_tilesize - 1) / c_tilesize);
 	}
 	// Clip rectangle to window's.
-	Rectangle clip_to_game(Rectangle const &r) const {
-		Rectangle wr = get_game_rect();
-		return (r.intersect(wr));
+	TileRect clip_to_game(TileRect const &r) const {
+		TileRect wr = get_game_rect();
+		return r.intersect(wr);
 	}
-	Rectangle clip_to_win(Rectangle const &r) const {
-		Rectangle wr = get_full_rect();
-		return (r.intersect(wr));
+	TileRect clip_to_win(TileRect const &r) const {
+		TileRect wr = get_full_rect();
+		return r.intersect(wr);
 	}
 	// Resize event occurred.
 	void resized(unsigned int neww, unsigned int newh, bool newfs,
@@ -319,13 +316,41 @@ public:
 	void set_sb_hide_missing_items(bool s) {
 		sb_hide_missing = s;
 	}
+	bool get_extended_intro() const {
+		return extended_intro;
+	}
+	void set_extended_intro(bool i) {
+		extended_intro = i;
+	}
+	/*
+	 * Touch options:
+ 	*/
+	bool get_item_menu() const {
+		return item_menu;
+	}
+	void set_item_menu(bool s) {
+		item_menu = s;
+	}
+	inline void set_dpad_location(int a) {
+		dpad_location = a;
+	}
+	inline int get_dpad_location() const {
+		return dpad_location;
+	}
+	bool get_touch_pathfind() const {
+		return touch_pathfind;
+	}
+	void set_touch_pathfind(bool s) {
+		touch_pathfind = s;
+	}
+
 	/*
 	 *  Game components:
 	 */
 	inline Game_map *get_map() const {
 		return map;
 	}
-	inline const vector<Game_map *> &get_maps() const {
+	inline const std::vector<Game_map *> &get_maps() const {
 		return maps;
 	}
 	inline Usecode_machine *get_usecode() const {
@@ -361,7 +386,7 @@ public:
 	/*
 	 *  ExultStudio support:
 	 */
-	Map_patch_collection *get_map_patches();
+	Map_patch_collection& get_map_patches();
 	// Locate shape (for EStudio).
 	bool locate_shape(int shapenum, bool upwards, int frnum, int qual);
 	void send_location();       // Send our location to EStudio.
@@ -412,12 +437,12 @@ public:
 		return ambient_light || special_light != 0;
 	}
 	// Light spell.
-	void add_special_light(int minutes);
+	void add_special_light(int units);
 	void toggle_ambient_light(bool state) {
 		ambient_light = state;
 	}
 	// Handle 'stop time' spell.
-	void set_time_stopped(long ticks);
+	void set_time_stopped(long delay);
 	long is_time_stopped() {
 		return !time_stopped ? 0 : check_time_stopped();
 	}
@@ -427,10 +452,7 @@ public:
 	void set_std_delay(int msecs) {
 		std_delay = msecs;
 	}
-	inline Actor *get_npc(long npc_num) const {
-		return (npc_num >= 0 && npc_num < static_cast<int>(npcs.size())) ?
-		       npcs[npc_num] : 0;
-	}
+	Actor *get_npc(long npc_num) const;
 	void locate_npc(int npc_num);
 	void set_body(int npc_num, Dead_body *body) {
 		if (npc_num >= static_cast<int>(bodies.size()))
@@ -466,7 +488,7 @@ public:
 	Actor *find_witness(Actor  *&closest_npc, int align);
 	void theft();           // Handle thievery.
 	static int get_guard_shape();
-	void call_guards(Actor *witness = 0, bool theft = false);
+	void call_guards(Actor *witness = nullptr, bool theft = false);
 	void stop_arresting();
 	void attack_avatar(int create_guards = 0, int align = 0);
 	bool is_hostile_nearby() const; // detects if hostiles are nearby for movement speed
@@ -476,7 +498,7 @@ public:
 	 *  Rendering:
 	 */
 	inline void set_painted() { // Force blit.
-		painted = 1;
+		painted = true;
 	}
 	inline bool was_painted() const {
 		return painted;
@@ -501,22 +523,22 @@ public:
 	                       int toptx, int topty, int skip_above = 31);
 	// Paint area of image.
 	void paint(int x, int y, int w, int h);
-	void paint(Rectangle &r) {
+	void paint(TileRect &r) {
 		paint(r.x, r.y, r.w, r.h);
 	}
 	void paint();           // Paint whole image.
 	// Paint 'dirty' rectangle.
 	void paint_dirty();
 	void set_all_dirty() {      // Whole window.
-		dirty = Rectangle(win->get_start_x(), win->get_start_y(), win->get_full_width(), win->get_full_height());
+		dirty = TileRect(win->get_start_x(), win->get_start_y(), win->get_full_width(), win->get_full_height());
 	}
-	void add_dirty(Rectangle const &r) { // Add rectangle to dirty area.
+	void add_dirty(TileRect const &r) { // Add rectangle to dirty area.
 		dirty = dirty.w > 0 ? dirty.add(r) : r;
 	}
 	// Add dirty rect. for obj. Rets. false
 	//   if not on screen.
 	bool add_dirty(const Game_object *obj) {
-		Rectangle rect = get_shape_rect(obj);
+		TileRect rect = get_shape_rect(obj);
 		rect.enlarge(1 + c_tilesize / 2);
 		rect = clip_to_win(rect);
 		if (rect.w > 0 && rect.h > 0) {
@@ -538,17 +560,15 @@ public:
 		if (a == camera_actor) return scroll_if_needed(t);
 		else return false;
 	}
-#if 1
 	// Show abs. location of mouse.
 	void show_game_location(int x, int y);
-#endif
 	// Get screen area of shape at pt.
-	Rectangle get_shape_rect(const Shape_frame *s, int x, int y) const {
-		return Rectangle(x - s->xleft, y - s->yabove,
+	TileRect get_shape_rect(const Shape_frame *s, int x, int y) const {
+		return TileRect(x - s->get_xleft(), y - s->get_yabove(),
 		                 s->get_width(), s->get_height());
 	}
 	// Get screen area used by object.
-	Rectangle get_shape_rect(const Game_object *obj) const;
+	TileRect get_shape_rect(const Game_object *obj) const;
 	// Get screen loc. of object.
 	void get_shape_location(const Game_object *obj, int &x, int &y);
 	void get_shape_location(Tile_coord const &t, int &x, int &y);
@@ -568,26 +588,26 @@ public:
 	void init_files(bool cycle = true); // Load all files
 
 	// From Gamedat
-	void get_saveinfo(Shape_file *&map,
+	void get_saveinfo(std::unique_ptr<Shape_file> &map,
 	                  SaveGame_Details *&details,
 	                  SaveGame_Party  *&party);
 	// From Savegame
 	bool get_saveinfo(int num, char *&name,
-	                  Shape_file *&map,
+	                  std::unique_ptr<Shape_file> &map,
 	                  SaveGame_Details *&details,
 	                  SaveGame_Party  *&party);
 	void read_saveinfo(IDataSource *in,
 	                   SaveGame_Details *&details,
 	                   SaveGame_Party  *&party);
-#ifdef HAVE_ZIP_SUPPORT
 private:
+#ifdef HAVE_ZIP_SUPPORT
 	bool get_saveinfo_zip(const char *fname, char *&name,
-	                      Shape_file *&map,
+	                      std::unique_ptr<Shape_file> &map,
 	                      SaveGame_Details *&details,
 	                      SaveGame_Party  *&party);
-	void restore_flex_files(IDataSource &ds, const char *basepath);
-public:
 #endif
+	void restore_flex_files(IDataSource &in, const char *basepath);
+public:
 	void write_saveinfo();      // Write the save info to gamedat
 	inline char *get_save_name(int i) const { // Get ->saved-game name.
 		return save_names[i];
@@ -608,7 +628,7 @@ public:
 #ifdef HAVE_ZIP_SUPPORT
 private:
 	bool save_gamedat_zip(const char *fname, const char *savename);
-	bool Restore_level2(void *unzipfile, const char *dirname, int dirlen);
+	bool Restore_level2(void *uzf, const char *dirname, int dirlen);
 	bool restore_gamedat_zip(const char *fname);
 public:
 #endif
@@ -637,15 +657,12 @@ public:
 		return allow_right_pathfind;
 	}
 	void teleport_party(Tile_coord const &t, bool skip_eggs = false,
-	                    int new_map = -1, bool no_status_check = true);
+	                    int newmap = -1, bool no_status_check = true);
 	bool activate_item(int shnum, int frnum = c_any_framenum,
 	                   int qual = c_any_qual); // Activate item in party.
 	// Find object (x, y) is in.
 	Game_object *find_object(int x, int y);
-#ifdef __IPHONEOS__
-	typedef std::map<Game_object *, int *> Game_object_map_xy;
-	void find_nearby_objects(Game_object_map_xy *mobjxy, int x, int y, Gump *gump = NULL);
-#endif
+	void find_nearby_objects(Game_object_map_xy& mobjxy, int x, int y, Gump *gump = nullptr);
 
 	// Show names of items clicked on.
 	void show_items(int x, int y, bool ctrl = false);
@@ -653,7 +670,6 @@ public:
 	void paused_combat_select(int x, int y);
 	ShapeID get_flat(int x, int y); // Return terrain (x, y) is in.
 	// Schedule object for deletion.
-	void delete_object(Game_object *obj);
 	// Handle a double-click in window.
 	void double_clicked(int x, int y);
 	bool start_dragging(int x, int y);
@@ -661,12 +677,12 @@ public:
 	bool drop_dragged(int x, int y, bool moved);// Done dragging.
 	void stop_dragging();
 	bool is_dragging() const {
-		return dragging != 0;
+		return dragging != nullptr;
 	}
 	int drop_at_lift(Game_object *to_drop, int x, int y, int at_lift);
 	Gump *get_dragging_gump();
 	// Create a mini-screenshot (96x60)
-	Shape_file *create_mini_screenshot();
+	std::unique_ptr<Shape_file> create_mini_screenshot();
 	/*
 	 *  Chunk-caching:
 	 */
@@ -677,11 +693,8 @@ public:
 	// Swapping a superchunk to disk emulation
 	void emulate_swapout(int scx, int scy);
 
-
-#ifdef RED_PLASMA
 	void setup_load_palette();
 	void cycle_load_palette();
-#endif
 
 private:
 	//
@@ -720,7 +733,4 @@ public:
 	}
 };
 
-void Set_renderer(Image_window8 *win, Palette *pal = 0, bool resize = false);
-bool Set_glpalette(Palette *pal = 0, bool rotation = false);
-
 #endif
