@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2000 The Exult Team
+Copyright (C) 2000-2022 The Exult Team
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -16,8 +16,8 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
-#ifndef _BOOK_GUMP_H_
-#define _BOOK_GUMP_H_
+#ifndef BOOK_GUMP_H
+#define BOOK_GUMP_H
 
 #include "Text_gump.h"
 
@@ -25,13 +25,10 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  *  A book shows text side-by-side.
  */
 class Book_gump : public Text_gump {
-	UNREPLICATABLE_CLASS_I(Book_gump, Text_gump(0))
-
 public:
 	Book_gump(int fnt = 4, int gump = -1);
-	virtual ~Book_gump() {  }
 	// Paint it and its contents.
-	virtual void paint();
+	void paint() override;
 };
 
 #endif
