@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2001 The Exult Team
+Copyright (C) 2001-2022 The Exult Team
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -26,7 +26,6 @@ class Actor;
 class Portrait_button;
 
 class Face_stats : public Gump {
-	UNREPLICATABLE_CLASS_I(Face_stats, Gump(0, 0, 0, 0, SF_GUMPS_VGA))
 	// Only allow for one to be made
 	static Face_stats   *self;
 	static int      mode;
@@ -44,39 +43,39 @@ class Face_stats : public Gump {
 	void        create_buttons();
 	void        delete_buttons();
 
-	Rectangle   region;
+	TileRect   region;
 
 public:
-	virtual ~Face_stats();
+	~Face_stats() override;
 	// Is a given point on a button?
-	virtual Gump_button *on_button(int mx, int my);
-	virtual void paint();
+	Gump_button *on_button(int mx, int my) override;
+	void paint() override;
 	// Don't close on end_gump_mode
-	virtual bool is_persistent() const {
+	bool is_persistent() const override {
 		return true;
 	}
 	// Can't be dragged with mouse
-	virtual bool is_draggable() const {
+	bool is_draggable() const override {
 		return false;
 	}
 	// Show the hand cursor
-	virtual bool no_handcursor() const {
+	bool no_handcursor() const override {
 		return true;
 	}
 
-	virtual Rectangle get_rect() const {
+	TileRect get_rect() const override {
 		return region;
 	}
-	virtual bool has_point(int x, int y) const;
+	bool has_point(int x, int y) const override;
 
 	// add dirty region, if dirty
-	virtual void update_gump();
+	void update_gump() override;
 
-	virtual int add(Game_object *obj, int mx = -1, int my = -1,
-	                int sx = -1, int sy = -1, bool dont_check = false,
-	                bool combine = false);
+	bool add(Game_object *obj, int mx = -1, int my = -1,
+	        int sx = -1, int sy = -1, bool dont_check = false,
+	        bool combine = false) override;
 
-	virtual Container_game_object *find_actor(int mx, int my);
+	Container_game_object *find_actor(int mx, int my) override;
 
 	static int get_state() {
 		return self ? mode : -1;
