@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2001-2013 The Exult Team
+Copyright (C) 2001-2022 The Exult Team
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -34,15 +34,17 @@ protected:
 
 public:
 	Text_button(Gump *p, const std::string &str, int x, int y, int w = 0, int h = 0);
-	virtual ~Text_button() {  }
-	virtual void paint();
+	void paint() override;
 
-	virtual int on_widget(int mx, int my) const;
+	bool on_widget(int mx, int my) const override;
 
-	virtual int on_button(int mx, int my) const {
+	bool on_button(int mx, int my) const override {
 		return on_widget(mx, my);
 	}
 
 };
 
+template <typename Parent, typename... Args>
+using CallbackTextButton = CallbackButtonBase<Parent, Text_button, Args...>;
+
 #endif
