@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2000-2013 The Exult Team
+Copyright (C) 2000-2022 The Exult Team
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -16,8 +16,8 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
-#ifndef _TEXT_GUMP_H_
-#define _TEXT_GUMP_H_
+#ifndef TEXT_GUMP_H
+#define TEXT_GUMP_H
 
 #include "Gump.h"
 
@@ -25,8 +25,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  *  A text gump is the base class for books and scrolls.
  */
 class Text_gump : public Gump {
-	UNREPLICATABLE_CLASS_I(Text_gump, Gump())
-
 protected:
 	char *text;         // The text.
 	int textlen;            // Length of text.
@@ -35,14 +33,14 @@ protected:
 	int font;       // The shape in fonts.vga to use
 
 public:
-	Text_gump(int shapenum, int fnt = 4) : Gump(0, shapenum),
-		text(0), textlen(0), curtop(0), curend(0), font(fnt)
+	Text_gump(int shapenum, int fnt = 4) : Gump(nullptr, shapenum),
+		text(nullptr), textlen(0), curtop(0), curend(0), font(fnt)
 	{  }
-	virtual ~Text_gump() {
+	~Text_gump() override {
 		delete [] text;
 	}
 	void add_text(const char *str); // Append text.
-	int paint_page(Rectangle const &box, int start);
+	int paint_page(TileRect const &box, int start);
 	// Next page of book/scroll.
 	int show_next_page();
 };
