@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2013  The Exult Team
+ *  Copyright (C) 2000-2022  The Exult Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,45 +16,31 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#ifndef _GUMP_UTILS_H_
-#define _GUMP_UTILS_H_
-
-#ifdef XWIN  /* Only needed in XWIN. */
-#include <sys/time.h>
-#endif
-
-#ifdef OPENBSD
-#include <sys/types.h>
-#endif
+#ifndef GUMP_UTILS_H
+#define GUMP_UTILS_H
 
 #include <unistd.h>
 
-#ifndef XWIN
 #include "SDL_timer.h"
-#endif
+#include "SDL_events.h"
 
 /*
  *  Delay between animations.
  */
 
+#define DELAY_TOTAL_MS 10
+#define DELAY_SINGLE_MS 1
+
 inline void Delay(
 ) {
-#ifdef XWIN
-	/*
-	 *  Here's a somewhat better way to delay in X:
-	 */
-	extern int xfd;
-	fd_set rfds;
-	struct timeval timer;
-	timer.tv_sec = 0;
-	timer.tv_usec = 50000;      // Try 1/50 second.
-	FD_ZERO(&rfds);
-	FD_SET(xfd, &rfds);
-	// Wait for timeout or event.
-	select(xfd + 1, &rfds, 0, 0, &timer);
-#else                   /* May use this for Linux too. */
-	SDL_Delay(10);          // Try 1/100 second.
-#endif
+	Uint32 expiration = DELAY_TOTAL_MS + SDL_GetTicks();
+	for (;;) {
+		SDL_PumpEvents();
+		if ((SDL_PeepEvents(nullptr, 0, SDL_PEEKEVENT, SDL_FIRSTEVENT, SDL_LASTEVENT) != 0) ||
+		    (static_cast<Sint32>(SDL_GetTicks()) >= static_cast<Sint32>(expiration))) return;
+
+		SDL_Delay(DELAY_SINGLE_MS);
+	}
 }
 
 #endif
