@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2016  The Exult Team
+ *  Copyright (C) 2016-2022  The Exult Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,8 +19,10 @@
 #ifndef ARRAY_SIZE_H
 #define ARRAY_SIZE_H
 
+#include <cstddef>
+
 template <typename T, size_t N>
-inline size_t array_size(const T(&)[N]) {
+constexpr inline size_t array_size(const T(&)[N]) {
 	return N;
 }
 
