@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2005 The Pentagram Team
-Copyright (C) 2010 The Exult Team
+Copyright (C) 2010-2022 The Exult Team
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -31,9 +31,9 @@ public:
 	virtual       uint32    ScaleBits() const;          //< bits for supported integer scaling
 	virtual       bool      ScaleArbitrary() const;     //< supports arbitrary scaling of any degree
 
-	virtual const char     *ScalerName() const;         //< Name Of the Scaler (1 word)
-	virtual const char     *ScalerDesc() const;         //< Desciption of the Scaler
-	virtual const char     *ScalerCopyright() const;    //< Scaler Copyright info
+	const char     *ScalerName() const override;         //< Name Of the Scaler (1 word)
+	const char     *ScalerDesc() const override;         //< Desciption of the Scaler
+	const char     *ScalerCopyright() const override;    //< Scaler Copyright info
 };
 
 }
