@@ -6,6 +6,7 @@
 
 /*
 Copyright (C) 1998 Jeffrey S. Freedman
+Copyright (C) 2001-2022 The Exult Team
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
@@ -39,7 +40,7 @@ Image_buffer::Image_buffer(
     int dpth            // Depth (bits/pixel).
 ) : width(w), height(h),
 	offset_x(0), offset_y(0),
-	depth(dpth), bits(0), line_width(w),
+	depth(dpth), bits(nullptr), line_width(w),
 	clipx(0), clipy(0),
 	clipw(w), cliph(h) {
 	switch (depth) {        // What depth?
