@@ -6,6 +6,7 @@
 
 /*
 Copyright (C) 1998 Jeffrey S. Freedman
+Copyright (C) 2000-2022 The Exult Team
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
@@ -49,20 +50,11 @@ Boston, MA  02111-1307, USA.
 #include "BilinearScaler.h"
 #include "PointScaler.h"
 
-#ifdef HAVE_OPENGL
-#ifdef MACOSX
-#include <OpenGL/gl.h>
-#else
-#include <GL/gl.h>
-#endif
-#include "shapes/glshape.h"
-#endif
-
 #include "SDL.h"
 
 #include "Configuration.h"
 
-bool SavePCX_RW(SDL_Surface *saveme, SDL_RWops *dst, bool freedst);
+bool SaveIMG_RW(SDL_Surface *saveme, SDL_RWops *dst, bool freedst, int guardband);
 
 using std::cout;
 using std::cerr;
@@ -86,8 +78,7 @@ const Image_window::ScalerConst Image_window::Hq4x("Hq4x");
 const Image_window::ScalerConst Image_window::_2xBR("2xBR");
 const Image_window::ScalerConst Image_window::_3xBR("3xBR");
 const Image_window::ScalerConst Image_window::_4xBR("4xBR");
-const Image_window::ScalerConst Image_window::OpenGL("OpenGL");
-const Image_window::ScalerConst Image_window::NumScalers(0);
+const Image_window::ScalerConst Image_window::NumScalers(nullptr);
 
 Image_window::ScalerVector Image_window::p_scalers;
 const Image_window::ScalerVector &Image_window::Scalers = Image_window::p_scalers;
@@ -102,10 +93,8 @@ int Image_window::desktop_depth = 0;
 int Image_window::windowed_8 = 0;
 int Image_window::windowed_16 = 0;
 int Image_window::windowed_32 = 0;
-#if SDL_VERSION_ATLEAST(2, 0, 1) && (defined(MACOSX) || defined(__IPHONEOS__))
-//When HighDPI is enabled we will end up with a different native scale factor, so we need to define the default
-float Image_window::nativescale = 1.0;
-#endif
+// When HighDPI is enabled we will end up with a different native scale factor, so we need to define the default
+float Image_window::nativescale = 1.0f;
 
 const int Image_window::guard_band = 4;
 
@@ -120,12 +109,12 @@ Image_window::ScalerVector::ScalerVector() {
 // This is all the names of the scalers. It needs to match the ScalerType enum
 	const ScalerInfo point = {
 		"Point", 0xFFFFFFFF, new Pentagram::PointScaler(),
-		0, 0, 0, 0, 0
+		nullptr, nullptr, nullptr, nullptr, nullptr
 	};
 	push_back(point);
 
 	const ScalerInfo Interlaced = {
-		"Interlaced", 0xFFFFFFFE, 0,
+		"Interlaced", 0xFFFFFFFE, nullptr,
 		&Image_window::show_scaled8to565_interlace,
 		&Image_window::show_scaled8to555_interlace,
 		&Image_window::show_scaled8to16_interlace,
@@ -136,52 +125,52 @@ Image_window::ScalerVector::ScalerVector() {
 
 	const ScalerInfo Bilinear = {
 		"Bilinear", 0xFFFFFFFF, new Pentagram::BilinearScaler(),
-		0, 0, 0, 0, 0
+		nullptr, nullptr, nullptr, nullptr, nullptr
 	};
 	push_back(Bilinear);
 
 	const ScalerInfo BilinearPlus = {
-		"BilinearPlus", SCALE_BIT(2), 0,
+		"BilinearPlus", SCALE_BIT(2), nullptr,
 		&Image_window::show_scaled8to565_BilinearPlus,
 		&Image_window::show_scaled8to555_BilinearPlus,
 		&Image_window::show_scaled8to16_BilinearPlus,
 		&Image_window::show_scaled8to32_BilinearPlus,
-		0
+		nullptr
 	};
 	push_back(BilinearPlus);
 
 	const ScalerInfo _2xSaI = {
-		"2xSaI", SCALE_BIT(2), 0,
+		"2xSaI", SCALE_BIT(2), nullptr,
 		&Image_window::show_scaled8to565_2xSaI,
 		&Image_window::show_scaled8to555_2xSaI,
 		&Image_window::show_scaled8to16_2xSaI,
 		&Image_window::show_scaled8to32_2xSaI,
-		0
+		nullptr
 	};
 	push_back(_2xSaI);
 
 	const ScalerInfo SuperEagle = {
-		"SuperEagle", SCALE_BIT(2), 0,
+		"SuperEagle", SCALE_BIT(2), nullptr,
 		&Image_window::show_scaled8to565_SuperEagle,
 		&Image_window::show_scaled8to555_SuperEagle,
 		&Image_window::show_scaled8to16_SuperEagle,
 		&Image_window::show_scaled8to32_SuperEagle,
-		0
+		nullptr
 	};
 	push_back(SuperEagle);
 
 	const ScalerInfo Super2xSaI = {
-		"Super2xSaI", SCALE_BIT(2), 0,
+		"Super2xSaI", SCALE_BIT(2), nullptr,
 		&Image_window::show_scaled8to565_Super2xSaI,
 		&Image_window::show_scaled8to555_Super2xSaI,
 		&Image_window::show_scaled8to16_Super2xSaI,
 		&Image_window::show_scaled8to32_Super2xSaI,
-		0
+		nullptr
 	};
 	push_back(Super2xSaI);
 
 	const ScalerInfo Scale2X = {
-		"Scale2X", SCALE_BIT(2), 0,
+		"Scale2X", SCALE_BIT(2), nullptr,
 		&Image_window::show_scaled8to565_2x_noblur,
 		&Image_window::show_scaled8to555_2x_noblur,
 		&Image_window::show_scaled8to16_2x_noblur,
@@ -192,87 +181,74 @@ Image_window::ScalerVector::ScalerVector() {
 
 #ifdef USE_HQ2X_SCALER
 	const ScalerInfo Hq2x = {
-		"Hq2x", SCALE_BIT(2), 0,
+		"Hq2x", SCALE_BIT(2), nullptr,
 		&Image_window::show_scaled8to565_Hq2x,
 		&Image_window::show_scaled8to555_Hq2x,
 		&Image_window::show_scaled8to16_Hq2x,
 		&Image_window::show_scaled8to32_Hq2x,
-		0
+		nullptr
 	};
 	push_back(Hq2x);
 #endif
 
 #ifdef USE_HQ3X_SCALER
 	const ScalerInfo Hq3x = {
-		"Hq3x", SCALE_BIT(3), 0,
+		"Hq3x", SCALE_BIT(3), nullptr,
 		&Image_window::show_scaled8to565_Hq3x,
 		&Image_window::show_scaled8to555_Hq3x,
 		&Image_window::show_scaled8to16_Hq3x,
 		&Image_window::show_scaled8to32_Hq3x,
-		0
+		nullptr
 	};
 	push_back(Hq3x);
 #endif
 
 #ifdef USE_HQ4X_SCALER
 	const ScalerInfo Hq4x = {
-		"Hq4x", SCALE_BIT(4), 0,
+		"Hq4x", SCALE_BIT(4), nullptr,
 		&Image_window::show_scaled8to565_Hq4x,
 		&Image_window::show_scaled8to555_Hq4x,
 		&Image_window::show_scaled8to16_Hq4x,
 		&Image_window::show_scaled8to32_Hq4x,
-		0
+		nullptr
 	};
 	push_back(Hq4x);
 #endif
 
 #ifdef USE_XBR_SCALER
 	const ScalerInfo _2xbr = {
-		"2xBR", SCALE_BIT(2), 0,
+		"2xBR", SCALE_BIT(2), nullptr,
 		&Image_window::show_scaled8to565_2xBR,
 		&Image_window::show_scaled8to555_2xBR,
 		&Image_window::show_scaled8to16_2xBR,
 		&Image_window::show_scaled8to32_2xBR,
-		0
+		nullptr
 	};
 	push_back(_2xbr);
 	const ScalerInfo _3xbr = {
-		"3xBR", SCALE_BIT(3), 0,
+		"3xBR", SCALE_BIT(3), nullptr,
 		&Image_window::show_scaled8to565_3xBR,
 		&Image_window::show_scaled8to555_3xBR,
 		&Image_window::show_scaled8to16_3xBR,
 		&Image_window::show_scaled8to32_3xBR,
-		0
+		nullptr
 	};
 	push_back(_3xbr);
 	const ScalerInfo _4xbr = {
-		"4xBR", SCALE_BIT(4), 0,
+		"4xBR", SCALE_BIT(4), nullptr,
 		&Image_window::show_scaled8to565_4xBR,
 		&Image_window::show_scaled8to555_4xBR,
 		&Image_window::show_scaled8to16_4xBR,
 		&Image_window::show_scaled8to32_4xBR,
-		0
+		nullptr
 	};
 	push_back(_4xbr);
 #endif
-
-#ifdef HAVE_OPENGL
-	const ScalerInfo opengl = {
-		"OpenGL", 0xFFFFFFFF, 0,
-		&Image_window::show_scaledOpenGL,
-		&Image_window::show_scaledOpenGL,
-		&Image_window::show_scaledOpenGL,
-		&Image_window::show_scaledOpenGL,
-		&Image_window::show_scaledOpenGL
-	};
-	push_back(opengl);
-#endif
 }
 
 Image_window::ScalerVector::~ScalerVector() {
-	for (std::vector<Image_window::ScalerInfo>::iterator it = begin();
-	        it != end(); ++it)
-		delete it->arb;
+	for (auto& it : *this)
+		delete it.arb;
 }
 
 Image_window::ScalerType Image_window::get_scaler_for_name(const char *scaler) {
@@ -295,19 +271,19 @@ int Image_window::Get_best_bpp(int w, int h, int bpp, uint32 flags) {
 
 	// Explicit BPP required
 	if (bpp != 0) {
-		if (!(flags & SDL_FULLSCREEN)) {
+		if (!(flags & SDL_WINDOW_FULLSCREEN_DESKTOP)) {
 			if (bpp == 16 &&  windowed_16 != 0) return 16;
 			else if (bpp == 32 && windowed_32 != 0) return 32;
 		}
 
-		if (SDL_VideoModeOK(w, h, bpp, flags) == 0)
+		if (VideoModeOK(w, h, bpp, flags) == 0)
 			return bpp;
 
-		cerr << "SDL Reports " << w << "x" << h << " " << bpp << " bpp " << ((flags & SDL_FULLSCREEN) ? "fullscreen" : "windowed") << " surface is not OK. Attmempting to use " << bpp << " bpp anyway." << endl;
+		cerr << "SDL Reports " << w << "x" << h << " " << bpp << " bpp " << ((flags & SDL_WINDOW_FULLSCREEN_DESKTOP) ? "fullscreen" : "windowed") << " surface is not OK. Attmempting to use " << bpp << " bpp anyway." << endl;
 		return bpp;
 	}
 
-	if (!(flags & SDL_FULLSCREEN)) {
+	if (!(flags & SDL_WINDOW_FULLSCREEN_DESKTOP)) {
 		if (desktop_depth == 16 && windowed_16 != 0) return 16;
 		else if (desktop_depth == 32 && windowed_32 != 0) return 32;
 		else if (windowed_16 == 16) return 16;
@@ -316,12 +292,12 @@ int Image_window::Get_best_bpp(int w, int h, int bpp, uint32 flags) {
 		else if (windowed_32 != 0) return 32;
 	}
 
-	if ((desktop_depth == 16 || desktop_depth == 32) && SDL_VideoModeOK(w, h, desktop_depth, flags)) {
+	if ((desktop_depth == 16 || desktop_depth == 32) && VideoModeOK(w, h, desktop_depth, flags)) {
 		return desktop_depth;
 	}
 
-	int desired16 = SDL_VideoModeOK(w, h, 16, flags);
-	int desired32 = SDL_VideoModeOK(w, h, 32, flags);
+	int desired16 = VideoModeOK(w, h, 16, flags);
+	int desired32 = VideoModeOK(w, h, 32, flags);
 
 	if (desired16 == 16)
 		return 16;
@@ -332,7 +308,7 @@ int Image_window::Get_best_bpp(int w, int h, int bpp, uint32 flags) {
 	else if (desired32 != 0)
 		return 32;
 
-	cerr << "SDL Reports " << w << "x" << h << " 16 bpp and 32 bpp " << ((flags & SDL_FULLSCREEN) ? "fullscreen" : "windowed") << " surfaces are not OK. Attempting to use 16 bpp. anyway" << endl;
+	cerr << "SDL Reports " << w << "x" << h << " 16 bpp and 32 bpp " << ((flags & SDL_WINDOW_FULLSCREEN_DESKTOP) ? "fullscreen" : "windowed") << " surfaces are not OK. Attempting to use 16 bpp. anyway" << endl;
 	return 16;
 }
 
@@ -346,11 +322,13 @@ void Image_window::static_init() {
 
 	cout << "Checking rendering support" << std::endl;
 
-#if SDL_VERSION_ATLEAST(2, 0, 0)
 	SDL_DisplayMode dispmode;
 	int bpp;
-	Uint32 Rmask, Gmask, Bmask, Amask;
-	if (SDL_GetDesktopDisplayMode(SDL_COMPAT_DISPLAY_INDEX, &dispmode) == 0
+	Uint32 Rmask;
+	Uint32 Gmask;
+	Uint32 Bmask;
+	Uint32 Amask;
+	if (SDL_GetDesktopDisplayMode(0, &dispmode) == 0
 		&& SDL_PixelFormatEnumToMasks(dispmode.format, &bpp, &Rmask, &Gmask, &Bmask, &Amask) == SDL_TRUE) {
 		desktop_displaymode = dispmode;
 		desktop_depth = bpp;
@@ -358,15 +336,9 @@ void Image_window::static_init() {
 		desktop_depth = 0;
 		cout << "Error: Couldn't get desktop display depth!" << std::endl;
 	}
-#else
-	SDL_PixelFormat format;
-	std::memset(&format, 0, sizeof(format));
-	desktop_depth = SDL_GetVideoInfo()->vfmt->BitsPerPixel;
-#endif
-
-	windowed_8 = SDL_VideoModeOK(640, 400, 8, SDL_SWSURFACE | SDL_HWPALETTE);
-	windowed_16 = SDL_VideoModeOK(640, 400, 16, SDL_SWSURFACE);
-	windowed_32 = SDL_VideoModeOK(640, 400, 32, SDL_SWSURFACE);
+	windowed_8 = VideoModeOK(640, 400, 8, SDL_SWSURFACE);
+	windowed_16 = VideoModeOK(640, 400, 16, SDL_SWSURFACE);
+	windowed_32 = VideoModeOK(640, 400, 32, SDL_SWSURFACE);
 
 	cout << ' ' << "Windowed" << '\t';
 	if (windowed_8)  cout << ' ' << 8 << ' ' << "bpp ok";
@@ -378,10 +350,9 @@ void Image_window::static_init() {
 
 	/* Get available fullscreen/hardware modes */
 	for (size_t i = 0; i < array_size(bpps); i++) {
-#if SDL_VERSION_ATLEAST(2, 0, 0)
 		for (int j = 0; j < SDL_GetNumDisplayModes(0); j++) {
 			SDL_DisplayMode dispmode;
-			if (SDL_GetDisplayMode(SDL_COMPAT_DISPLAY_INDEX, j, &dispmode) == 0) {
+			if (SDL_GetDisplayMode(0, j, &dispmode) == 0) {
 				Resolution res = { dispmode.w, dispmode.h, false, false, false};
 				p_resolutions[(res.width << 16) | res.height] = res;
 
@@ -389,31 +360,6 @@ void Image_window::static_init() {
 				cout << " Error getting display mode #" << j << ": " << SDL_GetError() << std::endl;
 			}
 		}
-#else
-		SDL_PixelFormat *pformat = 0;
-		if (bpps[i]) {
-			format.BitsPerPixel = bpps[i];
-			pformat = &format;
-		}
-		SDL_Rect **modes = SDL_ListModes(pformat, SDL_FULLSCREEN | SDL_SWSURFACE | ((bpps[i] == 8) ? SDL_HWPALETTE : 0));
-
-		// No mode for this bpp
-		if (modes == 0) {
-			continue;
-		}
-		// In theory this should never happen in fullscreen mode
-		else if (modes == reinterpret_cast<SDL_Rect **>(-1)) {
-			//if (bpps[i] == 8) any_mode8 = true;
-			//else any_mode == true;
-			continue;
-		}
-
-		for (; *modes; ++modes) {
-
-			Resolution res = { (*modes)->w, (*modes)->h, false, false, false};
-			p_resolutions[(res.width << 16) | res.height] = res;
-		}
-#endif
 	}
 
 	// It's empty, so add in some basic resolutions that would be nice to support
@@ -477,21 +423,21 @@ void Image_window::static_init() {
 	bool ok_pal = false;
 	bool ok_rgb = false;
 
-	for (std::map<uint32, Image_window::Resolution>::iterator it = p_resolutions.begin(); it != p_resolutions.end();) {
+	for (auto it = p_resolutions.begin(); it != p_resolutions.end();) {
 		Image_window::Resolution &res = it->second;
 		bool ok = false;
 
-		if (SDL_VideoModeOK(res.width, res.height, 8, SDL_FULLSCREEN | SDL_SWSURFACE | SDL_HWPALETTE)) {
+		if (VideoModeOK(res.width, res.height, 8, SDL_WINDOW_FULLSCREEN_DESKTOP | SDL_SWSURFACE)) {
 			res.palette = true;
 			ok_pal = true;
 			ok = true;
 		}
-		if (SDL_VideoModeOK(res.width, res.height, 16, SDL_FULLSCREEN | SDL_SWSURFACE)) {
+		if (VideoModeOK(res.width, res.height, 16, SDL_WINDOW_FULLSCREEN_DESKTOP | SDL_SWSURFACE)) {
 			res.rgb16 = true;
 			ok_rgb = true;
 			ok = true;
 		}
-		if (SDL_VideoModeOK(res.width, res.height, 32, SDL_FULLSCREEN | SDL_SWSURFACE)) {
+		if (VideoModeOK(res.width, res.height, 32, SDL_WINDOW_FULLSCREEN_DESKTOP | SDL_SWSURFACE)) {
 			res.rgb32 = true;
 			ok_rgb = true;
 			ok = true;
@@ -509,8 +455,10 @@ void Image_window::static_init() {
 		}
 	}
 
+#ifndef __IPHONEOS__
 	if (windowed_16 == 0 && windowed_32 == 0)
 		cerr << "SDL Reports 640x400 16 bpp and 32 bpp windowed surfaces are not OK. Windowed scalers may not work properly." << endl;
+#endif
 
 	if (!ok_pal && !ok_rgb)
 		cerr << "SDL Reports no usable fullscreen resolutions." << endl;
@@ -542,7 +490,7 @@ void Image_window::create_surface(
     unsigned int h
 ) {
 	uses_palette = true;
-	draw_surface = paletted_surface = inter_surface = display_surface = 0;
+	draw_surface = paletted_surface = inter_surface = display_surface = nullptr;
 
 	if (!Scalers[fill_scaler].arb) {
 		if (Scalers[scaler].arb)
@@ -553,65 +501,55 @@ void Image_window::create_surface(
 
 	get_draw_dims(w, h, scale, fill_mode, game_width, game_height, inter_width, inter_height);
 
-#if SDL_VERSION_ATLEAST(2, 0, 0)
-	if (try_scaler(w, h) == false) {
-#else
-	if ((game_width != inter_width || game_height != inter_height ||
-	        static_cast<int>(w) != game_width * scale || static_cast<int>(h) != game_height * scale ||
-	        force_bpp) && try_scaler(w, h) == false) {
-#endif
+	if (!try_scaler(w, h)) {
 		// Try fallback to point scaler if it failed, if it doesn't work, we probably can't run
 		scaler = point;
 		try_scaler(w, h);
 	}
 
 	if (!paletted_surface && !force_bpp) {      // No scaling, or failed?
-		uint32 flags = SDL_SWSURFACE | (fullscreen ? SDL_FULLSCREEN : 0);
-#if SDL_VERSION_ATLEAST(2, 0, 1)
-		flags |= SDL_WINDOW_ALLOW_HIGHDPI;
-#elif !SDL_VERSION_ATLEAST(2, 0, 0)
-		flags |= (ibuf->depth == 8 ? SDL_HWPALETTE : 0);
-#endif
-#if SDL_VERSION_ATLEAST(2, 0, 0)
-		if (screen_window != NULL) {
+		uint32 flags = SDL_SWSURFACE | SDL_WINDOW_ALLOW_HIGHDPI;
+		if (fullscreen) {
+			flags |= SDL_WINDOW_FULLSCREEN_DESKTOP;
+		}
+		if (screen_window != nullptr) {
 			SDL_SetWindowSize(screen_window, w / scale, h / scale);
 			SDL_SetWindowFullscreen(screen_window, flags);
 			SDL_DestroyTexture(screen_texture);
 			SDL_DestroyRenderer(screen_renderer);
 		} else
 			screen_window = SDL_CreateWindow("", SDL_WINDOWPOS_UNDEFINED, SDL_WINDOWPOS_UNDEFINED, w / scale, h / scale, flags);
-		if (screen_window == NULL)
+		if (screen_window == nullptr)
 			cout << "Couldn't create window: " << SDL_GetError() << std::endl;
 
 		screen_renderer = SDL_CreateRenderer(screen_window, -1, 0);
-		if (screen_renderer == NULL)
+		if (screen_renderer == nullptr)
 			cout << "Couldn't create renderer: " << SDL_GetError() << std::endl;
 
 		// Do an initial draw/fill
-		SDL_SetRenderDrawColor(screen_renderer, SDL2_INITIAL_FILL);
+		SDL_SetRenderDrawColor(screen_renderer, 0, 0, 0, 255);
 		SDL_RenderClear(screen_renderer);
 		SDL_RenderPresent(screen_renderer);
 
 		int sbpp;
-        	Uint32 sRmask, sGmask, sBmask, sAmask;
+        	Uint32 sRmask;
+        	Uint32 sGmask;
+        	Uint32 sBmask;
+        	Uint32 sAmask;
 		SDL_PixelFormatEnumToMasks(desktop_displaymode.format, &sbpp, &sRmask, &sGmask, &sBmask, &sAmask);
 		display_surface = SDL_CreateRGBSurface(0,
 				(w / scale), (h / scale), sbpp,
                                 sRmask, sGmask, sBmask, sAmask);
-		if (display_surface == NULL)
+		if (display_surface == nullptr)
 			cout << "Couldn't create display surface: " << SDL_GetError() << std::endl;
 		screen_texture = SDL_CreateTexture(screen_renderer,
                                 desktop_displaymode.format,
                                 SDL_TEXTUREACCESS_STREAMING,
                                 (w / scale), (h / scale));
-		if (screen_texture == NULL)
+		if (screen_texture == nullptr)
 			cout << "Couldn't create texture: " << SDL_GetError() << std::endl;
 
 		inter_surface = draw_surface = paletted_surface = display_surface;
-
-#else
-		inter_surface = draw_surface = paletted_surface = display_surface = SDL_SetVideoMode(w / scale, h / scale, ibuf->depth, flags);
-#endif
 		inter_width = w / scale;
 		inter_height = h / scale;
 		scale = 1;
@@ -631,7 +569,7 @@ void Image_window::create_surface(
 	ibuf->width = draw_surface->w;
 	ibuf->height = draw_surface->h;
 
-	if (draw_surface != display_surface && scaler != OpenGL) {
+	if (draw_surface != display_surface) {
 		ibuf->width -= guard_band * 2;
 		ibuf->height -= guard_band * 2;
 	}
@@ -644,7 +582,8 @@ void Image_window::create_surface(
 	ibuf->offset_y = (get_full_height() - get_game_height()) / 2;
 	ibuf->bits = static_cast<unsigned char *>(draw_surface->pixels) - get_start_x() - get_start_y() * ibuf->line_width;
 	// Scaler guardband is in effect
-	if (draw_surface != display_surface && scaler != OpenGL) ibuf->bits += guard_band + ibuf->line_width * guard_band;
+	if (draw_surface != display_surface)
+		ibuf->bits += guard_band + ibuf->line_width * guard_band;
 }
 
 /*
@@ -654,53 +593,39 @@ void Image_window::create_surface(
 
 bool Image_window::create_scale_surfaces(int w, int h, int bpp) {
 	int hwdepth = bpp;
-	uint32 flags = 0;
-
-	// Get best bpp
-	flags = SDL_SWSURFACE | (fullscreen ? SDL_FULLSCREEN : 0);
-#if SDL_VERSION_ATLEAST(2, 0, 1) && (defined(MACOSX) || defined(__IPHONEOS__))
-		flags |= SDL_WINDOW_ALLOW_HIGHDPI;
-#endif
-#ifdef __IPHONEOS__
-	// Turn on landscape mode if desired
-	if (w > h) {
-		SDL_SetHint(SDL_HINT_ORIENTATIONS, "LandscapeLeft LandscapeRight");
-#if !(SDL_VERSION_ATLEAST(2, 0, 0))
-		flags |= SDL_RESIZABLE;
-#endif
+	uint32 flags = SDL_SWSURFACE | SDL_WINDOW_ALLOW_HIGHDPI;
+	if (fullscreen) {
+		flags |= SDL_WINDOW_FULLSCREEN_DESKTOP;
 	}
-#endif
 	hwdepth = Get_best_bpp(w, h, hwdepth, flags);
 	if (!hwdepth) return false;
 
-#if SDL_VERSION_ATLEAST(2, 0, 0)
-	if (screen_window != NULL) {
+	if (screen_window != nullptr) {
 		SDL_SetWindowSize(screen_window, w, h);
 		SDL_SetWindowFullscreen(screen_window, flags);
 		SDL_DestroyTexture(screen_texture);
 		SDL_DestroyRenderer(screen_renderer);
 	} else
 		screen_window = SDL_CreateWindow("", SDL_WINDOWPOS_UNDEFINED, SDL_WINDOWPOS_UNDEFINED, w, h, flags);
-	if (screen_window == NULL)
+	if (screen_window == nullptr)
 		cout << "Couldn't create window: " << SDL_GetError() << std::endl;
 	screen_renderer = SDL_CreateRenderer(screen_window, -1, 0);
-	if (screen_renderer == NULL)
+	if (screen_renderer == nullptr)
 		cout << "Couldn't create renderer: " << SDL_GetError() << std::endl;
 
 	if (fullscreen) {
-		int dw, dh;
+		int dw;
+		int dh;
 		//with HighDPi this returns the higher resolutions
 		SDL_GetRendererOutputSize(screen_renderer, &dw, &dh);
 		w=dw;
 		h=dh;
 		Resolution res = { w, h, false, false, false};
 		p_resolutions[(w << 16) | h] = res;
- #if SDL_VERSION_ATLEAST(2, 0, 1) && (defined(MACOSX) || defined(__IPHONEOS__))
 		//getting new native scale when highdpi is active
 		int sw;
-		SDL_GetWindowSize(screen_window, &sw, 0);
-		nativescale = dw / sw;
-#endif
+		SDL_GetWindowSize(screen_window, &sw, nullptr);
+		nativescale = float(dw) / sw;
 		//high resolution fullscreen needs this to make the whole screen available
 		SDL_RenderSetLogicalSize(screen_renderer, w, h);
 	} else
@@ -708,28 +633,28 @@ bool Image_window::create_scale_surfaces(int w, int h, int bpp) {
 		SDL_SetWindowSize(screen_window, w, h);
 
 	// Do an initial draw/fill
-	SDL_SetRenderDrawColor(screen_renderer, SDL2_INITIAL_FILL);
+	SDL_SetRenderDrawColor(screen_renderer, 0, 0, 0, 255);
 	SDL_RenderClear(screen_renderer);
 	SDL_RenderPresent(screen_renderer);
 
 	int sbpp;
-       	Uint32 sRmask, sGmask, sBmask, sAmask;
+       	Uint32 sRmask;
+       	Uint32 sGmask;
+       	Uint32 sBmask;
+       	Uint32 sAmask;
 	SDL_PixelFormatEnumToMasks(desktop_displaymode.format, &sbpp, &sRmask, &sGmask, &sBmask, &sAmask);
 
 	display_surface = SDL_CreateRGBSurface(0,
 			w, h, sbpp,
                         sRmask, sGmask, sBmask, sAmask);
-	if (display_surface == NULL)
+	if (display_surface == nullptr)
 		cout << "Couldn't create display surface: " << SDL_GetError() << std::endl;
 	screen_texture = SDL_CreateTexture(screen_renderer,
                         desktop_displaymode.format,
                         SDL_TEXTUREACCESS_STREAMING,
                         w, h);
-	if (screen_texture == NULL)
+	if (screen_texture == nullptr)
 		cout << "Couldn't create texture: " << SDL_GetError() << std::endl;
-#else
-	display_surface = SDL_SetVideoMode(w, h, hwdepth, flags);
-#endif
 	if (!display_surface) {
 		cerr << "Unable to set video mode to" << w << "x" << h << " " << hwdepth << " bpp" << endl;
 		free_surface();
@@ -757,22 +682,9 @@ bool Image_window::create_scale_surfaces(int w, int h, int bpp) {
 		inter_surface = display_surface;
 	}
 
-
 	if ((uses_palette = (bpp == 8))) paletted_surface = display_surface;
 	else paletted_surface = draw_surface;
 
-#ifdef __IPHONEOS__
-	// Dirty little hack to help with landscape mode
-	// Something within the "create_surface" flow needs to be ran
-	// after landscape mode is "hinted" at.
-	// However, something within create_surface calls this function...
-	static bool RECREATED_ONCE = false;
-	if (w > h && !RECREATED_ONCE) {
-		RECREATED_ONCE = true;
-		free_surface();
-		create_surface(w, h);
-	}
-#endif
 	return true;
 }
 
@@ -781,97 +693,40 @@ bool Image_window::create_scale_surfaces(int w, int h, int bpp) {
 */
 
 bool Image_window::try_scaler(int w, int h) {
-	// OpenGL
-	if (scaler == OpenGL) {
-#ifdef HAVE_OPENGL
-		// Get info. about video.
-		const SDL_VideoInfo *vinfo = SDL_GetVideoInfo();
-		if (!vinfo) {
-			cout << "SDL_GetVideoInfo() failed: " << SDL_GetError()
-			     << endl;
-			return false;
-		}
-		// Set up SDL video flags.
-		int video_flags = SDL_OPENGL | SDL_GL_DOUBLEBUFFER |
-		                  SDL_HWPALETTE | SDL_RESIZABLE |
-		                  (fullscreen ? SDL_FULLSCREEN : 0);
-		// Can surface be in video RAM?
-		if (vinfo->hw_available)
-			video_flags |= SDL_HWSURFACE;
-		else
-			video_flags |= SDL_SWSURFACE;
-		if (vinfo->blit_hw) // Hardware blits?
-			video_flags |= SDL_HWACCEL;
-		// Want double-buffering.
-		SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
-		// Allocate surface.
-		int hwdepth = vinfo->vfmt->BitsPerPixel;
-		// +++++For now create 8-bit surface
-		//   to avoid crashing places we
-		//   haven't converted yet.
-		if ((display_surface = inter_surface = SDL_SetVideoMode(w, h,
-		                                       hwdepth, video_flags)) != 0 &&
-		        (draw_surface = paletted_surface = SDL_CreateRGBSurface(
-		                            SDL_SWSURFACE, w / scale, h / scale,
-		                            8, 0, 0, 0, 0)) != 0) {
-			inter_width = w;
-			inter_height = h;
-			game_width = draw_surface->w;
-			game_height = draw_surface->h;
-			//show_scaled = &Image_window::show_scaledOpenGL;
-			return true;
-		} else {
-			cerr << "Couldn't allocate surface: " <<  SDL_GetError() << endl;
-			free_surface();
-		}
-#else
-		cerr << "OpenGL not supported" << endl;
+	const ScalerInfo *info;
 
-#endif
-		scaler = point;
+	if (scaler < 0 || scaler >= NumScalers || scale == 1)
+		info = &Scalers[point];
+	else
+		info = &Scalers[scaler];
+
+	// Is the size supported, if not, default to point scaler
+	if (!(info->size_mask & SCALE_BIT(scale)))
+		return false;
+
+	bool has8 = ibuf->depth == 8 && info->fun8to8 && (force_bpp == 0 || force_bpp == 8);
+	bool has16 = ibuf->depth == 8 && info->fun8to16 && (force_bpp == 0 || force_bpp == 16);
+	bool has32 = ibuf->depth == 8 && info->fun8to32 && (force_bpp == 0 || force_bpp == 32);
+
+	if (info->arb) {
+		has8 |= (force_bpp == 0 || force_bpp == 8) && info->arb->Support8bpp(ibuf->depth);
+		has16 |= (force_bpp == 0 || force_bpp == 16) && info->arb->Support16bpp(ibuf->depth);
+		has32 |= (force_bpp == 0 || force_bpp == 32) && info->arb->Support32bpp(ibuf->depth);
 	}
 
-#if SDL_VERSION_ATLEAST(2, 0, 0)
-	if (true) {
-#else
-	if (game_width != inter_width || game_height != inter_height ||
-	        w != game_width * scale || h != game_height * scale || force_bpp) {
-#endif
-		const ScalerInfo *info;
+	// First try best of 16 bit/32 bit scaler
+	if (has16 && has32 && create_scale_surfaces(w, h, 0))
+		return true;
 
-		if (scaler < 0 || scaler >= NumScalers || scale == 1)
-			info = &Scalers[point];
-		else
-			info = &Scalers[scaler];
+	if (has16 && create_scale_surfaces(w, h, 16))
+		return true;
 
-		// Is the size supported, if not, default to point scaler
-		if (!(info->size_mask & SCALE_BIT(scale)))
-			return false;
+	if (has32 && create_scale_surfaces(w, h, 32))
+		return true;
 
-		bool has8 = ibuf->depth == 8 && info->fun8to8 && (force_bpp == 0 || force_bpp == 8);
-		bool has16 = ibuf->depth == 8 && info->fun8to16 && (force_bpp == 0 || force_bpp == 16);
-		bool has32 = ibuf->depth == 8 && info->fun8to32 && (force_bpp == 0 || force_bpp == 32);
-
-		if (info->arb) {
-			has8 |= (force_bpp == 0 || force_bpp == 8) && info->arb->Support8bpp(ibuf->depth);
-			has16 |= (force_bpp == 0 || force_bpp == 16) && info->arb->Support16bpp(ibuf->depth);
-			has32 |= (force_bpp == 0 || force_bpp == 32) && info->arb->Support32bpp(ibuf->depth);
-		}
-
-		// First try best of 16 bit/32 bit scaler
-		if (has16 && has32 && create_scale_surfaces(w, h, 0))
-			return true;
-
-		if (has16 && create_scale_surfaces(w, h, 16))
-			return true;
-
-		if (has32 && create_scale_surfaces(w, h, 32))
-			return true;
-
-		// 8bit display output is mostly deprecated!
-		if (has8 && create_scale_surfaces(w, h, 8))
-			return true;
-	}
+	// 8bit display output is mostly deprecated!
+	if (has8 && create_scale_surfaces(w, h, 8))
+		return true;
 
 	return false;
 }
@@ -881,24 +736,23 @@ bool Image_window::try_scaler(int w, int h) {
 */
 
 void Image_window::free_surface() {
-	if (draw_surface != 0 && draw_surface != display_surface && draw_surface != inter_surface) SDL_FreeSurface(draw_surface);
-	if (inter_surface != 0 && inter_surface != display_surface) SDL_FreeSurface(inter_surface);
-	paletted_surface = 0;
-	inter_surface = 0;
-	draw_surface = 0;
-	display_surface = 0;
-	ibuf->bits = 0;
+	if (draw_surface != nullptr && draw_surface != display_surface && draw_surface != inter_surface) SDL_FreeSurface(draw_surface);
+	if (inter_surface != nullptr && inter_surface != display_surface) SDL_FreeSurface(inter_surface);
+	paletted_surface = nullptr;
+	inter_surface = nullptr;
+	draw_surface = nullptr;
+	display_surface = nullptr;
+	ibuf->bits = nullptr;
 }
 
 /*
 *   Create a compatible buffer.
 */
 
-Image_buffer *Image_window::create_buffer(
+std::unique_ptr<Image_buffer> Image_window::create_buffer(
     int w, int h            // Dimensions.
 ) {
-	Image_buffer *newbuf = ibuf->create_another(w, h);
-	return (newbuf);
+	return ibuf->create_another(w, h);
 }
 
 /*
@@ -942,17 +796,13 @@ void Image_window::show(
 	if (!ready())
 		return;
 
-	int srcx = 0, srcy = 0;
+	int srcx = 0;
+	int srcy = 0;
 	if (!ibuf->clip(srcx, srcy, w, h, x, y))
 		return;
 	x -= get_start_x();
 	y -= get_start_y();
 
-	if (scaler == OpenGL) {
-		Image_window::show_scaledOpenGL(x, y, w, h);
-		return;
-	}
-
 	// Increase the area by 4 pixels
 	increase_area(x, y, w, h, 4, 4, 4, 4, get_full_width(), get_full_height());
 
@@ -975,7 +825,6 @@ void Image_window::show(
 
 	// Phase 1 blit from draw_surface to inter_surface
 	if (draw_surface != inter_surface) {
-		scalefun show_scaled;
 		const ScalerInfo &sel_scaler = Scalers[scaler];
 
 		// Need to apply an offset to compensate for the guard_band
@@ -986,6 +835,7 @@ void Image_window::show(
 			if (!sel_scaler.arb->Scale(draw_surface, x + guard_band, y + guard_band, w, h, inter_surface, scale * (x + guard_band), scale * (y + guard_band), scale * w, scale * h, false))
 				Scalers[point].arb->Scale(draw_surface, x + guard_band, y + guard_band, w, h, inter_surface, scale * (x + guard_band), scale * (y + guard_band), scale * w, scale * h, false);
 		} else {
+			scalefun show_scaled;
 			if (inter_surface->format->BitsPerPixel == 16 || inter_surface->format->BitsPerPixel == 15) {
 				int r = inter_surface->format->Rmask;
 				int g = inter_surface->format->Gmask;
@@ -993,9 +843,9 @@ void Image_window::show(
 
 				show_scaled =
 				    (r == 0xf800 && g == 0x7e0 && b == 0x1f) || (b == 0xf800 && g == 0x7e0 && r == 0x1f) ?
-				    (sel_scaler.fun8to565 != 0 ? sel_scaler.fun8to565 : sel_scaler.fun8to16) :
+				    (sel_scaler.fun8to565 != nullptr ? sel_scaler.fun8to565 : sel_scaler.fun8to16) :
 					    (r == 0x7c00 && g == 0x3e0 && b == 0x1f) || (b == 0x7c00 && g == 0x3e0 && r == 0x1f) ?
-					    (sel_scaler.fun8to555 != 0 ? sel_scaler.fun8to555 : sel_scaler.fun8to16) :
+					    (sel_scaler.fun8to555 != nullptr ? sel_scaler.fun8to555 : sel_scaler.fun8to16) :
 					    sel_scaler.fun8to16 ;
 			} else if (inter_surface->format->BitsPerPixel == 32) {
 				show_scaled = sel_scaler.fun8to32;
@@ -1041,20 +891,17 @@ void Image_window::show(
 		w = display_surface->w;
 		h = display_surface->h;
 	}
-	// Phase 3 notify SDL
-#if SDL_VERSION_ATLEAST(2, 0, 0)
-        UpdateRect(display_surface, x, y, w, h);
-#else
-	SDL_UpdateRect(display_surface, x, y, w, h);
-#endif
+	// Phase 3 blit high res draw surface on top of display_surface
+	// Phase 4 notify SDL
+	UpdateRect(display_surface, x, y, w, h);
 }
 
-
 /*
 *   Toggle fullscreen.
 */
 void Image_window::toggle_fullscreen() {
-	int w, h;
+	int w;
+	int h;
 
 	w = display_surface->w;
 	h = display_surface->h;
@@ -1066,7 +913,7 @@ void Image_window::toggle_fullscreen() {
 	}
 	/* First see if it's allowed.
 	* for now this is preventing the switch to fullscreen
-	*if ( SDL_VideoModeOK(w, h, bpp, flags) )
+	*if ( VideoModeOK(w, h, bpp, flags) )
 	*/
 	{
 		free_surface();     // Delete old.
@@ -1077,85 +924,13 @@ void Image_window::toggle_fullscreen() {
 
 bool Image_window::screenshot(SDL_RWops *dst) {
 	if (!paletted_surface) return false;
-#ifdef HAVE_OPENGL
-	if (GL_manager::get_instance()) {
-		int width = ibuf->width, height = ibuf->height;
-		GL_manager *glman = GL_manager::get_instance();
-		unsigned char *bits = glman->get_unscaled_rgb(width, height, false, true);
-		SDL_Surface *screenshot_surface = SDL_CreateRGBSurfaceFrom(bits,
-		                                  width, height, 24, 3 * width, 0, 0, 0, 0);
-		bool ret = SavePCX_RW(screenshot_surface, dst, true);
-		SDL_FreeSurface(screenshot_surface);
-		delete [] bits;
-		return ret;
-	}
-#endif
-	return SavePCX_RW(draw_surface, dst, true);
+	return SaveIMG_RW(draw_surface, dst, true, guard_band);
 }
 
 void Image_window::set_title(const char *title) {
-#if SDL_VERSION_ATLEAST(2, 0, 0)
 	SDL_SetWindowTitle(screen_window, title);
-#else
-	SDL_WM_SetCaption(title, 0);
-#endif
 }
 
-#ifdef HAVE_OPENGL
-/*
-*   Fill a rectangle with an 8-bit value.
-*/
-
-void Image_window::opengl_fill8(
-    unsigned char pix,
-    int srcw, int srch,
-    int destx, int desty
-) {
-	SDL_Color *colors = paletted_surface->format->palette->colors;
-	SDL_Color &color = colors[pix];
-	glDisable(GL_TEXTURE_2D);   // Disable texture-mapping.
-	glPushMatrix();
-	int x = destx;          // Left edge.
-	int y = -(desty + srch);
-	glTranslatef(static_cast<float>(x), static_cast<float>(y), 0);
-	glBegin(GL_QUADS);
-	{
-		glColor3ub(color.r, color.g, color.b);
-		glVertex3i(0, 0, 0);
-		glVertex3i(srcw, 0, 0);
-		glVertex3i(srcw, srch, 0);
-		glVertex3i(0, srch, 0);
-	}
-	glEnd();
-	glPopMatrix();
-}
-
-/*
-*   Apply a translucency table to a rectangle.
-*/
-
-void Image_window::opengl_fill_translucent8(
-    const unsigned char /* val */,    // Not used.
-    int srcw, int srch,
-    int destx, int desty,
-    const Xform_palette &xform        // Transform table.
-) {
-	glDisable(GL_TEXTURE_2D);   // Disable texture-mapping.
-	int x = destx;          // Left edge.
-	int y = -(desty + srch);
-	glBegin(GL_QUADS);
-	{
-		glColor4ub(xform.r, xform.g, xform.b, xform.a);
-		glVertex2i(x, y);
-		glVertex2i(x + srcw, y);
-		glVertex2i(x + srcw, y + srch);
-		glVertex2i(x, y + srch);
-	}
-	glEnd();
-}
-
-#endif
-
 int Image_window::get_display_width() {
 	return display_surface->w;
 }
@@ -1279,7 +1054,7 @@ Image_window::FillMode Image_window::string_to_fillmode(const char *str) {
 		if (*str != 'X' && *str != 'x') return static_cast<FillMode>(0);
 
 		++str;
-		if (*str < '0' || *str > '9') return static_cast<FillMode>(0);
+		if (!std::isdigit(static_cast<unsigned char>(*str))) return static_cast<FillMode>(0);
 
 		char *end;
 		unsigned long f = std::strtoul(str, &end, 10);
@@ -1293,7 +1068,7 @@ Image_window::FillMode Image_window::string_to_fillmode(const char *str) {
 		if (*str != 'X' && *str != 'x') return static_cast<FillMode>(0);
 
 		++str;
-		if (*str < '0' || *str > '9') return static_cast<FillMode>(0);
+		if (!std::isdigit(static_cast<unsigned char>(*str))) return static_cast<FillMode>(0);
 
 		char *end;
 		unsigned long f = std::strtoul(str, &end, 10);
@@ -1303,7 +1078,7 @@ Image_window::FillMode Image_window::string_to_fillmode(const char *str) {
 
 		return static_cast<FillMode>(AspectCorrectCentre + f * 2);
 	} else {
-		if (*str < '0' || *str > '9') return static_cast<FillMode>(0);
+		if (!std::isdigit(static_cast<unsigned char>(*str))) return static_cast<FillMode>(0);
 
 		char *end;
 		unsigned long fx = std::strtoul(str, &end, 10);
@@ -1312,7 +1087,7 @@ Image_window::FillMode Image_window::string_to_fillmode(const char *str) {
 		if (fx > 65535 || (*str != 'X' && *str != 'x')) return static_cast<FillMode>(0);
 
 		++str;
-		if (*str < '0' || *str > '9') return static_cast<FillMode>(0);
+		if (!std::isdigit(static_cast<unsigned char>(*str))) return static_cast<FillMode>(0);
 
 		unsigned long fy = std::strtoul(str, &end, 10);
 		str += (end - str);
@@ -1370,35 +1145,41 @@ bool Image_window::fillmode_to_string(FillMode fmode, std::string &str) {
 
 }
 
-#if SDL_VERSION_ATLEAST(2, 0, 0)
 void Image_window::UpdateRect(SDL_Surface *surf, int x, int y, int w, int h)
 {
 	// TODO: Only update the necessary portion of the screen.
 	// Seem to get flicker like crazy or some other ill effect no matter
 	// what I try. -Lanica 08/28/2013
-	SDL_UpdateTexture(screen_texture, NULL, surf->pixels, surf->pitch);
+	SDL_UpdateTexture(screen_texture, nullptr, surf->pixels, surf->pitch);
 	ignore_unused_variable_warning(x, y, w, h);
 	//SDL_Rect destRect = {x, y, w, h};
-	SDL_RenderCopy(screen_renderer, screen_texture, NULL, NULL);
+	SDL_RenderCopy(screen_renderer, screen_texture, nullptr, nullptr);
 	SDL_RenderPresent(screen_renderer);
 }
+
 int Image_window::VideoModeOK(int width, int height, int bpp, Uint32 flags)
 {
 	ignore_unused_variable_warning(bpp, flags);
-	int nbpp;
-	Uint32 Rmask, Gmask, Bmask, Amask;
- 	for (int j = 0; j < SDL_GetNumDisplayModes(0); j++)
-	{
+	if (height > width) {
+		// Reject portrait modes.
+		return 0;
+	}
+	const int num_dysplay_modes = SDL_GetNumDisplayModes(0);
+	for (int j = 0; j < num_dysplay_modes; j++) {
 		SDL_DisplayMode dispmode;
-		if (SDL_GetDisplayMode(SDL_COMPAT_DISPLAY_INDEX, j, &dispmode) == 0
-			&& SDL_PixelFormatEnumToMasks(dispmode.format, &nbpp, &Rmask, &Gmask, &Bmask, &Amask) == SDL_TRUE
-			&& dispmode.w == width
-			&& dispmode.h == height)
-		{
+		int nbpp;
+		Uint32 Rmask;
+		Uint32 Gmask;
+		Uint32 Bmask;
+		Uint32 Amask;
+		if (SDL_GetDisplayMode(0, j, &dispmode) == 0
+		        && SDL_PixelFormatEnumToMasks(dispmode.format, &nbpp, &Rmask, &Gmask, &Bmask, &Amask) == SDL_TRUE
+		        && dispmode.w == width
+		        && dispmode.h == height) {
 			return nbpp;
 		}
 	}
 	return 0;
 }
+
 SDL_DisplayMode Image_window::desktop_displaymode;
-#endif
