@@ -6,6 +6,7 @@
 
 /*
 Copyright (C) 1998 Jeffrey S. Freedman
+Copyright (C) 2000-2022 The Exult Team
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
@@ -30,22 +31,16 @@ Boston, MA  02111-1307, USA.
 #include "imagebuf.h"
 #include "common_types.h"
 #include <string>
-#include <vector>
 #include <map>
+#include <memory>
+#include <vector>
 
 #include "SDL_video.h"
-#include "sdl-compat.h"
 #include "ignore_unused_variable_warning.h"
 
 struct SDL_Surface;
 struct SDL_RWops;
 
-#ifdef HAVE_OPENGL
-#define IF_OPENGL(a,b) if (scaler == OpenGL) a; else b
-#else
-#define IF_OPENGL(a,b) b
-#endif
-
 /*
 *   Here's the top-level class to use for image buffers.  Image_window
 *   should be derived from it.
@@ -58,8 +53,8 @@ class ArbScaler;
 class Image_window {
 public:
 	// Firstly just some public scaler stuff
-	typedef void (Image_window::*scalefun)(int x, int y, int w, int h);
-	typedef int ScalerType;
+	using scalefun = void (Image_window::*)(int, int, int, int);
+	using ScalerType = int;
 
 	enum FillMode {
 	    Fill = 1,                   ///< Game area fills all of the display surface
@@ -117,7 +112,7 @@ public:
 	static const std::map<uint32, Image_window::Resolution> &Resolutions;
 	static const bool &AnyResAllowed;
 
-	static ScalerType get_scaler_for_name(const char *name);
+	static ScalerType get_scaler_for_name(const char *scaler);
 	inline static const char *get_name_for_scaler(int num) {
 		return Scalers[num].name;
 	}
@@ -127,7 +122,7 @@ public:
 		ScalerConst(const char *name) : Name(name) {
 		}
 		operator ScalerType() const {
-			if (Name == 0) return Scalers.size();
+			if (Name == nullptr) return Scalers.size();
 			return get_scaler_for_name(Name);
 		}
 	};
@@ -147,7 +142,6 @@ public:
 	static const ScalerConst    _2xBR;
 	static const ScalerConst    _3xBR;
 	static const ScalerConst    _4xBR;
-	static const ScalerConst    OpenGL;
 	static const ScalerConst    NumScalers;
 
 	// Gets the draw surface and intersurface dims.
@@ -176,14 +170,12 @@ protected:
 	FillMode fill_mode;
 	int fill_scaler;
 
-#if SDL_VERSION_ATLEAST(2, 0, 0)
 	static SDL_DisplayMode desktop_displaymode;
 	struct SDL_Window *screen_window;
 	struct SDL_Renderer *screen_renderer;
 	struct SDL_Texture *screen_texture;
 	static int VideoModeOK(int width, int height, int bpp, Uint32 flags);
 	void UpdateRect(SDL_Surface *surf, int x, int y, int w, int h);
-#endif
 
 	SDL_Surface *paletted_surface;  // Surface that palette is set on   (Example res)
 	SDL_Surface *display_surface;   // Final surface that is displayed  (1024x1024)
@@ -274,8 +266,6 @@ protected:
 	void show_scaled8to565_4xBR(int x, int y, int w, int h);
 	void show_scaled8to32_4xBR(int x, int y, int w, int h);
 
-	void show_scaledOpenGL(int x, int y, int w, int h);
-
 	/*
 	*   Image info.
 	*/
@@ -293,27 +283,21 @@ protected:
 	static int windowed_8;
 	static int windowed_16;
 	static int windowed_32;
-#if SDL_VERSION_ATLEAST(2, 0, 1) && (defined(MACOSX) || defined(__IPHONEOS__))
 	static float nativescale;
-#endif
 
 public:
-#if SDL_VERSION_ATLEAST(2, 0, 0)
-        inline struct SDL_Window *get_screen_window() const {
-                return screen_window;
-        }
-#endif
+	inline struct SDL_Window *get_screen_window() const {
+		return screen_window;
+	}
 	int Get_best_bpp(int w, int h, int bpp, uint32 flags);
 
 	// Create with given buffer.
 	Image_window(Image_buffer *ib, int w, int h, int gamew, int gameh, int scl = 1, bool fs = false, int sclr = point, FillMode fmode = AspectCorrectCentre, int fillsclr = point)
 		: ibuf(ib), scale(scl), scaler(sclr), uses_palette(true),
 		  fullscreen(fs), game_width(gamew), game_height(gameh),
-		  fill_mode(fmode), fill_scaler(fillsclr),
-		  paletted_surface(0), display_surface(0), inter_surface(0), draw_surface(0) {
-#if SDL_VERSION_ATLEAST(2, 0, 0)
-		screen_window = NULL;
-#endif
+		  fill_mode(fmode), fill_scaler(fillsclr), screen_window(nullptr),
+		  paletted_surface(nullptr), display_surface(nullptr),
+		  inter_surface(nullptr), draw_surface(nullptr) {
 		static_init();
 		create_surface(w, h);
 	}
@@ -337,21 +321,24 @@ public:
 			gy = (sy * inter_height) / (scale * get_display_height()) + get_start_y();
 		}
 	}
-#if SDL_VERSION_ATLEAST(2, 0, 1) && (defined(MACOSX) || defined(__IPHONEOS__))
+
 	void screen_to_game_hdpi(int sx, int sy, bool fast, int &gx, int &gy) {
+		int lgx;
+		int lgy;
+		screen_to_game(sx, sy, fast, lgx, lgy);
 		if (!fullscreen)
 			// reset nativescale to 1.0 for windowed gaming or toggling
 			// fullscreen/windowed mode uses the wrong nativescale
-			nativescale = 1.0;
-		if (fast) {
-			gx = (sx + get_start_x())* nativescale;
-			gy = (sy + get_start_y())* nativescale;
+			nativescale = 1.0f;
+		if (nativescale != 1.0f) {
+			gx = lgx * nativescale;
+			gy = lgy * nativescale;
 		} else {
-			gx = ((sx * inter_width) / (scale * get_display_width()) + get_start_x()) * nativescale;
-			gy = ((sy * inter_height) / (scale * get_display_height()) + get_start_y()) * nativescale;
+			gx = lgx;
+			gy = lgy;
 		}
 	}
-#endif
+
 	void game_to_screen(int gx, int gy, bool fast, int &sx, int &sy) {
 		if (fast) {
 			sx = gx - get_start_x();
@@ -374,7 +361,7 @@ public:
 	}
 
 	// Is rect. visible within clip?
-	int is_visible(int x, int y, int w, int h) {
+	bool is_visible(int x, int y, int w, int h) {
 		return ibuf->is_visible(x, y, w, h);
 	}
 	// Set title.
@@ -424,16 +411,16 @@ public:
 		return draw_surface;
 	}
 
-	int ready() {       // Ready to draw?
-		return (ibuf->bits != 0);
+	bool ready() {       // Ready to draw?
+		return ibuf->bits != nullptr;
 	}
 	bool is_fullscreen() {
 		return fullscreen;
 	}
 	// Create a compatible image buffer.
-	Image_buffer *create_buffer(int w, int h);
+	std::unique_ptr<Image_buffer> create_buffer(int w, int h);
 	// Resize event occurred.
-	void resized(unsigned int neww, unsigned int nehh, bool newfs, unsigned int newgw, unsigned int newgh, int newsc, int newscaler = point, FillMode fmode = AspectCorrectCentre, int fillsclr = point);
+	void resized(unsigned int neww, unsigned int newh, bool newfs, unsigned int newgw, unsigned int newgh, int newsc, int newscaler = point, FillMode fmode = AspectCorrectCentre, int fillsclr = point);
 	void show() {       // Repaint entire window.
 		show(get_start_x(), get_start_y(), get_full_width(), get_full_height());
 	}
@@ -478,14 +465,12 @@ public:
 	*/
 	// Fill with given (8-bit) value.
 	void fill8(unsigned char val) {
-		IF_OPENGL(opengl_fill8(val),
-		          ibuf->fill8(val));
+		ibuf->fill8(val);
 	}
 	// Fill rect. wth pixel.
 	void fill8(unsigned char val, int srcw, int srch,
 	           int destx, int desty) {
-		IF_OPENGL(opengl_fill8(val, srcw, srch, destx, desty),
-		          ibuf->fill8(val, srcw, srch, destx, desty));
+		ibuf->fill8(val, srcw, srch, destx, desty);
 	}
 	// Fill line with pixel.
 	void fill_line8(unsigned char val, int srcw,
@@ -519,9 +504,7 @@ public:
 	// Apply translucency to a rectangle
 	virtual void fill_translucent8(unsigned char val, int srcw, int srch,
 	                               int destx, int desty, const Xform_palette &xform) {
-		IF_OPENGL(opengl_fill_translucent8(val, srcw, srch,
-		                                   destx, desty, xform), ibuf->fill_translucent8(val,
-		                                           srcw, srch, destx, desty, xform));
+		ibuf->fill_translucent8(val, srcw, srch, destx, desty, xform);
 	}
 	// Copy rect. with transp. color.
 	void copy_transparent8(const unsigned char *src_pixels, int srcw,
@@ -529,19 +512,6 @@ public:
 		ibuf->copy_transparent8(src_pixels, srcw, srch,
 		                        destx, desty);
 	}
-	/*
-	*   OpenGL:
-	*/
-#ifdef HAVE_OPENGL
-	void opengl_fill8(unsigned char val) {
-		opengl_fill8(val, ibuf->width, ibuf->height, 0, 0);
-	}
-	// Fill rect. wth pixel.
-	void opengl_fill8(unsigned char val, int srcw, int srch,
-	                  int destx, int desty);
-	virtual void opengl_fill_translucent8(unsigned char val,
-	                                      int srcw, int srch, int destx, int desty, const Xform_palette &xform);
-#endif
 	/*
 	*   Depth-independent methods:
 	*/
