@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2001-2013 The Exult Team
+ *  Copyright (C) 2001-2022 The Exult Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -72,7 +72,7 @@ void ActionQuit(int const *params) {
 // { ActionOldFileGump, 0, "Save/restore", normal_keys, NONE },
 void ActionOldFileGump(int const *params) {
 	ignore_unused_variable_warning(params);
-	File_gump *fileio = new File_gump();
+	auto *fileio = new File_gump();
 	Game_window::get_instance()->get_gump_man()->do_modal_gump(fileio,
 	        Mouse::hand);
 	delete fileio;
@@ -81,7 +81,7 @@ void ActionOldFileGump(int const *params) {
 // { ActionMenuGump, 0, "GameMenu", normal_keys, NONE },
 void ActionMenuGump(int const *params) {
 	ignore_unused_variable_warning(params);
-	Gamemenu_gump *gmenu = new Gamemenu_gump();
+	auto *gmenu = new Gamemenu_gump();
 	Game_window::get_instance()->get_gump_man()->do_modal_gump(gmenu,
 	        Mouse::hand);
 	delete gmenu;
@@ -90,7 +90,7 @@ void ActionMenuGump(int const *params) {
 // { ActionFileGump, 0, "Save/restore", normal_keys, NONE },
 void ActionFileGump(int const *params) {
 	ignore_unused_variable_warning(params);
-	Newfile_gump *fileio = new Newfile_gump();
+	auto *fileio = new Newfile_gump();
 	Game_window::get_instance()->get_gump_man()->do_modal_gump(fileio,
 	        Mouse::hand);
 	delete fileio;
@@ -131,14 +131,15 @@ void ActionAbout(int const *params) {
 	scroll = new Scroll_gump();
 
 	scroll->add_text("Exult V" VERSION "\n");
-	scroll->add_text("(C) 1998-2020 Exult Team\n\n");
+	scroll->add_text("(C) 1998-2022 Exult Team\n\n");
 	scroll->add_text("Available under the terms of the ");
 	scroll->add_text("GNU General Public License\n\n");
-	scroll->add_text("http://exult.sourceforge.net\n");
+	scroll->add_text("http://exult.info\n");
 
 	scroll->paint();
 	do {
-		int x, y;
+		int x;
+		int y;
 		Get_click(x, y, Mouse::hand);
 	} while (scroll->show_next_page());
 	gwin->paint();
@@ -164,7 +165,7 @@ void ActionCloseOrMenu(int const *params) {
 	if (gwin->get_gump_man()->showing_gumps(true))
 		gwin->get_gump_man()->close_all_gumps();
 	else
-		ActionMenuGump(0);
+		ActionMenuGump(nullptr);
 }
 
 //  { ActionScreenshot, 0, "Take screenshot", normal_keys, NONE },
@@ -248,7 +249,7 @@ void ActionUseFood(int const *params) {
 // params[1] = event id.
 void ActionCallUsecode(int const *params) {
 	Usecode_machine *usecode = Game_window::get_instance()->get_usecode();
-	usecode->call_usecode(params[0], static_cast<Game_object *>(0),
+	usecode->call_usecode(params[0], nullptr,
 	                      static_cast<Usecode_machine::Usecode_events>(params[1]));
 
 	Mouse::mouse->set_speed_cursor();
@@ -273,7 +274,8 @@ void ActionCombatPause(int const *params) {
 //  { ActionTarget, 0, "Target mode", normal_keys, NONE },
 void ActionTarget(int const *params) {
 	ignore_unused_variable_warning(params);
-	int x, y;
+	int x;
+	int y;
 	if (!Get_click(x, y, Mouse::greenselect))
 		return;
 	Game_window::get_instance()->double_clicked(x, y);
@@ -325,8 +327,9 @@ void ActionInventory(int const *params) {
 void ActionTryKeys(int const *params) {
 	ignore_unused_variable_warning(params);
 	Game_window *gwin = Game_window::get_instance();
-	int x, y;           // Allow dragging.
-	if (!Get_click(x, y, Mouse::greenselect, 0, true))
+	int x;
+	int y;           // Allow dragging.
+	if (!Get_click(x, y, Mouse::greenselect, nullptr, true))
 		return;
 	// Look for obj. in open gump.
 	Gump *gump = gwin->get_gump_man()->find_gump(x, y);
@@ -404,7 +407,7 @@ void ActionCombatStats(int const *params) {
 	ignore_unused_variable_warning(params);
 	Game_window *gwin = Game_window::get_instance();
 	int cnt = gwin->get_party_man()->get_count();
-	gwin->get_gump_man()->add_gump(0, game->get_shape("gumps/cstats/1") + cnt);
+	gwin->get_gump_man()->add_gump(nullptr, game->get_shape("gumps/cstats/1") + cnt);
 }
 
 //  { ActionFaceStats, 0, "Change Face Stats State", normal_keys, NONE }
@@ -421,7 +424,8 @@ void ActionUseHealingItems(int const *params) {
 		return;
 
 	// Potions are wasted if at full health so we will check for that
-	int x, y;
+	int x;
+	int y;
 	if (!is_party_item(340, 1) || !Get_click(x, y, Mouse::greenselect))
 		return;
 	Game_object *obj = gwin->find_object(x, y);
@@ -507,7 +511,7 @@ int get_walking_speed(int const *params) {
 		speed = Mouse::medium_combat_speed_factor;
 	else
 		speed = parm == 1 ? Mouse::medium_speed_factor : Mouse::fast_speed_factor;
-	return (200 * gwin->get_std_delay() / speed);
+	return 200 * gwin->get_std_delay() / speed;
 }
 
 //  { ActionWalkWest, 0, "Walk west", normal_keys, NONE },
@@ -728,7 +732,8 @@ void ActionTeleport(int const *params) {
 
 void ActionTeleportTargetMode(int const *params) {
 	ignore_unused_variable_warning(params);
-	int x, y;
+	int x;
+	int y;
 	if (!Get_click(x, y, Mouse::redx))
 		return;
 	cheat.cursor_teleport();
