@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2001-2013 The Exult Team
+ *  Copyright (C) 2001-2022 The Exult Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,7 +20,6 @@
 #  include <config.h>
 #endif
 
-#include "sdl-compat.h"
 #include "SDL_keyboard.h"
 
 #include "actors.h"
@@ -45,7 +44,6 @@ using std::isspace;
 using std::strchr;
 using std::string;
 using std::strlen;
-using std::vector;
 
 static  class Chardata { // ctype-like character lists
 public:
@@ -57,7 +55,7 @@ public:
 	}
 } chardata;
 
-typedef void(*ActionFunc)(int const *);
+using ActionFunc = void (*)(int const *);
 
 const struct Action {
 	const char *s;
@@ -76,71 +74,71 @@ const struct Action {
 	bool allow_if_cant_act_charmed;
 	bool is_movement;
 } ExultActions[] = {
-	{ "QUIT", ActionQuit, 0, "Quit", Action::normal_keys, NONE, true, true, true, false },
-	{ "SAVE_RESTORE", ActionFileGump, 0, "Save/restore", Action::normal_keys, NONE, true, true, true, false },
-	{ "QUICKSAVE", ActionQuicksave, 0, "Quick-save", Action::normal_keys, NONE, false, true, true, false },
+	{ "QUIT", ActionQuit, nullptr, "Quit", Action::normal_keys, NONE, true, true, true, false },
+	{ "SAVE_RESTORE", ActionFileGump, nullptr, "Save/restore", Action::normal_keys, NONE, true, true, true, false },
+	{ "QUICKSAVE", ActionQuicksave, nullptr, "Quick-save", Action::normal_keys, NONE, false, true, true, false },
 	{
 		"QUICKRESTORE",
-		ActionQuickrestore, 0, "Quick-restore", Action::normal_keys, NONE, true, true, true, false
+		ActionQuickrestore, nullptr, "Quick-restore", Action::normal_keys, NONE, true, true, true, false
 	},
-	{ "ABOUT", ActionAbout, 0, "About Exult", Action::normal_keys, NONE, false, true, true, false },
-	{ "HELP", ActionHelp, 0, "List keys", Action::normal_keys, NONE, false, true, true, false },
-	{ "CLOSE_GUMPS", ActionCloseGumps, 0, "Close gumps", Action::dont_show, NONE, false, true, true, false },
-	{ "CLOSE_OR_MENU", ActionCloseOrMenu, 0, "Game menu", Action::normal_keys, NONE, true, true, true, false },
+	{ "ABOUT", ActionAbout, nullptr, "About Exult", Action::normal_keys, NONE, false, true, true, false },
+	{ "HELP", ActionHelp, nullptr, "List keys", Action::normal_keys, NONE, false, true, true, false },
+	{ "CLOSE_GUMPS", ActionCloseGumps, nullptr, "Close gumps", Action::dont_show, NONE, false, true, true, false },
+	{ "CLOSE_OR_MENU", ActionCloseOrMenu, nullptr, "Game menu", Action::normal_keys, NONE, true, true, true, false },
 	{
 		"SCREENSHOT",
-		ActionScreenshot, 0, "Take screenshot", Action::normal_keys, NONE, true, true, true, false
+		ActionScreenshot, nullptr, "Take screenshot", Action::normal_keys, NONE, true, true, true, false
 	},
-	{ "GAME_MENU", ActionMenuGump, 0, "Game Menu", Action::normal_keys, NONE, true, true, true, false },
+	{ "GAME_MENU", ActionMenuGump, nullptr, "Game Menu", Action::normal_keys, NONE, true, true, true, false },
 	{
 		"OLD_FILE_GUMP",
-		ActionOldFileGump, 0, "Save/restore", Action::normal_keys, NONE, true, true, true, false
+		ActionOldFileGump, nullptr, "Save/restore", Action::normal_keys, NONE, true, true, true, false
 	},
 	{
 		"SCALEVAL_INCREASE",
-		ActionScalevalIncrease, 0, "Increase scaleval", Action::mapedit_keys, NONE, true, true, true, false
+		ActionScalevalIncrease, nullptr, "Increase scaleval", Action::mapedit_keys, NONE, true, true, true, false
 	},
 	{
 		"SCALEVAL_DECREASE",
-		ActionScalevalDecrease, 0, "Decrease scaleval", Action::mapedit_keys, NONE, true, true, true, false
+		ActionScalevalDecrease, nullptr, "Decrease scaleval", Action::mapedit_keys, NONE, true, true, true, false
 	},
 	{
 		"BRIGHTER",
-		ActionBrighter, 0, "Increase brightness", Action::normal_keys, NONE, true, true, true, false
+		ActionBrighter, nullptr, "Increase brightness", Action::normal_keys, NONE, true, true, true, false
 	},
-	{ "DARKER", ActionDarker, 0, "Decrease brightness", Action::normal_keys, NONE, true, true, true, false },
+	{ "DARKER", ActionDarker, nullptr, "Decrease brightness", Action::normal_keys, NONE, true, true, true, false },
 	{
 		"TOGGLE_FULLSCREEN",
-		ActionFullscreen, 0, "Toggle fullscreen", Action::normal_keys, NONE, true, true, true, false
+		ActionFullscreen, nullptr, "Toggle fullscreen", Action::normal_keys, NONE, true, true, true, false
 	},
 
-	{ "USEITEM", ActionUseItem, 0, "Use item", Action::dont_show, NONE, false, false, false, false },
-	{ "USEFOOD", ActionUseFood, 0, "Feed", Action::normal_keys, NONE, false, false, false, false },
-	{ "CALL_USECODE", ActionCallUsecode, 0, "Call Usecode", Action::dont_show, NONE, false, false, false, false },
-	{ "TOGGLE_COMBAT", ActionCombat, 0, "Toggle combat", Action::normal_keys, NONE, false, false, true, false },
-	{ "PAUSE_COMBAT", ActionCombatPause, 0, "Pause combat", Action::normal_keys, NONE, false, false, true, false },
-	{ "TARGET_MODE", ActionTarget, 0, "Target mode", Action::normal_keys, NONE, false, false, true, false },
-	{ "INVENTORY", ActionInventory, 0, "Show inventory", Action::normal_keys, NONE, false, false, true, false },
-	{ "TRY_KEYS", ActionTryKeys, 0, "Try keys", Action::normal_keys, NONE, false, false, false, false },
-	{ "STATS", ActionStats, 0, "Show stats", Action::normal_keys, NONE, false, true, true, false },
+	{ "USEITEM", ActionUseItem, nullptr, "Use item", Action::dont_show, NONE, false, false, false, false },
+	{ "USEFOOD", ActionUseFood, nullptr, "Feed", Action::normal_keys, NONE, false, false, false, false },
+	{ "CALL_USECODE", ActionCallUsecode, nullptr, "Call Usecode", Action::dont_show, NONE, false, false, false, false },
+	{ "TOGGLE_COMBAT", ActionCombat, nullptr, "Toggle combat", Action::normal_keys, NONE, false, false, true, false },
+	{ "PAUSE_COMBAT", ActionCombatPause, nullptr, "Pause combat", Action::normal_keys, NONE, false, false, true, false },
+	{ "TARGET_MODE", ActionTarget, nullptr, "Target mode", Action::normal_keys, NONE, false, false, true, false },
+	{ "INVENTORY", ActionInventory, nullptr, "Show inventory", Action::normal_keys, NONE, false, false, true, false },
+	{ "TRY_KEYS", ActionTryKeys, nullptr, "Try keys", Action::normal_keys, NONE, false, false, false, false },
+	{ "STATS", ActionStats, nullptr, "Show stats", Action::normal_keys, NONE, false, true, true, false },
 	{
 		"COMBAT_STATS",
-		ActionCombatStats, 0, "Show combat stats", Action::normal_keys, SERPENT_ISLE, false, false, true, false
+		ActionCombatStats, nullptr, "Show combat stats", Action::normal_keys, SERPENT_ISLE, false, false, true, false
 	},
 	{
 		"FACE_STATS",
-		ActionFaceStats, 0, "Change Face Stats State", Action::normal_keys, NONE, false, true, true, false
+		ActionFaceStats, nullptr, "Change Face Stats State", Action::normal_keys, NONE, false, true, true, false
 	},
-	{ "USE_HEALING_ITEMS", ActionUseHealingItems, 0, "Use bandages or healing potions", Action::normal_keys, NONE, false, false, false, false },
+	{ "USE_HEALING_ITEMS", ActionUseHealingItems, nullptr, "Use bandages or healing potions", Action::normal_keys, NONE, false, false, false, false },
 	{
 		"SHOW_SI_INTRO",
-		ActionSIIntro, 0, "Show Alternate SI intro", Action::cheat_keys, SERPENT_ISLE, false, true, true, false
+		ActionSIIntro, nullptr, "Show Alternate SI intro", Action::cheat_keys, SERPENT_ISLE, false, true, true, false
 	},
-	{ "SHOW_ENDGAME", ActionEndgame, 0, "Show endgame", Action::cheat_keys, NONE, false, true, true, false },
-	{ "SCROLL_LEFT", ActionScrollLeft, 0, "Scroll left", Action::cheat_keys, NONE, false, true , true, false},
-	{ "SCROLL_RIGHT", ActionScrollRight, 0, "Scroll right", Action::cheat_keys, NONE, false, true, true, false },
-	{ "SCROLL_UP", ActionScrollUp, 0, "Scroll up", Action::cheat_keys, NONE, false, true, true, false },
-	{ "SCROLL_DOWN", ActionScrollDown, 0, "Scroll down", Action::cheat_keys, NONE, false, true, true, false },
+	{ "SHOW_ENDGAME", ActionEndgame, nullptr, "Show endgame", Action::cheat_keys, NONE, false, true, true, false },
+	{ "SCROLL_LEFT", ActionScrollLeft, nullptr, "Scroll left", Action::cheat_keys, NONE, false, true , true, false},
+	{ "SCROLL_RIGHT", ActionScrollRight, nullptr, "Scroll right", Action::cheat_keys, NONE, false, true, true, false },
+	{ "SCROLL_UP", ActionScrollUp, nullptr, "Scroll up", Action::cheat_keys, NONE, false, true, true, false },
+	{ "SCROLL_DOWN", ActionScrollDown, nullptr, "Scroll down", Action::cheat_keys, NONE, false, true, true, false },
 	{ "WALK_WEST", ActionWalkWest, ActionStopWalking, "Walk west", Action::normal_keys, NONE, false, false, false, true },
 	{ "WALK_EAST", ActionWalkEast, ActionStopWalking, "Walk east", Action::normal_keys, NONE, false, false, false, true },
 	{ "WALK_NORTH", ActionWalkNorth, ActionStopWalking, "Walk north", Action::normal_keys, NONE, false, false, false, true },
@@ -149,155 +147,155 @@ const struct Action {
 	{ "WALK_SOUTH_EAST", ActionWalkSouthEast, ActionStopWalking, "Walk south-east", Action::normal_keys, NONE, false, false, false, true },
 	{ "WALK_NORTH_WEST", ActionWalkNorthWest, ActionStopWalking, "Walk north-west", Action::normal_keys, NONE, false, false, false, true },
 	{ "WALK_SOUTH_WEST", ActionWalkSouthWest, ActionStopWalking, "Walk south-west", Action::normal_keys, NONE, false, false, false, true },
-	{ "CENTER_SCREEN", ActionCenter, 0, "Center screen", Action::cheat_keys, NONE, false, true, true, false },
+	{ "CENTER_SCREEN", ActionCenter, nullptr, "Center screen", Action::cheat_keys, NONE, false, true, true, false },
 	{
 		"SHAPE_BROWSER",
-		ActionShapeBrowser, 0, "Shape browser", Action::cheat_keys, NONE, false, true, true, false
+		ActionShapeBrowser, nullptr, "Shape browser", Action::cheat_keys, NONE, false, true, true, false
 	},
-	{ "SHAPE_BROWSER_HELP", ActionShapeBrowserHelp, 0, "List shape browser keys", Action::cheat_keys, NONE, false, true, true, false },
+	{ "SHAPE_BROWSER_HELP", ActionShapeBrowserHelp, nullptr, "List shape browser keys", Action::cheat_keys, NONE, false, true, true, false },
 	{
 		"CREATE_ITEM",
-		ActionCreateShape, 0, "Create last shape", Action::cheat_keys, NONE, false, true, true, false
+		ActionCreateShape, nullptr, "Create last shape", Action::cheat_keys, NONE, false, true, true, false
 	},
 	{
 		"DELETE_OBJECT",
-		ActionDeleteObject, 0, "Delete object", Action::cheat_keys, NONE, false, true, true, false
+		ActionDeleteObject, nullptr, "Delete object", Action::cheat_keys, NONE, false, true, true, false
 	},
 	{
 		"TOGGLE_EGGS",
-		ActionToggleEggs, 0, "Toggle egg display", Action::cheat_keys, NONE, false, true, true, false
+		ActionToggleEggs, nullptr, "Toggle egg display", Action::cheat_keys, NONE, false, true, true, false
 	},
 	{
 		"TOGGLE_GOD_MODE",
-		ActionGodMode, 0, "Toggle god mode", Action::cheat_keys, NONE, false, true, true, false
+		ActionGodMode, nullptr, "Toggle god mode", Action::cheat_keys, NONE, false, true, true, false
 	},
 	{
 		"CHANGE_GENDER",
-		ActionGender, 0, "Change gender", Action::cheat_keys, NONE, false, true, true, false
+		ActionGender, nullptr, "Change gender", Action::cheat_keys, NONE, false, true, true, false
 	},
 	{
 		"CHEAT_HELP",
-		ActionCheatHelp, 0, "List cheat keys", Action::cheat_keys, NONE, false, true, true, false
+		ActionCheatHelp, nullptr, "List cheat keys", Action::cheat_keys, NONE, false, true, true, false
 	},
 	{
 		"TOGGLE_INFRAVISION",
-		ActionInfravision, 0, "Toggle infravision", Action::cheat_keys, NONE, false, true, true, false
+		ActionInfravision, nullptr, "Toggle infravision", Action::cheat_keys, NONE, false, true, true, false
 	},
 	{
 		"TOGGLE_HACK_MOVER",
-		ActionHackMover, 0, "Toggle hack-mover mode", Action::cheat_keys, NONE, false, true, true, false
+		ActionHackMover, nullptr, "Toggle hack-mover mode", Action::cheat_keys, NONE, false, true, true, false
 	},
 	{
 		"MAP_TELEPORT",
-		ActionMapTeleport, 0, "Map teleport", Action::cheat_keys, NONE, false, true, true, false
+		ActionMapTeleport, nullptr, "Map teleport", Action::cheat_keys, NONE, false, true, true, false
 	},
 	{
 		"CURSOR_TELEPORT",
-		ActionTeleport, 0, "Teleport to cursor", Action::cheat_keys, NONE, false, true, true, false
+		ActionTeleport, nullptr, "Teleport to cursor", Action::cheat_keys, NONE, false, true, true, false
 	},
 	{
 		"TARGET_MODE_TELEPORT",
-		ActionTeleportTargetMode, 0, "Bring up cursor to teleport", Action::cheat_keys, NONE, false, true, true, false
+		ActionTeleportTargetMode, nullptr, "Bring up cursor to teleport", Action::cheat_keys, NONE, false, true, true, false
 	},
 	{
 		"NEXT_MAP_TELEPORT",
-		ActionNextMapTeleport, 0, "Teleport to next map", Action::cheat_keys, NONE, false, true, true, false
+		ActionNextMapTeleport, nullptr, "Teleport to next map", Action::cheat_keys, NONE, false, true, true, false
 	},
 	{
 		"NEXT_TIME_PERIOD",
-		ActionTime, 0, "Next time period", Action::cheat_keys, NONE, false, true, true, false
+		ActionTime, nullptr, "Next time period", Action::cheat_keys, NONE, false, true, true, false
 	},
 	{
 		"TOGGLE_WIZARD_MODE",
-		ActionWizard, 0, "Toggle archwizard mode", Action::cheat_keys, NONE, false, true, true, false
+		ActionWizard, nullptr, "Toggle archwizard mode", Action::cheat_keys, NONE, false, true, true, false
 	},
-	{ "PARTY_HEAL", ActionHeal, 0, "Heal party", Action::cheat_keys, NONE, false, true, true, false },
+	{ "PARTY_HEAL", ActionHeal, nullptr, "Heal party", Action::cheat_keys, NONE, false, true, true, false },
 	{
 		"PARTY_INCREASE_LEVEL",
-		ActionLevelup, 0, "Level-up party", Action::cheat_keys, NONE, false, true, true, false
+		ActionLevelup, nullptr, "Level-up party", Action::cheat_keys, NONE, false, true, true, false
 	},
 	{
 		"CHEAT_SCREEN",
-		ActionCheatScreen, 0, "Cheat Screen", Action::cheat_keys, NONE, true, true, true, false
+		ActionCheatScreen, nullptr, "Cheat Screen", Action::cheat_keys, NONE, true, true, true, false
 	},
 	{
 		"PICK_POCKET",
-		ActionPickPocket, 0, "Toggle Pick Pocket", Action::cheat_keys, NONE, false, true, true, false
+		ActionPickPocket, nullptr, "Toggle Pick Pocket", Action::cheat_keys, NONE, false, true, true, false
 	},
 	{
 		"NPC_NUMBERS",
-		ActionNPCNumbers, 0, "Toggle NPC Numbers", Action::cheat_keys, NONE, false, true, true, false
+		ActionNPCNumbers, nullptr, "Toggle NPC Numbers", Action::cheat_keys, NONE, false, true, true, false
 	},
 	{
 		"GRAB_ACTOR",
-		ActionGrabActor, 0, "Grab NPC for Cheat Screen", Action::cheat_keys, NONE, false, true, true, false
+		ActionGrabActor, nullptr, "Grab NPC for Cheat Screen", Action::cheat_keys, NONE, false, true, true, false
 	},
-	{ "PLAY_MUSIC", ActionPlayMusic, 0, "Play song", Action::cheat_keys, NONE, false, true, true, false },
+	{ "PLAY_MUSIC", ActionPlayMusic, nullptr, "Play song", Action::cheat_keys, NONE, false, true, true, false },
 	{
 		"TOGGLE_NAKED",
-		ActionNaked, 0, "Toggle naked mode", Action::cheat_keys, SERPENT_ISLE, false, true, true, false
+		ActionNaked, nullptr, "Toggle naked mode", Action::cheat_keys, SERPENT_ISLE, false, true, true, false
 	},
 	{
 		"TOGGLE_PETRA",
-		ActionPetra, 0, "Toggle Petra mode", Action::cheat_keys, SERPENT_ISLE, false, true, true, false
+		ActionPetra, nullptr, "Toggle Petra mode", Action::cheat_keys, SERPENT_ISLE, false, true, true, false
 	},
 	{
 		"CHANGE_SKIN",
-		ActionSkinColour, 0, "Change skin colour", Action::cheat_keys, NONE, false, true, true, false
+		ActionSkinColour, nullptr, "Change skin colour", Action::cheat_keys, NONE, false, true, true, false
 	},
 	{
-		"NOTEBOOK", ActionNotebook, 0, "Show notebook", Action::normal_keys,
+		"NOTEBOOK", ActionNotebook, nullptr, "Show notebook", Action::normal_keys,
 		NONE, false, false, false, false
 	},
 	{
 		"SOUND_TESTER",
-		ActionSoundTester, 0, "Sound tester", Action::cheat_keys, NONE, false, true, true, false
+		ActionSoundTester, nullptr, "Sound tester", Action::cheat_keys, NONE, false, true, true, false
 	},
-	{ "TEST", ActionTest, 0, "Test", Action::dont_show, NONE, false, true, true, false },
+	{ "TEST", ActionTest, nullptr, "Test", Action::dont_show, NONE, false, true, true, false },
 
 	{
 		"MAPEDIT_HELP",
-		ActionMapeditHelp, 0, "List mapedit keys", Action::mapedit_keys, NONE, false, true, true, false
+		ActionMapeditHelp, nullptr, "List mapedit keys", Action::mapedit_keys, NONE, false, true, true, false
 	},
 	{
 		"TOGGLE_MAP_EDITOR",
-		ActionMapEditor, 0, "Toggle map-editor mode", Action::mapedit_keys, NONE, true, true, true, false
+		ActionMapEditor, nullptr, "Toggle map-editor mode", Action::mapedit_keys, NONE, true, true, true, false
 	},
 	{
 		"SKIPLIFT_DECREMENT",
-		ActionSkipLift, 0, "Decrement skiplift", Action::mapedit_keys, NONE, false, true, true, false
+		ActionSkipLift, nullptr, "Decrement skiplift", Action::mapedit_keys, NONE, false, true, true, false
 	},
 	{
 		"CUT",
-		ActionCut, 0, "Cut Selected Objects", Action::mapedit_keys, NONE, true, true, true, false
+		ActionCut, nullptr, "Cut Selected Objects", Action::mapedit_keys, NONE, true, true, true, false
 	},
 	{
 		"COPY",
-		ActionCopy, 0, "Copy Selected Objects", Action::mapedit_keys, NONE, true, true, true, false
+		ActionCopy, nullptr, "Copy Selected Objects", Action::mapedit_keys, NONE, true, true, true, false
 	},
 	{
 		"PASTE",
-		ActionPaste, 0, "Paste Selected Objects", Action::mapedit_keys, NONE, true, true, true, false
+		ActionPaste, nullptr, "Paste Selected Objects", Action::mapedit_keys, NONE, true, true, true, false
 	},
 	{
 		"DELETE_SELECTED",
-		ActionDeleteSelected, 0, "Delete selected", Action::mapedit_keys, NONE, true, true, true, false
+		ActionDeleteSelected, nullptr, "Delete selected", Action::mapedit_keys, NONE, true, true, true, false
 	},
 	{
 		"MOVE_SELECTED",
-		ActionMoveSelected, 0, "Move selected", Action::mapedit_keys, NONE, true, true, true, false
+		ActionMoveSelected, nullptr, "Move selected", Action::mapedit_keys, NONE, true, true, true, false
 	},
 	{
 		"WRITE_MINIMAP",
-		ActionWriteMiniMap, 0, "Write minimap", Action::mapedit_keys, NONE, false, true, true, false
+		ActionWriteMiniMap, nullptr, "Write minimap", Action::mapedit_keys, NONE, false, true, true, false
 	},
-	{ "REPAINT", ActionRepaint, 0, "Repaint screen", Action::dont_show, NONE, true, true, true, false },
-	{ "", 0, 0, "", Action::dont_show, NONE, false, false, false, false } //terminator
+	{ "REPAINT", ActionRepaint, nullptr, "Repaint screen", Action::dont_show, NONE, true, true, true, false },
+	{ "", nullptr, nullptr, "", Action::dont_show, NONE, false, false, false, false } //terminator
 };
 
 const struct {
 	const char *s;
-	SDLKey k;
+	SDL_Keycode k;
 } SDLKeyStringTable[] = {
 	{"BACKSPACE", SDLK_BACKSPACE},
 	{"TAB",       SDLK_TAB},
@@ -306,7 +304,6 @@ const struct {
 	{"ESC",       SDLK_ESCAPE},
 	{"SPACE",     SDLK_SPACE},
 	{"DEL",       SDLK_DELETE},
-#if SDL_VERSION_ATLEAST(2, 0, 0)
 	{"KP0",       SDLK_KP_0},
 	{"KP1",       SDLK_KP_1},
 	{"KP2",       SDLK_KP_2},
@@ -318,17 +315,6 @@ const struct {
 	{"KP8",       SDLK_KP_8},
 	{"KP9",       SDLK_KP_9},
 	{"KP0",       SDLK_KP_0},
-#else
-	{"KP1",       SDLK_KP1},
-	{"KP2",       SDLK_KP2},
-	{"KP3",       SDLK_KP3},
-	{"KP4",       SDLK_KP4},
-	{"KP5",       SDLK_KP5},
-	{"KP6",       SDLK_KP6},
-	{"KP7",       SDLK_KP7},
-	{"KP8",       SDLK_KP8},
-	{"KP9",       SDLK_KP9},
-#endif
 	{"KP.",       SDLK_KP_PERIOD},
 	{"KP/",       SDLK_KP_DIVIDE},
 	{"KP*",       SDLK_KP_MULTIPLY},
@@ -363,8 +349,8 @@ const struct {
 };
 
 
-typedef std::map<std::string, SDLKey> ParseKeyMap;
-typedef std::map<std::string, const Action *> ParseActionMap;
+using ParseKeyMap = std::map<std::string, SDL_Keycode>;
+using ParseActionMap = std::map<std::string, const Action *>;
 
 static ParseKeyMap keys;
 static ParseActionMap actions;
@@ -374,24 +360,13 @@ KeyBinder::KeyBinder() {
 	FillParseMaps();
 }
 
-KeyBinder::~KeyBinder() {
-}
-
-void KeyBinder::AddKeyBinding(SDLKey key, int mod, const Action *action,
-                              int nparams, int *params) {
-	SDL_keysym k;
+void KeyBinder::AddKeyBinding(SDL_Keycode key, int mod, const Action *action,
+                              int nparams, const int *params) {
+	SDL_Keysym k;
 	ActionType a;
-
-#if SDL_VERSION_ATLEAST(2, 0, 0)
 	k.scancode = static_cast<SDL_Scancode>(0);
-#else
-	k.scancode = 0;
-#endif
 	k.sym      = key;
-	k.mod      = static_cast<SDLMod>(mod);
-#if !SDL_VERSION_ATLEAST(2, 0, 0)
-	k.unicode  = 0;
-#endif
+	k.mod      = static_cast<SDL_Keymod>(mod);
 	a.action    = action;
 	int i;  // For MSVC
 	for (i = 0; i < c_maxparams && i < nparams; i++)
@@ -435,7 +410,7 @@ bool KeyBinder::DoAction(ActionType const &a, bool press) const {
 			scroll->paint();
 			do {
 				int x, y;
-				Get_click(x, y, Mouse::hand, 0, false, scroll);
+				Get_click(x, y, Mouse::hand, nullptr, false, scroll);
 			} while (scroll->show_next_page());
 			Game_window::get_instance()->paint();
 			delete scroll;
@@ -449,7 +424,7 @@ bool KeyBinder::DoAction(ActionType const &a, bool press) const {
 		if (a.action->key_type == Action::mapedit_keys)
 			return true;
 #endif
-		if (a.action->func_release != NULL)
+		if (a.action->func_release != nullptr)
 			a.action->func_release(a.params);
 	}
 
@@ -457,30 +432,30 @@ bool KeyBinder::DoAction(ActionType const &a, bool press) const {
 }
 
 KeyMap::const_iterator KeyBinder::TranslateEvent(SDL_Event const &ev) const {
-	SDL_keysym key = ev.key.keysym;
+	SDL_Keysym key = ev.key.keysym;
 
 	if (ev.type != SDL_KEYDOWN && ev.type != SDL_KEYUP)
 		return bindings.end();
 
 	key.mod = KMOD_NONE;
 	if (ev.key.keysym.mod & KMOD_SHIFT)
-		key.mod = static_cast<SDLMod>(key.mod | KMOD_SHIFT);
+		key.mod = static_cast<SDL_Keymod>(key.mod | KMOD_SHIFT);
 	if (ev.key.keysym.mod & KMOD_CTRL)
-		key.mod = static_cast<SDLMod>(key.mod | KMOD_CTRL);
+		key.mod = static_cast<SDL_Keymod>(key.mod | KMOD_CTRL);
 #ifdef MACOSX
 	// map Meta to Alt on OS X
-	if (ev.key.keysym.mod & KMOD_META)
-		key.mod = (SDLMod)(key.mod | KMOD_ALT);
+	if (ev.key.keysym.mod & KMOD_GUI)
+		key.mod = static_cast<SDL_Keymod>(key.mod | KMOD_ALT);
 #else
 	if (ev.key.keysym.mod & KMOD_ALT)
-		key.mod = static_cast<SDLMod>(key.mod | KMOD_ALT);
+		key.mod = static_cast<SDL_Keymod>(key.mod | KMOD_ALT);
 #endif
 
 	return bindings.find(key);
 }
 
 bool KeyBinder::HandleEvent(SDL_Event const &ev) const {
-	KeyMap::const_iterator sdlkey_index = TranslateEvent(ev);
+	auto sdlkey_index = TranslateEvent(ev);
 	if (sdlkey_index != bindings.end())
 		return DoAction(sdlkey_index->second, ev.type == SDL_KEYDOWN);
 
@@ -488,7 +463,7 @@ bool KeyBinder::HandleEvent(SDL_Event const &ev) const {
 }
 
 bool KeyBinder::IsMotionEvent(SDL_Event const &ev) const {
-	KeyMap::const_iterator sdlkey_index = TranslateEvent(ev);
+	auto sdlkey_index = TranslateEvent(ev);
 	if (sdlkey_index == bindings.end())
 		return false;
 	ActionType const &act = sdlkey_index->second;
@@ -499,15 +474,14 @@ void KeyBinder::ShowHelp() const {
 	Scroll_gump *scroll;
 	scroll = new Scroll_gump();
 
-	std::vector<string>::const_iterator iter;
-
-	for (iter = keyhelp.begin(); iter != keyhelp.end(); ++iter)
-		scroll->add_text(iter->c_str());
+	for (const auto& iter : keyhelp)
+		scroll->add_text(iter.c_str());
 
 	scroll->paint();
 	do {
-		int x, y;
-		Get_click(x, y, Mouse::hand, 0, false, scroll);
+		int x;
+		int y;
+		Get_click(x, y, Mouse::hand, nullptr, false, scroll);
 	} while (scroll->show_next_page());
 	Game_window::get_instance()->paint();
 	delete scroll;
@@ -517,15 +491,14 @@ void KeyBinder::ShowCheatHelp() const {
 	Scroll_gump *scroll;
 	scroll = new Scroll_gump();
 
-	std::vector<string>::const_iterator iter;
-
-	for (iter = cheathelp.begin(); iter != cheathelp.end(); ++iter)
-		scroll->add_text(iter->c_str());
+	for (const auto& iter : cheathelp)
+		scroll->add_text(iter.c_str());
 
 	scroll->paint();
 	do {
-		int x, y;
-		Get_click(x, y, Mouse::hand, 0, false, scroll);
+		int x;
+		int y;
+		Get_click(x, y, Mouse::hand, nullptr, false, scroll);
 	} while (scroll->show_next_page());
 	Game_window::get_instance()->paint();
 	delete scroll;
@@ -535,22 +508,21 @@ void KeyBinder::ShowMapeditHelp() const {
 	Scroll_gump *scroll;
 	scroll = new Scroll_gump();
 
-	std::vector<string>::const_iterator iter;
-
-	for (iter = mapedithelp.begin(); iter != mapedithelp.end(); ++iter)
-		scroll->add_text(iter->c_str());
+	for (const auto& iter : mapedithelp)
+		scroll->add_text(iter.c_str());
 
 	scroll->paint();
 	do {
-		int x, y;
-		Get_click(x, y, Mouse::hand, 0, false, scroll);
+		int x;
+		int y;
+		Get_click(x, y, Mouse::hand, nullptr, false, scroll);
 	} while (scroll->show_next_page());
 	Game_window::get_instance()->paint();
 	delete scroll;
 }
 
 void KeyBinder::ShowBrowserKeys() const {
-	Scroll_gump *scroll = new Scroll_gump();
+	auto *scroll = new Scroll_gump();
 	scroll->add_text("Esc - Exits the shape browser");
 	scroll->add_text("down - Increase shape by 1");
 	scroll->add_text("S - Increase shape by 1");
@@ -575,15 +547,13 @@ void KeyBinder::ShowBrowserKeys() const {
 		strcpy(returned_key, "Error: No key assigned");
 	else {
 		strcpy(returned_key, "");   // prevent garbage text
-		std::vector<string>::const_iterator iter;
 		int extra_keys = 0;
-		for (iter = last_created_key.begin();
-		        iter != last_created_key.end(); ++iter) {
+		for (const auto& iter : last_created_key) {
 			if (extra_keys >= 5)
 				continue;
 			else if (extra_keys > 0)
 				strcat(returned_key, " or ");
-			strcat(returned_key, iter->c_str());
+			strcat(returned_key, iter.c_str());
 			extra_keys += 1;
 		}
 	}
@@ -591,21 +561,23 @@ void KeyBinder::ShowBrowserKeys() const {
 	scroll->add_text(returned_key);
 	scroll->paint();
 	do {
-		int x, y;
-		Get_click(x, y, Mouse::hand, 0, false, scroll);
+		int x;
+		int y;
+		Get_click(x, y, Mouse::hand, nullptr, false, scroll);
 	} while (scroll->show_next_page());
 	Game_window::get_instance()->paint();
 	delete scroll;
 }
 
 void KeyBinder::ParseText(char *text, int len) {
-	char *ptr, *end;
+	char *ptr;
+	char *end;
 	const char LF = '\n';
 
 	ptr = text;
 
 	// last (useful) line must end with LF
-	while ((ptr - text) < len && (end = strchr(ptr, LF)) != 0) {
+	while ((ptr - text) < len && (end = strchr(ptr, LF)) != nullptr) {
 		*end = '\0';
 		ParseLine(ptr);
 		ptr = end + 1;
@@ -621,12 +593,15 @@ static void skipspace(string &s) {
 
 void KeyBinder::ParseLine(char *line) {
 	size_t i;
-	SDL_keysym k;
+	SDL_Keysym k;
 	ActionType a;
 	k.sym      = SDLK_UNKNOWN;
 	k.mod      = KMOD_NONE;
-	string s = line, u;
-	string d, desc, keycode;
+	string s = line;
+	string u;
+	string d;
+	string desc;
+	string keycode;
 	bool show;
 
 	skipspace(s);
@@ -639,25 +614,24 @@ void KeyBinder::ParseLine(char *line) {
 	to_uppercase(u);
 
 	// get key
-	while (s.length() && !isspace(s[0])) {
+	while (s.length() && !isspace(static_cast<unsigned char>(s[0]))) {
 		// check modifiers
 		//    if (u.compare("ALT-",0,4) == 0) {
 		if (u.substr(0, 4) == "ALT-") {
-			k.mod = static_cast<SDLMod>(k.mod | KMOD_ALT);
+			k.mod = static_cast<SDL_Keymod>(k.mod | KMOD_ALT);
 			s.erase(0, 4);
 			u.erase(0, 4);
 			//    } else if (u.compare("CTRL-",0,5) == 0) {
 		} else if (u.substr(0, 5) == "CTRL-") {
-			k.mod = static_cast<SDLMod>(k.mod | KMOD_CTRL);
+			k.mod = static_cast<SDL_Keymod>(k.mod | KMOD_CTRL);
 			s.erase(0, 5);
 			u.erase(0, 5);
 			//    } else if (u.compare("SHIFT-",0,6) == 0) {
 		} else if (u.substr(0, 6) == "SHIFT-") {
-			k.mod = static_cast<SDLMod>(k.mod | KMOD_SHIFT);
+			k.mod = static_cast<SDL_Keymod>(k.mod | KMOD_SHIFT);
 			s.erase(0, 6);
 			u.erase(0, 6);
 		} else {
-
 			i = s.find_first_of(chardata.whitespace);
 
 			keycode = s.substr(0, i);
@@ -669,19 +643,17 @@ void KeyBinder::ParseLine(char *line) {
 				cerr << "Keybinder: parse error in line: " << s << endl;
 				return;
 			} else if (t.length() == 1) {
-				// translate 1-letter keys straight to SDLKey
-				char c = t[0];
-				if (c >= 33 && c <= 122 && c != 37) {
-					if (c >= 'A' && c <= 'Z')
-						c += 32; // need lowercase
-					k.sym = static_cast<SDLKey>(c);
+				// translate 1-letter keys straight to SDL_Keycode
+				auto c = static_cast<unsigned char>(t[0]);
+				if (std::isgraph(c) && c != '%' && c != '{' && c != '|' && c != '}' && c != '~') {
+					c = std::tolower(c);	// need lowercase
+					k.sym = static_cast<SDL_Keycode>(c);
 				} else {
 					cerr << "Keybinder: unsupported key: " << keycode << endl;
 				}
 			} else {
 				// lookup in table
-				ParseKeyMap::iterator key_index;
-				key_index = keys.find(t);
+				auto key_index = keys.find(t);
 				if (key_index != keys.end()) {
 					k.sym = (*key_index).second;
 				} else {
@@ -705,8 +677,7 @@ void KeyBinder::ParseLine(char *line) {
 	s.erase(0, i);
 	to_uppercase(t);
 
-	ParseActionMap::iterator action_index;
-	action_index = actions.find(t);
+	auto action_index = actions.find(t);
 	if (action_index != actions.end()) {
 		a.action = (*action_index).second;
 	} else {
@@ -840,11 +811,9 @@ void KeyBinder::LoadDefaults() {
 
 	U7object txtobj(resource.str, resource.num);
 	size_t len;
-	char *txt = txtobj.retrieve(len);
+	auto txt = txtobj.retrieve(len);
 	if (txt && len > 0)
-		ParseText(txt, len);
-
-	delete [] txt;
+		ParseText(reinterpret_cast<char*>(txt.get()), len);
 }
 
 // codes used in keybindings-files. (use uppercase here)
