@@ -5,7 +5,7 @@
  **/
 
 /*
-Copyright (C) 2001-2013 The Exult Team
+Copyright (C) 2001-2022 The Exult Team
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -23,44 +23,25 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
 #ifdef HAVE_CONFIG_H
-#  include <config.h>
+#	include <config.h>
 #endif
 
-#ifdef WIN32
-#include "Windrag.h"
-#endif
+#include "chunklst.h"
 
-#ifdef __GNUC__
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wold-style-cast"
-#pragma GCC diagnostic ignored "-Wcast-qual"
-#endif  // __GNUC__
-#include <gtk/gtk.h>
-#ifdef XWIN
-#include <gdk/gdkx.h>
-#endif
-#ifdef __GNUC__
-#pragma GCC diagnostic pop
-#endif  // __GNUC__
-#include "gtk_redefines.h"
+#include "exult_constants.h"
+#include "ibuf8.h"
+#include "shapegroup.h"
+#include "u7drag.h"
+#include "utils.h"
+#include "vgafile.h"
 
 #include <glib.h>
-#include "chunklst.h"
-#include "vgafile.h"
-#include "ibuf8.h"
-#include "u7drag.h"
-#include "exult_constants.h"
-#include "studio.h"
-#include "utils.h"
-#include "shapegroup.h"
-#include "ignore_unused_variable_warning.h"
-
 
 #include <iosfwd>
 
+using EStudio::Add_menu_item;
 using std::cout;
 using std::endl;
-using EStudio::Add_menu_item;
 
 const int border = 2;           // Border at bottom, sides of each
 //   chunk.
@@ -72,12 +53,17 @@ const int border = 2;           // Border at bottom, sides of each
 void Chunk_chooser::show(
     int x, int y, int w, int h  // Area to blit.
 ) {
-	Shape_draw::show(draw->window, x, y, w, h);
-	if (selected >= 0) {    // Show selected.
-		Rectangle b = info[selected].box;
+	Shape_draw::show(x, y, w, h);
+	if ((selected >= 0) && (drawgc != nullptr)) {    // Show selected.
+		TileRect b = info[selected].box;
 		// Draw yellow box.
-		gdk_draw_rectangle(draw->window, drawgc, FALSE,
-		                   b.x, b.y, b.w, b.h);
+		cairo_set_line_width(drawgc, 1.0);
+		cairo_set_source_rgb(drawgc,
+		                     ((drawfg >> 16) & 255) / 255.0,
+		                     ((drawfg >> 8) & 255) / 255.0,
+		                     (drawfg & 255) / 255.0);
+		cairo_rectangle(drawgc, b.x, b.y, b.w, b.h);
+		cairo_stroke(drawgc);
 	}
 }
 
@@ -112,7 +98,7 @@ void Chunk_chooser::select(
 	enable_controls();
 	int chunknum = info[selected].num;
 	// Remove prev. selection msg.
-//	gtk_statusbar_pop(GTK_STATUSBAR(sbar), sbar_sel);
+	//gtk_statusbar_pop(GTK_STATUSBAR(sbar), sbar_sel);
 	char buf[150];          // Show new selection.
 	g_snprintf(buf, sizeof(buf), "Chunk %d", chunknum);
 	gtk_statusbar_push(GTK_STATUSBAR(sbar), sbar_sel, buf);
@@ -125,14 +111,18 @@ void Chunk_chooser::select(
 void Chunk_chooser::render(
 ) {
 	// Look for selected frame.
-	int selchunk = -1, new_selected = -1;
+	int selchunk = -1;
+	int new_selected = -1;
 	if (selected >= 0)      // Save selection info.
 		selchunk = info[selected].num;
 	// Remove "selected" message.
 	//gtk_statusbar_pop(GTK_STATUSBAR(sbar), sbar_sel);
 	delete [] info;         // Delete old info. list.
 	// Get drawing area dimensions.
-	gint winw = draw->allocation.width, winh = draw->allocation.height;
+	GtkAllocation alloc = {0, 0, 0, 0};
+	gtk_widget_get_allocation(draw, &alloc);
+	gint winw = alloc.width;
+	gint winh = alloc.height;
 	// Provide more than enough room.
 	info = new Chunk_info[256];
 	info_cnt = 0;           // Count them.
@@ -140,7 +130,8 @@ void Chunk_chooser::render(
 	iwin->fill8(255);       // Background color.
 	int index = index0;
 	// 16x16 tiles, each 8x8 pixels.
-	const int chunkw = 128, chunkh = 128;
+	const int chunkw = 128;
+	const int chunkh = 128;
 	int total_cnt = get_count();
 	int y = border;
 	// Show bottom if at least 1/4 vis.
@@ -167,6 +158,7 @@ void Chunk_chooser::render(
 		unselect(false);
 	else
 		select(new_selected);
+	gtk_widget_queue_draw(draw);
 }
 
 /*
@@ -220,8 +212,8 @@ void Chunk_chooser::update_num_chunks(
 ) {
 	num_chunks = new_num_chunks;
 	GtkAdjustment *adj = gtk_range_get_adjustment(GTK_RANGE(vscroll));
-	adj->upper = num_chunks;
-	gtk_signal_emit_by_name(GTK_OBJECT(adj), "changed");
+	gtk_adjustment_set_upper(adj, num_chunks);
+	g_signal_emit_by_name(G_OBJECT(adj), "changed");
 }
 
 /*
@@ -273,7 +265,8 @@ void Chunk_chooser::render_chunk(
 		int x = c_tilesize;
 		for (int tx = 0; tx < c_tiles_per_chunk; tx++,
 		        x += c_tilesize) {
-			int shapenum, framenum;
+			int shapenum;
+			int framenum;
 			unsigned char l = *data++;
 			unsigned char h = *data++;
 			if (!headersz) {
@@ -309,24 +302,25 @@ gint Chunk_chooser::configure(
     gpointer data           // ->Chunk_chooser
 ) {
 	ignore_unused_variable_warning(widget);
-	Chunk_chooser *chooser = static_cast<Chunk_chooser *>(data);
+	auto *chooser = static_cast<Chunk_chooser *>(data);
 	chooser->Shape_draw::configure();
 	chooser->render();
 	// Set new scroll amounts.
-	int w = event->width, h = event->height;
+	int w = event->width;
+	int h = event->height;
 	int per_row = (w - border) / (128 + border);
 	int num_rows = (h - border) / (128 + border);
 	int page_size = per_row * num_rows;
 	GtkAdjustment *adj = gtk_range_get_adjustment(GTK_RANGE(
 	                         chooser->vscroll));
-	adj->step_increment = per_row;
-	adj->page_increment = page_size;
-	adj->page_size = page_size;
-	gtk_signal_emit_by_name(GTK_OBJECT(adj), "changed");
+	gtk_adjustment_set_step_increment(adj, per_row);
+	gtk_adjustment_set_page_increment(adj, page_size);
+	gtk_adjustment_set_page_size(adj, page_size);
+	g_signal_emit_by_name(G_OBJECT(adj), "changed");
 	if (chooser->group)     // Filtering?
 		chooser->enable_drop(); // Can drop chunks here.
 
-	return (TRUE);
+	return TRUE;
 }
 
 
@@ -336,73 +330,35 @@ gint Chunk_chooser::configure(
 
 gint Chunk_chooser::expose(
     GtkWidget *widget,      // The view window.
-    GdkEventExpose *event,
+    cairo_t *cairo,
     gpointer data           // ->Chunk_chooser.
 ) {
 	ignore_unused_variable_warning(widget);
-	Chunk_chooser *chooser = static_cast<Chunk_chooser *>(data);
-	chooser->show(event->area.x, event->area.y, event->area.width,
-	              event->area.height);
-	return (TRUE);
+	auto *chooser = static_cast<Chunk_chooser *>(data);
+	chooser->set_graphic_context(cairo);
+	GdkRectangle area = { 0, 0, 0, 0 };
+	gdk_cairo_get_clip_rectangle(cairo, &area);
+	chooser->show(area.x, area.y, area.width, area.height);
+	chooser->set_graphic_context(nullptr);
+	return TRUE;
 }
 
 /*
  *  Handle a mouse button press event.
  */
 
-#ifdef WIN32
-
-static bool win32_button = false;
-
-gint Chunk_chooser::win32_drag_motion(
-    GtkWidget *widget,      // The view window.
-    GdkEventMotion *event,
-    gpointer data           // ->Chunk_chooser.
-) {
-	ignore_unused_variable_warning(widget, event);
-	if (win32_button) {
-		win32_button = false;
-
-		// prepare the dragged data
-		windragdata wdata;
-
-		// This call allows us to recycle the data transfer initialization code.
-		//  It's clumsy, but far easier to maintain.
-		drag_data_get(NULL, NULL, (GtkSelectionData *) &wdata,
-		              U7_TARGET_CHUNKID, 0, data);
-
-		POINT pnt;
-		GetCursorPos(&pnt);
-
-		Windropsource idsrc(0, pnt.x, pnt.y);
-		LPDATAOBJECT idobj = (LPDATAOBJECT) new Winstudioobj(wdata);
-		DWORD dndout;
-
-		HRESULT res = DoDragDrop(idobj, &idsrc, DROPEFFECT_COPY, &dndout);
-		if (FAILED(res)) {
-			g_warning("Oops! Something is wrong with OLE2 DnD...");
-		}
-
-		idobj->Release();   // Not sure if we really need this. However, it doesn't hurt either.
-	}
-
-	return true;
-}
-
-#else
 gint Chunk_chooser::drag_motion(
     GtkWidget *widget,      // The view window.
     GdkEventMotion *event,
     gpointer data           // ->Shape_chooser.
 ) {
 	ignore_unused_variable_warning(widget);
-	Chunk_chooser *chooser = static_cast<Chunk_chooser *>(data);
+	auto *chooser = static_cast<Chunk_chooser *>(data);
 	if (!chooser->dragging && chooser->selected >= 0)
 		chooser->start_drag(U7_TARGET_CHUNKID_NAME,
 		                    U7_TARGET_CHUNKID, reinterpret_cast<GdkEvent *>(event));
 	return true;
 }
-#endif
 
 gint Chunk_chooser::mouse_press(
     GtkWidget *widget,      // The view window.
@@ -410,14 +366,14 @@ gint Chunk_chooser::mouse_press(
     gpointer data           // ->Chunk_chooser.
 ) {
 	ignore_unused_variable_warning(widget);
-	Chunk_chooser *chooser = static_cast<Chunk_chooser *>(data);
+	auto *chooser = static_cast<Chunk_chooser *>(data);
 
 	if (event->button == 4) {
 		chooser->scroll(true);
-		return(TRUE);
+		return TRUE;
 	} else if (event->button == 5) {
 		chooser->scroll(false);
-		return(TRUE);
+		return TRUE;
 	}
 
 	//int old_selected = chooser->selected;
@@ -429,19 +385,9 @@ gint Chunk_chooser::mouse_press(
 //			if (i == old_selected)
 //				return TRUE;
 			// Indicate we can dra.
-#ifdef WIN32
-// Here, we have to override GTK+'s Drag and Drop, which is non-OLE and
-// usually stucks outside the program window. I think it's because
-// the dragged shape only receives mouse motion events when the new mouse pointer
-// position is *still* inside the shape. So if you move the mouse too fast,
-// we are stuck.
-			win32_button = true;
-#endif
-
 			chooser->selected = i;
 			chooser->locate_cx = chooser->locate_cy = -1;
 			chooser->render();
-			chooser->show();
 			// Tell client.
 			if (chooser->sel_changed)
 				(*chooser->sel_changed)();
@@ -450,9 +396,9 @@ gint Chunk_chooser::mouse_press(
 	if (i == chooser->info_cnt && event->button == 1)
 		chooser->unselect(true);// Nothing under mouse.
 	else if (event->button == 3)
-		gtk_menu_popup(GTK_MENU(chooser->create_popup()), 0, 0, 0, 0,
-		               event->button, event->time);
-	return (TRUE);
+		gtk_menu_popup_at_pointer(GTK_MENU(chooser->create_popup()),
+		                          reinterpret_cast<GdkEvent *>(event));
+	return TRUE;
 }
 
 /*
@@ -464,7 +410,7 @@ static gint Mouse_release(
     gpointer data           // ->Shape_chooser.
 ) {
 	ignore_unused_variable_warning(widget, event);
-	Chunk_chooser *chooser = static_cast<Chunk_chooser *>(data);
+	auto *chooser = static_cast<Chunk_chooser *>(data);
 	chooser->mouse_up();
 	return true;
 }
@@ -482,40 +428,20 @@ void Chunk_chooser::drag_data_get(
     gpointer data           // ->Chunk_chooser.
 ) {
 	ignore_unused_variable_warning(widget, context, time);
-	cout << "In DRAG_DATA_GET" << endl;
-	Chunk_chooser *chooser = static_cast<Chunk_chooser *>(data);
+	cout << "In DRAG_DATA_GET of Chunk for '"
+	     << gdk_atom_name(gtk_selection_data_get_target(seldata))
+	     << "'" << endl;
+	auto *chooser = static_cast<Chunk_chooser *>(data);
 	if (chooser->selected < 0 || info != U7_TARGET_CHUNKID)
 		return;         // Not sure about this.
-	guchar buf[30];
+	guchar buf[U7DND_DATA_LENGTH(1)];
 	Chunk_info &shinfo = chooser->info[chooser->selected];
 	int len = Store_u7_chunkid(buf, shinfo.num);
 	cout << "Setting selection data (" << shinfo.num << ')' << endl;
-#ifdef WIN32
-	windragdata *wdata = (windragdata *)seldata;
-	wdata->assign(info, len, buf);
-#else
-	// Make us owner of xdndselection.
-	//gtk_selection_owner_set(widget, gdk_atom_intern("XdndSelection", 0),
-	//                          time);
 	// Set data.
 	gtk_selection_data_set(seldata,
-	                       gdk_atom_intern(U7_TARGET_CHUNKID_NAME, 0), 8, buf, len);
-#endif
-}
-
-/*
- *  Another app. has claimed the selection.
- */
-
-gint Chunk_chooser::selection_clear(
-    GtkWidget *widget,      // The view window.
-    GdkEventSelection *event,
-    gpointer data           // ->Chunk_chooser.
-) {
-	ignore_unused_variable_warning(widget, event, data);
-//	Chunk_chooser *chooser = static_cast<Chunk_chooser *>(data);
-	cout << "SELECTION_CLEAR" << endl;
-	return TRUE;
+	                       gtk_selection_data_get_target(seldata),
+	                       8, buf, len);
 }
 
 /*
@@ -528,50 +454,10 @@ gint Chunk_chooser::drag_begin(
     gpointer data           // ->Chunk_chooser.
 ) {
 	ignore_unused_variable_warning(widget, context);
-	cout << "In DRAG_BEGIN" << endl;
-	Chunk_chooser *chooser = static_cast<Chunk_chooser *>(data);
+	cout << "In DRAG_BEGIN of Chunk" << endl;
+	auto *chooser = static_cast<Chunk_chooser *>(data);
 	if (chooser->selected < 0)
 		return FALSE;       // ++++Display a halt bitmap.
-#if 0
-	// Get ->chunk.
-	Chunk_info &shinfo = chooser->info[chooser->selected];
-	Chunk_frame *chunk = chooser->ifile->get_chunk(shinfo.chunknum,
-	                     shinfo.framenum);
-	if (!chunk)
-		return FALSE;
-	int w = chunk->get_width(), h = chunk->get_height(),
-	    xright = chunk->get_xright(), ybelow = chunk->get_ybelow();
-	Image_buffer8 tbuf(w, h);   // Create buffer to render to.
-	tbuf.fill8(0xff);       // Fill with 'transparent' pixel.
-	unsigned char *tbits = tbuf.get_bits();
-	chunk->paint(&tbuf, w - 1 - xright, h - 1 - ybelow);
-	// Put chunk on a pixmap.
-	GdkPixmap *pixmap = gdk_pixmap_new(widget->window, w, h, -1);
-	gdk_draw_indexed_image(pixmap, chooser->drawgc, 0, 0, w, h,
-	                       GDK_RGB_DITHER_NORMAL, tbits,
-	                       tbuf.get_line_width(), chooser->palette);
-	int mask_stride = (w + 7) / 8;  // Round up to nearest byte.
-	char *mdata = new char[mask_stride * h];
-	for (int y = 0; y < h; y++) // Do each row.
-		// Do each byte.
-		for (int b = 0; b < mask_stride; b++) {
-			char bits = 0;
-			unsigned char *vals = tbits + y * w + b * 8;
-			for (int i = 0; i < 8; i++)
-				if (vals[i] != 0xff)
-					bits |= (1 << i);
-			mdata[y * mask_stride + b] = bits;
-		}
-	GdkBitmap *mask = gdk_bitmap_create_from_data(widget->window,
-	                  mdata, w, h);
-	delete mdata;
-	// This will be the chunk dragged.
-	gtk_drag_set_icon_pixmap(context,
-	                         gdk_window_get_colormap(widget->window), pixmap, mask,
-	                         w - 2 - xright, h - 2 - ybelow);
-	gdk_pixmap_unref(pixmap);
-	gdk_bitmap_unref(mask);
-#endif
 	return TRUE;
 }
 
@@ -587,15 +473,21 @@ void Chunk_chooser::drag_data_received(
     GtkSelectionData *seldata,
     guint info,
     guint time,
-    gpointer udata          // Should point to Shape_draw.
+    gpointer udata          // -> Chunk_chooser.
 ) {
 	ignore_unused_variable_warning(widget, context, x, y, info, time);
-	Chunk_chooser *chooser = static_cast<Chunk_chooser *>(udata);
-	cout << "Chunk drag_data_received" << endl;
-	if (seldata->type == gdk_atom_intern(U7_TARGET_CHUNKID_NAME, 0) &&
-	        seldata->format == 8 && seldata->length > 0) {
+	auto *chooser = static_cast<Chunk_chooser *>(udata);
+	cout << "In DRAG_DATA_RECEIVED of Chunk for '"
+	     << gdk_atom_name(gtk_selection_data_get_data_type(seldata))
+	     << "'" << endl;
+	if ((gtk_selection_data_get_data_type(seldata) == gdk_atom_intern(U7_TARGET_CHUNKID_NAME, 0) ||
+	     gtk_selection_data_get_data_type(seldata) == gdk_atom_intern(U7_TARGET_DROPFILE_NAME_MIME, 0) ||
+	     gtk_selection_data_get_data_type(seldata) == gdk_atom_intern(U7_TARGET_DROPFILE_NAME_MACOSX, 0)) &&
+	        Is_u7_chunkid(gtk_selection_data_get_data(seldata)) &&
+	        gtk_selection_data_get_format(seldata) == 8 &&
+	        gtk_selection_data_get_length(seldata) > 0) {
 		int cnum;
-		Get_u7_chunkid(seldata->data, cnum);
+		Get_u7_chunkid(gtk_selection_data_get_data(seldata), cnum);
 		chooser->group->add(cnum);
 		chooser->render();
 //		chooser->adjust_scrollbar(); ++++++Probably need to do this.
@@ -612,17 +504,21 @@ void Chunk_chooser::enable_drop(
 		return;
 	drop_enabled = true;
 	gtk_widget_realize(draw);//???????
-#ifndef WIN32
-	GtkTargetEntry tents[1];
+	GtkTargetEntry tents[3];
 	tents[0].target = const_cast<char *>(U7_TARGET_CHUNKID_NAME);
+	tents[1].target = const_cast<char *>(U7_TARGET_DROPFILE_NAME_MIME);
+	tents[2].target = const_cast<char *>(U7_TARGET_DROPFILE_NAME_MACOSX);
 	tents[0].flags = 0;
+	tents[1].flags = 0;
+	tents[2].flags = 0;
 	tents[0].info = U7_TARGET_CHUNKID;
-	gtk_drag_dest_set(draw, GTK_DEST_DEFAULT_ALL, tents, 1,
+	tents[1].info = U7_TARGET_CHUNKID;
+	tents[2].info = U7_TARGET_CHUNKID;
+	gtk_drag_dest_set(draw, GTK_DEST_DEFAULT_ALL, tents, 3,
 	                  static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE));
 
-	gtk_signal_connect(GTK_OBJECT(draw), "drag_data_received",
-	                   GTK_SIGNAL_FUNC(drag_data_received), this);
-#endif
+	g_signal_connect(G_OBJECT(draw), "drag-data-received",
+	                 G_CALLBACK(drag_data_received), this);
 }
 
 /*
@@ -638,7 +534,6 @@ void Chunk_chooser::scroll(
 	else if (index0 > newindex) // Backwards?
 		index0 = newindex >= 0 ? newindex : 0;
 	render();
-	show();
 }
 
 /*
@@ -649,12 +544,12 @@ void Chunk_chooser::scroll(
     bool upwards
 ) {
 	GtkAdjustment *adj = gtk_range_get_adjustment(GTK_RANGE(vscroll));
-	float delta = adj->step_increment;
+	gdouble delta = gtk_adjustment_get_step_increment(adj);
 	if (upwards)
 		delta = -delta;
-	adj->value += delta;
-	gtk_signal_emit_by_name(GTK_OBJECT(adj), "changed");
-	scroll(static_cast<gint>(adj->value));
+	gtk_adjustment_set_value(adj, gtk_adjustment_get_value(adj) + delta);
+	g_signal_emit_by_name(G_OBJECT(adj), "changed");
+	scroll(static_cast<gint>(gtk_adjustment_get_value(adj)));
 }
 
 /*
@@ -665,9 +560,9 @@ void Chunk_chooser::scrolled(
     GtkAdjustment *adj,     // The adjustment.
     gpointer data           // ->Chunk_chooser.
 ) {
-	Chunk_chooser *chooser = static_cast<Chunk_chooser *>(data);
-	cout << "Scrolled to " << adj->value << '\n';
-	gint newindex = static_cast<gint>(adj->value);
+	auto *chooser = static_cast<Chunk_chooser *>(data);
+	cout << "Scrolled to " << gtk_adjustment_get_value(adj) << '\n';
+	gint newindex = static_cast<gint>(gtk_adjustment_get_value(adj));
 	chooser->scroll(newindex);
 }
 
@@ -705,7 +600,7 @@ static void on_insert_empty(
     gpointer udata
 ) {
 	ignore_unused_variable_warning(item);
-	Chunk_chooser *chooser = static_cast<Chunk_chooser *>(udata);
+	auto *chooser = static_cast<Chunk_chooser *>(udata);
 	chooser->insert(false);
 }
 
@@ -714,7 +609,7 @@ static void on_insert_dup(
     gpointer udata
 ) {
 	ignore_unused_variable_warning(item);
-	Chunk_chooser *chooser = static_cast<Chunk_chooser *>(udata);
+	auto *chooser = static_cast<Chunk_chooser *>(udata);
 	chooser->insert(true);
 }
 static void on_delete(
@@ -722,7 +617,7 @@ static void on_delete(
     gpointer udata
 ) {
 	ignore_unused_variable_warning(item);
-	Chunk_chooser *chooser = static_cast<Chunk_chooser *>(udata);
+	auto *chooser = static_cast<Chunk_chooser *>(udata);
 	chooser->del();
 }
 
@@ -733,18 +628,18 @@ static void on_delete(
 GtkWidget *Chunk_chooser::create_popup(
 ) {
 	create_popup_internal(true); // Create popup with groups, files.
-	if (group != 0)         // Filtering?  Skip the rest.
+	if (group != nullptr)         // Filtering?  Skip the rest.
 		return popup;
 	GtkWidget *mitem = Add_menu_item(popup, "New...");
 	GtkWidget *new_menu = gtk_menu_new();
 	gtk_menu_item_set_submenu(GTK_MENU_ITEM(mitem), new_menu);
-	Add_menu_item(new_menu, "Empty", GTK_SIGNAL_FUNC(on_insert_empty),
+	Add_menu_item(new_menu, "Empty", G_CALLBACK(on_insert_empty),
 	              this);
 	if (selected >= 0) {
 		Add_menu_item(new_menu, "Duplicate",
-		              GTK_SIGNAL_FUNC(on_insert_dup), this);
+		              G_CALLBACK(on_insert_dup), this);
 		Add_menu_item(popup, "Delete",
-		              GTK_SIGNAL_FUNC(on_delete), this);
+		              G_CALLBACK(on_delete), this);
 	}
 	return popup;
 }
@@ -761,12 +656,10 @@ Chunk_chooser::Chunk_chooser(
     int w, int h,           // Dimensions.
     Shape_group *g          // Filter, or null.
 ) : Object_browser(g), Shape_draw(i, palbuf, gtk_drawing_area_new()),
-	chunkfile(cfile), chunksz(c_tiles_per_chunk *c_tiles_per_chunk * 2),
-	headersz(0), info(0), info_cnt(0), locate_cx(-1), locate_cy(-1),
-	drop_enabled(false), to_del(-1), sel_changed(0) {
-	static char v2hdr[] = { -1, -1, -1, -1, 'e', 'x', 'l', 't',
-	                        0, 0
-	                      };
+	chunkfile(cfile), chunksz(c_tiles_per_chunk * c_tiles_per_chunk * 2),
+	headersz(0), info(nullptr), info_cnt(0), locate_cx(-1), locate_cy(-1),
+	drop_enabled(false), to_del(-1), sel_changed(nullptr) {
+	static char v2hdr[] = {'\xff', '\xff', '\xff', '\xff', 'e', 'x', 'l', 't', 0, 0};
 	char v2buf[V2_CHUNK_HDR_SIZE];  // Check for V2 chunks.
 	chunkfile.seekg(0);
 	chunkfile.read(v2buf, sizeof(v2buf));
@@ -779,72 +672,66 @@ Chunk_chooser::Chunk_chooser(
 	chunkfile.seekg(0, std::ios::end);  // Figure total #chunks.
 	num_chunks = (static_cast<int>(chunkfile.tellg()) - headersz) / chunksz;
 	chunklist.resize(num_chunks);   // Init. list of ->'s to chunks.
+
 	// Put things in a vert. box.
-	GtkWidget *vbox = gtk_vbox_new(FALSE, 0);
+	GtkWidget *vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
+	gtk_box_set_homogeneous(GTK_BOX(vbox), FALSE);
 	set_widget(vbox); // This is our "widget"
 	gtk_widget_show(vbox);
 
-	GtkWidget *hbox = gtk_hbox_new(FALSE, 0);
+	GtkWidget *hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
+	gtk_box_set_homogeneous(GTK_BOX(hbox), FALSE);
 	gtk_widget_show(hbox);
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
 
 	// A frame looks nice.
-	GtkWidget *frame = gtk_frame_new(NULL);
+	GtkWidget *frame = gtk_frame_new(nullptr);
 	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
+	widget_set_margins(frame, 2*HMARGIN, 2*HMARGIN, 2*VMARGIN, 2*VMARGIN);
 	gtk_widget_show(frame);
 	gtk_box_pack_start(GTK_BOX(hbox), frame, TRUE, TRUE, 0);
+
 	// NOTE:  draw is in Shape_draw.
 	// Indicate the events we want.
 	gtk_widget_set_events(draw, GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK
 	                      | GDK_BUTTON_RELEASE_MASK
-	                      | GDK_POINTER_MOTION_HINT_MASK |
-	                      GDK_BUTTON1_MOTION_MASK);
+	                      | GDK_BUTTON1_MOTION_MASK);
 	// Set "configure" handler.
-	gtk_signal_connect(GTK_OBJECT(draw), "configure_event",
-	                   GTK_SIGNAL_FUNC(configure), this);
-	// Set "expose" handler.
-	gtk_signal_connect(GTK_OBJECT(draw), "expose_event",
-	                   GTK_SIGNAL_FUNC(expose), this);
+	g_signal_connect(G_OBJECT(draw), "configure-event",
+	                 G_CALLBACK(configure), this);
+	// Set "expose-event" - "draw" handler.
+	g_signal_connect(G_OBJECT(draw), "draw",
+	                 G_CALLBACK(expose), this);
 	// Set mouse click handler.
-	gtk_signal_connect(GTK_OBJECT(draw), "button_press_event",
-	                   GTK_SIGNAL_FUNC(mouse_press), this);
-	gtk_signal_connect(GTK_OBJECT(draw), "button_release_event",
-	                   GTK_SIGNAL_FUNC(Mouse_release), this);
+	g_signal_connect(G_OBJECT(draw), "button-press-event",
+	                 G_CALLBACK(mouse_press), this);
+	g_signal_connect(G_OBJECT(draw), "button-release-event",
+	                 G_CALLBACK(Mouse_release), this);
 	// Mouse motion.
-	gtk_signal_connect(GTK_OBJECT(draw), "drag_begin",
-	                   GTK_SIGNAL_FUNC(drag_begin), this);
-#ifdef WIN32
-// required to override GTK+ Drag and Drop
-	gtk_signal_connect(GTK_OBJECT(draw), "motion_notify_event",
-	                   GTK_SIGNAL_FUNC(win32_drag_motion), this);
-#else
-	gtk_signal_connect(GTK_OBJECT(draw), "motion_notify_event",
-	                   GTK_SIGNAL_FUNC(drag_motion), this);
-#endif
-	gtk_signal_connect(GTK_OBJECT(draw), "drag_data_get",
-	                   GTK_SIGNAL_FUNC(drag_data_get), this);
-	gtk_signal_connect(GTK_OBJECT(draw), "selection_clear_event",
-	                   GTK_SIGNAL_FUNC(selection_clear), this);
+	g_signal_connect(G_OBJECT(draw), "drag-begin",
+	                 G_CALLBACK(drag_begin), this);
+	g_signal_connect(G_OBJECT(draw), "motion-notify-event",
+	                 G_CALLBACK(drag_motion), this);
+	g_signal_connect(G_OBJECT(draw), "drag-data-get",
+	                 G_CALLBACK(drag_data_get), this);
 	gtk_container_add(GTK_CONTAINER(frame), draw);
-	gtk_drawing_area_size(GTK_DRAWING_AREA(draw), w, h);
+	widget_set_margins(draw, 2*HMARGIN, 2*HMARGIN, 2*VMARGIN, 2*VMARGIN);
+	gtk_widget_set_size_request(draw, w, h);
 	gtk_widget_show(draw);
 	// Want a scrollbar for the chunks.
-	GtkObject *chunk_adj = gtk_adjustment_new(0, 0,
-	                       num_chunks, 1,
-	                       4, 1.0);
-	vscroll = gtk_vscrollbar_new(GTK_ADJUSTMENT(chunk_adj));
-#if 0
-	// Update window when it stops.
-	gtk_range_set_update_policy(GTK_RANGE(vscroll),
-	                            GTK_UPDATE_DELAYED);
-#endif
+	GtkAdjustment *chunk_adj = GTK_ADJUSTMENT(gtk_adjustment_new(0, 0,
+	                           num_chunks, 1,
+	                           4, 1.0));
+	vscroll = gtk_scrollbar_new(GTK_ORIENTATION_VERTICAL, GTK_ADJUSTMENT(chunk_adj));
 	gtk_box_pack_start(GTK_BOX(hbox), vscroll, FALSE, TRUE, 0);
 	// Set scrollbar handler.
-	gtk_signal_connect(GTK_OBJECT(chunk_adj), "value_changed",
-	                   GTK_SIGNAL_FUNC(scrolled), this);
+	g_signal_connect(G_OBJECT(chunk_adj), "value-changed",
+	                 G_CALLBACK(scrolled), this);
 	gtk_widget_show(vscroll);
+
 	// At the bottom, status bar:
-	GtkWidget *hbox1 = gtk_hbox_new(FALSE, 0);
+	GtkWidget *hbox1 = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
+	gtk_box_set_homogeneous(GTK_BOX(hbox1), FALSE);
 	gtk_box_pack_start(GTK_BOX(vbox), hbox1, FALSE, FALSE, 0);
 	gtk_widget_show(hbox1);
 	// At left, a status bar.
@@ -852,6 +739,7 @@ Chunk_chooser::Chunk_chooser(
 	sbar_sel = gtk_statusbar_get_context_id(GTK_STATUSBAR(sbar),
 	                                        "selection");
 	gtk_box_pack_start(GTK_BOX(hbox1), sbar, TRUE, TRUE, 0);
+	widget_set_margins(sbar, 2*HMARGIN, 2*HMARGIN, 2*VMARGIN, 2*VMARGIN);
 	gtk_widget_show(sbar);
 	// Add locate/move controls to bottom.
 	gtk_box_pack_start(GTK_BOX(vbox),
@@ -899,7 +787,6 @@ bool Chunk_chooser::server_response(
 	case Exult_server::send_terrain:
 		set_chunk(data, datalen);
 		render();
-		show();
 		return true;
 	default:
 		return false;
@@ -915,10 +802,9 @@ void Chunk_chooser::end_terrain_editing(
 	// Clear out cache of chunks.
 	for (int i = 0; i < num_chunks; i++) {
 		delete chunklist[i];
-		chunklist[i] = 0;
+		chunklist[i] = nullptr;
 	}
 	render();
-	show();
 }
 
 /*
@@ -933,15 +819,13 @@ void Chunk_chooser::unselect(
 		locate_cx = locate_cy = -1;
 		if (need_render) {
 			render();
-			show();
 		}
 		if (sel_changed)    // Tell client.
 			(*sel_changed)();
 	}
 	enable_controls();      // Enable/disable controls.
-	char buf[150];          // Show new selection.
 	if (info_cnt > 0) {
-//		gtk_statusbar_pop(GTK_STATUSBAR(sbar), sbar_sel);
+		char buf[150];          // Show new selection.
 		g_snprintf(buf, sizeof(buf), "Chunks %d to %d",
 		           info[0].num, info[info_cnt - 1].num);
 		gtk_statusbar_push(GTK_STATUSBAR(sbar), sbar_sel, buf);
@@ -962,7 +846,8 @@ void Chunk_chooser::locate(
 	unsigned char data[Exult_server::maxlength];
 	unsigned char *ptr = &data[0];
 	int tnum = info[selected].num;  // Terrain #.
-	int cx = locate_cx, cy = locate_cy;
+	int cx = locate_cx;
+	int cy = locate_cy;
 	if (dir == 0) {
 		cx = cy = -1;
 	} else if (dir == -1)
@@ -998,8 +883,8 @@ void Chunk_chooser::locate_response(
 		to_del = -1;
 		return;         // Not the current selection.
 	}
-	short cx = static_cast<short>(Read2(ptr));  // Get chunk found.
-	short cy = static_cast<short>(Read2(ptr));
+	auto cx = static_cast<short>(Read2(ptr));  // Get chunk found.
+	auto cy = static_cast<short>(Read2(ptr));
 	ptr++;              // Skip upwards flag.
 	if (!*ptr) {
 		if (to_del >= 0 && to_del == tnum) {
@@ -1062,12 +947,12 @@ void Chunk_chooser::insert_response(
 	ignore_unused_variable_warning(datalen);
 	const unsigned char *ptr = data;
 	int tnum = static_cast<short>(Read2(ptr));
-	bool dup = *ptr++ ? true : false;
+	bool dup = *ptr++ != 0;
 	if (!*ptr)
 		EStudio::Alert("Terrain insert failed.");
 	else {
 		// Insert in our list.
-		unsigned char *data = new unsigned char[chunksz];
+		auto *data = new unsigned char[chunksz];
 		if (dup && tnum >= 0 && tnum < num_chunks && chunklist[tnum])
 			memcpy(data, chunklist[tnum], chunksz);
 		else
@@ -1078,7 +963,6 @@ void Chunk_chooser::insert_response(
 			chunklist.push_back(data);
 		update_num_chunks(num_chunks + 1);
 		render();
-		show();
 	}
 }
 
@@ -1101,7 +985,6 @@ void Chunk_chooser::delete_response(
 		chunklist.erase(chunklist.begin() + tnum);
 		update_num_chunks(num_chunks - 1);
 		render();
-		show();
 	}
 }
 
@@ -1157,6 +1040,5 @@ void Chunk_chooser::swap_response(
 			}
 		}
 		render();
-		show();
 	}
 }
