@@ -8,7 +8,7 @@
 #define INCL_COMBO_H    1
 
 /*
-Copyright (C) 2002-2013 The Exult Team
+Copyright (C) 2002-2022 The Exult Team
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -25,11 +25,13 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
+#include "objbrowse.h"
+#include "rect.h"
+#include "shapedraw.h"
+
+#include <memory>
 #include <string>
 #include <vector>
-#include "objbrowse.h"
-#include "shapedraw.h"
-#include "rect.h"
 
 class Shapes_vga_file;
 class Flex_file_info;
@@ -61,9 +63,9 @@ class Combo {
 	//   use.
 	short starttx, startty;     // Offset represented by top-left.
 	std::string name;       // Name given by user.
-	Rectangle tilefoot;     // Footprint in tiles.
+	TileRect tilefoot;     // Footprint in tiles.
 	// Get footprint of given member.
-	Rectangle get_member_footprint(int i);
+	TileRect get_member_footprint(int i);
 public:
 	friend class Combo_editor;
 	friend class Combo_chooser;
@@ -71,7 +73,8 @@ public:
 	Combo(const Combo &c2);     // Copy.
 	~Combo();
 	Combo_member *get(int i) {
-		return i >= 0 && static_cast<unsigned>(i) < members.size() ? members[i] : 0;
+		return i >= 0 && static_cast<unsigned>(i) < members.size() ?
+		       members[i] : nullptr;
 	}
 	// Add a new object.
 	void add(int tx, int ty, int tz, int shnum, int frnum, bool toggle);
@@ -81,7 +84,7 @@ public:
 	          int xoff = 0, int yoff = 0);
 	int find(int mx, int my);   // Find at mouse position.
 	// Serialize:
-	unsigned char *write(int &datalen);
+	std::unique_ptr<unsigned char[]> write(int &datalen);
 	const unsigned char *read(const unsigned char *buf, int bufsize);
 };
 
@@ -100,11 +103,13 @@ class Combo_editor : public Shape_draw {
 public:
 	friend class Combo_chooser;
 	Combo_editor(Shapes_vga_file *svga, unsigned char *palbuf);
-	~Combo_editor();
+	~Combo_editor() override;
+	static gboolean on_combo_draw_expose_event(
+	    GtkWidget *widget, cairo_t *cairo, gpointer data);
 	void show(bool tf);     // Show/hide.
 	void render_area(GdkRectangle *area);
-	virtual void render() {
-		render_area(0);
+	void render() override {
+		render_area(nullptr);
 	}
 	void set_controls();        // Set controls to selected entry.
 	// Handle mouse.
@@ -116,7 +121,7 @@ public:
 	void remove();          // Remove selected.
 	void save();            // Save it.
 	bool is_visible() {
-		return GTK_WIDGET_VISIBLE(win);
+		return gtk_widget_get_visible(GTK_WIDGET(win));
 	}
 };
 
@@ -130,11 +135,10 @@ public:
 class Combo_info {
 	friend class Combo_chooser;
 	int num;
-	Rectangle box;          // Box where drawn.
-	Combo_info() {  }
+	TileRect box;          // Box where drawn.
 	void set(int n, int rx, int ry, int rw, int rh) {
 		num = n;
-		box = Rectangle(rx, ry, rw, rh);
+		box = TileRect(rx, ry, rw, rh);
 	}
 };
 
@@ -152,18 +156,17 @@ class Combo_chooser: public Object_browser, public Shape_draw {
 	int info_cnt;           // # entries in info.
 	void (*sel_changed)();      // Called when selection changes.
 	// Blit onto screen.
-	virtual void show(int x, int y, int w, int h);
-	virtual void show() {
-		Combo_chooser::show(0, 0,
-		                    draw->allocation.width, draw->allocation.height);
-	}
+	void show(int x, int y, int w, int h) override;
 	void select(int new_sel);   // Show new selection.
-	virtual void load();        // Load from file data.
-	virtual void render();      // Draw list.
-	virtual void set_background_color(guint32 c) {
+	void load() override {
+		load_internal();
+	}
+	void load_internal();        // Load from file data.
+	void render() override;      // Draw list.
+	void set_background_color(guint32 c) override {
 		Shape_draw::set_background_color(c);
 	}
-	virtual int get_selected_id() {
+	int get_selected_id() override {
 		return selected < 0 ? -1 : info[selected].num;
 	}
 	void scroll(int newindex);  // Scroll.
@@ -173,11 +176,11 @@ class Combo_chooser: public Object_browser, public Shape_draw {
 public:
 	Combo_chooser(Vga_file *i, Flex_file_info *flinfo,
 	              unsigned char *palbuf, int w, int h,
-	              Shape_group *g = 0);
-	virtual ~Combo_chooser();
+	              Shape_group *g = nullptr);
+	~Combo_chooser() override;
 	// Turn off selection.
 	void unselect(bool need_render = true);
-	int is_selected() {     // Is a combo selected?
+	bool is_selected() {     // Is a combo selected?
 		return selected >= 0;
 	}
 	void set_selected_callback(void (*fun)()) {
@@ -191,30 +194,23 @@ public:
 	static gint configure(GtkWidget *widget, GdkEventConfigure *event,
 	                      gpointer data);
 	// Blit to screen.
-	static gint expose(GtkWidget *widget, GdkEventExpose *event,
-	                   gpointer data);
+	static gint expose(
+	    GtkWidget *widget, cairo_t *cairo, gpointer data);
 	// Handle mouse press.
 	static gint mouse_press(GtkWidget *widget, GdkEventButton *event,
 	                        gpointer data);
 	// Give dragged combo.
 	static void drag_data_get(GtkWidget *widget, GdkDragContext *context,
-	                          GtkSelectionData *selection_data, guint info, guint time, gpointer data);
-	// Someone else selected.
-	static gint selection_clear(GtkWidget *widget,
-	                            GdkEventSelection *event, gpointer data);
+	                          GtkSelectionData *seldata,
+	                          guint info, guint time, gpointer data);
 	static gint drag_begin(GtkWidget *widget, GdkDragContext *context,
 	                       gpointer data);
 	// Handle scrollbar.
 	static void scrolled(GtkAdjustment *adj, gpointer data);
-	virtual void move(bool upwards);// Move current selected combo.
-	virtual void search(const char *srch, int dir);
-#ifdef WIN32
-	static gint win32_drag_motion(GtkWidget *widget, GdkEventMotion *event,
-	                              gpointer data);
-#else
+	void move(bool upwards) override;// Move current selected combo.
+	void search(const char *srch, int dir) override;
 	static gint drag_motion(GtkWidget *widget, GdkEventMotion *event,
 	                        gpointer data);
-#endif
 };
 
 
