@@ -5,7 +5,7 @@
  **/
 
 /*
-Copyright (C) 2002-2013 The Exult Team
+Copyright (C) 2002-2022 The Exult Team
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -23,15 +23,14 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
 #ifdef HAVE_CONFIG_H
-#  include <config.h>
+#	include <config.h>
 #endif
 
-#include <string>
-#include "studio.h"
+#include "execbox.h"
 #include "exult_constants.h"
 #include "utils.h"
-#include "execbox.h"
-#include "ignore_unused_variable_warning.h"
+
+#include <string>
 
 using std::string;
 
@@ -84,13 +83,13 @@ void Ucc_done(
 void ExultStudio::open_compile_window(
 ) {
 	if (!compilewin) {      // First time?
-		compilewin = glade_xml_get_widget(app_xml, "compile_win");
+		compilewin = get_widget("compile_win");
 		compile_box = new Exec_box(
 		    GTK_TEXT_VIEW(
-		        glade_xml_get_widget(app_xml, "compile_msgs")),
+		        get_widget("compile_msgs")),
 		    GTK_STATUSBAR(
-		        glade_xml_get_widget(app_xml, "compile_status")),
-		    Ucc_done, 0);
+		        get_widget("compile_status")),
+		    Ucc_done, nullptr);
 	}
 	gtk_widget_show(compilewin);
 }
@@ -132,7 +131,7 @@ void ExultStudio::compile(
 	argv[2] = obj.c_str();
 	argv[3] = source.c_str();   // What to compile.
 	argv[4] = incdir.c_str();   // Include dir
-	argv[5] = 0;            // NULL.
+	argv[5] = nullptr;            // nullptr.
 	if (!compile_box->exec("ucc", argv))
 		EStudio::Alert("Error executing usecode compiler ('ucc')");
 }
