@@ -5,7 +5,7 @@
  **/
 
 /*
-Copyright (C) 2002-2013 The Exult Team
+Copyright (C) 2002-2022 The Exult Team
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -23,35 +23,35 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
 #ifdef HAVE_CONFIG_H
-#  include <config.h>
+#	include <config.h>
 #endif
 
 #include "execbox.h"
-#include "ignore_unused_variable_warning.h"
+
+#include <cstdlib>
+#include <cstring>
 #include <iostream> /* Debugging only */
 #include <string>
-#include <cstring>
-#include <cstdlib>
+
+#ifndef _WIN32
+#	ifdef HAVE_SYS_TYPES_H
+#		include <sys/types.h>
+#	endif
+#	include <sys/wait.h>
+#	include <unistd.h>
+
+#	include <csignal>
 
 using std::cout;
 using std::endl;
 
-#ifndef WIN32
-
-#ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
-#include <unistd.h>
-#include <signal.h>
-#include <sys/wait.h>
-
 /*
  *  Create.
  */
 
 Exec_process::Exec_process(
 ) : child_stdin(-1), child_stdout(-1), child_stderr(-1),
-	child_pid(-1), stdout_tag(-1), stderr_tag(-1), reader(0) {
+	child_pid(-1), stdout_tag(-1), stderr_tag(-1), reader(nullptr) {
 }
 
 /*
@@ -78,9 +78,9 @@ void Exec_process::kill_child(
 	if (child_stderr >= 0)
 		close(child_stderr);
 	if (stdout_tag >= 0)
-		gdk_input_remove(stdout_tag);
+		g_source_remove(stdout_tag);
 	if (stderr_tag >= 0)
-		gdk_input_remove(stderr_tag);
+		g_source_remove(stderr_tag);
 	child_pid = child_stdin = child_stdout = child_stderr =
 	                              stdout_tag = stderr_tag = -1;
 }
@@ -89,14 +89,15 @@ void Exec_process::kill_child(
  *  Read from child & call client routine.
  */
 
-static void Read_from_child(
-    gpointer data,          // ->Exex_process
-    gint id,            // Pipe ID.
-    GdkInputCondition condition
+static gboolean Read_from_child(
+    GIOChannel  *source,
+    GIOCondition condition,
+    gpointer     data           // ->Exex_process
 ) {
 	ignore_unused_variable_warning(condition);
-	Exec_process *ex = static_cast<Exec_process *>(data);
-	ex->read_from_child(id);
+	auto *ex = static_cast<Exec_process *>(data);
+	ex->read_from_child(g_io_channel_unix_get_fd(source));
+	return TRUE;
 }
 void Exec_process::read_from_child(
     int id              // Pipe to read from.
@@ -110,7 +111,7 @@ void Exec_process::read_from_child(
 	if (!check_child(exit_code)) {  // Child done?
 		kill_child();       // Clean up.
 		if (reader)     // Tell client.
-			(*reader)(0, 0, exit_code, reader_data);
+			(*reader)(nullptr, 0, exit_code, reader_data);
 	}
 }
 
@@ -169,7 +170,9 @@ bool Exec_process::exec(
 	reader = rfun;          // Store callback.
 	reader_data = udata;
 	// Pipes for talking to child:
-	int stdin_pipe[2], stdout_pipe[2], stderr_pipe[2];
+	int stdin_pipe[2];
+	int stdout_pipe[2];
+	int stderr_pipe[2];
 	stdin_pipe[0] = stdin_pipe[1] = stdout_pipe[0] = stdout_pipe[1] =
 	                                    stderr_pipe[0] = stderr_pipe[1] = -1;
 	kill_child();           // Kill running process.
@@ -186,9 +189,9 @@ bool Exec_process::exec(
 		return false;
 	}
 	if (child_pid == 0) {   // Are we the child?
-		if (dup_wrapper(stdin_pipe , 0, true ) ||
-		    dup_wrapper(stdout_pipe, 1, false) ||
-		    dup_wrapper(stderr_pipe, 2, false)) {
+		if (dup_wrapper(stdin_pipe, 0, true) ||
+		        dup_wrapper(stdout_pipe, 1, false) ||
+		        dup_wrapper(stderr_pipe, 2, false)) {
 			Close_pipes(stdin_pipe, stdout_pipe, stderr_pipe);
 			return false;
 		}
@@ -204,10 +207,16 @@ bool Exec_process::exec(
 	close(stderr_pipe[1]);
 	cout << "Child_stdout is " << child_stdout << ", Child_stderr is " <<
 	     child_stderr << endl;
-	stdout_tag = gdk_input_add(child_stdout,
-	                           GDK_INPUT_READ, Read_from_child, this);
-	stderr_tag = gdk_input_add(child_stderr,
-	                           GDK_INPUT_READ, Read_from_child, this);
+	GIOChannel *gio_out = g_io_channel_unix_new(child_stdout);
+	GIOChannel *gio_err = g_io_channel_unix_new(child_stderr);
+	stdout_tag = g_io_add_watch(gio_out,
+	                            static_cast<GIOCondition>(G_IO_IN | G_IO_HUP | G_IO_ERR),
+	                            Read_from_child, this);
+	stderr_tag = g_io_add_watch(gio_err,
+	                            static_cast<GIOCondition>(G_IO_IN | G_IO_HUP | G_IO_ERR),
+	                            Read_from_child, this);
+	g_io_channel_unref(gio_out);
+	g_io_channel_unref(gio_err);
 	return true;
 }
 
@@ -281,16 +290,16 @@ void Exec_box::show_status(
  */
 
 static void Exec_callback(
-    char *data,         // Data read, or NULL.
+    char *data,         // Data read, or nullptr.
     int datalen,            // Length, or 0 if child exited.
     int exit_code,          // Exit code if datalen = 0.
     void *user_data         // ->Exex_box
 ) {
-	Exec_box *box = static_cast<Exec_box *>(user_data);
+	auto *box = static_cast<Exec_box *>(user_data);
 	box->read_from_child(data, datalen, exit_code);
 }
 void Exec_box::read_from_child(
-    char *data,         // Data read, or NULL.
+    char *data,         // Data read, or nullptr.
     int datalen,            // Length, or 0 if child exited.
     int exit_code           // Exit code if datalen = 0.
 ) {
@@ -330,9 +339,7 @@ bool Exec_box::exec(
 ) {
 	GtkTextBuffer *buffer = gtk_text_view_get_buffer(box);
 	gtk_text_buffer_set_text(buffer, "", 0);    // Clear out old text
-	if (!executor->exec(file, argv, Exec_callback, this))
-		return false;
-	return true;
+	return executor->exec(file, argv, Exec_callback, this);
 }
 
 /*
