@@ -5,7 +5,7 @@
  **/
 
 /*
-Copyright (C) 2000-2013 The Exult Team
+Copyright (C) 2000-2022 The Exult Team
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -23,29 +23,25 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
 #ifdef HAVE_CONFIG_H
-#  include <config.h>
+#	include <config.h>
 #endif
 
-#include <cstring>
-#include <cstdlib>
-
+#include "exult_constants.h"
+#include "npclst.h"
+#include "objserial.h"
+#include "servemsg.h"
+#include "shapedraw.h"
+#include "shapefile.h"
+#include "shapevga.h"
 #include "studio.h"
 #include "u7drag.h"
-#include "servemsg.h"
-#include "objserial.h"
-#include "exult_constants.h"
 #include "utils.h"
-#include "shapefile.h"
-#include "shapedraw.h"
-#include "npclst.h"
-#include "ignore_unused_variable_warning.h"
 
-#ifdef WIN32
-#include "windrag.h"
-#endif
+#include <cstdlib>
+#include <cstring>
 
-using   std::cout;
-using   std::endl;
+using std::cout;
+using std::endl;
 
 class Actor;
 
@@ -97,8 +93,8 @@ C_EXPORT void on_npc_show_gump_clicked(
 	cout << "In on_npc_show_gump_clicked()" << endl;
 	unsigned char data[Exult_server::maxlength];
 	// Get container address.
-	uintptr addr = reinterpret_cast<uintptr>(gtk_object_get_user_data(
-	                         GTK_OBJECT(gtk_widget_get_toplevel(GTK_WIDGET(btn)))));
+	auto addr = reinterpret_cast<uintptr>(g_object_get_data(
+	        G_OBJECT(gtk_widget_get_toplevel(GTK_WIDGET(btn))), "user_data"));
 	unsigned char *ptr = &data[0];
 	Serial_out io(ptr);
 	io << addr;
@@ -134,79 +130,8 @@ C_EXPORT void on_npc_usecode_browse_clicked(
 		studio->set_entry("npc_usecode_entry", uc, true);
 }
 
-/*
- *  Draw shape in NPC shape area.
- */
-C_EXPORT gboolean on_npc_draw_expose_event(
-    GtkWidget *widget,      // The view window.
-    GdkEventExpose *event,
-    gpointer data           // ->Shape_chooser.
-) {
-	ignore_unused_variable_warning(widget, data);
-	ExultStudio::get_instance()->show_npc_shape(
-	    event->area.x, event->area.y, event->area.width,
-	    event->area.height);
-	return (TRUE);
-}
-
-/*
- *  Npc shape # lost focus, so update shape displayed.
- */
-C_EXPORT gboolean on_npc_shape_focus_out_event(
-    GtkWidget *widget,
-    GdkEventFocus *event,
-    gpointer user_data
-) {
-	ignore_unused_variable_warning(widget, event, user_data);
-	ExultStudio::get_instance()->show_npc_shape();
-	return FALSE;
-}
-
-/*
- *  Callback for when a shape is dropped on the NPC draw area.
- */
-
-static void Npc_shape_dropped(
-    int file,           // U7_SHAPE_SHAPES.
-    int shape,
-    int frame,
-    void *udata
-) {
-	if (file == U7_SHAPE_SHAPES && shape >= 150 && shape < c_max_shapes)
-		static_cast<ExultStudio *>(udata)->set_npc_shape(shape, frame);
-}
-
 // Schedule names.
-/*
- *  Draw face.
- */
-C_EXPORT gboolean on_npc_face_draw_expose_event(
-    GtkWidget *widget,      // The view window.
-    GdkEventExpose *event,
-    gpointer data           // ->Shape_chooser.
-) {
-	ignore_unused_variable_warning(widget, data);
-	ExultStudio::get_instance()->show_npc_face(
-	    event->area.x, event->area.y, event->area.width,
-	    event->area.height);
-	return (TRUE);
-}
 
-/*
- *  Callback for when a shape is dropped on the NPC face area.
- */
-
-static void Npc_face_dropped(
-    int file,           // U7_SHAPE_FACES
-    int shape,
-    int frame,
-    void *udata
-) {
-	if (file == U7_SHAPE_FACES && shape >= 0 && shape < 1024)
-		static_cast<ExultStudio *>(udata)->set_npc_face(shape, frame);
-}
-
-// Schedule names.
 static const char *sched_names[32] = {
 	"Combat", "Horiz. Pace", "Vert. Pace", "Talk", "Dance",
 	"Eat", "Farm", "Tend Shop", "Miner", "Hound", "Stand",
@@ -221,31 +146,33 @@ static const char *sched_names[32] = {
  */
 
 static void Set_schedule_line(
-    GladeXML *app_xml,
+    ExultStudio *studio,
     int time,           // 0-7.
     int type,           // Activity (0-31, or -1 for none).
     int tx, int ty, int tz = 0  // Location.
 ) {
 	char *lname = g_strdup_printf("npc_sched%d", time);
-	GtkLabel *label = GTK_LABEL(glade_xml_get_widget(app_xml, lname));
+	GtkLabel *label = GTK_LABEL(studio->get_widget(lname));
 	g_free(lname);
 	// User data = schedule #.
-	gtk_object_set_user_data(GTK_OBJECT(label), reinterpret_cast<gpointer>(uintptr(type)));
+	g_object_set_data(G_OBJECT(label), "user_data", reinterpret_cast<gpointer>(uintptr(type)));
 	gtk_label_set_text(label,
 	                   type >= 0 && type < 32 ? sched_names[type] : "-----");
 	// Set location.
 	char *locname = g_strdup_printf("sched_loc%d", time);
-	GtkBox *box = GTK_BOX(glade_xml_get_widget(app_xml, locname));
+	GtkBox *box = GTK_BOX(studio->get_widget(locname));
 	g_free(locname);
-	GList *list = g_list_first(box->children);
-	GtkWidget *spin = static_cast<GtkBoxChild *>(list->data)->widget;
+	GList *children = g_list_first(gtk_container_get_children(GTK_CONTAINER(box)));
+	GList *list = children;
+	GtkWidget *spin = GTK_WIDGET(list->data);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin), tx);
 	list = g_list_next(list);
-	spin = static_cast<GtkBoxChild *>(list->data)->widget;
+	spin = GTK_WIDGET(list->data);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin), ty);
 	list = g_list_next(list);
-	spin = static_cast<GtkBoxChild *>(list->data)->widget;
+	spin = GTK_WIDGET(list->data);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin), tz);
+	g_list_free(children);
 }
 
 /*
@@ -255,31 +182,33 @@ static void Set_schedule_line(
  */
 
 static bool Get_schedule_line(
-    GladeXML *app_xml,
+    ExultStudio *studio,
     int time,           // 0-7.
     Serial_schedule &sched      // Filled in if 'true' returned.
 ) {
 	char *lname = g_strdup_printf("npc_sched%d", time);
-	GtkLabel *label = GTK_LABEL(glade_xml_get_widget(app_xml, lname));
+	GtkLabel *label = GTK_LABEL(studio->get_widget(lname));
 	g_free(lname);
 	// User data = schedule #.
-	sched.type = reinterpret_cast<sintptr>(gtk_object_get_user_data(GTK_OBJECT(label)));
+	sched.type = reinterpret_cast<sintptr>(g_object_get_data(G_OBJECT(label), "user_data"));
 	if (sched.type < 0 || sched.type > 31)
 		return false;
 	// Get location.
 	char *locname = g_strdup_printf("sched_loc%d", time);
-	GtkBox *box = GTK_BOX(glade_xml_get_widget(app_xml, locname));
+	GtkBox *box = GTK_BOX(studio->get_widget(locname));
 	g_free(locname);
-	GList *list = g_list_first(box->children);
-	GtkWidget *spin = static_cast<GtkBoxChild *>(list->data)->widget;
+	GList *children = g_list_first(gtk_container_get_children(GTK_CONTAINER(box)));
+	GList *list = children;
+	GtkWidget *spin = GTK_WIDGET(list->data);
 	sched.tx = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin));
 	list = g_list_next(list);
-	spin = static_cast<GtkBoxChild *>(list->data)->widget;
+	spin = GTK_WIDGET(list->data);
 	sched.ty = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin));
 	list = g_list_next(list);
-	spin = static_cast<GtkBoxChild *>(list->data)->widget;
+	spin = GTK_WIDGET(list->data);
 	sched.tz = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin));
 	sched.time = time;
+	g_list_free(children);
 	return true;
 }
 
@@ -287,58 +216,42 @@ static bool Get_schedule_line(
  *  Open the npc-editing window.
  */
 
-#ifdef WIN32
-
-static void Drop_dragged_shape(int shape, int frame, int x, int y, void *data) {
-	ignore_unused_variable_warning(x, y);
-	if (shape < 150)
-		return;
-	cout << "Dropped a shape: " << shape << "," << frame << " " << data << endl;
-
-	Npc_shape_dropped(U7_SHAPE_SHAPES, shape, frame, data);
-}
-
-static void Drop_dragged_face(int shape, int frame, int x, int y, void *data) {
-	cout << "Dropped a face: " << shape << "," << frame << " " << data << endl;
-	ignore_unused_variable_warning(x, y);
-	Npc_face_dropped(U7_SHAPE_FACES, shape, frame, data);
-}
-
-#endif
-
 void ExultStudio::open_npc_window(
     unsigned char *data,        // Serialized npc, or null.
     int datalen
 ) {
-#ifdef WIN32
-	bool first_time = false;
-#endif
 	if (!npcwin) {          // First time?
-#ifdef WIN32
-		first_time = true;
-#endif
-		npcwin = glade_xml_get_widget(app_xml, "npc_window");
+		npcwin = get_widget("npc_window");
 
 		if (vgafile && palbuf) {
-			npc_draw = new Shape_draw(vgafile->get_ifile(), palbuf,
-			                          glade_xml_get_widget(app_xml, "npc_draw"));
-			npc_draw->enable_drop(Npc_shape_dropped, this);
+			npc_single = new Shape_single(
+			    get_widget("npc_shape"), nullptr,
+			    [](int shnum)->bool{ return (shnum >= c_first_obj_shape) &&
+			                                (shnum < c_max_shapes); },
+			    get_widget("npc_frame"),
+			    U7_SHAPE_SHAPES,
+			    vgafile->get_ifile(),
+			    palbuf.get(),
+			    get_widget("npc_draw"));
 		}
 		if (facefile && palbuf) {
-			npc_face_draw = new Shape_draw(facefile->get_ifile(),
-			                               palbuf,
-			                               glade_xml_get_widget(app_xml, "npc_face_draw"));
-			npc_face_draw->enable_drop(Npc_face_dropped, this);
+			npc_face_single = new Shape_single(
+			    get_widget("npc_face_frame"), nullptr,
+			    [](int shnum)->bool{ return (shnum >= 0) && (shnum < 1024); },
+			    nullptr,
+			    U7_SHAPE_FACES,
+			    facefile->get_ifile(),
+			    palbuf.get(),
+			    get_widget("npc_face_draw"));
 		}
 		npc_ctx = gtk_statusbar_get_context_id(
-		              GTK_STATUSBAR(glade_xml_get_widget(
-		                                app_xml, "npc_status")), "Npc Editor");
+		              GTK_STATUSBAR(get_widget("npc_status")), "Npc Editor");
 		for (int i = 0; i < 24 / 3; i++) // Init. schedules' user_data.
-			Set_schedule_line(app_xml, i, -1, 0, 0, 0);
+			Set_schedule_line(this, i, -1, 0, 0, 0);
 
 	}
 	// Init. npc address to null.
-	gtk_object_set_user_data(GTK_OBJECT(npcwin), 0);
+	g_object_set_data(G_OBJECT(npcwin), "user_data", nullptr);
 	// Make 'apply', 'cancel' sensitive.
 	set_sensitive("npc_apply_btn", true);
 	set_sensitive("npc_cancel_btn", true);
@@ -349,11 +262,6 @@ void ExultStudio::open_npc_window(
 	} else
 		init_new_npc();
 	gtk_widget_show(npcwin);
-#ifdef WIN32
-	if (first_time || !npcdnd)
-		Windnd::CreateStudioDropDest(npcdnd, npchwnd, Drop_dragged_shape, NULL, Drop_dragged_face, (void *) this);
-
-#endif
 }
 
 /*
@@ -364,9 +272,6 @@ void ExultStudio::close_npc_window(
 ) {
 	if (npcwin) {
 		gtk_widget_hide(npcwin);
-#ifdef WIN32
-		Windnd::DestroyStudioDropDest(npcdnd, npchwnd);
-#endif
 	}
 }
 
@@ -382,16 +287,15 @@ static bool Get_prop_spin(
     GtkSpinButton  *&spin,      // Spin button returned.
     int &pnum           // Property number (0-11) returned.
 ) {
-	GtkTableChild *ent = static_cast<GtkTableChild *>(list->data);
-	GtkBin *frame = GTK_BIN(ent->widget);
-	spin = GTK_SPIN_BUTTON(frame->child);
-	assert(spin != 0);
-	const char *name = glade_get_widget_name(GTK_WIDGET(spin));
+	GtkBin *frame = GTK_BIN(list->data);
+	spin = GTK_SPIN_BUTTON(gtk_bin_get_child(frame));
+	assert(spin != nullptr);
+	const char *name = gtk_buildable_get_name(GTK_BUILDABLE(spin));
 	// Names: npc_prop_nn.
 	if (strncmp(name, "npc_prop_", 9) != 0)
 		return false;
 	pnum = atoi(name + 9);
-	return (pnum >= 0 && pnum < 12);
+	return pnum >= 0 && pnum < 12;
 }
 
 /*
@@ -410,10 +314,9 @@ static bool Get_flag_cbox(
     unsigned long  *&bits,      // ->one of 3 flags above.
     int &fnum           // Flag # (0-31) returned.
 ) {
-	GtkTableChild *ent = static_cast<GtkTableChild *>(list->data);
-	cbox = GTK_CHECK_BUTTON(ent->widget);
-	assert(cbox != 0);
-	const char *name = glade_get_widget_name(GTK_WIDGET(cbox));
+	cbox = GTK_CHECK_BUTTON(list->data);
+	assert(cbox != nullptr);
+	const char *name = gtk_buildable_get_name(GTK_BUILDABLE(cbox));
 	// Names: npc_flag_xx_nn, where
 	//   xx = si, of, tf.
 	if (strncmp(name, "npc_flag_", 9) != 0)
@@ -428,7 +331,7 @@ static bool Get_flag_cbox(
 	else
 		return false;
 	fnum = atoi(name + 9 + 3);
-	return (fnum >= 0 && fnum < 32);
+	return fnum >= 0 && fnum < 32;
 }
 
 /*
@@ -446,15 +349,22 @@ void ExultStudio::init_new_npc(
 	Exult_server::wait_for_response(server_socket, 100);
 	Exult_server::Receive_data(server_socket, id, data, sizeof(data));
 	const unsigned char *ptr = &data[0];
-	Read2(ptr);	// Snip number of NPCs
+	Read2(ptr); // Snip number of NPCs
 	int first_unused = Read2(ptr);
 	npc_num = first_unused;
 	set_entry("npc_num_entry", npc_num, true, false);
 	// Usually, usecode = 0x400 + num.
 	set_entry("npc_usecode_entry", 0x400 + npc_num, true,
-	          npc_num >= 256 ? true : false);
+	          npc_num >= 256);
 	// Usually, face = npc_num.
-	set_npc_face(npc_num, 0);
+	{
+		GtkWidget *widget = get_widget("npc_face_frame");
+		g_object_set_data(G_OBJECT(widget), "user_data",
+		    reinterpret_cast<gpointer>(uintptr(npc_num)));
+		char *label = g_strdup_printf("Face #%d", npc_num);
+		gtk_frame_set_label(GTK_FRAME(widget), label);
+		g_free(label);
+	}
 	set_entry("npc_name_entry", "");
 	set_entry("npc_ident_entry", 0);
 	set_entry("npc_shape", -1);
@@ -462,30 +372,29 @@ void ExultStudio::init_new_npc(
 	set_optmenu("npc_attack_mode", 0);
 	set_optmenu("npc_alignment", 0);
 	// Clear flag buttons.
-	GtkTable *ftable = GTK_TABLE(
-	                       glade_xml_get_widget(app_xml, "npc_flags_table"));
+	GtkContainer *ftable = GTK_CONTAINER(get_widget("npc_flags_table"));
 	// Set flag checkboxes.
-	for (GList *list = g_list_first(ftable->children); list;
-	        list = g_list_next(list)) {
-		GtkTableChild *ent = static_cast<GtkTableChild *>(list->data);
-		GtkCheckButton *cbox = GTK_CHECK_BUTTON(ent->widget);
+	GList *children = g_list_first(gtk_container_get_children(ftable));
+	for (GList *list = children; list; list = g_list_next(list)) {
+		GtkCheckButton *cbox = GTK_CHECK_BUTTON(list->data);
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(cbox), false);
 	}
+	g_list_free(children);
 	// Make sure the "default" NPC can walk around.
 	set_toggle("npc_flag_tf_05", true);
 	// Set properties.
-	GtkTable *ptable = GTK_TABLE(
-	                       glade_xml_get_widget(app_xml, "npc_props_table"));
-	for (GList *list = g_list_first(ptable->children); list;
-	        list = g_list_next(list)) {
+	GtkContainer *ptable = GTK_CONTAINER(get_widget("npc_props_table"));
+	children = g_list_first(gtk_container_get_children(ptable));
+	for (GList *list = children; list; list = g_list_next(list)) {
 		GtkSpinButton *spin;
 		int pnum;
 		if (Get_prop_spin(list, spin, pnum))
 			gtk_spin_button_set_value(spin, 12);
 	}
+	g_list_free(children);
 	// Clear schedules.
 	for (int i = 0; i < 24 / 3; i++)
-		Set_schedule_line(app_xml, i, -1, 0, 0, 0);
+		Set_schedule_line(this, i, -1, 0, 0, 0);
 }
 
 /*
@@ -499,14 +408,20 @@ int ExultStudio::init_npc_window(
     int datalen
 ) {
 	Actor *addr;
-	int tx, ty, tz;
-	int shape, frame, face;
+	int tx;
+	int ty;
+	int tz;
+	int shape;
+	int frame;
+	int face;
 	std::string name;
-	short npc_num, ident;
+	short npc_num;
+	short ident;
 	int usecode;
 	std::string usecodefun;
 	int properties[12];
-	short attack_mode, alignment;
+	short attack_mode;
+	short alignment;
 	unsigned long oflags;       // Object flags.
 	unsigned long xflags;       // Extra object flags.
 	unsigned long type_flags;   // Movement flags.
@@ -520,7 +435,7 @@ int ExultStudio::init_npc_window(
 		return 0;
 	}
 	// Store address with window.
-	gtk_object_set_user_data(GTK_OBJECT(npcwin), addr);
+	g_object_set_data(G_OBJECT(npcwin), "user_data", addr);
 	// Store name, ident, num.
 	utf8Str utf8name(name.c_str());
 	set_entry("npc_name_entry", utf8name);
@@ -528,8 +443,16 @@ int ExultStudio::init_npc_window(
 	set_entry("npc_num_entry", npc_num, true, false);
 	set_entry("npc_ident_entry", ident);
 	// Shape/frame.
-	set_npc_shape(shape, frame);
-	set_npc_face(face, 0);
+	set_entry("npc_shape", shape);
+	set_entry("npc_frame", 16);
+	{
+		GtkWidget *widget = get_widget("npc_face_frame");
+		g_object_set_data(G_OBJECT(widget), "user_data",
+		    reinterpret_cast<gpointer>(uintptr(face)));
+		char *label = g_strdup_printf("Face #%d", face);
+		gtk_frame_set_label(GTK_FRAME(widget), label);
+		g_free(label);
+	}
 	// Usecode #.
 	if (npc_num >= 256 && !usecodefun.empty())
 		set_entry("npc_usecode_entry", usecodefun.c_str(), true);
@@ -539,11 +462,10 @@ int ExultStudio::init_npc_window(
 	set_optmenu("npc_attack_mode", attack_mode);
 	set_optmenu("npc_alignment", alignment);
 	// Set flag buttons.
-	GtkTable *ftable = GTK_TABLE(
-	                       glade_xml_get_widget(app_xml, "npc_flags_table"));
+	GtkContainer *ftable = GTK_CONTAINER(get_widget("npc_flags_table"));
 	// Set flag checkboxes.
-	for (GList *list = g_list_first(ftable->children); list;
-	        list = g_list_next(list)) {
+	GList *children = g_list_first(gtk_container_get_children(ftable));
+	for (GList *list = children; list; list = g_list_next(list)) {
 		GtkCheckButton *cbox;
 		unsigned long *bits;
 		int fnum;
@@ -552,25 +474,26 @@ int ExultStudio::init_npc_window(
 			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(cbox),
 			                             (*bits & (1 << fnum)) != 0);
 	}
+	g_list_free(children);
 	// Set properties.
-	GtkTable *ptable = GTK_TABLE(
-	                       glade_xml_get_widget(app_xml, "npc_props_table"));
-	for (GList *list = g_list_first(ptable->children); list;
-	        list = g_list_next(list)) {
+	GtkContainer *ptable = GTK_CONTAINER(get_widget("npc_props_table"));
+	children = g_list_first(gtk_container_get_children(ptable));
+	for (GList *list = children; list; list = g_list_next(list)) {
 		GtkSpinButton *spin;
 		int pnum;
 		if (Get_prop_spin(list, spin, pnum))
 			gtk_spin_button_set_value(spin, properties[pnum]);
 	}
+	g_list_free(children);
 	// Set schedules.
 	for (int i = 0; i < 24 / 3; i++) // First init. to empty.
-		Set_schedule_line(app_xml, i, -1, 0, 0, 0);
+		Set_schedule_line(this, i, -1, 0, 0, 0);
 	for (int i = 0; i < num_schedules; i++) {
 		Serial_schedule &sched = schedules[i];
 		int time = sched.time;  // 0-7.
 		if (time < 0 || time > 7)
 			continue;   // Invalid.
-		Set_schedule_line(app_xml, time, sched.type, sched.tx, sched.ty,
+		Set_schedule_line(this, time, sched.type, sched.tx, sched.ty,
 		                  sched.tz);
 	}
 	return 1;
@@ -583,7 +506,7 @@ int ExultStudio::init_npc_window(
  * close. You can't call it in Npc_response because browser is private.
  */
 void ExultStudio::update_npc() { // update npc browser display (if open)
-	Npc_chooser *npcchoose = dynamic_cast<Npc_chooser *>(browser);
+	auto *npcchoose = dynamic_cast<Npc_chooser *>(browser);
 	if (npcchoose) {
 		short npc_num = get_num_entry("npc_num_entry");
 		npcchoose->update_npc(npc_num);
@@ -619,16 +542,18 @@ int ExultStudio::save_npc_window(
 ) {
 	cout << "In save_npc_window()" << endl;
 	// Get npc (null if creating new).
-	Actor *addr = static_cast<Actor*>(gtk_object_get_user_data(GTK_OBJECT(npcwin)));
-	int tx = -1, ty = -1, tz = -1;  // +++++For now.
+	auto *addr = static_cast<Actor *>(g_object_get_data(G_OBJECT(npcwin), "user_data"));
+	int tx = -1;
+	int ty = -1;
+	int tz = -1;  // +++++For now.
 	codepageStr locname(get_text_entry("npc_name_entry"));
 	std::string name(locname);
 	short npc_num = get_num_entry("npc_num_entry");
 	short ident = get_num_entry("npc_ident_entry");
 	int shape = get_num_entry("npc_shape");
 	int frame = get_num_entry("npc_frame");
-	GtkWidget *fw = glade_xml_get_widget(app_xml, "npc_face_frame");
-	int face = reinterpret_cast<sintptr>(gtk_object_get_user_data(GTK_OBJECT(fw)));
+	GtkWidget *fw = get_widget("npc_face_frame");
+	int face = reinterpret_cast<sintptr>(g_object_get_data(G_OBJECT(fw), "user_data"));
 	int usecode = get_num_entry("npc_usecode_entry");
 	std::string usecodefun;
 	if (!usecode)
@@ -645,11 +570,10 @@ int ExultStudio::save_npc_window(
 		return 0;
 	}
 	// Set flag buttons.
-	GtkTable *ftable = GTK_TABLE(
-	                       glade_xml_get_widget(app_xml, "npc_flags_table"));
+	GtkContainer *ftable = GTK_CONTAINER(get_widget("npc_flags_table"));
 	// Get flags.
-	for (GList *list = g_list_first(ftable->children); list;
-	        list = g_list_next(list)) {
+	GList *children = g_list_first(gtk_container_get_children(ftable));
+	for (GList *list = children; list; list = g_list_next(list)) {
 		GtkCheckButton *cbox;
 		unsigned long *bits;
 		int fnum;
@@ -659,21 +583,22 @@ int ExultStudio::save_npc_window(
 			            GTK_TOGGLE_BUTTON(cbox)))
 				*bits |= (1 << fnum);
 	}
+	g_list_free(children);
 	int properties[12];     // Get properties.
-	GtkTable *ptable = GTK_TABLE(
-	                       glade_xml_get_widget(app_xml, "npc_props_table"));
-	for (GList *list = g_list_first(ptable->children); list;
-	        list = g_list_next(list)) {
+	GtkContainer *ptable = GTK_CONTAINER(get_widget("npc_props_table"));
+	children = g_list_first(gtk_container_get_children(ptable));
+	for (GList *list = children; list; list = g_list_next(list)) {
 		GtkSpinButton *spin;
 		int pnum;
 		if (Get_prop_spin(list, spin, pnum))
 			properties[pnum] =
 			    gtk_spin_button_get_value_as_int(spin);
 	}
+	g_list_free(children);
 	short num_schedules = 0;
 	Serial_schedule schedules[8];
 	for (int i = 0; i < 8; i++)
-		if (Get_schedule_line(app_xml, i, schedules[num_schedules]))
+		if (Get_schedule_line(this, i, schedules[num_schedules]))
 			num_schedules++;
 	if (Npc_actor_out(server_socket, addr, tx, ty, tz, shape, frame, face,
 	                  name, npc_num, ident, usecode, usecodefun,
@@ -694,88 +619,13 @@ int ExultStudio::save_npc_window(
 		return 1;       // Leave window open.
 	}
 	// Update NPC browser information.
-	Npc_chooser *npcchoose = dynamic_cast<Npc_chooser *>(browser);
+	auto *npcchoose = dynamic_cast<Npc_chooser *>(browser);
 	if (npcchoose)
 		npcchoose->update_npc(npc_num);
 	close_npc_window();
 	return 1;
 }
 
-/*
- *  Paint the shape in the NPC draw area.
- */
-
-void ExultStudio::show_npc_shape(
-    int x, int y, int w, int h  // Rectangle. w=-1 to show all.
-) {
-	if (!npc_draw)
-		return;
-	npc_draw->configure();
-	// Yes, this is kind of redundant...
-	int shnum = get_num_entry("npc_shape");
-	int frnum = get_num_entry("npc_frame");
-	if (!shnum)         // Don't draw shape 0.
-		shnum = -1;
-	npc_draw->draw_shape_centered(shnum, frnum);
-	if (w != -1)
-		npc_draw->show(x, y, w, h);
-	else
-		npc_draw->show();
-}
-
-/*
- *  Set NPC shape.
- */
-
-void ExultStudio::set_npc_shape(
-    int shape,
-    int frame
-) {
-	set_entry("npc_shape", shape);
-	set_entry("npc_frame", frame);
-	show_npc_shape();
-}
-
-/*
- *  Paint the face.
- */
-
-void ExultStudio::show_npc_face(
-    int x, int y, int w, int h  // Rectangle. w=-1 to show all.
-) {
-	if (!npc_face_draw)
-		return;
-	npc_face_draw->configure();
-	GtkWidget *frame = glade_xml_get_widget(app_xml, "npc_face_frame");
-	int shnum = reinterpret_cast<sintptr>(gtk_object_get_user_data(GTK_OBJECT(frame)));
-	npc_face_draw->draw_shape_centered(shnum, 0);
-	if (w != -1)
-		npc_face_draw->show(x, y, w, h);
-	else
-		npc_face_draw->show();
-}
-
-/*
- *  Set NPC face.
- */
-
-void ExultStudio::set_npc_face(
-    int shape,
-    int frame
-) {
-	ignore_unused_variable_warning(frame);
-//	set_entry("npc_shape", shape);
-//	set_entry("npc_frame", frame);
-	if (shape < 0)
-		shape = 1;      // Default to 1st after Avatar.
-	GtkWidget *widget = glade_xml_get_widget(app_xml, "npc_face_frame");
-	gtk_object_set_user_data(GTK_OBJECT(widget), reinterpret_cast<gpointer>(uintptr(shape)));
-	char *label = g_strdup_printf("Face #%d", shape);
-	gtk_frame_set_label(GTK_FRAME(widget), label);
-	g_free(label);
-	show_npc_face();
-}
-
 /*
  *  A choice was clicked in the 'schedule' dialog.
  */
@@ -786,19 +636,18 @@ void ExultStudio::schedule_btn_clicked(
 ) {
 	ExultStudio *studio = ExultStudio::get_instance();
 	// Get name assigned in Glade.
-	const char *name = glade_get_widget_name(btn);
+	const char *name = gtk_buildable_get_name(GTK_BUILDABLE(btn));
 	const char *numptr = name + 5;  // Past "sched".
 	int num = atoi(numptr);
-	GtkWidget *schedwin = static_cast<GtkWidget *>(data);
-	GtkLabel *label = static_cast<GtkLabel *>(gtk_object_get_user_data(
-	                      GTK_OBJECT(schedwin)));
+	auto *schedwin = static_cast<GtkWidget *>(data);
+	auto *label = static_cast<GtkLabel *>(g_object_get_data(
+	        G_OBJECT(schedwin), "user_data"));
 	// User data = schedule #.
-	gtk_object_set_user_data(GTK_OBJECT(label), reinterpret_cast<gpointer>(uintptr(num)));
+	g_object_set_data(G_OBJECT(label), "user_data", reinterpret_cast<gpointer>(uintptr(num)));
 	gtk_label_set_text(label, num >= 0 && num < 32
 	                   ? sched_names[num] : "-----");
 	cout << "Chose schedule " << num << endl;
-	gtk_widget_hide(glade_xml_get_widget(studio->get_xml(),
-	                                     "schedule_dialog"));
+	gtk_widget_hide(studio->get_widget("schedule_dialog"));
 }
 
 /*
@@ -809,8 +658,8 @@ static void Set_sched_btn(
     GtkWidget *btn,
     gpointer data
 ) {
-	gtk_signal_connect(GTK_OBJECT(btn), "clicked",
-	                   GTK_SIGNAL_FUNC(ExultStudio::schedule_btn_clicked), data);
+	g_signal_connect(G_OBJECT(btn), "clicked",
+	                 G_CALLBACK(ExultStudio::schedule_btn_clicked), data);
 }
 
 /*
@@ -822,16 +671,16 @@ C_EXPORT void on_npc_set_sched(
 ) {
 	ignore_unused_variable_warning(user_data);
 	static int first = 1;       // To initialize signal handlers.
-	const char *name = glade_get_widget_name(btn);
+	const char *name = gtk_buildable_get_name(GTK_BUILDABLE(btn));
 	const char *numptr = name + strlen(name) - 1;
-	GladeXML *xml = ExultStudio::get_instance()->get_xml();
 	char lname[20];         // Set up label name.
 	strcpy(lname, "npc_sched");
 	strcat(lname, numptr);      // Same number as button.
-	GtkLabel *label = GTK_LABEL(glade_xml_get_widget(xml, lname));
+	ExultStudio *studio = ExultStudio::get_instance();
+	GtkLabel *label = GTK_LABEL(studio->get_widget(lname));
 	GtkContainer *btns = GTK_CONTAINER(
-	                         glade_xml_get_widget(xml, "sched_btns"));
-	GtkWidget *schedwin = glade_xml_get_widget(xml, "schedule_dialog");
+	                         studio->get_widget("sched_btns"));
+	GtkWidget *schedwin = studio->get_widget("schedule_dialog");
 	if (!label || !btns || !schedwin)
 		return;
 	if (first) {        // First time?  Set handlers.
@@ -839,7 +688,7 @@ C_EXPORT void on_npc_set_sched(
 		gtk_container_foreach(btns, Set_sched_btn, schedwin);
 	}
 	// Store label as dialog's data.
-	gtk_object_set_user_data(GTK_OBJECT(schedwin), label);
+	g_object_set_data(G_OBJECT(schedwin), "user_data", label);
 	gtk_widget_show(schedwin);
 }
 
@@ -856,19 +705,21 @@ static void Game_loc_response(
 	if (id != Exult_server::game_pos)
 		return;
 	// Get box with loc. spin btns.
-	GtkBox *box = static_cast<GtkBox *>(client);
+	auto *box = static_cast<GtkBox *>(client);
 	int tx = Read2(data);
 	int ty = Read2(data);
 	int tz = Read2(data);
-	GList *list = g_list_first(box->children);
-	GtkWidget *spin = static_cast<GtkBoxChild *>(list->data)->widget;
+	GList *children = g_list_first(gtk_container_get_children(GTK_CONTAINER(box)));
+	GList *list = children;
+	GtkWidget *spin = GTK_WIDGET(list->data);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin), tx);
 	list = g_list_next(list);
-	spin = static_cast<GtkBoxChild *>(list->data)->widget;
+	spin = GTK_WIDGET(list->data);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin), ty);
 	list = g_list_next(list);
-	spin = static_cast<GtkBoxChild *>(list->data)->widget;
+	spin = GTK_WIDGET(list->data);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin), tz);
+	g_list_free(children);
 }
 
 /*
