@@ -5,7 +5,7 @@
  **/
 
 /*
-Copyright (C) 2005-2013  The Exult Team
+Copyright (C) 2005-2022 The Exult Team
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -23,52 +23,34 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
 #ifdef HAVE_CONFIG_H
-#  include <config.h>
+#	include <config.h>
 #endif
 
-#ifdef WIN32
-#include "Windrag.h"
-#include <windows.h>
-#endif
+#include "npclst.h"
 
-#ifdef __GNUC__
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wold-style-cast"
-#pragma GCC diagnostic ignored "-Wcast-qual"
-#endif  // __GNUC__
-#include <gtk/gtk.h>
-#include <gdk/gdkkeysyms.h>
-#ifdef XWIN
-#include <gdk/gdkx.h>
-#endif
-#ifdef __GNUC__
-#pragma GCC diagnostic pop
-#endif  // __GNUC__
-#include "gtk_redefines.h"
+#include "fontgen.h"
+#include "ibuf8.h"
+#include "pngio.h"
+#include "shapefile.h"
+#include "shapegroup.h"
+#include "shapevga.h"
+#include "u7drag.h"
+#include "utils.h"
 
 #include <glib.h>
-#include <stdlib.h>
 #include <sys/stat.h>
 #include <unistd.h>
-#include "npclst.h"
-#include "shapevga.h"
-#include "ibuf8.h"
-#include "u7drag.h"
-#include "studio.h"
-#include "utils.h"
-#include "shapegroup.h"
-#include "shapefile.h"
-#include "pngio.h"
-#include "fontgen.h"
-#include "ignore_unused_variable_warning.h"
 
+#include <cmath>
+#include <cstdlib>
+
+using EStudio::Add_menu_item;
+using EStudio::Alert;
 using std::cout;
 using std::cerr;
 using std::endl;
 using std::string;
 using std::vector;
-using EStudio::Alert;
-using EStudio::Add_menu_item;
 
 /*
  *  Blit onto screen.
@@ -77,12 +59,17 @@ using EStudio::Add_menu_item;
 void Npc_chooser::show(
     int x, int y, int w, int h  // Area to blit.
 ) {
-	Shape_draw::show(draw->window, x, y, w, h);
-	if (selected >= 0) {    // Show selected.
-		Rectangle b = info[selected].box;
+	Shape_draw::show(x, y, w, h);
+	if ((selected >= 0) && (drawgc != nullptr)) {    // Show selected.
+		TileRect b = info[selected].box;
 		// Draw yellow box.
-		gdk_draw_rectangle(draw->window, drawgc, FALSE,
-		                   b.x, b.y - voffset, b.w, b.h);
+		cairo_set_line_width(drawgc, 1.0);
+		cairo_set_source_rgb(drawgc,
+		                     ((drawfg >> 16) & 255) / 255.0,
+		                     ((drawfg >> 8) & 255) / 255.0,
+		                     (drawfg & 255) / 255.0);
+		cairo_rectangle(drawgc, b.x, b.y - voffset, b.w, b.h);
+		cairo_stroke(drawgc);
 	}
 }
 
@@ -107,7 +94,9 @@ void Npc_chooser::render(
 ) {
 	vector<Estudio_npc> &npcs = get_npcs();
 	// Get drawing area dimensions.
-	gint winh = draw->allocation.height;
+	GtkAllocation alloc = {0, 0, 0, 0};
+	gtk_widget_get_allocation(draw, &alloc);
+	gint winh = alloc.height;
 	// Clear window first.
 	iwin->fill8(255);       // Set to background_color.
 	int curr_y = -row0_voffset;
@@ -120,9 +109,11 @@ void Npc_chooser::render(
 		for (unsigned index = row.index0; cols; --cols, ++index) {
 			int npcnum = info[index].npcnum;
 			int shapenum = npcs[npcnum].shapenum;
-			Shape_frame *shape = ifile->get_shape(shapenum, 0);
+			int framenum = info[index].framenum;
+			Shape_frame *shape = ifile->get_shape(shapenum,
+			                                      framenum);
 			if (shape) {
-				int sx = info[index].box.x;
+				int sx = info[index].box.x - hoffset;
 				int sy = info[index].box.y - voffset;
 				shape->paint(iwin, sx + shape->get_xleft(),
 				             sy + shape->get_yabove());
@@ -135,6 +126,53 @@ void Npc_chooser::render(
 		}
 		curr_y += rows[rownum].height;
 	}
+	gtk_widget_queue_draw(draw);
+}
+
+/*
+ *  Get maximum shape height for all its frames.
+ */
+
+static int Get_max_height(
+    Shape *shape
+) {
+	int cnt = shape->get_num_frames();
+	int maxh = 0;
+	for (int i = 0; i < cnt; i++) {
+		Shape_frame *frame = shape->get_frame(i);
+		if (!frame) {
+			continue;
+		}
+		int ht = frame->get_height();
+		if (ht > maxh)
+			maxh = ht;
+	}
+	return maxh;
+}
+
+/*
+ *  Get the x-offset in pixels where a frame will be drawn.
+ *
+ *  Output: Offset from left edge of (virtual) drawing area.
+ */
+
+static int Get_x_offset(
+    Shape *shape,
+    int framenum
+) {
+	if (!shape)
+		return 0;
+	int nframes = shape->get_num_frames();
+	if (framenum >= nframes)
+		framenum = nframes - 1;
+	int xoff = 0;
+	for (int i = 0; i < framenum; i++) {
+		Shape_frame *fr = shape->get_frame(i);
+		if (fr) {
+			xoff += fr->get_width() + border;
+		}
+	}
+	return xoff;
 }
 
 /*
@@ -154,14 +192,17 @@ void Npc_chooser::setup_info(
 	index0 = 0;
 	voffset = 0;
 	total_height = 0;
-	setup_shapes_info();
+	if (frames_mode)
+		setup_frames_info();
+	else
+		setup_shapes_info();
 	setup_vscrollbar();
 	if (savepos)
 		goto_index(oldind);
 }
 
 /*
- *  Setup info.
+ *  Setup info when not in 'frames' mode.
  */
 
 void Npc_chooser::setup_shapes_info(
@@ -169,12 +210,21 @@ void Npc_chooser::setup_shapes_info(
 	vector<Estudio_npc> &npcs = get_npcs();
 	if (npcs.empty())       // No NPC's?  Try to get them.
 		static_cast<Npcs_file_info *>(file_info)->setup();
+	int selnpc   = -1;
+	int selframe = -1;
+	if (selected >= 0) {    // Save selection info.
+		selnpc   = info[selected].npcnum;
+		selframe = info[selected].framenum;
+	}
 	// Get drawing area dimensions.
-	gint winw = draw->allocation.width;
+	GtkAllocation alloc = {0, 0, 0, 0};
+	gtk_widget_get_allocation(draw, &alloc);
+	gint winw = alloc.width;
 	int x = 0;
 	int curr_y = 0;
 	int row_h = 0;
-	int total_cnt = get_count(), num_shapes = ifile->get_num_shapes();
+	int total_cnt = get_count();
+	int num_shapes = ifile->get_num_shapes();
 	//   filter (group).
 	rows.resize(1);         // Start 1st row.
 	rows[0].index0 = 0;
@@ -184,13 +234,14 @@ void Npc_chooser::setup_shapes_info(
 		if (npcnum >= 356 && npcnum <= 359)
 			continue;
 		int shapenum = npcs[npcnum].shapenum;
+		int framenum = npcnum == selnpc ? selframe : framenum0;
 		if (shapenum < 0 || shapenum >= num_shapes)
 			continue;
-		Shape_frame *shape = ifile->get_shape(shapenum, 0);
+		Shape_frame *shape = ifile->get_shape(shapenum, framenum);
 		if (!shape)
 			continue;
-		int sh = shape->get_height(),
-		    sw = shape->get_width();
+		int sh = shape->get_height();
+		int sw = shape->get_width();
 		// Check if we've exceeded max width
 		if (x + sw > winw && x) {   // But don't leave row empty.
 			// Next line.
@@ -198,7 +249,7 @@ void Npc_chooser::setup_shapes_info(
 			curr_y += row_h + border;
 			row_h = 0;
 			x = 0;
-			rows.push_back(Npc_row());
+			rows.emplace_back();
 			rows.back().index0 = info.size();
 			rows.back().y = curr_y;
 		}
@@ -206,14 +257,104 @@ void Npc_chooser::setup_shapes_info(
 			row_h = sh;
 		int sy = curr_y + border; // Get top y-coord.
 		// Store info. about where drawn.
-		info.push_back(Npc_entry());
-		info.back().set(npcnum, x, sy, sw, sh);
+		info.emplace_back();
+		info.back().set(npcnum, framenum, x, sy, sw, sh);
 		x += sw + border;
 	}
 	rows.back().height = row_h + border;
 	total_height = curr_y + rows.back().height + border;
 }
 
+/*
+ *  Setup one NPC per row, showing its frames from left to right.
+ */
+
+void Npc_chooser::setup_frames_info(
+) {
+	vector<Estudio_npc> &npcs = get_npcs();
+	if (npcs.empty())       // No NPC's?  Try to get them.
+		static_cast<Npcs_file_info *>(file_info)->setup();
+	// Get drawing area dimensions.
+	int curr_y = 0;
+	int maxw = 0;
+	unsigned total_cnt = get_count();
+	int num_shapes = ifile->get_num_shapes();
+	//   filter (group).
+	for (unsigned index = index0; index < total_cnt; index++) {
+		int npcnum = group ? (*group)[index] : index;
+		if (npcnum >= 356 && npcnum <= 359)
+			continue;
+		int shapenum = npcs[npcnum].shapenum;
+		if (shapenum < 0 || shapenum >= num_shapes)
+			continue;
+		// Get all frames.
+		Shape *shape = ifile->extract_shape(shapenum);
+		int nframes = shape ? shape->get_num_frames() : 0;
+		if (!nframes)
+			continue;
+		int row_h = Get_max_height(shape);
+		rows.emplace_back();
+		rows.back().index0 = info.size();
+		rows.back().y = curr_y;
+		rows.back().height = row_h + border;
+		int x = 0;
+		int sw;
+		int sh;
+		for (int framenum = 0; framenum < nframes; framenum++,
+		        x += sw + border) {
+			Shape_frame *frame = shape->get_frame(framenum);
+			if (!frame) {
+				sw = 0;
+				continue;
+			}
+			sh = frame->get_height();
+			sw = frame->get_width();
+			int sy = curr_y + border; // Get top y-coord.
+			// Store info. about where drawn.
+			info.emplace_back();
+			info.back().set(npcnum, framenum, x, sy, sw, sh);
+		}
+		if (x > maxw)
+			maxw = x;
+		// Next line.
+		curr_y += row_h + border;
+	}
+	total_height = curr_y + border;
+	setup_hscrollbar(maxw);
+}
+
+/*
+ *  Horizontally scroll so that the selected frame is visible (in frames
+ *  mode).
+ */
+
+void Npc_chooser::scroll_to_frame(
+) {
+	if (selected >= 0) {    // Save selection info.
+		vector<Estudio_npc> &npcs = get_npcs();
+		int selnpc   = info[selected].npcnum;
+		int selframe = info[selected].framenum;
+		Shape *shape = ifile->extract_shape(npcs[selnpc].shapenum);
+		int xoff = Get_x_offset(shape, selframe);
+		if (xoff < hoffset) // Left of visual area?
+			hoffset = xoff > border ? xoff - border : 0;
+		else {
+			GtkAllocation alloc = {0, 0, 0, 0};
+			gtk_widget_get_allocation(draw, &alloc);
+			gint winw = alloc.width;
+			Shape_frame *fr = shape->get_frame(selframe);
+			if (fr) {
+				int sw = fr->get_width();
+				if (xoff + sw + border - hoffset > winw)
+					hoffset = xoff + sw + border - winw;
+			}
+		}
+		GtkAdjustment *adj = gtk_range_get_adjustment(
+		                         GTK_RANGE(hscroll));
+		gtk_adjustment_set_value(adj, hoffset);
+	}
+}
+
 /*
  *  Scroll so a desired index is in view.
  */
@@ -226,10 +367,11 @@ void Npc_chooser::goto_index(
 	Npc_entry &inf = info[index];   // Already in view?
 	int midx = inf.box.x + inf.box.w / 2;
 	int midy = inf.box.y + inf.box.h / 2;
-	Rectangle winrect(0, voffset, config_width, config_height);
+	TileRect winrect(0, voffset, config_width, config_height);
 	if (winrect.has_point(midx, midy))
 		return;
-	unsigned start = 0, count = rows.size();
+	unsigned start = 0;
+	unsigned count = rows.size();
 	while (count > 1) {     // Binary search.
 		unsigned mid = start + count / 2;
 		if (index < rows[mid].index0)
@@ -261,7 +403,8 @@ int Npc_chooser::find_npc(
 				return i;
 		return -1;
 	}
-	unsigned start = 0, count = info.size();
+	unsigned start = 0;
+	unsigned count = info.size();
 	while (count > 1) {     // Binary search.
 		unsigned mid = start + count / 2;
 		if (npcnum < info[mid].npcnum)
@@ -287,7 +430,7 @@ static gint Configure_chooser(
     gpointer data           // ->Npc_chooser
 ) {
 	ignore_unused_variable_warning(widget);
-	Npc_chooser *chooser = static_cast<Npc_chooser *>(data);
+	auto *chooser = static_cast<Npc_chooser *>(data);
 	return chooser->configure(event);
 }
 gint Npc_chooser::configure(
@@ -305,7 +448,7 @@ gint Npc_chooser::configure(
 		render();       // Same size?  Just render it.
 	if (group)          // Filtering?
 		enable_drop();      // Can drop NPCs here.
-	return (TRUE);
+	return TRUE;
 }
 
 /*
@@ -314,72 +457,35 @@ gint Npc_chooser::configure(
 
 gint Npc_chooser::expose(
     GtkWidget *widget,      // The view window.
-    GdkEventExpose *event,
+    cairo_t *cairo,
     gpointer data           // ->Npc_chooser.
 ) {
 	ignore_unused_variable_warning(widget);
-	Npc_chooser *chooser = static_cast<Npc_chooser *>(data);
-	chooser->show(event->area.x, event->area.y, event->area.width,
-	              event->area.height);
-	return (TRUE);
+	auto *chooser = static_cast<Npc_chooser *>(data);
+	chooser->set_graphic_context(cairo);
+	GdkRectangle area = { 0, 0, 0, 0 };
+	gdk_cairo_get_clip_rectangle(cairo, &area);
+	chooser->show(area.x, area.y, area.width, area.height);
+	chooser->set_graphic_context(nullptr);
+	return TRUE;
 }
 
 /*
  *  Handle a mouse drag event.
  */
 
-#ifdef WIN32
-
-static bool win32_button = false;
-
-gint Npc_chooser::win32_drag_motion(
-    GtkWidget *widget,      // The view window.
-    GdkEventMotion *event,
-    gpointer data           // ->Npc_chooser.
-) {
-	ignore_unused_variable_warning(widget, event);
-	if (win32_button) {
-		win32_button = false;
-
-		// prepare the dragged data
-		windragdata wdata;
-
-		// This call allows us to recycle the data transfer initialization code.
-		//  It's clumsy, but far easier to maintain.
-		drag_data_get(NULL, NULL, (GtkSelectionData *) &wdata,
-		              U7_TARGET_NPCID, 0, data);
-
-		POINT pnt;
-		GetCursorPos(&pnt);
-
-		Windropsource idsrc(0, pnt.x, pnt.y);
-		LPDATAOBJECT idobj = (LPDATAOBJECT) new Winstudioobj(wdata);
-		DWORD dndout;
-
-		HRESULT res = DoDragDrop(idobj, &idsrc, DROPEFFECT_COPY, &dndout);
-		if (FAILED(res)) {
-			g_warning("Oops! Something is wrong with OLE2 DnD..");
-		}
-
-		idobj->Release();   // Not sure if we really need this. However, it doesn't hurt either.
-	}
-
-	return true;
-}
-#else
 gint Npc_chooser::drag_motion(
     GtkWidget *widget,      // The view window.
     GdkEventMotion *event,
     gpointer data           // ->Npc_chooser.
 ) {
 	ignore_unused_variable_warning(widget);
-	Npc_chooser *chooser = static_cast<Npc_chooser *>(data);
+	auto *chooser = static_cast<Npc_chooser *>(data);
 	if (!chooser->dragging && chooser->selected >= 0)
 		chooser->start_drag(U7_TARGET_NPCID_NAME,
 		                    U7_TARGET_NPCID, reinterpret_cast<GdkEvent *>(event));
 	return true;
 }
-#endif
 
 /*
  *  Handle a mouse button-press event.
@@ -393,27 +499,21 @@ gint Npc_chooser::mouse_press(
 	if (event->button == 4) {
 		if (row0 > 0)
 			scroll_row_vertical(row0 - 1);
-		return(TRUE);
+		return TRUE;
 	} else if (event->button == 5) {
 		scroll_row_vertical(row0 + 1);
-		return(TRUE);
+		return TRUE;
 	}
-	int old_selected = selected, new_selected = -1;
+	int old_selected = selected;
+	int new_selected = -1;
 	unsigned i;              // Search through entries.
 	unsigned infosz = info.size();
-	int absx = static_cast<int>(event->x), absy = static_cast<int>(event->y) + voffset;
+	int absx = static_cast<int>(event->x);
+	int absy = static_cast<int>(event->y) + voffset;
 	for (i = rows[row0].index0; i < infosz; i++) {
 		if (info[i].box.has_point(absx, absy)) {
 			// Found the box?
 			// Indicate we can drag.
-#ifdef WIN32
-// Here, we have to override GTK+'s Drag and Drop, which is non-OLE and
-// usually stucks outside the program window. I think it's because
-// the dragged shape only receives mouse motion events when the new mouse pointer
-// position is *still* inside the shape. So if you move the mouse too fast,
-// we are stuck.
-			win32_button = true;
-#endif
 			new_selected = i;
 			break;
 		} else if (info[i].box.y - voffset >= config_height)
@@ -422,7 +522,6 @@ gint Npc_chooser::mouse_press(
 	if (new_selected >= 0) {
 		select(new_selected);
 		render();
-		show();
 		if (sel_changed)    // Tell client.
 			(*sel_changed)();
 	}
@@ -434,9 +533,9 @@ gint Npc_chooser::mouse_press(
 			edit_npc();
 	}
 	if (event->button == 3)
-		gtk_menu_popup(GTK_MENU(create_popup()),
-		               0, 0, 0, 0, event->button, event->time);
-	return (TRUE);
+		gtk_menu_popup_at_pointer(GTK_MENU(create_popup()),
+		                          reinterpret_cast<GdkEvent *>(event));
+	return TRUE;
 }
 
 /*
@@ -447,7 +546,7 @@ static gint Mouse_press(
     GdkEventButton *event,
     gpointer data           // ->Npc_chooser.
 ) {
-	Npc_chooser *chooser = static_cast<Npc_chooser *>(data);
+	auto *chooser = static_cast<Npc_chooser *>(data);
 	return chooser->mouse_press(widget, event);
 }
 static gint Mouse_release(
@@ -456,7 +555,7 @@ static gint Mouse_release(
     gpointer data           // ->Npc_chooser.
 ) {
 	ignore_unused_variable_warning(widget, event);
-	Npc_chooser *chooser = static_cast<Npc_chooser *>(data);
+	auto *chooser = static_cast<Npc_chooser *>(data);
 	chooser->mouse_up();
 	return true;
 }
@@ -470,16 +569,6 @@ on_npc_draw_key_press(GtkEntry   *entry,
                       gpointer    user_data) {
 	ignore_unused_variable_warning(entry, event, user_data);
 	//Npc_chooser *chooser = static_cast<Npc_chooser *>(user_data);
-#if 0
-	switch (event->keyval) {
-	case GDK_Delete:
-		chooser->del_frame();
-		return TRUE;
-	case GDK_Insert:
-		chooser->new_frame();
-		return TRUE;
-	}
-#endif
 	return FALSE;           // Let parent handle it.
 }
 
@@ -492,7 +581,8 @@ void Npc_chooser::edit_npc(
 	ExultStudio *studio = ExultStudio::get_instance();
 	int npcnum = info[selected].npcnum;
 	//Estudio_npc &npc = get_npcs()[npcnum];
-	unsigned char buf[Exult_server::maxlength], *ptr;
+	unsigned char buf[Exult_server::maxlength];
+	unsigned char *ptr;
 	ptr = &buf[0];
 	Write2(ptr, npcnum);
 	if (!studio->send_to_server(Exult_server::edit_npc, buf, ptr - buf))
@@ -533,40 +623,20 @@ void Npc_chooser::drag_data_get(
     gpointer data           // ->Npc_chooser.
 ) {
 	ignore_unused_variable_warning(widget, context, time);
-	cout << "In DRAG_DATA_GET" << endl;
-	Npc_chooser *chooser = static_cast<Npc_chooser *>(data);
+	cout << "In DRAG_DATA_GET of Npc for '"
+	     << gdk_atom_name(gtk_selection_data_get_target(seldata))
+	     << "'" << endl;
+	auto *chooser = static_cast<Npc_chooser *>(data);
 	if (chooser->selected < 0 || info != U7_TARGET_NPCID)
 		return;         // Not sure about this.
-	guchar buf[30];
+	guchar buf[U7DND_DATA_LENGTH(1)];
 	int npcnum = chooser->info[chooser->selected].npcnum;
 	int len = Store_u7_npcid(buf, npcnum);
 	cout << "Setting selection data (" << npcnum << ')' << endl;
-#ifdef WIN32
-	windragdata *wdata = (windragdata *)seldata;
-	wdata->assign(info, len, buf);
-#else
-	// Make us owner of xdndselection.
-	//gtk_selection_owner_set(widget, gdk_atom_intern("XdndSelection", 0),
-	//                          time);
 	// Set data.
 	gtk_selection_data_set(seldata,
-	                       gdk_atom_intern(U7_TARGET_NPCID_NAME, 0), 8, buf, len);
-#endif
-}
-
-/*
- *  Another app. has claimed the selection.
- */
-
-gint Npc_chooser::selection_clear(
-    GtkWidget *widget,      // The view window.
-    GdkEventSelection *event,
-    gpointer data           // ->Npc_chooser.
-) {
-	ignore_unused_variable_warning(widget, event, data);
-//	Npc_chooser *chooser = static_cast<Npc_chooser *>(data);
-	cout << "SELECTION_CLEAR" << endl;
-	return TRUE;
+	                       gtk_selection_data_get_target(seldata),
+	                       8, buf, len);
 }
 
 /*
@@ -579,8 +649,8 @@ gint Npc_chooser::drag_begin(
     gpointer data           // ->Npc_chooser.
 ) {
 	ignore_unused_variable_warning(widget);
-	cout << "In DRAG_BEGIN" << endl;
-	Npc_chooser *chooser = static_cast<Npc_chooser *>(data);
+	cout << "In DRAG_BEGIN of Npc" << endl;
+	auto *chooser = static_cast<Npc_chooser *>(data);
 	if (chooser->selected < 0)
 		return FALSE;       // ++++Display a halt bitmap.
 	// Get ->npc.
@@ -605,15 +675,21 @@ void Npc_chooser::drag_data_received(
     GtkSelectionData *seldata,
     guint info,
     guint time,
-    gpointer udata          // Should point to Shape_draw.
+    gpointer udata          // -> Npc_chooser.
 ) {
 	ignore_unused_variable_warning(widget, context, x, y, info, time);
-	Npc_chooser *chooser = static_cast<Npc_chooser *>(udata);
-	cout << "Npc drag_data_received" << endl;
-	if (seldata->type == gdk_atom_intern(U7_TARGET_NPCID_NAME, 0) &&
-	        seldata->format == 8 && seldata->length > 0) {
+	auto *chooser = static_cast<Npc_chooser *>(udata);
+	cout << "In DRAG_DATA_RECEIVED of Npc for '"
+	     << gdk_atom_name(gtk_selection_data_get_data_type(seldata))
+	     << "'" << endl;
+	if ((gtk_selection_data_get_data_type(seldata) == gdk_atom_intern(U7_TARGET_NPCID_NAME, 0) ||
+	     gtk_selection_data_get_data_type(seldata) == gdk_atom_intern(U7_TARGET_DROPFILE_NAME_MIME, 0) ||
+	     gtk_selection_data_get_data_type(seldata) == gdk_atom_intern(U7_TARGET_DROPFILE_NAME_MACOSX, 0)) &&
+	        Is_u7_npcid(gtk_selection_data_get_data(seldata)) &&
+	        gtk_selection_data_get_format(seldata) == 8 &&
+	        gtk_selection_data_get_length(seldata) > 0) {
 		int npcnum;
-		Get_u7_npcid(seldata->data, npcnum);
+		Get_u7_npcid(gtk_selection_data_get_data(seldata), npcnum);
 		chooser->group->add(npcnum);
 		chooser->setup_info(true);
 		chooser->render();
@@ -630,17 +706,21 @@ void Npc_chooser::enable_drop(
 		return;
 	drop_enabled = true;
 	gtk_widget_realize(draw);//???????
-#ifndef WIN32
-	GtkTargetEntry tents[1];
+	GtkTargetEntry tents[3];
 	tents[0].target = const_cast<char *>(U7_TARGET_NPCID_NAME);
+	tents[1].target = const_cast<char *>(U7_TARGET_DROPFILE_NAME_MIME);
+	tents[2].target = const_cast<char *>(U7_TARGET_DROPFILE_NAME_MACOSX);
 	tents[0].flags = 0;
+	tents[1].flags = 0;
+	tents[2].flags = 0;
 	tents[0].info = U7_TARGET_NPCID;
-	gtk_drag_dest_set(draw, GTK_DEST_DEFAULT_ALL, tents, 1,
+	tents[1].info = U7_TARGET_NPCID;
+	tents[2].info = U7_TARGET_NPCID;
+	gtk_drag_dest_set(draw, GTK_DEST_DEFAULT_ALL, tents, 3,
 	                  static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE));
 
-	gtk_signal_connect(GTK_OBJECT(draw), "drag_data_received",
-	                   GTK_SIGNAL_FUNC(drag_data_received), this);
-#endif
+	g_signal_connect(G_OBJECT(draw), "drag-data-received",
+	                 G_CALLBACK(drag_data_received), this);
 }
 
 /*
@@ -655,7 +735,6 @@ void Npc_chooser::scroll_row_vertical(
 	row0 = newrow;
 	row0_voffset = 0;
 	render();
-	show();
 }
 
 /*
@@ -692,7 +771,7 @@ void Npc_chooser::scroll_vertical(
 			delta += row0_voffset;
 			row0_voffset = 0;
 		} else {
-			if (row0 <= 0)
+			if (row0 == 0)
 				break;
 			--row0;
 			row0_voffset = 0;
@@ -706,7 +785,6 @@ void Npc_chooser::scroll_vertical(
 		}
 	}
 	render();
-	show();
 	update_statusbar();
 }
 
@@ -718,13 +796,33 @@ void Npc_chooser::setup_vscrollbar(
 ) {
 	GtkAdjustment *adj = gtk_range_get_adjustment(
 	                         GTK_RANGE(vscroll));
-	adj->value = 0;
-	adj->lower = 0;
-	adj->upper = total_height;
-	adj->step_increment = 16;   // +++++FOR NOW.
-	adj->page_increment = config_height;
-	adj->page_size = config_height;
-	gtk_signal_emit_by_name(GTK_OBJECT(adj), "changed");
+	gtk_adjustment_set_value(adj, 0);
+	gtk_adjustment_set_lower(adj, 0);
+	gtk_adjustment_set_upper(adj, total_height);
+	gtk_adjustment_set_step_increment(adj, 16);   // +++++FOR NOW.
+	gtk_adjustment_set_page_increment(adj, config_height);
+	gtk_adjustment_set_page_size(adj, config_height);
+	g_signal_emit_by_name(G_OBJECT(adj), "changed");
+}
+
+/*
+ *  Adjust horizontal scroll amounts.
+ */
+
+void Npc_chooser::setup_hscrollbar(
+    int newmax          // New max., or -1 to leave alone.
+) {
+	GtkAdjustment *adj = gtk_range_get_adjustment(
+	                         GTK_RANGE(hscroll));
+	if (newmax > 0)
+		gtk_adjustment_set_upper(adj, newmax);
+	GtkAllocation alloc = {0, 0, 0, 0};
+	gtk_widget_get_allocation(draw, &alloc);
+	gtk_adjustment_set_page_increment(adj, alloc.width);
+	gtk_adjustment_set_page_size(adj, alloc.width);
+	if (gtk_adjustment_get_page_size(adj) > gtk_adjustment_get_upper(adj))
+		gtk_adjustment_set_upper(adj, gtk_adjustment_get_page_size(adj));
+	g_signal_emit_by_name(G_OBJECT(adj), "changed");
 }
 
 /*
@@ -735,11 +833,70 @@ void Npc_chooser::vscrolled(    // For vertical scrollbar.
     GtkAdjustment *adj,     // The adjustment.
     gpointer data           // ->Npc_chooser.
 ) {
-	Npc_chooser *chooser = static_cast<Npc_chooser *>(data);
-	cout << "Scrolled to " << adj->value << '\n';
-	gint newindex = static_cast<gint>(adj->value);
+	auto *chooser = static_cast<Npc_chooser *>(data);
+	cout << "Scrolled to " << gtk_adjustment_get_value(adj) << '\n';
+	gint newindex = static_cast<gint>(gtk_adjustment_get_value(adj));
 	chooser->scroll_vertical(newindex);
 }
+void Npc_chooser::hscrolled(      // For horizontal scrollbar.
+    GtkAdjustment *adj,     // The adjustment.
+    gpointer data           // ->Npc_chooser.
+) {
+	auto *chooser = static_cast<Npc_chooser *>(data);
+	chooser->hoffset = static_cast<gint>(gtk_adjustment_get_value(adj));
+	chooser->render();
+}
+
+/*
+ *  Handle a change to the 'frame' spin button.
+ */
+
+void Npc_chooser::frame_changed(
+    GtkAdjustment *adj,     // The adjustment.
+    gpointer data           // ->Npc_chooser.
+) {
+	auto *chooser = static_cast<Npc_chooser *>(data);
+	gint newframe = static_cast<gint>(gtk_adjustment_get_value(adj));
+	if (chooser->selected >= 0) {
+		Npc_entry &npcinfo = chooser->info[chooser->selected];
+		vector<Estudio_npc> &npcs = chooser->get_npcs();
+		int nframes = chooser->ifile->get_num_frames(npcs[npcinfo.npcnum].shapenum);
+		if (newframe >= nframes)    // Just checking
+			return;
+		npcinfo.framenum = newframe;
+		if (chooser->frames_mode)   // Get sel. frame in view.
+			chooser->scroll_to_frame();
+		chooser->render();
+		chooser->update_statusbar();
+	}
+}
+
+/*
+ *  'All frames' toggled.
+ */
+
+void Npc_chooser::all_frames_toggled(
+    GtkToggleButton *btn,
+    gpointer data
+) {
+	auto *chooser = static_cast<Npc_chooser *>(data);
+	if (chooser->info.empty()) return;
+	bool on = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(btn));
+	chooser->frames_mode = on;
+	if (on)             // Frame => show horiz. scrollbar.
+		gtk_widget_show(chooser->hscroll);
+	else
+		gtk_widget_hide(chooser->hscroll);
+	// The old index is no longer valid, so we need to remember the shape.
+	int indx = chooser->selected >= 0 ? chooser->selected
+	           : static_cast<int>(chooser->rows[chooser->row0].index0);
+	int npcnum = chooser->info[indx].npcnum;
+	chooser->selected = -1;
+	chooser->setup_info();
+	indx = chooser->find_npc(npcnum);
+	if (indx >= 0)          // Get back to given shape.
+		chooser->goto_index(indx);
+}
 
 /*
  *  Get # shapes we can display.
@@ -779,7 +936,7 @@ void Npc_chooser::search(
 	for (i = start; i != stop; i += dir) {
 		unsigned npcnum = info[i].npcnum;
 		const char *nm = npcnum < npcs.size() ?
-		                 npcs[npcnum].name.c_str() : 0;
+		                 npcs[npcnum].name.c_str() : nullptr;
 		if (nm && search_name(nm, srch))
 			break;      // Found it.
 	}
@@ -787,7 +944,7 @@ void Npc_chooser::search(
 		return;         // Not found.
 	goto_index(i);
 	select(i);
-	show();
+	render();
 }
 
 /*
@@ -835,7 +992,7 @@ GtkWidget *Npc_chooser::create_popup(
 	create_popup_internal(false);
 	if (selected >= 0) {    // Add editing choices.
 		Add_menu_item(popup, "Edit...",
-		              GTK_SIGNAL_FUNC(on_npc_popup_edit_activate),
+		              G_CALLBACK(on_npc_popup_edit_activate),
 		              this);
 	}
 	return popup;
@@ -853,75 +1010,84 @@ Npc_chooser::Npc_chooser(
     Shape_file_info *fi
 ) : Object_browser(g, fi),
 	Shape_draw(i, palbuf, gtk_drawing_area_new()),
+	framenum0(0),
 	info(0), rows(0), row0(0),
 	row0_voffset(0), total_height(0),
-	voffset(0), status_id(-1), drop_enabled(false), sel_changed(0) {
+	frames_mode(false), hoffset(0),
+	voffset(0), status_id(-1), drop_enabled(false), sel_changed(nullptr) {
 	rows.reserve(40);
+
 	// Put things in a vert. box.
-	GtkWidget *vbox = gtk_vbox_new(FALSE, 0);
+	GtkWidget *vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
+	gtk_box_set_homogeneous(GTK_BOX(vbox), FALSE);
 	set_widget(vbox); // This is our "widget"
 	gtk_widget_show(vbox);
 
-	GtkWidget *hbox = gtk_hbox_new(FALSE, 0);
+	GtkWidget *hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
+	gtk_box_set_homogeneous(GTK_BOX(hbox), FALSE);
 	gtk_widget_show(hbox);
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
 
 	// A frame looks nice.
-	GtkWidget *frame = gtk_frame_new(NULL);
+	GtkWidget *frame = gtk_frame_new(nullptr);
 	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
+	widget_set_margins(frame, 2*HMARGIN, 2*HMARGIN, 2*VMARGIN, 2*VMARGIN);
 	gtk_widget_show(frame);
 	gtk_box_pack_start(GTK_BOX(hbox), frame, TRUE, TRUE, 0);
+
 	// NOTE:  draw is in Shape_draw.
 	// Indicate the events we want.
 	gtk_widget_set_events(draw, GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK
 	                      | GDK_BUTTON_RELEASE_MASK
-	                      | GDK_POINTER_MOTION_HINT_MASK |
-	                      GDK_BUTTON1_MOTION_MASK | GDK_KEY_PRESS_MASK);
+	                      | GDK_BUTTON1_MOTION_MASK | GDK_KEY_PRESS_MASK);
 	// Set "configure" handler.
-	gtk_signal_connect(GTK_OBJECT(draw), "configure_event",
-	                   GTK_SIGNAL_FUNC(Configure_chooser), this);
-	// Set "expose" handler.
-	gtk_signal_connect(GTK_OBJECT(draw), "expose_event",
-	                   GTK_SIGNAL_FUNC(expose), this);
+	g_signal_connect(G_OBJECT(draw), "configure-event",
+	                 G_CALLBACK(Configure_chooser), this);
+	// Set "expose-event" - "draw" handler.
+	g_signal_connect(G_OBJECT(draw), "draw",
+	                 G_CALLBACK(expose), this);
 	// Keystroke.
-	gtk_signal_connect(GTK_OBJECT(draw), "key-press-event",
-	                   GTK_SIGNAL_FUNC(on_npc_draw_key_press),
-	                   this);
-	GTK_WIDGET_SET_FLAGS(draw, GTK_CAN_FOCUS);
+	g_signal_connect(G_OBJECT(draw), "key-press-event",
+	                 G_CALLBACK(on_npc_draw_key_press),
+	                 this);
+	gtk_widget_set_can_focus(GTK_WIDGET(draw), TRUE);
 	// Set mouse click handler.
-	gtk_signal_connect(GTK_OBJECT(draw), "button_press_event",
-	                   GTK_SIGNAL_FUNC(Mouse_press), this);
-	gtk_signal_connect(GTK_OBJECT(draw), "button_release_event",
-	                   GTK_SIGNAL_FUNC(Mouse_release), this);
+	g_signal_connect(G_OBJECT(draw), "button-press-event",
+	                 G_CALLBACK(Mouse_press), this);
+	g_signal_connect(G_OBJECT(draw), "button-release-event",
+	                 G_CALLBACK(Mouse_release), this);
 	// Mouse motion.
-	gtk_signal_connect(GTK_OBJECT(draw), "drag_begin",
-	                   GTK_SIGNAL_FUNC(drag_begin), this);
-#ifdef WIN32
-// required to override GTK+ Drag and Drop
-	gtk_signal_connect(GTK_OBJECT(draw), "motion_notify_event",
-	                   GTK_SIGNAL_FUNC(win32_drag_motion), this);
-#else
-	gtk_signal_connect(GTK_OBJECT(draw), "motion_notify_event",
-	                   GTK_SIGNAL_FUNC(drag_motion), this);
-#endif
-	gtk_signal_connect(GTK_OBJECT(draw), "drag_data_get",
-	                   GTK_SIGNAL_FUNC(drag_data_get), this);
-	gtk_signal_connect(GTK_OBJECT(draw), "selection_clear_event",
-	                   GTK_SIGNAL_FUNC(selection_clear), this);
+	g_signal_connect(G_OBJECT(draw), "drag-begin",
+	                 G_CALLBACK(drag_begin), this);
+	g_signal_connect(G_OBJECT(draw), "motion-notify-event",
+	                 G_CALLBACK(drag_motion), this);
+	g_signal_connect(G_OBJECT(draw), "drag-data-get",
+	                 G_CALLBACK(drag_data_get), this);
 	gtk_container_add(GTK_CONTAINER(frame), draw);
-	gtk_drawing_area_size(GTK_DRAWING_AREA(draw), w, h);
+	widget_set_margins(draw, 2*HMARGIN, 2*HMARGIN, 2*VMARGIN, 2*VMARGIN);
+	gtk_widget_set_size_request(draw, w, h);
 	gtk_widget_show(draw);
 	// Want vert. scrollbar for the shapes.
-	GtkObject *shape_adj = gtk_adjustment_new(0, 0,
-	                       get_count() / 4, 1, 1, 1);
-	vscroll = gtk_vscrollbar_new(GTK_ADJUSTMENT(shape_adj));
+	GtkAdjustment *shape_adj = GTK_ADJUSTMENT(gtk_adjustment_new(0, 0,
+	                           std::ceil(get_count() / 4.0), 1, 1, 1));
+	vscroll = gtk_scrollbar_new(GTK_ORIENTATION_VERTICAL, GTK_ADJUSTMENT(shape_adj));
 	gtk_box_pack_start(GTK_BOX(hbox), vscroll, FALSE, TRUE, 0);
 	// Set scrollbar handler.
-	gtk_signal_connect(GTK_OBJECT(shape_adj), "value_changed",
-	                   GTK_SIGNAL_FUNC(vscrolled), this);
+	g_signal_connect(G_OBJECT(shape_adj), "value-changed",
+	                 G_CALLBACK(vscrolled), this);
 	gtk_widget_show(vscroll);
+	// Horizontal scrollbar.
+	shape_adj = GTK_ADJUSTMENT(gtk_adjustment_new(0, 0, 1600, 8, 16, 16));
+	hscroll = gtk_scrollbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_ADJUSTMENT(shape_adj));
+	gtk_box_pack_start(GTK_BOX(vbox), hscroll, FALSE, TRUE, 0);
+	// Set scrollbar handler.
+	g_signal_connect(G_OBJECT(shape_adj), "value-changed",
+	                 G_CALLBACK(hscrolled), this);
+//++++  gtk_widget_hide(hscroll);   // Only shown in 'frames' mode.
+
 	// At the bottom, status bar & frame:
-	GtkWidget *hbox1 = gtk_hbox_new(FALSE, 0);
+	GtkWidget *hbox1 = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
+	gtk_box_set_homogeneous(GTK_BOX(hbox1), FALSE);
 	gtk_box_pack_start(GTK_BOX(vbox), hbox1, FALSE, FALSE, 0);
 	gtk_widget_show(hbox1);
 	// At left, a status bar.
@@ -929,10 +1095,30 @@ Npc_chooser::Npc_chooser(
 	sbar_sel = gtk_statusbar_get_context_id(GTK_STATUSBAR(sbar),
 	                                        "selection");
 	gtk_box_pack_start(GTK_BOX(hbox1), sbar, TRUE, TRUE, 0);
+	widget_set_margins(sbar, 2*HMARGIN, 2*HMARGIN, 2*VMARGIN, 2*VMARGIN);
 	gtk_widget_show(sbar);
 	GtkWidget *label = gtk_label_new("Frame:");
-	gtk_box_pack_start(GTK_BOX(hbox1), label, FALSE, FALSE, 4);
+	gtk_box_pack_start(GTK_BOX(hbox1), label, FALSE, FALSE, 0);
+	widget_set_margins(label, 2*HMARGIN, 1*HMARGIN, 2*VMARGIN, 2*VMARGIN);
 	gtk_widget_show(label);
+	// A spin button for frame#.
+	frame_adj = GTK_ADJUSTMENT(gtk_adjustment_new(0, 0,
+	                           16, 1,
+	                           0, 0.0));
+	fspin = gtk_spin_button_new(GTK_ADJUSTMENT(frame_adj),
+	                            1, 0);
+	g_signal_connect(G_OBJECT(frame_adj), "value-changed",
+	                 G_CALLBACK(frame_changed), this);
+	gtk_box_pack_start(GTK_BOX(hbox1), fspin, FALSE, FALSE, 0);
+	widget_set_margins(fspin, 1*HMARGIN, 2*HMARGIN, 2*VMARGIN, 2*VMARGIN);
+	gtk_widget_show(fspin);
+	// A toggle for 'All Frames'.
+	GtkWidget *allframes = gtk_toggle_button_new_with_label("Frames");
+	gtk_box_pack_start(GTK_BOX(hbox1), allframes, FALSE, FALSE, 0);
+	widget_set_margins(allframes, 2*HMARGIN, 2*HMARGIN, 2*VMARGIN, 2*VMARGIN);
+	gtk_widget_show(allframes);
+	g_signal_connect(G_OBJECT(allframes), "toggled",
+	                 G_CALLBACK(all_frames_toggled), this);
 	// Add search controls to bottom.
 	gtk_box_pack_start(GTK_BOX(vbox),
 	                   create_controls(find_controls | locate_controls),
@@ -960,7 +1146,6 @@ void Npc_chooser::unselect(
 		selected = -1;
 		if (need_render) {
 			render();
-			show();
 		}
 		if (sel_changed)    // Tell client.
 			(*sel_changed)();
