@@ -5,7 +5,7 @@
  **/
 
 /*
-Copyright (C) 2002-2013 The Exult Team
+Copyright (C) 2002-2022 The Exult Team
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -23,28 +23,32 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
 #ifdef HAVE_CONFIG_H
-#  include <config.h>
+#	include <config.h>
 #endif
 
 #include "shapefile.h"
-#include "u7drag.h"
-#include "shapegroup.h"
-#include "shapevga.h"
-#include "shapelst.h"
+
+#include "Flex.h"
 #include "chunklst.h"
+#include "combo.h"
+#include "exceptions.h"
 #include "npclst.h"
 #include "paledit.h"
-#include "utils.h"
-#include "Flex.h"
-#include "exceptions.h"
-#include "combo.h"
+#include "shapegroup.h"
+#include "shapelst.h"
+#include "shapevga.h"
 #include "studio.h"
+#include "u7drag.h"
+#include "utils.h"
+
+#include <algorithm>
 
-using std::vector;
-using std::string;
 using std::cerr;
 using std::endl;
 using std::ofstream;
+using std::string;
+using std::unique_ptr;
+using std::vector;
 
 /*
  *  Delete file and groups.
@@ -68,9 +72,9 @@ Object_browser *Shape_file_info::get_browser(
 ) {
 	if (browser)
 		return browser;     // Okay.
-	browser = create_browser(vgafile, palbuf, 0);
+	browser = create_browser(vgafile, palbuf, nullptr);
 	// Add a reference (us).
-	gtk_widget_ref(browser->get_widget());
+	g_object_ref(browser->get_widget());
 	return browser;
 }
 
@@ -92,8 +96,8 @@ Object_browser *Image_file_info::create_browser(
     unsigned char *palbuf,      // Palette for displaying.
     Shape_group *g          // Group, or 0.
 ) {
-	Shape_chooser *chooser = new Shape_chooser(ifile, palbuf, 400, 64,
-	        g, this);
+	auto *chooser = new Shape_chooser(ifile, palbuf, 400, 64,
+	                                  g, this);
 	// Fonts?  Show 'A' as the default.
 	if (strcasecmp(basename.c_str(), "fonts.vga") == 0)
 		chooser->set_framenum0('A');
@@ -115,14 +119,18 @@ void Image_file_info::flush(
 	modified = false;
 	int nshapes = ifile->get_num_shapes();
 	int shnum;          // First read all entries.
-	Shape **shapes = new Shape *[nshapes];
+	vector<Shape *> shapes(nshapes);
 	for (shnum = 0; shnum < nshapes; shnum++)
 		shapes[shnum] = ifile->extract_shape(shnum);
 	string filestr("<PATCH>/"); // Always write to 'patch'.
 	filestr += basename;
 	// !flex means single-shape.
-	write_file(filestr.c_str(), shapes, nshapes, !ifile->is_flex());
-	delete [] shapes;
+	try {
+		write_file(filestr.c_str(), shapes.data(), nshapes, !ifile->is_flex());
+	} catch (exult_exception &e) {
+		EStudio::Alert("Error writing '%s'", filestr.c_str());
+		return;
+	}
 	// Tell Exult to reload this file.
 	unsigned char buf[Exult_server::maxlength];
 	unsigned char *ptr = &buf[0];
@@ -157,26 +165,21 @@ void Image_file_info::write_file(
     int nshapes,            // # shapes.
     bool single         // Don't write a FLEX file.
 ) {
-	ofstream out;
-	U7open(out, pathname);      // May throw exception.
+	OFileDataSource out(pathname);      // May throw exception.
 	if (single) {
 		if (nshapes)
 			shapes[0]->write(out);
 		out.flush();
-		if (!out.good())
-			throw file_write_exception(pathname);
-		out.close();
 		return;
 	}
 	Flex_writer writer(out, "Written by ExultStudio", nshapes);
 	// Write all out.
 	for (int shnum = 0; shnum < nshapes; shnum++) {
 		if (shapes[shnum]->get_modified() || shapes[shnum]->get_from_patch())
-			shapes[shnum]->write(out);
-		writer.mark_section_done();
+			writer.write_object(shapes[shnum]);
+		else
+			writer.empty_object();
 	}
-	if (!writer.close())
-		throw file_write_exception(pathname);
 }
 
 /*
@@ -237,15 +240,14 @@ bool Npcs_file_info::read_npc(unsigned num) {
 	int server_socket = studio->get_server_socket();
 	unsigned char buf[Exult_server::maxlength];
 	Exult_server::Msg_type id;
-	int datalen;
 	unsigned char *ptr;
 	const unsigned char *newptr;
 	newptr = ptr = &buf[0];
 	Write2(ptr, num);
 	if (!studio->send_to_server(Exult_server::npc_info, buf, ptr - buf) ||
 	        !Exult_server::wait_for_response(server_socket, 100) ||
-	        (datalen = Exult_server::Receive_data(server_socket,
-	                   id, buf, sizeof(buf))) == -1 ||
+	        Exult_server::Receive_data(server_socket,
+	                   id, buf, sizeof(buf)) == -1 ||
 	        id != Exult_server::npc_info ||
 	        Read2(newptr) != num)
 		return false;
@@ -275,11 +277,11 @@ void Npcs_file_info::setup(
 	// Should get immediate answer.
 	unsigned char buf[Exult_server::maxlength];
 	Exult_server::Msg_type id;
-	int num_npcs, datalen;
+	int num_npcs;
 	if (Send_data(server_socket, Exult_server::npc_unused) == -1 ||
 	        !Exult_server::wait_for_response(server_socket, 100) ||
-	        (datalen = Exult_server::Receive_data(server_socket,
-	                   id, buf, sizeof(buf))) == -1 ||
+	        Exult_server::Receive_data(server_socket,
+	                   id, buf, sizeof(buf)) == -1 ||
 	        id != Exult_server::npc_unused) {
 		cerr << "Error sending data to server." << endl;
 		return;
@@ -294,8 +296,8 @@ void Npcs_file_info::setup(
 		if (!studio->send_to_server(Exult_server::npc_info,
 		                            buf, ptr - buf) ||
 		        !Exult_server::wait_for_response(server_socket, 100) ||
-		        (datalen = Exult_server::Receive_data(server_socket,
-		                   id, buf, sizeof(buf))) == -1 ||
+		        Exult_server::Receive_data(server_socket,
+		                   id, buf, sizeof(buf)) == -1 ||
 		        id != Exult_server::npc_info ||
 		        Read2(newptr) != i) {
 			npcs.resize(0);
@@ -336,14 +338,12 @@ Flex_file_info::Flex_file_info(
     const char *bnm,        // Basename,
     const char *pnm,        // Full pathname,
     unsigned size           // File size.
-) : Shape_file_info(bnm, pnm, 0), flex(0), write_flat(true) {
-	entries.resize(size > 0);
+) : Shape_file_info(bnm, pnm, nullptr), flex(nullptr), write_flat(true) {
+	entries.resize(static_cast<size_t>(size > 0));
 	lengths.resize(entries.size());
 	if (size > 0) {         // Read in whole thing.
-		std::ifstream in;
-		U7open(in, pnm);    // Shouldn't fail.
-		entries[0] = new char[size];
-		in.read(entries[0], size);
+		IFileDataSource in(pnm);
+		entries[0] = in.readN(size);
 		lengths[0] = size;
 	}
 }
@@ -355,16 +355,13 @@ Flex_file_info::Flex_file_info(
 Flex_file_info::~Flex_file_info(
 ) {
 	delete flex;
-	int cnt = entries.size();
-	for (int i = 0; i < cnt; i++)
-		delete entries[i];
 }
 
 /*
  *  Get i'th entry.
  */
 
-char *Flex_file_info::get(
+unsigned char *Flex_file_info::get(
     unsigned i,
     size_t &len
 ) {
@@ -374,9 +371,9 @@ char *Flex_file_info::get(
 			lengths[i] = len;
 		}
 		len = lengths[i];
-		return entries[i];
+		return entries[i].get();
 	} else
-		return 0;
+		return nullptr;
 }
 
 /*
@@ -385,17 +382,16 @@ char *Flex_file_info::get(
 
 void Flex_file_info::set(
     unsigned i,
-    char *newentry,         // Allocated data that we'll own.
+    unique_ptr<unsigned char[]> newentry,         // Allocated data that we'll own.
     int entlen          // Length.
 ) {
 	if (i > entries.size())
 		return;
 	if (i == entries.size()) {  // Appending?
-		entries.push_back(newentry);
+		entries.push_back(std::move(newentry));
 		lengths.push_back(entlen);
 	} else {
-		delete entries[i];
-		entries[i] = newentry;
+		entries[i] = std::move(newentry);
 		lengths[i] = entlen;
 	}
 }
@@ -408,12 +404,8 @@ void Flex_file_info::swap(
     unsigned i
 ) {
 	assert(i < entries.size() - 1);
-	char *tmpent = entries[i];
-	int tmplen = lengths[i];
-	entries[i] = entries[i + 1];
-	lengths[i] = lengths[i + 1];
-	entries[i + 1] = tmpent;
-	lengths[i + 1] = tmplen;
+	std::swap(entries[i], entries[i + 1]);
+	std::swap(lengths[i], lengths[i + 1]);
 }
 
 /*
@@ -424,7 +416,6 @@ void Flex_file_info::remove(
     unsigned i
 ) {
 	assert(i < entries.size());
-	delete entries[i];      // Free memory.
 	entries.erase(entries.begin() + i);
 	lengths.erase(lengths.begin() + i);
 }
@@ -462,26 +453,19 @@ void Flex_file_info::flush(
 		if (!entries[i])
 			get(i, len);
 	}
-	ofstream out;
 	string filestr("<PATCH>/"); // Always write to 'patch'.
 	filestr += basename;
-	U7open(out, filestr.c_str());   // May throw exception.
+	OFileDataSource ds(filestr.c_str());    // Throws exception on failure
 	if (cnt <= 1 && write_flat) { // Write flat file.
 		if (cnt)
-			out.write(entries[0], lengths[0]);
-		out.close();
-		if (!out.good())
-			throw file_write_exception(filestr);
+			ds.write(entries[0].get(), lengths[0]);
 		return;
 	}
-	Flex_writer writer(out, "Written by ExultStudio", cnt);
+	Flex_writer writer(ds, "Written by ExultStudio", cnt);
 	// Write all out.
 	for (int i = 0; i < cnt; i++) {
-		out.write(entries[i], lengths[i]);
-		writer.mark_section_done();
+		writer.write_object(entries[i].get(), lengths[i]);
 	}
-	if (!writer.close())
-		throw file_write_exception(filestr);
 }
 
 /*
@@ -497,8 +481,7 @@ bool Flex_file_info::revert(
 	modified = false;
 	int cnt = entries.size();
 	for (int i = 0; i < cnt; i++) {
-		delete entries[i];
-		entries[i] = 0;
+		entries[i].reset();
 		lengths[i] = 0;
 	}
 	if (flex) {
@@ -506,17 +489,13 @@ bool Flex_file_info::revert(
 		entries.resize(cnt);
 		lengths.resize(entries.size());
 	} else {            // Single palette.
-		std::ifstream in;
-		U7open(in, pathname.c_str());   // Shouldn't fail.
-		in.seekg(0, std::ios::end); // Figure size.
-		int sz = in.tellg();
+		IFileDataSource in(pathname);
+		const int sz = in.getSize();
 		cnt = sz > 0 ? 1 : 0;
 		entries.resize(cnt);
 		lengths.resize(entries.size());
-		in.seekg(0);
 		if (cnt) {
-			entries[0] = new char[sz];
-			in.read(entries[0], sz);
+			entries[0] = in.readN(sz);
 			lengths[0] = sz;
 		}
 	}
@@ -529,9 +508,8 @@ bool Flex_file_info::revert(
 
 Shape_file_set::~Shape_file_set(
 ) {
-	for (vector<Shape_file_info *>::iterator it = files.begin();
-	        it != files.end(); ++it)
-		delete(*it);
+	for (auto *file : files)
+		delete file;
 }
 
 /*
@@ -544,23 +522,24 @@ static bool Create_file(
     const char *basename,       // Base file name.
     const string &pathname      // Full name.
 ) {
-	int namelen = strlen(basename);
-	if (strcasecmp(".flx", basename + namelen - 4) == 0) {
-		// We can create an empty flx.
-		ofstream out;
-		U7open(out, pathname.c_str());  // May throw exception.
-		Flex_writer writer(out, "Written by ExultStudio", 0);
-		if (!writer.close())
-			throw file_write_exception(pathname);
-		return true;
-	} else if (strcasecmp(".pal", basename + namelen - 4) == 0) {
-		// Empty 1-palette file.
-		ofstream out;
-		U7open(out, pathname.c_str());  // May throw exception.
-		out.close();        // Empty file.
-		return true;
-	} else if (strcasecmp("npcs", basename) == 0)
-		return true;        // Don't need file.
+	try {
+		int namelen = strlen(basename);
+		if (strcasecmp(".flx", basename + namelen - 4) == 0) {
+			// We can create an empty flx.
+			OFileDataSource out(pathname.c_str());  // May throw exception.
+			Flex_writer writer(out, "Written by ExultStudio", 0);
+			return true;
+		} else if (strcasecmp(".pal", basename + namelen - 4) == 0) {
+			// Empty 1-palette file.
+			ofstream out;
+			U7open(out, pathname.c_str());  // May throw exception.
+			out.close();        // Empty file.
+			return true;
+		} else if (strcasecmp("npcs", basename) == 0)
+			return true;        // Don't need file.
+	} catch (exult_exception &e) {
+		EStudio::Alert("Error writing '%s'", pathname.c_str());
+	}
 	return false;           // Might add more later.
 }
 
@@ -574,24 +553,24 @@ Shape_file_info *Shape_file_set::create(
     const char *basename        // Like 'shapes.vga'.
 ) {
 	// Already have it open?
-	for (vector<Shape_file_info *>::iterator it = files.begin();
-	        it != files.end(); ++it)
-		if (strcasecmp((*it)->basename.c_str(), basename) == 0)
-			return *it; // Found it.
+	for (auto *file : files)
+		if (strcasecmp(file->basename.c_str(), basename) == 0)
+			return file; // Found it.
 	// Look in 'static', 'patch'.
 	string sstr = string("<STATIC>/") + basename;
 	string pstr = string("<PATCH>/") + basename;
-	const char *spath = sstr.c_str(), *ppath = pstr.c_str();
+	const char *spath = sstr.c_str();
+	const char *ppath = pstr.c_str();
 	bool sexists = U7exists(spath);
 	bool pexists = U7exists(ppath);
 	if (!sexists && !pexists)   // Neither place.  Try to create.
 		if (!(pexists = Create_file(basename, ppath)))
-			return 0;
+			return nullptr;
 	// Use patch file if it exists.
 	const char *fullname = pexists ? ppath : spath;
 	string group_name(basename);    // Create groups file.
 	group_name += ".grp";
-	Shape_group_file *groups = new Shape_group_file(group_name.c_str());
+	auto *groups = new Shape_group_file(group_name.c_str());
 	if (strcasecmp(basename, "shapes.vga") == 0)
 		return append(new Image_file_info(basename, fullname,
 		                                  new Shapes_vga_file(spath, U7_SHAPE_SHAPES, ppath),
@@ -612,7 +591,7 @@ Shape_file_info *Shape_file_set::create(
 		return append(new Image_file_info(basename, fullname,
 		                                  new Vga_file(spath, U7_SHAPE_FONTS, ppath), groups));
 	else if (strcasecmp(basename, "u7chunks") == 0) {
-		std::ifstream *file = new std::ifstream;
+		auto *file = new std::ifstream;
 		U7open(*file, fullname);
 		return append(new Chunks_file_info(basename, fullname,
 		                                   file, groups));
@@ -632,7 +611,7 @@ Shape_file_info *Shape_file_set::create(
 		return append(new Flex_file_info(basename, fullname, sz));
 	} else {            // Not handled above?
 		// Get image file for this path.
-		Vga_file *ifile = new Vga_file(spath, U7_SHAPE_UNK, ppath);
+		auto *ifile = new Vga_file(spath, U7_SHAPE_UNK, ppath);
 		if (ifile->is_good())
 			return append(new Image_file_info(basename, fullname,
 			                                  ifile, groups));
@@ -641,22 +620,21 @@ Shape_file_info *Shape_file_set::create(
 		delete ifile;
 	}
 	cerr << "Error opening image file '" << basename << "'.\n";
-	return 0;
+	return nullptr;
 }
 
 /*
  *  Locates the NPC browser.
  *
- *  Output: ->file info, or 0 if not found.
+ *  Output: ->file info, or nullptr if not found.
  */
 
 Shape_file_info *Shape_file_set::get_npc_browser(
 ) {
-	for (vector<Shape_file_info *>::iterator it = files.begin();
-	        it != files.end(); ++it)
-		if (strcasecmp((*it)->basename.c_str(), "npcs") == 0)
-			return *it; // Found it.
-	return 0;   // Doesn't exist yet.
+	for (auto *file : files)
+		if (strcasecmp(file->basename.c_str(), "npcs") == 0)
+			return file; // Found it.
+	return nullptr;   // Doesn't exist yet.
 }
 
 /*
@@ -665,9 +643,8 @@ Shape_file_info *Shape_file_set::get_npc_browser(
 
 void Shape_file_set::flush(
 ) {
-	for (vector<Shape_file_info *>::iterator it = files.begin();
-	        it != files.end(); ++it)
-		(*it)->flush();
+	for (auto *file : files)
+		file->flush();
 }
 
 /*
@@ -676,9 +653,7 @@ void Shape_file_set::flush(
 
 bool Shape_file_set::is_modified(
 ) {
-	for (vector<Shape_file_info *>::iterator it = files.begin();
-	        it != files.end(); ++it)
-		if ((*it)->modified)
-			return true;
-	return false;
+	return std::any_of(files.cbegin(), files.cend(), [](auto* file) {
+		return file->modified;
+	});
 }
