@@ -6,6 +6,7 @@
 
 /*
 Copyright (C) 2000  Jeffrey S. Freedman
+Copyright (C) 2001-2022  The Exult Team
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -54,7 +55,7 @@ using std::ostream;
 using std::cout;
 using std::endl;
 
-Barge_object *Barge_object::editing = 0;
+Game_object_shared Barge_object::editing;
 
 /*
  *  Rotate a point 90 degrees to the right around a point.
@@ -68,7 +69,8 @@ inline Tile_coord Rotate90r(
     Tile_coord const &c         // Center to rotate around.
 ) {
 	// Get cart. coords. rel. to center.
-	int rx = t.tx - c.tx, ry = c.ty - t.ty;
+	int rx = t.tx - c.tx;
+	int ry = c.ty - t.ty;
 	return Tile_coord(c.tx + ry, c.ty + rx, t.tz);
 }
 
@@ -84,7 +86,8 @@ inline Tile_coord Rotate90l(
     Tile_coord const &c         // Center to rotate around.
 ) {
 	// Get cart. coords. rel. to center.
-	int rx = t.tx - c.tx, ry = c.ty - t.ty;
+	int rx = t.tx - c.tx;
+	int ry = c.ty - t.ty;
 	return Tile_coord(c.tx - ry, c.ty - rx, t.tz);
 }
 
@@ -100,7 +103,8 @@ inline Tile_coord Rotate180(
     Tile_coord const &c         // Center to rotate around.
 ) {
 	// Get cart. coords. rel. to center.
-	int rx = t.tx - c.tx, ry = c.ty - t.ty;
+	int rx = t.tx - c.tx;
+	int ry = c.ty - t.ty;
 	return Tile_coord(c.tx - rx, c.ty + ry, t.tz);
 }
 
@@ -182,11 +186,12 @@ inline void Barge_object::swap_dims(
  *  Get footprint in tiles.
  */
 
-Rectangle Barge_object::get_tile_footprint(
+TileRect Barge_object::get_tile_footprint(
 ) {
 	Tile_coord pos = get_tile();
-	int xts = get_xtiles(), yts = get_ytiles();
-	Rectangle foot((pos.tx - xts + 1 + c_num_tiles) % c_num_tiles,
+	int xts = get_xtiles();
+	int yts = get_ytiles();
+	TileRect foot((pos.tx - xts + 1 + c_num_tiles) % c_num_tiles,
 	               (pos.ty - yts + 1 + c_num_tiles) % c_num_tiles, xts, yts);
 	return foot;
 }
@@ -214,10 +219,11 @@ int Barge_object::okay_to_rotate(
 	// Special case for carpet.
 	int move_type = (lift > 0) ? (MOVE_LEVITATE) : MOVE_ALL_TERRAIN;
 	// Get footprint in tiles.
-	Rectangle foot = get_tile_footprint();
-	int xts = get_xtiles(), yts = get_ytiles();
+	TileRect foot = get_tile_footprint();
+	int xts = get_xtiles();
+	int yts = get_ytiles();
 	// Get where new footprint will be.
-	Rectangle newfoot(pos.tx - yts + 1, pos.ty - xts + 1, yts, xts);
+	TileRect newfoot(pos.tx - yts + 1, pos.ty - xts + 1, yts, xts);
 	int new_lift;
 	if (newfoot.y < foot.y)     // Got a piece above the old one?
 		// Check area.  (No dropping allowed.)
@@ -268,19 +274,20 @@ void Barge_object::gather(
 	ice_raft = false;       // We'll just detect it each time.
 	objects.resize(perm_count); // Start fresh.
 	// Get footprint in tiles.
-	Rectangle foot = get_tile_footprint();
+	TileRect foot = get_tile_footprint();
 	int lift = get_lift();      // How high we are.
 	// Go through intersected chunks.
 	Chunk_intersect_iterator next_chunk(foot);
-	Rectangle tiles;
-	int cx, cy;
+	TileRect tiles;
+	int cx;
+	int cy;
 	while (next_chunk.get_next(tiles, cx, cy)) {
 		Map_chunk *chunk = gmap->get_chunk(cx, cy);
 		tiles.x += cx * c_tiles_per_chunk;
 		tiles.y += cy * c_tiles_per_chunk;
 		Game_object *obj;
 		Object_iterator next(chunk->get_objects());
-		while ((obj = next.get_next()) != 0) {
+		while ((obj = next.get_next()) != nullptr) {
 			// Look at each object.
 			if (obj == this)
 				continue;
@@ -292,11 +299,11 @@ void Barge_object::gather(
 				continue;
 			const Shape_info &info = obj->get_info();
 			// Above barge, within 5-tiles up?
-			bool isbarge = info.is_barge_part() != 0 /*+++ || !info.get_weight() */;
+			bool isbarge = info.is_barge_part() /*+++ || !info.get_weight() */;
 			if (t.tz + info.get_3d_height() > lift &&
 			        ((isbarge && t.tz >= lift - 1) ||
 			         (t.tz < lift + 5 && t.tz >= lift /*+++ + 1 */))) {
-				objects.push_back(obj);
+				objects.push_back(obj->shared_from_this());
 				int btype = obj->get_info().get_barge_type();
 				if (btype == Shape_info::barge_raft)
 					ice_raft = true;
@@ -309,7 +316,7 @@ void Barge_object::gather(
 	// Test for boat.
 	Map_chunk *chunk = gmap->get_chunk_safely(
 	                       center.tx / c_tiles_per_chunk, center.ty / c_tiles_per_chunk);
-	if (boat == -1 && chunk != 0) {
+	if (boat == -1 && chunk != nullptr) {
 		ShapeID flat = chunk->get_flat(center.tx % c_tiles_per_chunk,
 		                               center.ty % c_tiles_per_chunk);
 		if (flat.is_invalid())
@@ -328,10 +335,12 @@ void Barge_object::gather(
 
 void Barge_object::add_dirty(
 ) {
-	int x, y;           // Get lower-right corner.
+	int x;
+	int y;           // Get lower-right corner.
 	gwin->get_shape_location(this, x, y);
-	int w = xtiles * c_tilesize, h = ytiles * c_tilesize;
-	Rectangle box(x - w, y - h, w, h);
+	int w = xtiles * c_tilesize;
+	int h = ytiles * c_tilesize;
+	TileRect box(x - w, y - h, w, h);
 	const int barge_enlarge = (c_tilesize + c_tilesize / 4);
 	const int barge_stretch = (4 * c_tilesize + c_tilesize / 2);
 	box.enlarge(barge_enlarge);     // Make it a bit bigger.
@@ -402,12 +411,12 @@ void Barge_object::travel_to_tile(
 	if (!path)
 		path = new Zombie();
 	// Set up new path.
-	if (path->NewPath(get_tile(), dest, 0)) {
+	if (path->NewPath(get_tile(), dest, nullptr)) {
 		frame_time = speed;
 		// Figure new direction.
 		Tile_coord cur = get_tile();
-		int dy = Tile_coord::delta(cur.ty, dest.ty),
-		    dx = Tile_coord::delta(cur.tx, dest.tx);
+		int dy = Tile_coord::delta(cur.ty, dest.ty);
+		int dx = Tile_coord::delta(cur.tx, dest.tx);
 		int ndir = Get_direction4(-dy, dx);
 		if (!ice_raft)      // Ice-raft doesn't rotate.
 			face_direction(ndir);
@@ -433,16 +442,17 @@ void Barge_object::turn_right(
 	dir = (dir + 1) % 4;    // Increment direction.
 	int cnt = objects.size();   // We'll move each object.
 	// But 1st, remove & save new pos.
-	Tile_coord *positions = new Tile_coord[cnt];
+	auto *positions = new Tile_coord[cnt];
 	for (int i = 0; i < cnt; i++) {
 		Game_object *obj = get_object(i);
 		int frame = obj->get_framenum();
 		const Shape_info &info = obj->get_info();
 		positions[i] = Rotate90r(gwin, obj, info.get_3d_xtiles(frame),
 		                         info.get_3d_ytiles(frame), center);
-		obj->remove_this(1);    // Remove object from world.
+        Game_object_shared keep;
+		obj->remove_this(&keep);    // Remove object from world.
 		// Set to rotated frame.
-		obj->set_frame(obj->get_rotated_frame(1));
+		obj->change_frame(obj->get_rotated_frame(1));
 		obj->set_invalid(); // So it gets added back right.
 	}
 	finish_move(positions);     // Add back & del. positions.
@@ -464,16 +474,17 @@ void Barge_object::turn_left(
 	dir = (dir + 3) % 4;    // Increment direction.
 	int cnt = objects.size();   // We'll move each object.
 	// But 1st, remove & save new pos.
-	Tile_coord *positions = new Tile_coord[cnt];
+	auto *positions = new Tile_coord[cnt];
 	for (int i = 0; i < cnt; i++) {
 		Game_object *obj = get_object(i);
 		int frame = obj->get_framenum();
 		const Shape_info &info = obj->get_info();
 		positions[i] = Rotate90l(gwin, obj, info.get_3d_xtiles(frame),
 		                         info.get_3d_ytiles(frame), center);
-		obj->remove_this(1);    // Remove object from world.
+		Game_object_shared keep;
+		obj->remove_this(&keep);    // Remove object from world.
 		// Set to rotated frame.
-		obj->set_frame(obj->get_rotated_frame(3));
+		obj->change_frame(obj->get_rotated_frame(3));
 		obj->set_invalid(); // So it gets added back right.
 	}
 	finish_move(positions);     // Add back & del. positions.
@@ -492,16 +503,17 @@ void Barge_object::turn_around(
 	dir = (dir + 2) % 4;    // Increment direction.
 	int cnt = objects.size();   // We'll move each object.
 	// But 1st, remove & save new pos.
-	Tile_coord *positions = new Tile_coord[cnt];
+	auto *positions = new Tile_coord[cnt];
 	for (int i = 0; i < cnt; i++) {
 		Game_object *obj = get_object(i);
 		int frame = obj->get_framenum();
 		const Shape_info &info = obj->get_info();
 		positions[i] = Rotate180(gwin, obj, info.get_3d_xtiles(frame),
 		                         info.get_3d_ytiles(frame), center);
-		obj->remove_this(1);    // Remove object from world.
+		Game_object_shared keep;
+		obj->remove_this(&keep);    // Remove object from world.
 		// Set to rotated frame.
-		obj->set_frame(obj->get_rotated_frame(2));
+		obj->change_frame(obj->get_rotated_frame(2));
 		obj->set_invalid(); // So it gets added back right.
 	}
 	finish_move(positions);     // Add back & del. positions.
@@ -522,7 +534,7 @@ void Barge_object::done(
 		// Pretend they were clicked on.
 		int cnt = objects.size();   // Look for open sail.
 		for (int i = 0; i < cnt; i++) {
-			Game_object *obj = objects[i];
+			Game_object *obj = objects[i].get();
 			if (obj->get_info().get_barge_type() == Shape_info::barge_sails &&
 			        (obj->get_framenum() & 7) < 4) {
 				obj->activate();
@@ -539,12 +551,13 @@ void Barge_object::done(
 
 int Barge_object::okay_to_land(
 ) {
-	Rectangle foot = get_tile_footprint();
+	TileRect foot = get_tile_footprint();
 	int lift = get_lift();      // How high we are.
 	// Go through intersected chunks.
 	Chunk_intersect_iterator next_chunk(foot);
-	Rectangle tiles;
-	int cx, cy;
+	TileRect tiles;
+	int cx;
+	int cy;
 	while (next_chunk.get_next(tiles, cx, cy)) {
 		// Check each tile.
 		Map_chunk *chunk = gmap->get_chunk(cx, cy);
@@ -552,9 +565,9 @@ int Barge_object::okay_to_land(
 			for (int tx = tiles.x; tx < tiles.x + tiles.w; tx++)
 				if (chunk->get_highest_blocked(lift, tx, ty) != -1 ||
 				        chunk->get_flat(tx, ty).get_info().is_water())
-					return (0);
+					return 0;
 	}
-	return (1);
+	return 1;
 }
 
 /*
@@ -610,10 +623,12 @@ void Barge_object::move(
 	Container_game_object::move(newtx, newty, newlift, newmap);
 	set_center();
 	// Get deltas.
-	int dx = newtx - old.tx, dy = newty - old.ty, dz = newlift - old.tz;
+	int dx = newtx - old.tx;
+	int dy = newty - old.ty;
+	int dz = newlift - old.tz;
 	int cnt = objects.size();   // We'll move each object.
 	// But 1st, remove & save new pos.
-	Tile_coord *positions = new Tile_coord[cnt];
+	auto *positions = new Tile_coord[cnt];
 	int i;
 	for (i = 0; i < cnt; i++) {
 		Game_object *obj = get_object(i);
@@ -623,17 +638,18 @@ void Barge_object::move(
 		                   (ot.tx + dx + c_num_tiles) % c_num_tiles,
 		                   (ot.ty + dy + c_num_tiles) % c_num_tiles,
 		                   ot.tz + dz);
-		obj->remove_this(1);    // Remove object from world.
+		Game_object_shared keep;
+		obj->remove_this(&keep);    // Remove object from world.
 		obj->set_invalid(); // So it gets added back right.
 		if (!taking_2nd_step) {
 			// Animate a few shapes.
 			int frame = obj->get_framenum();
 			switch (obj->get_info().get_barge_type()) {
 			case Shape_info::barge_wheel:       // Cart wheel.
-				obj->set_frame(((frame + 1) & 3) | (frame & 32));
+				obj->change_frame(((frame + 1) & 3) | (frame & 32));
 				break;
 			case Shape_info::barge_draftanimal:     // Draft horse.
-				obj->set_frame(((frame + 4) & 15) | (frame & 32));
+				obj->change_frame(((frame + 4) & 15) | (frame & 32));
 				break;
 			}
 		}
@@ -648,8 +664,9 @@ void Barge_object::move(
 void Barge_object::remove(
     Game_object *obj
 ) {
-	obj->set_owner(0);
-	obj->remove_this(1);        // Now remove from outside world.
+	obj->set_owner(nullptr);
+	Game_object_shared keep;
+	obj->remove_this(&keep);        // Now remove from outside world.
 }
 
 /*
@@ -666,8 +683,8 @@ bool Barge_object::add(
     bool noset      // True to prevent actors from setting sched. weapon.
 ) {
 	ignore_unused_variable_warning(dont_check, combine, noset);
-	objects.push_back(obj);     // Add to list.
-	return (false);         // We want it added to the chunk.
+	objects.push_back(obj->shared_from_this());     // Add to list.
+	return false;         // We want it added to the chunk.
 }
 
 /*
@@ -677,9 +694,8 @@ bool Barge_object::add(
 bool Barge_object::contains(
     Game_object *obj
 ) {
-	for (Game_object_vector::const_iterator it = objects.begin();
-	        it != objects.end(); ++it)
-		if (obj == *it)
+	for (auto& object : objects)
+		if (obj == object.get())
 			return true;
 	return false;
 }
@@ -690,11 +706,11 @@ bool Barge_object::contains(
  *  Output: 0 to reject, 1 to accept.
  */
 
-int Barge_object::drop(
+bool Barge_object::drop(
     Game_object *obj
 ) {
 	ignore_unused_variable_warning(obj);
-	return 0;           //++++++Later.
+	return false;           //++++++Later.
 }
 
 /*
@@ -708,7 +724,10 @@ void Barge_object::paint(
 	if (gwin->paint_eggs) {
 		Container_game_object::paint();
 		int pix = sman->get_special_pixel(CURSED_PIXEL);
-		int rx, by, lx, ty; // Right, bottom, left, top.
+		int rx;
+		int by;
+		int lx;
+		int ty; // Right, bottom, left, top.
 		gwin->get_shape_location(this, rx, by);
 		lx = rx - xtiles * c_tilesize + 1;
 		ty = by - ytiles * c_tilesize + 1;
@@ -746,13 +765,13 @@ bool Barge_object::edit(
 #ifdef USE_EXULTSTUDIO
 	if (client_socket >= 0 &&   // Talking to ExultStudio?
 	        cheat.in_map_editor()) {
-		editing = 0;
+		editing.reset();
 		Tile_coord t = get_tile();
 		if (Barge_object_out(client_socket, this, t.tx, t.ty, t.tz,
 		                     get_shapenum(), get_framenum(),
 		                     xtiles, ytiles, dir) != -1) {
 			cout << "Sent barge data to ExultStudio" << endl;
-			editing = this;
+			editing = shared_from_this();
 		} else
 			cout << "Error sending barge data to ExultStudio" <<
 			     endl;
@@ -773,22 +792,29 @@ void Barge_object::update_from_studio(
 ) {
 #ifdef USE_EXULTSTUDIO
 	Barge_object *barge;
-	int tx, ty, tz;
-	int shape, frame;
-	int xtiles, ytiles, dir;
+	int tx;
+	int ty;
+	int tz;
+	int shape;
+	int frame;
+	int xtiles;
+	int ytiles;
+	int dir;
 	if (!Barge_object_in(data, datalen, barge, tx, ty, tz, shape, frame,
 	                     xtiles, ytiles, dir)) {
 		cout << "Error decoding barge" << endl;
 		return;
 	}
-	if (barge && barge != editing) {
+	if (barge && barge != editing.get()) {
 		cout << "Barge from ExultStudio is not being edited" << endl;
 		return;
 	}
-	editing = 0;
+	// Keeps NPC alive until end of function
+	Game_object_shared keep = std::move(editing);
 	if (!barge) {       // Create a new one?
-		int x, y;
-		if (!Get_click(x, y, Mouse::hand, 0)) {
+		int x;
+		int y;
+		if (!Get_click(x, y, Mouse::hand, nullptr)) {
 			if (client_socket >= 0)
 				Exult_server::Send_data(client_socket,
 				                        Exult_server::cancel);
@@ -797,7 +823,9 @@ void Barge_object::update_from_studio(
 		if (shape == -1)
 			shape = 961;    // FOR NOW.
 		// Create.  Gets initialized below.
-		barge = new Barge_object(shape, 0, 0, 0, 0, 0, 0, 0);
+		auto obj = std::make_shared<Barge_object>(shape, 0, 0, 0, 0, 0, 0, 0);
+		barge = obj.get();
+		keep = std::move(obj);
 		int lift;       // Try to drop at increasing hts.
 		for (lift = 0; lift < 12; lift++)
 			if (gwin->drop_at_lift(barge, x, y, lift) == 1)
@@ -828,11 +856,11 @@ void Barge_object::update_from_studio(
 /*
  *  Step onto an adjacent tile.
  *
- *  Output: 0 if blocked.
+ *  Output: false if blocked.
  *      Dormant is set if off screen.
  */
 
-int Barge_object::step(
+bool Barge_object::step(
     Tile_coord t,           // Tile to step onto.
     int frame,              // Frame (ignored).
     bool force              // Forces the step to happen.
@@ -854,13 +882,13 @@ int Barge_object::step(
 	// No rising/dropping.
 	if (Map_chunk::is_blocked(get_xtiles(), get_ytiles(),
 	                          4, cur, t, move_type, 0, 0))
-		return (0);     // Done.
+		return false;     // Done.
 	move(t.tx, t.ty, t.tz);     // Move it & its objects.
 	// Near an egg?
 	Map_chunk *nlist = gmap->get_chunk(get_cx(), get_cy());
 	nlist->activate_eggs(gwin->get_main_actor(), t.tx, t.ty, t.tz,
 	                     cur.tx, cur.ty);
-	return (1);         // Add back to queue for next time.
+	return true;         // Add back to queue for next time.
 }
 
 /*
@@ -878,9 +906,9 @@ void Barge_object::write_ireg(
 	*ptr++ = 0;         // Unknown.
 	// Flags (quality).  Taking B3 to in-
 	//   dicate barge mode.
-	*ptr++ = (dir << 1) | ((gwin->get_moving_barge() == this) << 3);
+	*ptr++ = (dir << 1) | ((gwin->get_moving_barge() == this) ? (1 << 3) : 0);
 	*ptr++ = 0;         // (Quantity).
-	*ptr++ = (get_lift() & 15) << 4;
+	*ptr++ = nibble_swap(get_lift());
 	*ptr++ = 0;         // Data2.
 	*ptr++ = 0;         //
 	out->write(reinterpret_cast<char *>(buf), ptr - buf);
