@@ -5,7 +5,7 @@
  **/
 
 /*
-Copyright (C) 2000 The Exult Team
+Copyright (C) 2000-2022 The Exult Team
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -32,6 +32,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "gamemap.h"
 #include "shapeinf.h"
 #include "citerate.h"
+#include "databuf.h"
 #include "egg.h"
 #include "objiter.h"
 #include "objs.h"
@@ -43,47 +44,31 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "miscinf.h"
 #include "ignore_unused_variable_warning.h"
 
-#if 0
-#include <iostream>
-using std::cout;
-using std::endl;
-#endif
-
 using std::memset;
 using std::rand;
+using std::vector;
 
 /*
  *  Create the cached data storage for a chunk.
  */
 
 Chunk_cache::Chunk_cache(
-) : egg_objects(4) {
-	memset(&eggs[0], 0, sizeof(eggs));
-}
-
-/*
- *  Delete cache.
- */
-
-Chunk_cache::~Chunk_cache(
-) {
-	for (vector<blocked8z>::iterator it = blocked.begin();
-	        it != blocked.end(); ++it)
-		delete[] *it;
+) : egg_objects(4), eggs{} {
 }
 
 /*
  *  This mask gives the low bits (b0) for a given # of ztiles.
  */
-unsigned long tmasks[8] = {     0x0L,
-                                0x1L,
-                                0x5L,
-                                0x15L,
-                                0x55L,
-                                0x155L,
-                                0x555L,
-                                0x1555L
-                          };
+unsigned long tmasks[8] = {
+    0x0L,
+    0x1L,
+    0x5L,
+    0x15L,
+    0x55L,
+    0x155L,
+    0x555L,
+    0x1555L
+};
 
 /*
  *  Set (actually, increment count) for a given tile.
@@ -93,21 +78,21 @@ unsigned long tmasks[8] = {     0x0L,
  *      newb1 =  b1 OR b0
  */
 inline void Set_blocked_tile(
-    uint16 *blocked,        // 16x16 flags,
+    Chunk_cache::blocked8z &blocked,        // 16x16 flags,
     int tx, int ty,         // Tile #'s (0-15).
     int lift,           // Starting lift to set.
     int ztiles          // # tiles along z-axis.
 ) {
-	uint16 &val = blocked[ty * c_tiles_per_chunk + tx];
+	uint16 val = blocked[ty * c_tiles_per_chunk + tx];
 	// Get mask for the bit0's:
-	uint16 mask0 = static_cast<uint16>(tmasks[ztiles] << 2 * lift);
+	auto mask0 = static_cast<uint16>(tmasks[ztiles] << 2 * lift);
 	uint16 mask1 = mask0 << 1;  // Mask for the bit1's.
 	uint16 val0s = val & mask0;
 	uint16 Nval0s = (~val)&mask0;
 	uint16 val1s = val & mask1;
 	uint16 newval = val1s | (val0s << 1) | Nval0s | (val1s >> 1);
 	// Replace old values with new.
-	val = (val&~(mask0 | mask1)) | newval;
+	blocked[ty * c_tiles_per_chunk + tx] = (val&~(mask0 | mask1)) | newval;
 }
 
 /*
@@ -118,21 +103,21 @@ inline void Set_blocked_tile(
  *      newb1 =  b1 AND  b0
  */
 inline void Clear_blocked_tile(
-    uint16 *blocked,        // 16x16 flags,
+    Chunk_cache::blocked8z &blocked,        // 16x16 flags,
     int tx, int ty,         // Tile #'s (0-15).
     int lift,           // Starting lift to set.
     int ztiles          // # tiles along z-axis.
 ) {
-	uint16 &val = blocked[ty * c_tiles_per_chunk + tx];
+	uint16 val = blocked[ty * c_tiles_per_chunk + tx];
 	// Get mask for the bit0's:
-	uint16 mask0 = static_cast<uint16>(tmasks[ztiles] << 2 * lift);
+	auto mask0 = static_cast<uint16>(tmasks[ztiles] << 2 * lift);
 	uint16 mask1 = mask0 << 1;  // Mask for the bit1's.
 	uint16 val0s = val & mask0;
 	uint16 Nval0s = (~val)&mask0;
 	uint16 val1s = val & mask1;
 	uint16 newval = (val1s & (val0s << 1)) | ((val1s >> 1) & Nval0s);
 	// Replace old values with new.
-	val = (val&~(mask0 | mask1)) | newval;
+	blocked[ty * c_tiles_per_chunk + tx] = (val&~(mask0 | mask1)) | newval;
 }
 
 /*
@@ -140,16 +125,13 @@ inline void Clear_blocked_tile(
  *  covers 8 lifts.
  */
 
-Chunk_cache::blocked8z Chunk_cache::new_blocked_level(
+Chunk_cache::blocked8z &Chunk_cache::new_blocked_level(
     int zlevel
 ) {
 	if (static_cast<unsigned>(zlevel) >= blocked.size())
 		blocked.resize(zlevel + 1);
-	blocked8z block = blocked[zlevel] = new uint16[256];
-//	std::cout << "***Creating block for level " << zlevel << ", cache = "
-//		<< (void*)this << std::endl;
-	memset(block, 0, 256 * sizeof(uint16));
-	return block;
+	blocked[zlevel] = std::make_unique<uint16[]>(256);
+	return blocked[zlevel];
 }
 
 /*
@@ -163,10 +145,12 @@ void Chunk_cache::set_blocked(
 ) {
 	int z = lift;
 	while (ztiles) {
-		int zlevel = z / 8, thisz = z % 8, zcnt = 8 - thisz;
+		int zlevel = z / 8;
+		int thisz = z % 8;
+		int zcnt = 8 - thisz;
 		if (ztiles < zcnt)
 			zcnt = ztiles;
-		uint16 *block = need_blocked_level(zlevel);
+		auto &block = need_blocked_level(zlevel);
 		for (int y = starty; y <= endy; y++)
 			for (int x = startx; x <= endx; x++)
 				Set_blocked_tile(block, x, y, thisz, zcnt);
@@ -183,12 +167,13 @@ void Chunk_cache::clear_blocked(
 	int z = lift;
 	while (ztiles) {
 		unsigned int zlevel = z / 8;
-		int thisz = z % 8, zcnt = 8 - thisz;
+		int thisz = z % 8;
+		int zcnt = 8 - thisz;
 		if (zlevel >= blocked.size())
 			break;      // All done.
 		if (ztiles < zcnt)
 			zcnt = ztiles;
-		uint16 *block = blocked[zlevel];
+		auto &block = blocked[zlevel];
 		if (block) {
 			for (int y = starty; y <= endy; y++)
 				for (int x = startx; x <= endx; x++)
@@ -237,11 +222,12 @@ void Chunk_cache::update_object(
 			                   endx, endy, lift % 8, ztiles);
 		return;
 	}
-	Rectangle footprint = obj->get_footprint();
+	TileRect footprint = obj->get_footprint();
 	// Go through interesected chunks.
 	Chunk_intersect_iterator next_chunk(footprint);
-	Rectangle tiles;
-	int cx, cy;
+	TileRect tiles;
+	int cx;
+	int cy;
 	while (next_chunk.get_next(tiles, cx, cy))
 		gmap->get_chunk(cx, cy)->set_blocked(tiles.x, tiles.y,
 		                                     tiles.x + tiles.w - 1, tiles.y + tiles.h - 1, lift,
@@ -255,17 +241,17 @@ void Chunk_cache::update_object(
 
 void Chunk_cache::set_egged(
     Egg_object *egg,
-    Rectangle &tiles,       // Range of tiles within chunk.
+    TileRect &tiles,       // Range of tiles within chunk.
     bool add                // 1 to add, 0 to remove.
 ) {
 	// Egg already there?
-	int eggnum = -1, spot = -1;
-	for (Egg_vector::const_iterator it = egg_objects.begin();
-	        it != egg_objects.end(); ++it) {
+	int eggnum = -1;
+	int spot = -1;
+	for (auto it = egg_objects.begin(); it != egg_objects.end(); ++it) {
 		if (*it == egg) {
 			eggnum = it - egg_objects.begin();
 			break;
-		} else if (*it == 0 && spot == -1)
+		} else if (*it == nullptr && spot == -1)
 			spot = it - egg_objects.begin();
 	}
 	if (add) {
@@ -279,26 +265,26 @@ void Chunk_cache::set_egged(
 		if (eggnum > 15)    // We only have 16 bits.
 			eggnum = 15;
 		short mask = (1 << eggnum);
-		int stopx = tiles.x + tiles.w, stopy = tiles.y + tiles.h;
+		int stopx = tiles.x + tiles.w;
+		int stopy = tiles.y + tiles.h;
 		for (int ty = tiles.y; ty < stopy; ++ty)
 			for (int tx = tiles.x; tx < stopx; ++tx)
 				eggs[ty * c_tiles_per_chunk + tx] |= mask;
 	} else {            // Remove.
 		if (eggnum < 0)
 			return;     // Not there.
-		egg_objects[eggnum] = NULL;
+		egg_objects[eggnum] = nullptr;
 		if (eggnum >= 15) { // We only have 16 bits.
 			// Last one at 15 or above?
-			for (Egg_vector::const_iterator it =
-			            egg_objects.begin() + 15;
-			        it != egg_objects.end(); ++it)
-				if (*it != 0)
+			for (auto it = egg_objects.begin() + 15; it != egg_objects.end(); ++it)
+				if (*it != nullptr)
 					// No, so leave bits alone.
 					return;
 			eggnum = 15;
 		}
 		short mask = ~(1 << eggnum);
-		int stopx = tiles.x + tiles.w, stopy = tiles.y + tiles.h;
+		int stopx = tiles.x + tiles.w;
+		int stopy = tiles.y + tiles.h;
 		for (int ty = tiles.y; ty < stopy; ty++)
 			for (int tx = tiles.x; tx < stopx; tx++)
 				eggs[ty * c_tiles_per_chunk + tx] &= mask;
@@ -316,11 +302,12 @@ void Chunk_cache::update_egg(
 ) {
 	ignore_unused_variable_warning(chunk);
 	// Get footprint with abs. tiles.
-	Rectangle foot = egg->get_area();
+	TileRect foot = egg->get_area();
 	if (!foot.w)
 		return;         // Empty (probability = 0).
-	Rectangle crect;        // Gets tiles within each chunk.
-	int cx, cy;
+	TileRect crect;        // Gets tiles within each chunk.
+	int cx;
+	int cy;
 	if (egg->is_solid_area()) {
 		// Do solid rectangle.
 		Chunk_intersect_iterator all(foot);
@@ -329,10 +316,10 @@ void Chunk_cache::update_egg(
 		return;
 	}
 	// Just do the perimeter.
-	Rectangle top(foot.x, foot.y, foot.w, 1);
-	Rectangle bottom(foot.x, foot.y + foot.h - 1, foot.w, 1);
-	Rectangle left(foot.x, foot.y + 1, 1, foot.h - 2);
-	Rectangle right(foot.x + foot.w - 1, foot.y + 1, 1, foot.h - 2);
+	TileRect top(foot.x, foot.y, foot.w, 1);
+	TileRect bottom(foot.x, foot.y + foot.h - 1, foot.w, 1);
+	TileRect left(foot.x, foot.y + 1, 1, foot.h - 2);
+	TileRect right(foot.x + foot.w - 1, foot.y + 1, 1, foot.h - 2);
 	// Go through intersected chunks.
 	Chunk_intersect_iterator tops(top);
 	while (tops.get_next(crect, cx, cy))
@@ -358,7 +345,7 @@ void Chunk_cache::setup(
 ) {
 	Game_object *obj;       // Set 'blocked' tiles.
 	Object_iterator next(chunk->get_objects());
-	while ((obj = next.get_next()) != 0)
+	while ((obj = next.get_next()) != nullptr)
 		if (obj->is_egg())
 			update_egg(chunk, obj->as_egg(), true);
 		else
@@ -377,13 +364,14 @@ inline void Chunk_cache::set_tflags(
     int tx, int ty,
     int maxz
 ) {
-	int zlevel = maxz / 8, bsize = blocked.size();
+	int zlevel = maxz / 8;
+	int bsize = blocked.size();
 	if (zlevel >= bsize) {
 		memset(tflags + bsize, 0, (zlevel - bsize + 1)*sizeof(uint16));
 		zlevel = bsize - 1;
 	}
 	while (zlevel >= 0) {
-		uint16 *block = blocked[zlevel];
+		auto &block = blocked[zlevel];
 		tflags[zlevel--] = block ? block[ty * c_tiles_per_chunk + tx] : 0;
 	}
 	tflags_maxz = maxz;
@@ -473,12 +461,12 @@ inline void Check_terrain(
 /*
  *  Is a given square occupied at a given lift?
  *
- *  Output: 1 if so, else 0.
- *      If 0 (tile is free), new_lift contains the new height that
+ *  Output: true if so, else false.
+ *      If false (tile is free), new_lift contains the new height that
  *         an actor will be at if he walks onto the tile.
  */
 
-int Chunk_cache::is_blocked(
+bool Chunk_cache::is_blocked(
     int height,         // Height (in tiles) of obj. being
     //   tested.
     int lift,           // Given lift.
@@ -494,11 +482,19 @@ int Chunk_cache::is_blocked(
 	// and can only move horizontally
 	if (move_flags & MOVE_ETHEREAL) {
 		new_lift = lift;
-		return 0;
+		return false;
 	}
 	// Figure max lift allowed.
-	if (max_rise == -1)
-		max_rise = (move_flags & MOVE_FLY) ? max_drop : 1;
+	if (max_rise == -1) {
+		if ((move_flags & (MOVE_FLY | MOVE_MAPEDIT)) != 0) {
+			max_rise = max_drop;
+		} else if ((move_flags & MOVE_WALK)) {
+			max_rise = 1;
+		} else {
+			// Swim.
+			max_rise = 0;
+		}
+	}
 	int max_lift = lift + max_rise;
 	if (max_lift > 255)
 		max_lift = 255;     // As high as we can go.
@@ -516,10 +512,10 @@ int Chunk_cache::is_blocked(
 		// Look downwards.
 		new_lift = get_highest_blocked(lift) + 1;
 		if (new_lift >= lift)   // Couldn't drop?
-			return 1;
+			return true;
 		int new_high = get_lowest_blocked(new_lift);
 		if (new_high != -1 && new_high < new_lift + height)
-			return 1;   // Still blocked above.
+			return true;   // Still blocked above.
 	}
 	if (new_lift <= lift) {     // Not going up?  See if falling.
 		new_lift = (move_flags & MOVE_LEVITATE) ? lift :
@@ -530,13 +526,13 @@ int Chunk_cache::is_blocked(
 			if (move_flags & MOVE_MAPEDIT)
 				new_lift = lift - max_drop;
 			else
-				return 1;
+				return true;
 		}
 		int new_high = get_lowest_blocked(new_lift);
 
 		// Make sure that where we want to go is tall enough for us
 		if (new_high != -1 && new_high < (new_lift + height))
-			return 1;
+			return true;
 	}
 
 	// Found a new place to go, lets test if we can actually move there
@@ -544,27 +540,29 @@ int Chunk_cache::is_blocked(
 	// Lift 0 tests
 	if (new_lift == 0) {
 		if (move_flags & MOVE_MAPEDIT)
-			return 0;   // Map-editor, so anything is okay.
+			return false;   // Map-editor, so anything is okay.
 		int ter = 0;
 		Check_terrain(obj_list, tx, ty, ter);
-		if (ter & 2) {      // Water
-			if (move_flags & (MOVE_FLY | MOVE_SWIM))
-				return 0;
-			else
-				return 1;
-		} else if (ter & 1) { // Land
-			if (move_flags & (MOVE_FLY | MOVE_WALK))
-				return 0;
-			else
-				return 1;
-		} else if (ter & 4) { // Blocked
-			return 1;
-		} else  // Other
-			return 0;
+		bool const can_walk = (move_flags & MOVE_WALK) != 0;
+		bool const can_swim = (move_flags & MOVE_SWIM) != 0;
+		bool const can_fly  = (move_flags & MOVE_FLY ) != 0;
+		if (can_swim && !can_walk && !can_fly && (ter & 2) == 0) {
+			// Can only swim; do not allow to move outside of water.
+			return true;
+		}
+		if (can_walk && !can_swim && !can_fly && (ter & 2) != 0) {
+			// Can only walk; do not allow to move into water.
+			return true;
+		}
+		if (!can_swim && !can_fly && (ter & 4) != 0) {
+			// Can only walk and terrain is solid (and 0-height).
+			return true;
+		}
+		return false;
 	} else if (move_flags & (MOVE_FLY | MOVE_WALK))
-		return 0;
+		return false;
 
-	return 1;
+	return true;
 }
 
 /*
@@ -579,6 +577,8 @@ void Chunk_cache::activate_eggs(
     unsigned short eggbits,     // Eggs[tile].
     bool now            // Do them immediately.
 ) {
+	// Ensure we exist until the end of the function.
+	auto ownHandle = shared_from_this();
 	size_t i;               // Go through eggs.
 	for (i = 0; i < 8 * sizeof(eggbits) - 1 && eggbits;
 	        i++, eggbits = eggbits >> 1) {
@@ -614,11 +614,10 @@ void Chunk_cache::activate_eggs(
 Game_object *Chunk_cache::find_door(
     const Tile_coord& tile
 ) {
-	for (std::set<Game_object *>::iterator it = doors.begin();
-	        it != doors.end(); ++it)
-		if ((*it)->blocks(tile))
-			return *it; // Found it.
-	return 0;
+	for (auto *door : doors)
+		if (door->blocks(tile))
+			return door; // Found it.
+	return nullptr;
 }
 
 /*
@@ -629,23 +628,12 @@ Map_chunk::Map_chunk(
     Game_map *m,            // Map we'll belong to.
     int chunkx, int chunky      // Absolute chunk coords.
 ) : map(m),
-	terrain(0), objects(0), first_nonflat(0), from_below(0),
+	terrain(nullptr), objects(nullptr), first_nonflat(nullptr), from_below(0),
 	from_right(0), from_below_right(0), ice_dungeon(0x00),
-	dungeon_levels(0), cache(0), roof(0),
-	dungeon_lights(0), non_dungeon_lights(0),
+	dungeon_levels(nullptr), cache(nullptr), roof(0),
 	cx(chunkx), cy(chunky),  selected(false) {
 }
 
-/*
- *  Delete all objects contained within.
- */
-
-Map_chunk::~Map_chunk(
-) {
-	delete cache;
-	delete [] dungeon_levels;
-}
-
 /*
  *  Set terrain.  Even if the terrain is the same, it still reloads the
  *  'flat' objects.
@@ -662,15 +650,14 @@ void Map_chunk::set_terrain(
 			// Separate scope for Object_iterator.
 			Object_iterator it(get_objects());
 			Game_object *each;
-			while ((each = it.get_next()) != 0)
+			while ((each = it.get_next()) != nullptr)
 				// Kind of nasty, I know:
 				if (each->as_terrain())
 					removes.push_back(each);
 		}
-		for (Game_object_vector::const_iterator it = removes.begin();
-		        it != removes.end(); ++it)
+		for (auto *remove : removes)
 			// We don't want to edit the chunks here:
-			(*it)->Game_object::remove_this();
+			remove->Game_object::remove_this();
 	}
 	terrain = ter;
 	terrain->add_client();
@@ -680,15 +667,15 @@ void Map_chunk::set_terrain(
 			ShapeID id = ter->get_flat(tilex, tiley);
 			Shape_frame *shape = id.get_shape();
 			if (shape && shape->is_rle()) {
-				int shapenum = id.get_shapenum(),
-				    framenum = id.get_framenum();
+				int shapenum = id.get_shapenum();
+				int framenum = id.get_framenum();
 				const Shape_info &info = id.get_info();
-				Game_object *obj = info.is_animated() ?
-				                   new Animated_object(shapenum,
-				                                       framenum, tilex, tiley)
-				                   : new Terrain_game_object(shapenum,
+				Game_object_shared obj = info.is_animated() ?
+				            std::make_shared<Animated_object>(shapenum,
+				                              framenum, tilex, tiley)
+				          : std::make_shared<Terrain_game_object>(shapenum,
 				                           framenum, tilex, tiley);
-				add(obj);
+				add(obj.get());
 			}
 		}
 }
@@ -703,7 +690,7 @@ void Map_chunk::add_dependencies(
 ) {
 	Game_object *obj;       // Figure dependencies.
 	Nonflat_object_iterator next(this);
-	while ((obj = next.get_next()) != 0) {
+	while ((obj = next.get_next()) != nullptr) {
 		//cout << "Here " << __LINE__ << " " << obj << endl;
 		/* Compare returns -1 if lt, 0 if dont_care, 1 if gt. */
 		int cmp = Game_object::compare(newinfo, obj);
@@ -752,11 +739,12 @@ void Map_chunk::add(
 ) {
 	newobj->chunk = this;       // Set object's chunk.
 	Ordering_info ord(gwin, newobj);
+	Game_object_shared newobj_shared = newobj->shared_from_this();
 	// Put past flats.
 	if (first_nonflat)
-		objects.insert_before(newobj, first_nonflat);
+		objects.insert_before(newobj_shared, first_nonflat);
 	else
-		objects.append(newobj);
+		objects.append(newobj_shared);
 	// Not flat?
 	if (newobj->get_lift() || ord.info.get_3d_height()) {
 		// Deal with dependencies.
@@ -793,9 +781,9 @@ void Map_chunk::add(
 	if (ord.info.is_light_source()) { // Count light sources.
 		if (dungeon_levels && is_dungeon(newobj->get_tx(),
 		                                 newobj->get_ty()))
-			dungeon_lights++;
+			dungeon_lights.insert(newobj);
 		else
-			non_dungeon_lights++;
+			non_dungeon_lights.insert(newobj);
 	}
 	if (newobj->get_lift() >= 5) {  // Looks like a roof?
 		if (ord.info.get_shape_class() == Shape_info::building)
@@ -823,14 +811,14 @@ void Map_chunk::add_egg(
 void Map_chunk::remove_egg(
     Egg_object *egg
 ) {
-	remove(egg);            // Remove it normally.
 	if (cache)          // Remove from cache.
 		cache->update_egg(this, egg, false);
+	remove(egg);            // Remove it normally.
 }
 
 /*
  *  Remove a game object from this list.  The object's 'chunk' field
- *  is set to NULL.
+ *  is set to nullptr.
  */
 
 void Map_chunk::remove(
@@ -843,8 +831,9 @@ void Map_chunk::remove(
 	Game_map *gmap = gwin->get_map();
 	const Shape_info &info = remove->get_info();
 	// See if it extends outside.
-	int frame = remove->get_framenum(), tx = remove->get_tx(),
-	    ty = remove->get_ty();
+	int frame = remove->get_framenum();
+	int tx = remove->get_tx();
+	int ty = remove->get_ty();
 	/* Let's try boundary. YES.  Helps with statues through roofs. */
 	bool ext_left = (tx - info.get_3d_xtiles(frame)) < 0 && cx > 0;
 	bool ext_above = (ty - info.get_3d_ytiles(frame)) < 0 && cy > 0;
@@ -857,29 +846,29 @@ void Map_chunk::remove(
 		gmap->get_chunk(cx, cy - 1)->from_below--;
 	if (info.is_light_source()) { // Count light sources.
 		if (dungeon_levels && is_dungeon(tx, ty))
-			dungeon_lights--;
+			dungeon_lights.erase(remove);
 		else
-			non_dungeon_lights--;
+			non_dungeon_lights.erase(remove);
 	}
 	if (remove == first_nonflat) {  // First nonflat?
 		// Update.
 		first_nonflat = remove->get_next();
 		if (first_nonflat == objects.get_first())
-			first_nonflat = 0;
+			first_nonflat = nullptr;
 	}
-	objects.remove(remove);     // Remove from list.
 	remove->set_invalid();      // No longer part of world.
+	objects.remove(remove);     // Remove from list.
 }
 
 /*
  *  Is a given rectangle of tiles blocked at a given lift?
  *
- *  Output: 1 if so, else 0.
- *      If 0 (tile is free), new_lift contains the new height that
+ *  Output: true if so, else false.
+ *      If false (tile is free), new_lift contains the new height that
  *         an actor will be at if he walks onto the tile.
  */
 
-int Map_chunk::is_blocked(
+bool Map_chunk::is_blocked(
     int height,         // Height (along lift) to check.
     int lift,           // Starting lift.
     int startx, int starty,     // Starting tile coords.
@@ -891,15 +880,17 @@ int Map_chunk::is_blocked(
     //   viour (max_drop if FLY, else 1).
 ) {
 	Game_map *gmap = gwin->get_map();
-	int tx, ty;
+	int tx;
+	int ty;
 	new_lift = 0;
 	startx = (startx + c_num_tiles) % c_num_tiles;      // Watch for wrapping.
 	starty = (starty + c_num_tiles) % c_num_tiles;
-	int stopy = (starty + ytiles) % c_num_tiles,
-	    stopx = (startx + xtiles) % c_num_tiles;
+	int stopy = (starty + ytiles) % c_num_tiles;
+	int stopx = (startx + xtiles) % c_num_tiles;
 	for (ty = starty; ty != stopy; ty = INCR_TILE(ty)) {
 		// Get y chunk, tile-in-chunk.
-		int cy = ty / c_tiles_per_chunk, rty = ty % c_tiles_per_chunk;
+		int cy = ty / c_tiles_per_chunk;
+		int rty = ty % c_tiles_per_chunk;
 		for (tx = startx; tx != stopx; tx = INCR_TILE(tx)) {
 			int this_lift;
 			Map_chunk *olist = gmap->get_chunk(
@@ -908,23 +899,23 @@ int Map_chunk::is_blocked(
 			if (olist->is_blocked(height, lift,
 			                      tx % c_tiles_per_chunk,
 			                      rty, this_lift, move_flags, max_drop, max_rise))
-				return (1);
+				return true;
 			// Take highest one.
 			new_lift = this_lift > new_lift ?
 			           this_lift : new_lift;
 		}
 	}
-	return (0);
+	return false;
 }
 
 /*
  *  Check an absolute tile position.
  *
- *  Output: 1 if blocked, 0 otherwise.
+ *  Output: true if blocked, false otherwise.
  *      Tile.tz may be updated for stepping onto square.
  */
 
-int Map_chunk::is_blocked(
+bool Map_chunk::is_blocked(
     Tile_coord &tile,
     int height,         // Height in tiles to check.
     const int move_flags,
@@ -937,15 +928,15 @@ int Map_chunk::is_blocked(
 	Map_chunk *chunk = gmap->get_chunk_safely(
 	                       tile.tx / c_tiles_per_chunk, tile.ty / c_tiles_per_chunk);
 	if (!chunk)         // Outside the world?
-		return 0;       // Then it's not blocked.
+		return false;       // Then it's not blocked.
 	chunk->setup_cache();       // Be sure cache is present.
 	int new_lift;           // Check it within chunk.
 	if (chunk->is_blocked(height, tile.tz, tile.tx % c_tiles_per_chunk,
 	                      tile.ty % c_tiles_per_chunk, new_lift, move_flags, max_drop,
 	                      max_rise))
-		return (1);
+		return true;
 	tile.tz = new_lift;
-	return (0);
+	return false;
 }
 
 /*
@@ -953,7 +944,7 @@ int Map_chunk::is_blocked(
  *  step onto an adjacent square.
  */
 
-int Map_chunk::is_blocked(
+bool Map_chunk::is_blocked(
     // Object dims:
     int xtiles, int ytiles, int ztiles,
     Tile_coord const &from,     // Stepping from here.
@@ -963,13 +954,17 @@ int Map_chunk::is_blocked(
     int max_rise            // Max. rise, or -1 to use old beha-
     //   viour (max_drop if FLY, else 1).
 ) {
-	int vertx0, vertx1;     // Get x-coords. of vert. block
+	int vertx0;
+	int vertx1;     // Get x-coords. of vert. block
 	//   to right/left.
-	int horizx0, horizx1;       // Get x-coords of horiz. block
+	int horizx0;
+	int horizx1;       // Get x-coords of horiz. block
 	//   above/below.
-	int verty0, verty1;     // Get y-coords of horiz. block
+	int verty0;
+	int verty1;     // Get y-coords of horiz. block
 	//   above/below.
-	int horizy0, horizy1;       // Get y-coords of vert. block
+	int horizy0;
+	int horizy1;       // Get y-coords of vert. block
 	//   to right/left.
 	// !Watch for wrapping.
 	horizx0 = (to.tx + 1 - xtiles + c_num_tiles) % c_num_tiles;
@@ -996,7 +991,8 @@ int Map_chunk::is_blocked(
 		// Includes top of vert. area.
 		verty0 = INCR_TILE(verty0);
 	}
-	int x, y;           // Go through horiz. part.
+	int x;
+	int y;           // Go through horiz. part.
 	int new_lift = from.tz;
 	int new_lift0 = -1;     // All lift changes must be same.
 #ifdef DEBUG
@@ -1007,7 +1003,8 @@ int Map_chunk::is_blocked(
 #endif
 	for (y = horizy0; y != horizy1; y = INCR_TILE(y)) {
 		// Get y chunk, tile-in-chunk.
-		int cy = y / c_tiles_per_chunk, rty = y % c_tiles_per_chunk;
+		int cy = y / c_tiles_per_chunk;
+		int rty = y % c_tiles_per_chunk;
 		for (x = horizx0; x != horizx1; x = INCR_TILE(x)) {
 			Map_chunk *olist = gmap->get_chunk(
 			                       x / c_tiles_per_chunk, cy);
@@ -1015,19 +1012,20 @@ int Map_chunk::is_blocked(
 			int rtx = x % c_tiles_per_chunk;
 			if (olist->is_blocked(ztiles, from.tz, rtx, rty,
 			                      new_lift, move_flags, max_drop, max_rise))
-				return 1;
+				return true;
 			if (new_lift != from.tz) {
 				if (new_lift0 == -1)
 					new_lift0 = new_lift;
 				else if (new_lift != new_lift0)
-					return (1);
+					return true;
 			}
 		}
 	}
 	// Do vert. block.
 	for (x = vertx0; x != vertx1; x = INCR_TILE(x)) {
 		// Get x chunk, tile-in-chunk.
-		int cx = x / c_tiles_per_chunk, rtx = x % c_tiles_per_chunk;
+		int cx = x / c_tiles_per_chunk;
+		int rtx = x % c_tiles_per_chunk;
 		for (y = verty0; y != verty1; y = INCR_TILE(y)) {
 			Map_chunk *olist = gmap->get_chunk(
 			                       cx, y / c_tiles_per_chunk);
@@ -1035,17 +1033,17 @@ int Map_chunk::is_blocked(
 			int rty = y % c_tiles_per_chunk;
 			if (olist->is_blocked(ztiles, from.tz, rtx, rty,
 			                      new_lift, move_flags, max_drop, max_rise))
-				return 1;
+				return true;
 			if (new_lift != from.tz) {
 				if (new_lift0 == -1)
 					new_lift0 = new_lift;
 				else if (new_lift != new_lift0)
-					return (1);
+					return true;
 			}
 		}
 	}
 	to.tz = new_lift;
-	return (0);         // All clear.
+	return false;         // All clear.
 }
 
 /*
@@ -1055,32 +1053,36 @@ int Map_chunk::is_blocked(
  *         clockwise.  List is on heap.
  */
 
-static Tile_coord *Get_square(
-    Tile_coord &pos,        // Center of square.
+static auto Get_square(
+    Tile_coord &pos,    // Center of square.
     int dist            // Distance to perimeter (>0)
 ) {
-	Tile_coord *square = new Tile_coord[8 * dist];
+	std::vector<Tile_coord> square;
+	square.reserve(8 * dist);
 	// Upper left corner:
-	square[0] = Tile_coord(DECR_TILE(pos.tx, dist),
-	                       DECR_TILE(pos.ty, dist), pos.tz);
-	int i;              // Start with top row.
-	int len = 2 * dist + 1;
+	square.emplace_back(DECR_TILE(pos.tx, dist),
+	                    DECR_TILE(pos.ty, dist), pos.tz);
+	const int len = 2 * dist + 1;
 	int out = 1;
-	for (i = 1; i < len; i++, out++)
-		square[out] = Tile_coord(INCR_TILE(square[out - 1].tx),
-		                         square[out - 1].ty, pos.tz);
+	for (int i = 1; i < len; i++, out++) {
+		const auto &back = square.back();
+		square.emplace_back(INCR_TILE(back.tx), back.ty, pos.tz);
+	}
 	// Down right side.
-	for (i = 1; i < len; i++, out++)
-		square[out] = Tile_coord(square[out - 1].tx,
-		                         INCR_TILE(square[out - 1].ty), pos.tz);
+	for (int i = 1; i < len; i++, out++) {
+		const auto &back = square.back();
+		square.emplace_back(back.tx, INCR_TILE(back.ty), pos.tz);
+	}
 	// Bottom, going back to left.
-	for (i = 1; i < len; i++, out++)
-		square[out] = Tile_coord(DECR_TILE(square[out - 1].tx),
-		                         square[out - 1].ty, pos.tz);
+	for (int i = 1; i < len; i++, out++) {
+		const auto &back = square.back();
+		square.emplace_back(DECR_TILE(back.tx), back.ty, pos.tz);
+	}
 	// Left side, going up.
-	for (i = 1; i < len - 1; i++, out++)
-		square[out] = Tile_coord(square[out - 1].tx,
-		                         DECR_TILE(square[out - 1].ty), pos.tz);
+	for (int i = 1; i < len - 1; i++, out++) {
+		const auto &back = square.back();
+		square.emplace_back(back.tx, DECR_TILE(back.ty), pos.tz);
+	}
 	return square;
 }
 
@@ -1095,7 +1097,8 @@ inline bool Check_spot(
     int tx, int ty, int tz
 ) {
 	Game_map *gmap = Game_window::get_instance()->get_map();
-	int cx = tx / c_tiles_per_chunk, cy = ty / c_tiles_per_chunk;
+	int cx = tx / c_tiles_per_chunk;
+	int cy = ty / c_tiles_per_chunk;
 	Map_chunk *chunk = gmap->get_chunk_safely(cx, cy);
 	if (!chunk)
 		return false;
@@ -1139,24 +1142,21 @@ Tile_coord Map_chunk::find_spot(
 	for (int d = 1; d <= dist; d++) { // Look outwards.
 		int square_cnt = 8 * d    ; // # tiles in square's perim.
 		// Get square (starting in NW).
-		Tile_coord *square = Get_square(pos, d);
+		const auto square = Get_square(pos, d);
 		int index = dir * d; // Get index of preferred spot.
 		// Get start of preferred range.
 		index = (index - d / 2 + square_cnt) % square_cnt;
 		for (int cnt = square_cnt; cnt; cnt--, index++) {
-			Tile_coord &p = square[index % square_cnt];
+			const Tile_coord &p = square[index % square_cnt];
 			if (!Map_chunk::is_blocked(zs, p.tz, p.tx - xs + 1,
 			                           p.ty - ys + 1, xs, ys, new_lift, mflags,
 			                           max_drop) &&
 			        (where == anywhere ||
 			         Check_spot(where, p.tx, p.ty, new_lift))) {
 				// Use tile before deleting.
-				Tile_coord ret(p.tx, p.ty, new_lift);
-				delete [] square;
-				return ret;
+				return Tile_coord(p.tx, p.ty, new_lift);
 			}
 		}
-		delete [] square;
 	}
 	return Tile_coord(-1, -1, -1);
 }
@@ -1191,15 +1191,16 @@ Tile_coord Map_chunk::find_spot(
 
 int Map_chunk::find_in_area(
     Game_object_vector &vec,    // Returned here.
-    Rectangle const &area,          // Area to search.
+    TileRect const &area,          // Area to search.
     int shapenum,
     int framenum
 ) {
 	int savesize = vec.size();
 	// Go through interesected chunks.
 	Chunk_intersect_iterator next_chunk(area);
-	Rectangle tiles;        // (Tiles within intersected chunk).
-	int eachcx, eachcy;
+	TileRect tiles;        // (Tiles within intersected chunk).
+	int eachcx;
+	int eachcy;
 	Game_map *gmap = gwin->get_map();
 	while (next_chunk.get_next(tiles, eachcx, eachcy)) {
 		Map_chunk *chunk = gmap->get_chunk_safely(eachcx, eachcy);
@@ -1207,7 +1208,7 @@ int Map_chunk::find_in_area(
 			continue;
 		Object_iterator next(chunk->objects);
 		Game_object *each;
-		while ((each = next.get_next()) != 0)
+		while ((each = next.get_next()) != nullptr)
 			if (each->get_shapenum() == shapenum &&
 			        each->get_framenum() == framenum &&
 			        tiles.has_world_point(each->get_tx(), each->get_ty()))
@@ -1234,11 +1235,12 @@ void Map_chunk::try_all_eggs(
 	Game_map *gmap = gwin->get_map();
 	Tile_coord pos = obj->get_tile();
 	const int dist = 32;        // See if this works okay.
-	Rectangle area(pos.tx - dist, pos.ty - dist, 2 * dist, 2 * dist);
+	TileRect area(pos.tx - dist, pos.ty - dist, 2 * dist, 2 * dist);
 	// Go through interesected chunks.
 	Chunk_intersect_iterator next_chunk(area);
-	Rectangle tiles;        // (Ignored).
-	int eachcx, eachcy;
+	TileRect tiles;        // (Ignored).
+	int eachcx;
+	int eachcy;
 	Egg_vector eggs;        // Get them here first, as activating
 	eggs.reserve(40);
 	//   an egg could affect chunk's list.
@@ -1249,7 +1251,7 @@ void Map_chunk::try_all_eggs(
 		chunk->setup_cache();   // I think we should do this.
 		Object_iterator next(chunk->objects);
 		Game_object *each;
-		while ((each = next.get_next()) != 0)
+		while ((each = next.get_next()) != nullptr)
 			if (each->is_egg()) {
 				Egg_object *egg = each->as_egg();
 				// Music eggs are causing problems.
@@ -1260,9 +1262,8 @@ void Map_chunk::try_all_eggs(
 					eggs.push_back(egg);
 			}
 	}
-	for (Egg_vector::const_iterator it = eggs.begin(); it != eggs.end();
-	        ++it)
-		(*it)->hatch(obj);
+	for (auto *egg : eggs)
+		egg->hatch(obj);
 	norecurse--;
 }
 
@@ -1271,26 +1272,21 @@ void Map_chunk::try_all_eggs(
  */
 
 void Map_chunk::add_dungeon_levels(
-    Rectangle &tiles, unsigned int lift
+    TileRect &tiles, unsigned int lift
 ) {
 	if (!dungeon_levels) {
 		// First one found.
-		dungeon_levels = new unsigned char[256 / 2];
-		memset(dungeon_levels, 0, 256 / 2);
+		dungeon_levels = std::make_unique<unsigned char[]>(256);
 	}
-	int endy = tiles.y + tiles.h, endx = tiles.x + tiles.w;
+	int endy = tiles.y + tiles.h;
+	int endx = tiles.x + tiles.w;
 	for (int ty = tiles.y; ty < endy; ty++) {
 		for (int tx = tiles.x; tx < endx; tx++) {
-			int tnum = (ty * c_tiles_per_chunk + tx) / 2;
-			if (GAME_SI)    // SI has roofs at random levels!!
+			if (GAME_SI) {
+				// SI has roofs at random levels!!
 				lift = 5;
-			if (tx % 2) {
-				dungeon_levels[tnum] &= 0x0F;
-				dungeon_levels[tnum] |= lift << 4;
-			} else {
-				dungeon_levels[tnum] &= 0xF0;
-				dungeon_levels[tnum] |= lift;
 			}
+			dungeon_levels[ty * c_tiles_per_chunk + tx] = lift;
 		}
 	}
 }
@@ -1305,25 +1301,26 @@ void Map_chunk::setup_dungeon_levels(
 
 	Object_iterator next(objects);
 	Game_object *each;
-	while ((each = next.get_next()) != 0) {
+	while ((each = next.get_next()) != nullptr) {
 		// Test for mountain-tops.
 		const Shape_info &shinf = each->get_info();
 		if (shinf.get_shape_class() == Shape_info::building &&
 		        shinf.get_mountain_top_type() == Shape_info::normal_mountain_top) {
 			// SI shape 941, frame 0 => do whole chunk (I think).
-			Rectangle area =
+			TileRect area =
 			    (shinf.has_translucency()
 			     && each->get_framenum() == 0)
-			    ? Rectangle(cx * c_tiles_per_chunk,
-			                cy * c_tiles_per_chunk,
-			                c_tiles_per_chunk,
-			                c_tiles_per_chunk)
+			    ? TileRect(cx * c_tiles_per_chunk,
+			               cy * c_tiles_per_chunk,
+			               c_tiles_per_chunk,
+			               c_tiles_per_chunk)
 			    : each->get_footprint();
 
 			// Go through interesected chunks.
 			Chunk_intersect_iterator next_chunk(area);
-			Rectangle tiles;// Rel. tiles.
-			int cx, cy;
+			TileRect tiles;// Rel. tiles.
+			int cx;
+			int cy;
 			while (next_chunk.get_next(tiles, cx, cy))
 				gmap->get_chunk(cx, cy)->add_dungeon_levels(
 				    tiles, each->get_lift());
@@ -1334,26 +1331,28 @@ void Map_chunk::setup_dungeon_levels(
 			// This is not exactly accurate.
 			ice_dungeon |= 1 << ((each->get_tx() >> 3) + 2 * (each->get_ty() >> 3));
 
-			Rectangle area = each->get_footprint();
+			TileRect area = each->get_footprint();
 
 			// Go through interesected chunks.
 			Chunk_intersect_iterator next_chunk(area);
-			Rectangle tiles;// Rel. tiles.
-			int cx, cy;
+			TileRect tiles;// Rel. tiles.
+			int cx;
+			int cy;
 			while (next_chunk.get_next(tiles, cx, cy))
 				gmap->get_chunk(cx, cy)->add_dungeon_levels(
 				    tiles, each->get_lift());
 		}
 	}
 	if (dungeon_levels) {   // Recount lights.
-		dungeon_lights = non_dungeon_lights = 0;
+		dungeon_lights.clear();
+		non_dungeon_lights.clear();
 		next.reset();
-		while ((each = next.get_next()) != 0)
+		while ((each = next.get_next()) != nullptr)
 			if (each->get_info().is_light_source()) {
 				if (is_dungeon(each->get_tx(), each->get_ty()))
-					dungeon_lights++;
+					dungeon_lights.insert(each);
 				else
-					non_dungeon_lights++;
+					non_dungeon_lights.insert(each);
 			}
 	}
 }
@@ -1364,27 +1363,29 @@ void Map_chunk::setup_dungeon_levels(
  */
 
 void Map_chunk::gravity(
-    Rectangle const &area,          // Unblocked tiles (in abs. coords).
+    TileRect const &area,          // Unblocked tiles (in abs. coords).
     int lift            // Lift where tiles are free.
 ) {
 	Game_object_vector dropped; // Gets list of objs. that dropped.
 	dropped.reserve(20);
 	// Go through interesected chunks.
 	Chunk_intersect_iterator next_chunk(area);
-	Rectangle tiles;        // Rel. tiles.  Not used.
-	int cx, cy, new_lift;
+	TileRect tiles;        // Rel. tiles.  Not used.
+	int cx;
+	int cy;
+	int new_lift;
 	while (next_chunk.get_next(tiles, cx, cy)) {
 		Map_chunk *chunk = gmap->get_chunk(cx, cy);
 		Object_iterator objs(chunk->objects);
 		Game_object *obj;
-		while ((obj = objs.get_next()) != 0) {
+		while ((obj = objs.get_next()) != nullptr) {
 			// We DO want NPC's to fall.
 			if (!obj->is_dragable() &&
 			        !obj->get_info().is_npc())
 				continue;
 			Tile_coord t = obj->get_tile();
 			// Get footprint.
-			Rectangle foot = obj->get_footprint();
+			TileRect foot = obj->get_footprint();
 			// Above area?
 			if (t.tz >= lift && foot.intersects(area) &&
 			        // Unblocked below itself?
@@ -1395,13 +1396,11 @@ void Map_chunk::gravity(
 				dropped.push_back(obj);
 		}
 	}
-	Game_object_vector::const_iterator it;
 	// Drop each one found.
-	for (it = dropped.begin(); it != dropped.end(); ++it) {
-		Game_object *obj = *it;
+	for (auto *obj : dropped) {
 		Tile_coord t = obj->get_tile();
 		// Get footprint.
-		Rectangle foot = obj->get_footprint();
+		TileRect foot = obj->get_footprint();
 		// Let drop as far as possible.
 		if (!is_blocked(1, t.tz - 1, foot.x, foot.y,
 		                foot.w, foot.h, new_lift, MOVE_ALL_TERRAIN, 100) && new_lift < t.tz) {
@@ -1411,16 +1410,6 @@ void Map_chunk::gravity(
 			        obj->get_info().get_3d_height());
 		}
 	}
-#if 0
-	for (it = dropped.begin(); it != dropped.end(); ++it) {
-		// Recurse on each one.
-		Game_object *obj = *it;
-		// Get footprint.
-		Rectangle foot = obj->get_footprint();
-		gravity(foot, obj->get_lift() +
-		        obj->get_info().get_3d_height());
-	}
-#endif
 }
 
 /*
@@ -1434,27 +1423,23 @@ void Map_chunk::gravity(
  *
  */
 int Map_chunk::is_roof(int tx, int ty, int lift) {
-#if 1       /* Might be lying on bed at lift==2. */
 	int height = get_lowest_blocked(lift + 4, tx, ty);
-#else       /* But this is even worse! */
-	int height = get_lowest_blocked(lift + 2, tx, ty);
-#endif
 	if (height == -1) return 255;
 	return height;
 }
 
 void Map_chunk::kill_cache() {
 	// Get rid of terrain
-	if (terrain) terrain->remove_client();
-	terrain = 0;
+	if (terrain) {
+		terrain->remove_client();
+	}
+	terrain = nullptr;
 
 	// Now remove the cachce
-	delete cache;
-	cache = 0;
+	cache.reset();
 
 	// Delete dungeon bits
-	delete [] dungeon_levels;
-	dungeon_levels = 0;
+	dungeon_levels.reset();
 }
 
 int Map_chunk::get_obj_actors(vector<Game_object *> &removes,
@@ -1465,11 +1450,11 @@ int Map_chunk::get_obj_actors(vector<Game_object *> &removes,
 	// Separate scope for Object_iterator.
 	Object_iterator it(get_objects());
 	Game_object *each;
-	while ((each = it.get_next()) != 0) {
+	while ((each = it.get_next()) != nullptr) {
 		Actor *actor = each->as_actor();
 
 		// Normal objects and monsters
-		if (actor == 0 || (each->is_monster() && each->get_flag(Obj_flags::is_temporary))) {
+		if (actor == nullptr || (each->is_monster() && each->get_flag(Obj_flags::is_temporary))) {
 			removes.push_back(each);
 			int ireg_size = each->get_ireg_size();
 
@@ -1484,3 +1469,11 @@ int Map_chunk::get_obj_actors(vector<Game_object *> &removes,
 
 	return failed ? -1 : buf_size;
 }
+
+void Map_chunk::write(ODataSource& out, bool v2) {
+	// Restore original order (sort of).
+	Object_iterator_backwards next(this);
+	Game_object *obj;
+	while ((obj = next.get_next()) != nullptr)
+		obj->write_ifix(&out, v2);
+}
