@@ -2,7 +2,7 @@
  *  contain.h - Container objects.
  *
  *  Copyright (C) 1998-1999  Jeffrey S. Freedman
- *  Copyright (C) 2000-2013  The Exult Team
+ *  Copyright (C) 2000-2022  The Exult Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,41 +29,36 @@
  *  A container object:
  */
 class Container_game_object : public Ireg_game_object {
-	int volume_used;        // Amount of volume occupied.
-	char resistance;    // Resistance to attack.
+	int volume_used = 0;        // Amount of volume occupied.
+	char resistance = 0;        // Resistance to attack.
 protected:
-	Object_list objects;        // ->first object.
+	Object_list objects = nullptr;        // ->first object.
 	int get_max_volume() const;
-	int  gumpX;
-	int  gumpY;
-	bool gumpInit;
+	int  gumpX = 0;
+	int  gumpY = 0;
+	bool gumpInit = false;
 public:
-	virtual void setGumpXY(int x, int y) { 
-		gumpX = x; 
-		gumpY = y;	
+	void setGumpXY(int x, int y) {
+		gumpX = x;
+		gumpY = y;
 		gumpInit = true;
 	}
-	virtual int  getGumpX() {
+	int  getGumpX() {
 		return gumpX;
 	}
-	virtual int  getGumpY() {
+	int  getGumpY() {
 		return gumpY;
 	}
-	virtual bool validGumpXY() {
+	bool validGumpXY() {
 		return gumpInit;
 	}
 	Container_game_object(int shapenum, int framenum, unsigned int tilex,
 	                      unsigned int tiley, unsigned int lft,
 	                      char res = 0)
 		: Ireg_game_object(shapenum, framenum, tilex, tiley, lft),
-		  volume_used(0), resistance(res), objects(0) {
-			gumpInit = false;
-	}
-	Container_game_object() : volume_used(0), resistance(0),
-		objects(0) {
-			gumpInit = false;
-	}
-	virtual ~Container_game_object();
+		  resistance(res), objects(nullptr)
+	{  }
+	Container_game_object() = default;
 	Object_list &get_objects() {
 		return objects;
 	}
@@ -72,15 +67,15 @@ public:
 		volume_used += delta;
 	}
 	// Room for this object?
-	int has_room(Game_object *obj) const {
+	bool has_room(Game_object *obj) const {
 		return get_max_volume() <= 0 ||
 		       obj->get_volume() + volume_used <= get_max_volume();
 	}
 	// Remove an object.
 	virtual void remove(Game_object *obj);
 	// Add an object.
-	virtual bool add(Game_object *obj, bool dont_check = false,
-	                 bool combine = false, bool noset = false);
+	bool add(Game_object *obj, bool dont_check = false,
+	         bool combine = false, bool noset = false) override;
 	// Change member shape.
 	virtual void change_member_shape(Game_object *obj, int newshape);
 	// Find object's spot.
@@ -90,61 +85,61 @@ public:
 	}
 	virtual Game_object *get_readied(int index) const {
 		ignore_unused_variable_warning(index);
-		return 0;
+		return nullptr;
 	}
 	virtual void call_readied_usecode(int index,
 	                                  Game_object *obj, int eventid) {
 		ignore_unused_variable_warning(index, obj, eventid);
 	}
 	// Add/remove quantities of objs.
-	virtual int add_quantity(int delta, int shapenum,
-	                         int qual = c_any_qual,
-	                         int framenum = c_any_framenum, bool dontcreate = false, bool temporary = false);
-	virtual int create_quantity(int delta, int shapenum, int qual,
-	                            int framenum, bool temporary = false);
-	virtual int remove_quantity(int delta, int shapenum, int qual,
-	                            int framenum);
-	virtual Game_object *find_item(int shapenum, int qual, int framenum);
+	int add_quantity(int delta, int shapenum,
+	                 int qual = c_any_qual,
+	                 int framenum = c_any_framenum, bool dontcreate = false, bool temporary = false) override;
+	int create_quantity(int delta, int shnum, int qual,
+	                    int frnum, bool temporary = false) override;
+	int remove_quantity(int delta, int shapenum, int qual,
+	                    int framenum) override;
+	Game_object *find_item(int shapenum, int qual, int framenum) override;
 	bool show_gump(int event = 1);
 	// Run usecode function.
-	virtual void activate(int event = 1);
-	virtual bool edit();        // Edit in ExultStudio.
+	void activate(int event = 1) override;
+	bool edit() override;        // Edit in ExultStudio.
 	// Saved from ExultStudio.
 	static void update_from_studio(unsigned char *data, int datalen);
-	virtual int get_weight();
+	int get_weight() override;
 	// Drop another onto this.
-	virtual int drop(Game_object *obj);
+	bool drop(Game_object *obj) override;
 	// Count contained objs.
-	virtual int count_objects(int shapenum, int qual = c_any_qual,
-	                          int framenum = c_any_framenum);
+	int count_objects(int shapenum, int qual = c_any_qual,
+	                          int framenum = c_any_framenum) override;
 	// Get contained objs.
-	virtual int get_objects(Game_object_vector &vec, int shapenum,
-	                        int qual, int framenum);
-	virtual void set_flag_recursively(int flag);
+	int get_objects(Game_object_vector &vec, int shapenum,
+	                int qual, int framenum) override;
+	void set_flag_recursively(int flag) override;
 	// Write out to IREG file.
-	virtual void write_ireg(ODataSource *out);
+	void write_ireg(ODataSource *out) override;
 	// Get size of IREG. Returns -1 if can't write to buffer
-	virtual int get_ireg_size();
+	int get_ireg_size() override;
 	// Write contents in IREG format.
 	virtual void write_contents(ODataSource *out);
 
-	virtual int get_obj_hp() const {
+	int get_obj_hp() const override {
 		return resistance;
 	}
-	virtual void set_obj_hp(int hp) {
+	void set_obj_hp(int hp) override {
 		resistance = static_cast<char>(hp);
 	}
 
-	virtual Game_object *find_weapon_ammo(int weapon, int needed = 1,
-	                                      bool recursive = false);
+	Game_object *find_weapon_ammo(int weapon, int needed = 1,
+	                              bool recursive = false) override;
 
 	bool extract_contents(Container_game_object *targ);
 
-	virtual void delete_contents();
+	void delete_contents() override;
 
-	virtual void remove_this(int nodel = 0);
+	void remove_this(Game_object_shared *keep = nullptr) override;
 
-	virtual Container_game_object *as_container() {
+	Container_game_object *as_container() override {
 		return this;
 	}
 
