@@ -1,6 +1,6 @@
 /*
  *  Copyright (C) 1998-1999  Jeffrey S. Freedman
- *  Copyright (C) 2000-2013  The Exult Team
+ *  Copyright (C) 2000-2022  The Exult Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,6 +21,8 @@
 #  include <config.h>
 #endif
 
+#include <memory>
+
 #include "Audio.h"
 #include "monsters.h"
 #include "cheat.h"
@@ -58,7 +60,7 @@ using std::rand;
 using std::ostream;
 using std::string;
 
-Egg_object *Egg_object::editing = 0;
+Game_object_shared Egg_object::editing;
 
 /*
  *  Timer for a missile egg (type-6 egg).
@@ -80,13 +82,9 @@ public:
 			range = winfo->get_range();
 		}
 		int crit = e->get_criteria();
-		if (crit == Egg_object::party_near || crit == Egg_object::avatar_near) {
-			chk_range = true;
-		} else {
-			chk_range = false;
-		}
+		chk_range = crit == Egg_object::party_near || crit == Egg_object::avatar_near;
 	}
-	virtual void handle_event(unsigned long curtime, uintptr udata);
+	void handle_event(unsigned long curtime, uintptr udata) override;
 };
 
 /*
@@ -108,16 +106,18 @@ void Missile_launcher::handle_event(
 			return;
 		}
 	}
-	Projectile_effect *proj = 0;
+	std::unique_ptr<Projectile_effect> proj{nullptr};
 	if (dir < 8) {          // Direction given?
 		// Get adjacent tile in direction.
 		Tile_coord adj = src.get_neighbor(dir % 8);
 		// Make it go (range) tiles.
-		int dx = adj.tx - src.tx, dy = adj.ty - src.ty;
+		int dx = adj.tx - src.tx;
+		int dy = adj.ty - src.ty;
 		Tile_coord dest = src;
 		dest.tx += range * dx;
 		dest.ty += range * dy;
-		proj = new Projectile_effect(egg, dest, weapon, shapenum, shapenum);
+		proj = std::make_unique<Projectile_effect>(egg,
+				dest, weapon, shapenum, shapenum);
 	} else {            // Target a party member.
 		Actor *party[9];
 		int psize = gwin->get_party(party, 1);
@@ -127,11 +127,11 @@ void Missile_launcher::handle_event(
 		for (int i = n; !proj && cnt; cnt--, i = (i + 1) % psize)
 			if (Fast_pathfinder_client::is_straight_path(src,
 			        party[i]->get_tile()))
-				proj = new Projectile_effect(
+				proj = std::make_unique<Projectile_effect>(
 				    src, party[i], weapon, shapenum, shapenum);
 	}
 	if (proj)
-		eman->add_effect(proj);
+		eman->add_effect(std::move(proj));
 	if (chk_range)
 		gwin->get_tqueue()->add(curtime + (delay > 0 ? delay : 1), this, udata);
 }
@@ -150,7 +150,7 @@ public:
 		: Egg_object(shnum, frnum, tx, ty, tz, itype, prob, d1, 0),
 		  score(d1 & 0xff), continuous(((d1 >> 8) & 1) != 0)
 	{  }
-	virtual void hatch_now(Game_object *obj, bool must) {
+	void hatch_now(Game_object *obj, bool must) override {
 		ignore_unused_variable_warning(obj, must);
 #ifdef DEBUG
 		cout << "Audio parameters might be: " << (data1 & 0xff) <<
@@ -167,7 +167,7 @@ public:
 	             unsigned char prob, uint16 d1)
 		: Jukebox_egg(shnum, frnum, tx, ty, tz, itype, prob, d1)
 	{  }
-	virtual void hatch_now(Game_object *obj, bool must) {
+	void hatch_now(Game_object *obj, bool must) override {
 		ignore_unused_variable_warning(obj, must);
 		Audio::get_ptr()->play_sound_effect(score, this, AUDIO_MAX_VOLUME,
 		                                    continuous);
@@ -181,7 +181,7 @@ public:
 	          unsigned char prob, uint16 d1)
 		: Egg_object(shnum, frnum, tx, ty, tz, itype, prob, d1, 0)
 	{  }
-	virtual void hatch_now(Game_object *obj, bool must) {
+	void hatch_now(Game_object *obj, bool must) override {
 		ignore_unused_variable_warning(obj, must);
 		ucmachine->do_speech(data1 & 0xff);
 	}
@@ -195,11 +195,12 @@ class Monster_egg : public Egg_object {
 		Tile_coord dest = Map_chunk::find_spot(
 		                      get_tile(), 5, mshape, 0, 1);
 		if (dest.tx != -1) {
-			Monster_actor *monster =
+			Game_object_shared new_monster =
 			    Monster_actor::create(mshape, dest, sched, align);
+			Game_object *monster = new_monster.get();
 			monster->change_frame(mframe);
 			gwin->add_dirty(monster);
-			gwin->add_nearby_npc(monster);
+			gwin->add_nearby_npc(monster->as_npc());
 		}
 	}
 public:
@@ -217,7 +218,7 @@ public:
 			mframe = d2 >> 10;
 		}
 	}
-	virtual void hatch_now(Game_object *obj, bool must) {
+	void hatch_now(Game_object *obj, bool must) override {
 		ignore_unused_variable_warning(obj, must);
 		const Shape_info &info = ShapeID::get_info(mshape);
 		if (info.is_npc()) {
@@ -229,13 +230,13 @@ public:
 			while (num--)
 				create_monster();
 		} else {        // Create item.
-			Game_object *nobj = get_map()->create_ireg_object(info,
+			Game_object_shared nobj = get_map()->create_ireg_object(info,
 			                    mshape, mframe, get_tx(), get_ty(), get_lift());
 			if (nobj->is_egg())
 				chunk->add_egg(nobj->as_egg());
 			else
-				chunk->add(nobj);
-			gwin->add_dirty(nobj);
+				chunk->add(nobj.get());
+			gwin->add_dirty(nobj.get());
 			nobj->set_flag(Obj_flags::okay_to_take);
 			// Objects are created temporary
 			nobj->set_flag(Obj_flags::is_temporary);
@@ -255,23 +256,23 @@ public:
 		set_quality(d1 & 0xff);
 		Usecode_egg::set_str1(fnm);
 	}
-	virtual void set_str1(const char *s) {
+	void set_str1(const char *s) override {
 		fun_name = s ? s : "";
 		if (s && *s)
 			fun = 0;    // Want to look this up.
 	}
-	virtual const char *get_str1() {
+	const char *get_str1() const override {
 		return fun_name.c_str();
 	}
-	virtual int get_usecode() const {
+	int get_usecode() const override {
 	    return fun;
 	}
-	virtual bool set_usecode(int funid, const char *nm = 0) {
+	bool set_usecode(int funid, const char *nm = nullptr) override {
 	    fun = funid;
 		fun_name = nm;
 		return true;
 	}
-	virtual void hatch_now(Game_object *obj, bool must) {
+	void hatch_now(Game_object *obj, bool must) override {
 		ignore_unused_variable_warning(obj);
 		if (!fun && !fun_name.empty())
 			fun = ucmachine->find_function(fun_name.c_str());
@@ -279,7 +280,7 @@ public:
 			ucmachine->call_usecode(fun, this,
 			                        Usecode_machine::egg_proximity);
 		else {          // Do on next animation frame.
-			Usecode_script *scr = new Usecode_script(this);
+			auto *scr = new Usecode_script(this);
 			scr->add(Ucscript::usecode, fun);
 			if (flags & (1 << static_cast<int>(once))) {
 				// Don't remove until done.
@@ -300,38 +301,38 @@ public:
 	            unsigned int tz, unsigned short itype,
 	            unsigned char prob, uint16 d1, uint16 d2)
 		: Egg_object(shnum, frnum, tx, ty, tz, itype, prob, d1, d2),
-		  weapon(d1), dir(d2 & 0xff), delay(d2 >> 8), launcher(0)
+		  weapon(d1), dir(d2 & 0xff), delay(d2 >> 8), launcher(nullptr)
 	{  }
-	virtual ~Missile_egg() {
+	~Missile_egg() override {
 		if (launcher) {
 			gwin->get_tqueue()->remove(launcher);
 			delete launcher;
 		}
 	}
-	virtual void remove_this(int nodel) {
+	void remove_this(Game_object_shared *keep) override {
 		if (launcher) {     // Stop missiles.
 			gwin->get_tqueue()->remove(launcher);
 			delete launcher;
-			launcher = 0;
+			launcher = nullptr;
 		}
-		Egg_object::remove_this(nodel);
+		Egg_object::remove_this(keep);
 	}
-	virtual void paint() {
+	void paint() override {
 		// Make sure launcher is active.
 		if (launcher && !launcher->in_queue() &&
 		    (criteria == party_near || criteria == avatar_near))
 			gwin->get_tqueue()->add(0L, launcher);
 		Egg_object::paint();
 	}
-	virtual void set(int crit, int dist) {
+	void set(int crit, int dist) override {
 		if (crit == external_criteria && launcher) {    // Cancel trap.
 			gwin->get_tqueue()->remove(launcher);
 			delete launcher;
-			launcher = 0;
+			launcher = nullptr;
 		}
 		Egg_object::set(crit, dist);
 	}
-	virtual void hatch_now(Game_object *obj, bool must) {
+	void hatch_now(Game_object *obj, bool must) override {
 		ignore_unused_variable_warning(obj, must);
 		const Shape_info &info = ShapeID::get_info(weapon);
 		const Weapon_info *winf = info.get_weapon_info();
@@ -367,7 +368,7 @@ public:
 		desty = (schunk / 12) * c_tiles_per_schunk + (d2 >> 8);
 		destz = d3 & 0xff;
 	}
-	virtual void hatch_now(Game_object *obj, bool must) {
+	void hatch_now(Game_object *obj, bool must) override {
 		ignore_unused_variable_warning(must);
 		Tile_coord pos(-1, -1, -1); // Get position to jump to.
 		int eggnum = 255;
@@ -403,7 +404,7 @@ public:
 		if (!len)       // Means continuous.
 			len = 120;  // How about a couple game hours?
 	}
-	virtual void hatch_now(Game_object *obj, bool must) {
+	void hatch_now(Game_object *obj, bool must) override {
 		ignore_unused_variable_warning(obj, must);
 		set_weather(weather, len, this);
 	}
@@ -418,19 +419,17 @@ public:
 		: Egg_object(shnum, frnum, tx, ty, tz, itype, prob, d1, d2),
 		  dist(d1 & 0xff)
 	{  }
-	virtual void hatch_now(Game_object *obj, bool must) {
+	void hatch_now(Game_object *obj, bool must) override {
 		ignore_unused_variable_warning(must);
 		Egg_vector eggs;
 		find_nearby_eggs(eggs, 275, dist);
 		find_nearby_eggs(eggs, 200, dist);
-		for (Egg_vector::const_iterator it = eggs.begin();
-		        it != eggs.end(); ++it) {
-			Egg_object *egg = *it;
+		for (auto *egg : eggs) {
 			if (egg != this &&
 			        egg->criteria == external_criteria &&
 			        // Attempting to fix problem in Silver Seed
 			        !(egg->flags & (1 << static_cast<int>(hatched))))
-				egg->hatch(obj, 0);
+				egg->hatch(obj, false);
 		}
 	}
 };
@@ -449,8 +448,8 @@ public:
  *  Create an "egg" from Ireg data.
  */
 
-Egg_object *Egg_object::create_egg(
-    unsigned char *entry,       // 12+ byte ireg entry.
+Egg_object_shared Egg_object::create_egg(
+    const unsigned char *entry,       // 12+ byte ireg entry.
     int entlen,
     bool animated,
     int shnum,
@@ -468,7 +467,7 @@ Egg_object *Egg_object::create_egg(
 	                  data1, data2, data3);
 }
 
-Egg_object *Egg_object::create_egg(
+Egg_object_shared Egg_object::create_egg(
     bool animated,
     int shnum, int frnum,
     unsigned int tx, unsigned int ty, unsigned int tz,
@@ -482,57 +481,57 @@ Egg_object *Egg_object::create_egg(
 	if (type == teleport && frnum == 6 && shnum == 275)
 		type = path;        // (Mountains N. of Vesper).
 
-	Egg_object *obj = 0;
+	Egg_object_shared obj;
 	switch (type) {     // The type:
 	case monster:
-		obj = new Monster_egg(shnum, frnum, tx, ty, tz, itype, prob,
+		obj = std::make_shared<Monster_egg>(shnum, frnum, tx, ty, tz, itype, prob,
 		                      data1, data2, data3);
 		break;
 	case jukebox:
-		obj = new Jukebox_egg(shnum, frnum, tx, ty, tz, itype, prob,
+		obj = std::make_shared<Jukebox_egg>(shnum, frnum, tx, ty, tz, itype, prob,
 		                      data1);
 		break;
 	case soundsfx:
-		obj = new Soundsfx_egg(shnum, frnum, tx, ty, tz, itype, prob,
+		obj = std::make_shared<Soundsfx_egg>(shnum, frnum, tx, ty, tz, itype, prob,
 		                       data1);
 		break;
 	case voice:
-		obj = new Voice_egg(shnum, frnum, tx, ty, tz, itype, prob,
+		obj = std::make_shared<Voice_egg>(shnum, frnum, tx, ty, tz, itype, prob,
 		                    data1);
 		break;
 	case usecode:
-		obj = new Usecode_egg(shnum, frnum, tx, ty, tz, itype, prob,
+		obj = std::make_shared<Usecode_egg>(shnum, frnum, tx, ty, tz, itype, prob,
 		                      data1, data2, str1);
 		break;
 	case missile:
-		obj = new Missile_egg(shnum, frnum, tx, ty, tz, itype, prob,
+		obj = std::make_shared<Missile_egg>(shnum, frnum, tx, ty, tz, itype, prob,
 		                      data1, data2);
 		break;
 	case teleport:
 	case intermap:
-		obj = new Teleport_egg(shnum, frnum, tx, ty, tz, itype, prob,
+		obj = std::make_shared<Teleport_egg>(shnum, frnum, tx, ty, tz, itype, prob,
 		                       data1, data2, data3);
 		break;
 	case weather:
-		obj = new Weather_egg(shnum, frnum, tx, ty, tz, itype, prob,
+		obj = std::make_shared<Weather_egg>(shnum, frnum, tx, ty, tz, itype, prob,
 		                      data1, data2);
 		break;
 	case path:
-		obj = new Path_egg(shnum, frnum, tx, ty, tz, itype, prob,
+		obj = std::make_shared<Path_egg>(shnum, frnum, tx, ty, tz, itype, prob,
 		                   data1, data2);
 		break;
 	case button:
-		obj = new Button_egg(shnum, frnum, tx, ty, tz, itype, prob,
+		obj = std::make_shared<Button_egg>(shnum, frnum, tx, ty, tz, itype, prob,
 		                     data1, data2);
 		break;
 	default:
 		cerr << "Illegal egg itype:  " << type << endl;
-		obj = new Egg_object(shnum, frnum, tx, ty, tz, itype, prob,
-		                     data1, data2, data3);
+		obj = std::make_shared<Egg_object>(shnum, frnum, tx, ty, tz, itype,
+			  									  prob, data1, data2, data3);
 	}
 	if (animated)
-		obj->set_animator(new Frame_animator(obj));
-	return (obj);
+		obj->set_animator(new Frame_animator(obj.get()));
+	return obj;
 }
 
 /*
@@ -549,7 +548,7 @@ void Egglike_game_object::paint(
  *  Can this be clicked on?
  */
 
-int Egglike_game_object::is_findable(
+bool Egglike_game_object::is_findable(
 ) {
 	return gwin->paint_eggs && Ireg_game_object::is_findable();
 }
@@ -567,7 +566,7 @@ Egg_object::Egg_object(
     short d1, short d2, short d3
 ) : Egglike_game_object(shapenum, framenum, tilex, tiley, lft),
 	probability(prob), data1(d1), data2(d2), data3(d3),
-	area(Rectangle(0, 0, 0, 0)), animator(0) {
+	area(TileRect(0, 0, 0, 0)), animator(nullptr) {
 	type = itype & 0xf;
 	// Teleport destination?
 	if (type == teleport && framenum == 6 && shapenum == 275)
@@ -600,11 +599,11 @@ inline void Egg_object::init_field(
 	type = ty;
 	probability = 100;
 	data1 = data2 = 0;
-	area = Rectangle(0, 0, 0, 0);
+	area = TileRect(0, 0, 0, 0);
 	criteria = party_footpad;
 	distance = 0;
 	solid_area = 0;
-	animator = 0;
+	animator = nullptr;
 	flags = (1 << auto_reset);
 }
 
@@ -638,26 +637,26 @@ Egg_object::~Egg_object(
 void Egg_object::set_area(
 ) {
 	if (!probability || type == path) { // No chance of normal activation?
-		area = Rectangle(0, 0, 0, 0);
+		area = TileRect(0, 0, 0, 0);
 		return;
 	}
 	Tile_coord t = get_tile();  // Get absolute tile coords.
 	switch (criteria) {     // Set up active area.
 	case cached_in:         // Make it really large.
-		area = Rectangle(t.tx - 32, t.ty - 32, 64, 64);
+		area = TileRect(t.tx - 32, t.ty - 32, 64, 64);
 		break;
 	case avatar_footpad:
 	case party_footpad: {
 		const Shape_info &info = get_info();
 		int frame = get_framenum();
-		int xtiles = info.get_3d_xtiles(frame),
-		    ytiles = info.get_3d_ytiles(frame);
-		area = Rectangle(t.tx - xtiles + 1, t.ty - ytiles + 1,
+		int xtiles = info.get_3d_xtiles(frame);
+		int ytiles = info.get_3d_ytiles(frame);
+		area = TileRect(t.tx - xtiles + 1, t.ty - ytiles + 1,
 		                 xtiles, ytiles);
 		break;
 	}
 	case avatar_far:        // Make it 1 tile bigger each dir.
-		area = Rectangle(t.tx - distance - 1, t.ty - distance - 1,
+		area = TileRect(t.tx - distance - 1, t.ty - distance - 1,
 		                 2 * distance + 3, 2 * distance + 3);
 		break;
 	default: {
@@ -668,13 +667,13 @@ void Egg_object::set_area(
 			if (criteria == external_criteria)
 				width += 2;
 		}
-		area = Rectangle(t.tx - distance, t.ty - distance,
+		area = TileRect(t.tx - distance, t.ty - distance,
 		                 width, width);
 		break;
 	}
 	}
 	// Don't go outside the world.
-	Rectangle world(0, 0, c_num_chunks * c_tiles_per_chunk,
+	TileRect world(0, 0, c_num_chunks * c_tiles_per_chunk,
 	                c_num_chunks * c_tiles_per_chunk);
 	area = area.intersect(world);
 }
@@ -682,7 +681,7 @@ void Egg_object::set_area(
 /*
  *  Can this be clicked on?
  */
-int Egg_object::is_findable() {
+bool Egg_object::is_findable() {
 	if (animator)
 		return Ireg_game_object::is_findable();
 	else
@@ -709,23 +708,23 @@ void Egg_object::set(
  *  on the spot?
  */
 
-int Egg_object::is_active(
+bool Egg_object::is_active(
     Game_object *obj,       // Object placed (or Actor).
     int tx, int ty, int tz,     // Tile stepped onto.
     int from_tx, int from_ty    // Tile stepped from.
 ) {
 	if (cheat.in_map_editor())
-		return 0;       // Disable in map-editor.
+		return false;       // Disable in map-editor.
 	if ((flags & (1 << static_cast<int>(hatched))) &&
 	        !(flags & (1 << static_cast<int>(auto_reset))))
-		return (0);     // For now... Already hatched.
+		return false;     // For now... Already hatched.
 	if (flags & (1 << static_cast<int>(nocturnal))) {
 		// Nocturnal.
 		int hour = gclock->get_hour();
 		if (!(hour >= 21 || hour <= 4))
-			return (0); // It's not night.
+			return false; // It's not night.
 	}
-	Egg_criteria cri = static_cast<Egg_criteria>(get_criteria());
+	auto cri = static_cast<Egg_criteria>(get_criteria());
 
 	int deltaz = tz - get_lift();
 	switch (cri) {
@@ -737,46 +736,44 @@ int Egg_object::is_active(
 			// Mark hatched if not auto-reset.
 			if (!(flags & (1 << static_cast<int>(auto_reset))))
 				flags |= (1 << static_cast<int>(hatched));
-			return 0;
+			return false;
 		}
 		if (obj != gwin->get_main_actor() || !area.has_world_point(tx, ty))
-			return 0;   // Not in square.
+			return false;   // Not in square.
 		if (!(flags & (1 << static_cast<int>(hatched))))
-			return 1;   // First time.
+			return true;   // First time.
 		// Must have autoreset.
 		// Just activate when reentering.
 		return !area.has_world_point(from_tx, from_ty);
 	}
 	case avatar_near:
 		if (obj != gwin->get_main_actor())
-			return 0;
+			return false;
 #ifdef DEBUG
 		print_debug();
 #endif
 		// fall through
 	case party_near:        // Avatar or party member.
 		if (!obj->get_flag(Obj_flags::in_party))
-			return 0;
+			return false;
 		if (type == teleport || // Teleports:  Any tile, exact lift.
 		        type == intermap)
 			return deltaz == 0 && area.has_world_point(tx, ty);
 		else if (type == jukebox || type == soundsfx || type == voice)
 			// Guessing. Fixes shrine of Spirituality and Sacrifice.
 			return area.has_world_point(tx, ty);
-		if (!((deltaz / 2 == 0 ||
+		return (deltaz / 2 == 0 ||
 		        // Using trial&error here:
 		        (Game::get_game_type() == SERPENT_ISLE &&
 		         type != missile) ||
 		        (type == missile && deltaz / 5 == 0)) &&
 		        // New tile is in, old is out.
 		        area.has_world_point(tx, ty) &&
-		        !area.has_world_point(from_tx, from_ty)))
-			return 0;
-		return 1;
+		        !area.has_world_point(from_tx, from_ty);
 	case avatar_far: {      // New tile is outside, old is inside.
 		if (obj != gwin->get_main_actor() || !area.has_world_point(tx, ty))
-			return (0);
-		Rectangle inside(area.x + 1, area.y + 1,
+			return false;
+		TileRect inside(area.x + 1, area.y + 1,
 		                 area.w - 2, area.h - 2);
 		return inside.has_world_point(from_tx, from_ty) &&
 		       !inside.has_world_point(tx, ty);
@@ -793,11 +790,14 @@ int Egg_object::is_active(
 			return area.has_world_point(tx, ty) && deltaz == 0 &&
 			       obj->get_flag(Obj_flags::in_party);;
 	case something_on:
+		// force close gumps when gumps pause game
+		if (!gumpman->gumps_dont_pause_game())
+			gumpman->close_all_gumps();
 		return          // Guessing.  At SI end, deltaz == -1.
 		    deltaz / 4 == 0 && area.has_world_point(tx, ty) && !obj->as_actor();
 	case external_criteria:
 	default:
-		return 0;
+		return false;
 	}
 }
 
@@ -840,7 +840,7 @@ void Egg_object::activate(
     int /* event */
 ) {
 	if (!edit())
-		hatch(0, 0);
+		hatch(nullptr, false);
 	if (animator)
 		flags &= ~(1 << static_cast<int>(hatched)); // Moongate:  reset always.
 }
@@ -856,7 +856,7 @@ bool Egg_object::edit(
 #ifdef USE_EXULTSTUDIO
 	if (client_socket >= 0 &&   // Talking to ExultStudio?
 	        cheat.in_map_editor()) {
-		editing = 0;
+		editing.reset();
 		Tile_coord t = get_tile();
 		// Usecode function name.
 		string str1 = get_str1();
@@ -867,7 +867,7 @@ bool Egg_object::edit(
 		                   (flags >> hatched) & 1, (flags >> auto_reset) & 1,
 		                   data1, data2, data3, str1) != -1) {
 			cout << "Sent egg data to ExultStudio" << endl;
-			editing = this;
+			editing = shared_from_this();
 		} else
 			cout << "Error sending egg data to ExultStudio" << endl;
 		return true;
@@ -886,16 +886,25 @@ void Egg_object::update_from_studio(
     int datalen
 ) {
 #ifdef USE_EXULTSTUDIO
-	int x, y;           // Mouse click for new egg.
+	int x;
+	int y;           // Mouse click for new egg.
 	Egg_object *oldegg;
-	int tx, ty, tz;
-	int shape, frame;
+	int tx;
+	int ty;
+	int tz;
+	int shape;
+	int frame;
 	int type;
 	int criteria;
 	int probability;
 	int distance;
-	bool nocturnal, once, hatched, auto_reset;
-	int data1, data2, data3;
+	bool nocturnal;
+	bool once;
+	bool hatched;
+	bool auto_reset;
+	int data1;
+	int data2;
+	int data3;
 	string str1;
 	if (!Egg_object_in(data, datalen, oldegg, tx, ty, tz, shape, frame,
 	                   type, criteria, probability, distance,
@@ -904,13 +913,14 @@ void Egg_object::update_from_studio(
 		cout << "Error decoding egg" << endl;
 		return;
 	}
-	if (oldegg && oldegg != editing) {
+	if (oldegg && oldegg != editing.get()) {
 		cout << "Egg from ExultStudio is not being edited" << endl;
 		return;
 	}
-	editing = 0;
+	// Keeps NPC alive until end of function
+	Game_object_shared keep = std::move(editing);
 	if (!oldegg) {          // Creating a new one?  Get loc.
-		if (!Get_click(x, y, Mouse::hand, 0)) {
+		if (!Get_click(x, y, Mouse::hand, nullptr)) {
 			if (client_socket >= 0)
 				Exult_server::Send_data(client_socket,
 				                        Exult_server::cancel);
@@ -959,18 +969,18 @@ void Egg_object::update_from_studio(
 
 	const Shape_info &info = ShapeID::get_info(shape);
 	bool anim = info.is_animated() || info.has_sfx();
-	Egg_object *egg = create_egg(anim, shape, frame, tx, ty, tz, type,
+	Egg_object_shared egg = create_egg(anim, shape, frame, tx, ty, tz, type,
 	                             probability, data1, data2, data3, str1.c_str());
 	if (!oldegg) {
 		int lift;       // Try to drop at increasing hts.
 		for (lift = 0; lift < 12; lift++)
-			if (gwin->drop_at_lift(egg, x, y, lift) == 1)
+			if (gwin->drop_at_lift(egg.get(), x, y, lift) == 1)
 				break;
 		if (lift == 12) {
 			if (client_socket >= 0)
 				Exult_server::Send_data(client_socket,
 				                        Exult_server::cancel);
-			delete egg;
+			egg = nullptr;
 			return;
 		}
 		if (client_socket >= 0)
@@ -980,7 +990,7 @@ void Egg_object::update_from_studio(
 		Tile_coord pos = oldegg->get_tile();
 		egg->move(pos.tx, pos.ty, pos.tz);
 	}
-	gwin->add_dirty(egg);
+	gwin->add_dirty(egg.get());
 	egg->criteria = criteria & 7;
 	egg->distance = distance & 31;
 	egg->probability = probability;
@@ -992,8 +1002,8 @@ void Egg_object::update_from_studio(
 	if (oldegg)
 		oldegg->remove_this();
 	Map_chunk *echunk = egg->get_chunk();
-	echunk->remove_egg(egg);    // Got to add it back.
-	echunk->add_egg(egg);
+	echunk->remove_egg(egg.get());    // Got to add it back.
+	echunk->add_egg(egg.get());
 	cout << "Egg updated" << endl;
 #else
 	ignore_unused_variable_warning(data, datalen);
@@ -1040,9 +1050,15 @@ void Egg_object::hatch(
 	int roll = must ? 0 : 1 + rand() % 100;
 	if (roll <= probability) {
 		// Time to hatch the egg.
+		// Watch it in case it gets deleted.
+		Game_object_weak watch = weak_from_this();
 		hatch_now(obj, must);
+		if (watch.expired()) {
+			// We have been deleted, so just leave.
+			return;
+		}
 		if (flags & (1 << static_cast<int>(once))) {
-			remove_this(0);
+		    remove_this(nullptr);
 			return;
 		}
 	}
@@ -1091,21 +1107,21 @@ void Egg_object::set_weather(
 		eman->remove_weather_effects();
 		break;
 	case 1:     // Snow.
-		eman->add_effect(new Snowstorm_effect(len, 0, egg));
+		eman->add_effect(std::make_unique<Snowstorm_effect>(len, 0, egg));
 		break;
 	case 2:     // Storm.
-		eman->add_effect(new Storm_effect(len, 0, egg));
+		eman->add_effect(std::make_unique<Storm_effect>(len, 0, egg));
 		break;
 	case 3:     // (On Ambrosia).
 		eman->remove_weather_effects();
-		eman->add_effect(new Sparkle_effect(len, 0, egg));
+		eman->add_effect(std::make_unique<Sparkle_effect>(len, 0, egg));
 		break;
 	case 4:     // Fog.
-		eman->add_effect(new Fog_effect(len, 0, egg));
+		eman->add_effect(std::make_unique<Fog_effect>(len, 0, egg));
 		break;
 	case 5:     // Overcast.
 	case 6:     // Clouds.
-		eman->add_effect(new Clouds_effect(len, 0, egg, weather));
+		eman->add_effect(std::make_unique<Clouds_effect>(len, 0, egg, weather));
 		break;
 	default:
 		break;
@@ -1124,13 +1140,15 @@ void Egg_object::move(
     int newmap
 ) {
 	// Figure new chunk.
-	int newcx = newtx / c_tiles_per_chunk, newcy = newty / c_tiles_per_chunk;
+	int newcx = newtx / c_tiles_per_chunk;
+	int newcy = newty / c_tiles_per_chunk;
 	Game_map *eggmap = newmap >= 0 ? gwin->get_map(newmap) : get_map();
 	if (!eggmap) eggmap = gmap;
 	Map_chunk *newchunk = eggmap->get_chunk_safely(newcx, newcy);
 	if (!newchunk)
 		return;         // Bad loc.
-	remove_this(1);         // Remove from old.
+	Game_object_shared keep;
+	remove_this(&keep);         // Remove from old.
 	set_lift(newlift);      // Set new values.
 	set_shape_pos(newtx % c_tiles_per_chunk, newty % c_tiles_per_chunk);
 	newchunk->add_egg(this);    // Updates cx, cy.
@@ -1142,8 +1160,10 @@ void Egg_object::move(
  */
 
 void Egg_object::remove_this(
-    int nodel                       // 1 to not delete.
+    Game_object_shared *keep     // Non-null to not delete.
 ) {
+    if (keep)
+	    *keep = shared_from_this();
 	if (get_owner())        // Watch for this.
 		get_owner()->remove(this);
 	else {
@@ -1152,8 +1172,6 @@ void Egg_object::remove_this(
 			chunk->remove_egg(this);
 		}
 	}
-	if (!nodel)
-		gwin->delete_object(this);
 }
 
 /*
@@ -1176,7 +1194,7 @@ void Egg_object::write_ireg(
 	Write2(ptr, tword);
 	*ptr++ = probability;
 	Write2(ptr, data1);
-	*ptr++ = (get_lift() & 15) << 4;
+	*ptr++ = nibble_swap(get_lift());
 	Write2(ptr, data2);
 	if (data3 > 0)
 		Write2(ptr, data3);
@@ -1195,7 +1213,7 @@ int Egg_object::get_ireg_size() {
 		return -1;
 	const char *str1 = get_str1();
 	return 8 + get_common_ireg_size() + ((data3 > 0) ? 2 : 0)
-	       + (*str1 ? Game_map::write_string(0, str1) : 0);
+	       + (*str1 ? Game_map::write_string(nullptr, str1) : 0);
 }
 
 /*
@@ -1254,10 +1272,9 @@ bool Field_object::field_effect(
 		return false;
 	}
 
-	Field_frame_animator *ani;
-	if (!del && (ani = dynamic_cast<Field_frame_animator *>(animator)) != 0)
+	if (!del && animator)
 		// Tell animator to keep checking.
-		ani->activated = true;
+		animator->activate_animator();
 	return del;
 }
 
@@ -1289,12 +1306,11 @@ void Field_object::activate(
 	Actor_vector npcs;      // Find all nearby NPC's.
 	gwin->get_nearby_npcs(npcs);
 	npcs.push_back(gwin->get_main_actor()); // Include Avatar.
-	Rectangle eggfoot = get_footprint();
+	TileRect eggfoot = get_footprint();
 	// Clear flag to check.
-	static_cast<Field_frame_animator *>(animator)->activated = false;
-	for (Actor_vector::const_iterator it = npcs.begin();
-	        it != npcs.end(); ++it) {
-		Actor *actor = *it;
+	if (animator)
+		animator->deactivate_animator();
+	for (auto *actor : npcs) {
 		if (actor->is_dead() || Game_object::distance(actor) > 4)
 			continue;
 		if (actor->get_footprint().intersects(eggfoot))
@@ -1311,7 +1327,7 @@ void Field_object::hatch(
     bool /* must */         // If 1, skip dice roll.
 ) {
 	if (field_effect(obj->as_actor()))// Apply field.
-		remove_this(0);     // Delete sleep/poison if applied.
+		remove_this(nullptr);     // Delete sleep/poison if applied.
 }
 
 /*
@@ -1376,27 +1392,27 @@ void Mirror_object::hatch(Game_object *obj, bool must) {
 }
 
 // Can it be activated?
-int Mirror_object::is_active(Game_object *obj, int tx, int ty, int tz, int from_tx, int from_ty) {
+bool Mirror_object::is_active(Game_object *obj, int tx, int ty, int tz, int from_tx, int from_ty) {
 	ignore_unused_variable_warning(obj, tx, ty, tz, from_tx, from_ty);
 	// These are broken, so dont touch
 	int frnum = get_framenum();
-	if (frnum % 3 == 2)  return 0;
+	if (frnum % 3 == 2)  return false;
 	if (frnum >= 3 && GAME_BG)  // Demon mirror in FOV.
-		return 0;
+		return false;
 
-	return 1;
+	return true;
 }
 
 // Set up active area.
 void Mirror_object::set_area() {
 	// These are broken, so dont touch
-	if ((get_framenum() % 3) == 2) area = Rectangle(0, 0, 0, 0);
+	if ((get_framenum() % 3) == 2) area = TileRect(0, 0, 0, 0);
 
 	Tile_coord t = get_tile();  // Get absolute tile coords.
 
 	// To left or above?
-	if (get_shapenum() == 268) area = Rectangle(t.tx - 1, t.ty - 3, 6, 6);
-	else  area = Rectangle(t.tx - 3 , t.ty - 1, 6, 6);
+	if (get_shapenum() == 268) area = TileRect(t.tx - 1, t.ty - 3, 6, 6);
+	else  area = TileRect(t.tx - 3 , t.ty - 1, 6, 6);
 }
 
 void Mirror_object::paint() {
