@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002  The Exult Team
+ *  Copyright (C) 2002-2022  The Exult Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -39,12 +39,6 @@
 #endif
 
 using std::string;
-#ifndef HAVE_SNPRINTF
-extern int snprintf(char *, size_t, const char *, /*args*/ ...) ATTR_PRINTF(3,4);
-namespace std {
-using ::snprintf;
-}
-#endif
 using std::strchr;
 
 /*
@@ -148,7 +142,7 @@ string Game_object::get_name(
 	int shnum = get_shapenum();
 	const char *name;
 	const char *shpname = (shnum >= 0 && shnum < get_num_item_names()) ?
-	                      get_item_name(shnum) : 0;
+	                      get_item_name(shnum) : nullptr;
 	int type = nminf ? nminf->get_type() : -255;
 	int msgid;
 	if (type < 0 && type != -255)   // This is a "catch all" default.
@@ -200,7 +194,7 @@ string Game_object::get_name(
 	}
 	int quantity;
 	string display_name;
-	if (name == 0)
+	if (name == nullptr)
 		return "";
 
 	if (ShapeID::get_info(shnum).has_quantity())
@@ -209,7 +203,7 @@ string Game_object::get_name(
 		quantity = 1;
 
 	// If there are no slashes then it is simpler
-	if (strchr(name, '/') == 0) {
+	if (strchr(name, '/') == nullptr) {
 		if (quantity <= 1)
 			display_name = name;
 		else {
