@@ -2,7 +2,7 @@
  *  objs.h - Game objects.
  *
  *  Copyright (C) 1998-1999  Jeffrey S. Freedman
- *  Copyright (C) 2000-2013  The Exult Team
+ *  Copyright (C) 2000-2022  The Exult Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 
 #include <string>   // STL string
 #include <set>
+#include <memory>
 #include "exult_constants.h"
 #include "common_types.h"
 #include "flags.h"
@@ -43,29 +44,37 @@ class Barge_object;
 class Game_window;
 class Npc_actor;
 class PathFinder;
-class Rectangle;
 class Schedule;
+class TileRect;
 class Usecode_machine;
 class Vga_file;
 class ODataSource;
 class Game_map;
 class Object_client;
+class Spellbook_object;
+class Virtue_stone_object;
+class Dead_body;
+class Ordering_info;
 
 template<class T>
 class T_Object_list;
 
-typedef std::vector<Game_object *>   Game_object_vector;
-typedef std::vector<Egg_object *>    Egg_vector;
-typedef std::vector<Actor *>     Actor_vector;
+using Game_object_vector = std::vector<Game_object *>;
+using Egg_vector = std::vector<Egg_object *>;
+using Actor_vector = std::vector<Actor *>;
+using Game_object_shared = std::shared_ptr<Game_object>;
+using Game_object_weak = std::weak_ptr<Game_object>;
+using Game_object_shared_vector = std::vector<Game_object_shared>;
 
 /*
  *  A game object is a shape from shapes.vga along with info. about its
  *  position within its chunk.
  */
-class Game_object : public ShapeID {
+class Game_object : public ShapeID,
+					public std::enable_shared_from_this<Game_object> {
 protected:
-	static Game_object *editing;    // Obj. being edited by ExultStudio.
-	Map_chunk *chunk;       // Chunk we're in, or NULL.
+	static Game_object_shared editing;    // Obj. being edited by ExultStudio.
+	Map_chunk *chunk = nullptr;       // Chunk we're in, or nullptr.
 	unsigned char tx, ty;       // (X,Y) of shape within chunk, or if
 	//   in a container, coords. within
 	//   gump's rectangle.
@@ -74,11 +83,11 @@ protected:
 	short quality;          // Some sort of game attribute.
 	int get_cxi() const;
 	int get_cyi() const;
-	void remove_clients();
 private:
-	Game_object *next, *prev;   // ->next in chunk list or container.
+	Game_object_shared next;   // ->next in chunk list or container.
+    Game_object *prev;
 public:
-	typedef std::set<Game_object *> Game_object_set;
+	using Game_object_set = std::set<Game_object *>;
 private:
 	Game_object_set dependencies;   // Objects which must be painted before
 	//   this can be rendered.
@@ -86,28 +95,25 @@ private:
 	static unsigned char rotate[8]; // For getting rotated frame #.
 	std::vector <Object_client *> clients; // Notify when deleted.
 public:
-	uint32 render_seq;      // Render sequence #.
-public:
-	friend class T_Object_list<Game_object *>;
-	friend class T_Object_iterator<Game_object *>;
-	friend class T_Flat_object_iterator<Game_object *, Map_chunk *>;
-	friend class T_Object_iterator_backwards < Game_object *,
+	uint32 render_seq = 0;      // Render sequence #.
+	friend class T_Object_list<Game_object>;
+	friend class T_Object_iterator<Game_object>;
+	friend class T_Flat_object_iterator<Game_object, Map_chunk *>;
+	friend class T_Object_iterator_backwards < Game_object,
 			Map_chunk * >;
 	friend class Map_chunk;
 	Game_object(int shapenum, int framenum, unsigned int tilex,
 	            unsigned int tiley, unsigned int lft = 0)
-		: ShapeID(shapenum, framenum), chunk(0), tx(tilex), ty(tiley),
-		  lift(lft), quality(0), render_seq(0)
+		: ShapeID(shapenum, framenum), tx(tilex), ty(tiley),
+		  lift(lft), quality(0)
 	{  }
 	// Copy constructor.
-	Game_object(const Game_object &obj2)
-		: ShapeID(obj2), chunk(obj2.chunk), tx(obj2.tx), ty(obj2.ty),
-		  lift(obj2.lift), quality(obj2.quality), render_seq(0)
-	{  }
-	Game_object() : ShapeID(), chunk(0), render_seq(0)  // Create fake entry.
-	{  }
-	virtual ~Game_object()
-	{  }
+	Game_object(const Game_object &obj2) = delete;
+	Game_object() = default; // Create fake entry.
+	~Game_object() override = default;
+    Game_object_weak weak_from_this() {
+	    return std::weak_ptr<Game_object>(shared_from_this());
+	}
 	int get_tx() const {    // Get tile (0-15) within chunk.
 		return tx;
 	}
@@ -122,7 +128,7 @@ public:
 	// Get missile start location in abs. tiles given dir.
 	Tile_coord get_missile_tile(int dir) const;
 	// Get distance to another object.
-	int distance(Game_object *o2) const;
+	int distance(const Game_object *o2) const;
 	// Get direction to another object.
 	int distance(Tile_coord t2) const;
 	// Get direction to another object.
@@ -148,7 +154,7 @@ public:
 	virtual void set_obj_hp(int hp);
 	int get_volume() const;     // Get space taken.
 	// Add/remove to/from quantity.
-	int modify_quantity(int delta, bool *del = 0);
+	int modify_quantity(int delta, bool *del = nullptr);
 	// Set shape coord. in chunk/gump.
 	void set_shape_pos(unsigned int shapex, unsigned int shapey) {
 		tx = shapex;
@@ -157,20 +163,20 @@ public:
 	void set_lift(int l) {
 		lift = l;
 	}
-	Game_object *get_next() {
-		return next;
+	Game_object *get_next() const {
+		return next.get();
 	}
-	Game_object *get_prev() {
+	Game_object *get_prev() const {
 		return prev;
 	}
 	// Compare for render order.
-	static int compare(class Ordering_info &inf1, Game_object *obj2);
+	static int compare(Ordering_info &inf1, Game_object *obj2);
 	int lt(Game_object &obj2);  // Is this less than another in pos.?
 	void set_invalid() {    // Set to invalid position.
-		chunk = 0;
+		chunk = nullptr;
 	}
 	bool is_pos_invalid() const {
-		return chunk == 0;
+		return chunk == nullptr;
 	}
 	bool inside_locked() const;
 	void set_chunk(Map_chunk *c) {
@@ -191,11 +197,9 @@ public:
 	void move(Tile_coord const &t, int newmap = -1) {
 		move(t.tx, t.ty, t.tz, newmap);
 	}
-	bool add_client(Object_client *c);
-	void remove_client(Object_client *c);
-	void change_frame(int frnum);   // Change frame & set to repaint.
+	virtual void change_frame(int frnum);   // Change frame & set to repaint.
 	// Swap positions.
-	int swap_positions(Game_object *obj2);
+	bool swap_positions(Game_object *obj2);
 	Game_object_set &get_dependencies() {
 		return dependencies;
 	}
@@ -231,17 +235,19 @@ public:
 	                              int shapenum, int delta, int mask = 8);
 	static int find_nearby_eggs(Egg_vector &vec, Tile_coord const &pos,
 	                            int shapenum, int delta, int qual = c_any_qual,
-	                            int framenum = c_any_framenum);
+	                            int frnum = c_any_framenum);
 	static int find_nearby(Game_object_vector &vec, Tile_coord const &pos,
 	                       int shapenum, int delta, int mask, int qual = c_any_qual,
-	                       int framenum = c_any_framenum, bool exclude_okay_to_take = false);
+	                       int frnum = c_any_framenum,
+						   bool exclude_okay_to_take = false);
+    static void obj_vec_to_weak(std::vector<Game_object_weak> &dest,
+													Game_object_vector &src);
 	int find_nearby_actors(Actor_vector &vec, int shapenum, int delta,
 	                       int mask = 8) const;
 	int find_nearby_eggs(Egg_vector &vec, int shapenum, int delta,
 	                     int qual = c_any_qual, int frnum = c_any_framenum) const;
 	int find_nearby(Game_object_vector &vec, int shapenum, int delta,
 	                int mask, int qual = c_any_qual, int framenum = c_any_framenum) const;
-
 	Game_object *find_closest(Game_object_vector &vec,
 	                          int *shapenums, int num_shapes, int dist = 24);
 	static Game_object *find_closest(Tile_coord const &pos,
@@ -258,13 +264,13 @@ public:
 	                                 int dist = 24) {
 		return find_closest(pos, &shapenum, 1, dist);
 	}
-	Rectangle get_footprint();  // Get tile footprint.
-	Block get_block();
-	bool blocks(Tile_coord const &tile);    // Do we block a given tile?
+	TileRect get_footprint();  // Get tile footprint.
+	Block get_block() const;
+	bool blocks(Tile_coord const &tile) const;    // Do we block a given tile?
 	// Find object blocking given tile.
 	static Game_object *find_blocking(Tile_coord tile);
 	static Game_object *find_door(Tile_coord tile);
-	int is_closed_door() const; // Checking for a closed door.
+	bool is_closed_door() const; // Checking for a closed door.
 	Game_object *get_outermost();   // Get top 'owner' of this object.
 	void say(const char *text);     // Put text up by item.
 	void say(int msgnum);       // Show given text msg.
@@ -275,8 +281,8 @@ public:
 	// Make this class abstract.
 	virtual void paint_terrain() = 0;
 	// Can this be clicked on?
-	virtual int is_findable() {
-		return 1;
+	virtual bool is_findable() {
+		return true;
 	}
 	// Run usecode function.
 	virtual void activate(int event = 1);
@@ -285,9 +291,9 @@ public:
 	static void update_from_studio(unsigned char *data, int datalen);
 	virtual std::string get_name() const;
 	// Remove/delete this object.
-	virtual void remove_this(int nodel = 0);
+	virtual void remove_this(Game_object_shared *keep = nullptr);
 	virtual Container_game_object *get_owner() const {
-		return 0;
+		return nullptr;
 	}
 	virtual void set_owner(Container_game_object *o) {
 		ignore_unused_variable_warning(o);
@@ -295,9 +301,9 @@ public:
 	static int get_weight(int shnum, int quant = 1);
 	virtual int get_weight();
 	virtual int get_max_weight() const;   // Get max. weight allowed.
-	virtual int is_dragable() const;// Can this be dragged?
+	virtual bool is_dragable() const;// Can this be dragged?
 	// Drop another onto this.
-	virtual int drop(Game_object *obj);
+	virtual bool drop(Game_object *obj);
 	// Set/clear/get actor flag.
 	virtual void set_flag(int flag) {
 		ignore_unused_variable_warning(flag);
@@ -305,35 +311,44 @@ public:
 	virtual void clear_flag(int flag) {
 		ignore_unused_variable_warning(flag);
 	}
-	virtual int get_flag(int flag) const  {
+	virtual bool get_flag(int flag) const  {
 		ignore_unused_variable_warning(flag);
-		return 0;
+		return false;
 	}
 	virtual void set_flag_recursively(int flag) {
 		ignore_unused_variable_warning(flag);
 	}
-	virtual int get_type_flag(int flag) const {
+	virtual bool get_type_flag(int flag) const {
 		ignore_unused_variable_warning(flag);
-		return 0;
+		return false;
 	}
 
 	virtual Actor *as_actor() {
-		return 0;
+		return nullptr;
 	}
 	virtual Npc_actor *as_npc() {
-		return 0;
+		return nullptr;
 	}
 	virtual Barge_object *as_barge() {
-		return 0;
+		return nullptr;
 	}
 	virtual Terrain_game_object *as_terrain() {
-		return 0;
+		return nullptr;
 	}
 	virtual Container_game_object *as_container() {
-		return 0;
+		return nullptr;
 	}
 	virtual Egg_object *as_egg() {
-		return 0;
+		return nullptr;
+	}
+	virtual Spellbook_object *as_spellbook() {
+		return nullptr;
+	}
+	virtual Virtue_stone_object *as_virtstone() {
+		return nullptr;
+	}
+	virtual Dead_body *as_body() {
+		return nullptr;
 	}
 	virtual int is_egg() const { // An egg?
 		return 0;
@@ -357,10 +372,10 @@ public:
 	virtual bool add(Game_object *obj, bool dont_check = false,
 	                 bool combine = false, bool noset = false);
 	// Add to NPC 'ready' spot.
-	virtual int add_readied(Game_object *obj, int index,
-	                        int dont_check = 0, int force_pos = 0, bool noset = false) {
+	virtual bool add_readied(Game_object *obj, int index,
+	                        bool dont_check = false, bool force_pos = false, bool noset = false) {
 		ignore_unused_variable_warning(index, force_pos);
-		return add(obj, dont_check != 0, false, noset);
+		return add(obj, dont_check, false, noset);
 	}
 	virtual int add_quantity(int delta, int shapenum, int qual = c_any_qual,
 	                         int framenum = c_any_framenum, bool dontcreate = false, bool temporary = false) {
@@ -379,35 +394,35 @@ public:
 	}
 	virtual Game_object *find_item(int shapenum, int qual, int framenum) {
 		ignore_unused_variable_warning(shapenum, qual, framenum);
-		return 0;
+		return nullptr;
 	}
 	// Get coord. where this was placed.
 	virtual Tile_coord get_original_tile_coord() const {
 		return get_tile();
 	}
 	// Move out of the way.
-	virtual int move_aside(Actor *for_actor, int dir) {
+	virtual bool move_aside(Actor *for_actor, int dir) {
 		ignore_unused_variable_warning(for_actor, dir);
-		return 0;    // For now.
+		return false;    // For now.
 	}
 	// Get frame if rotated clockwise.
-	virtual int get_rotated_frame(int quads);
+	virtual int get_rotated_frame(int quads) const;
 	// Step onto an (adjacent) tile.
-	virtual int step(Tile_coord t, int frame, bool force = false) {
+	virtual bool step(Tile_coord t, int frame, bool force = false) {
 		ignore_unused_variable_warning(t, frame, force);
-		return 0;
+		return false;
 	}
-	virtual int is_monster() {
-		return 0;
+	virtual bool is_monster() const {
+		return false;
 	}
 	virtual Game_object *find_weapon_ammo(int weapon, int needed = 1,
 	                                      bool recursive = false) {
 		ignore_unused_variable_warning(weapon, needed, recursive);
-		return 0;
+		return nullptr;
 	}
-	virtual int get_effective_range(const Weapon_info *winf = 0, int reach = -1);
+	virtual int get_effective_range(const Weapon_info *winf = nullptr, int reach = -1) const;
 	int get_weapon_ammo(int weapon, int family, int proj, bool ranged,
-	                    Game_object **ammo = 0, bool recursive = false);
+	                    Game_object **ammo = nullptr, bool recursive = false);
 	void play_hit_sfx(int weapon, bool ranged);
 	virtual bool try_to_hit(Game_object *attacker, int attval) {
 		ignore_unused_variable_warning(attacker, attval);
@@ -420,9 +435,9 @@ public:
 	virtual int figure_hit_points(Game_object *attacker, int weapon_shape = -1,
 	                              int ammo_shape = -1, bool explosion = false);
 	virtual int apply_damage(Game_object *attacker, int str,
-	                         int wpoints, int type, int bias = 0, int *exp = 0);
-	virtual int reduce_health(int delta, int damage_type, Game_object *attacker = 0,
-	                          int *exp = 0);
+	                         int wpoints, int type, int bias = 0, int *exp = nullptr);
+	virtual int reduce_health(int delta, int damage_type, Game_object *attacker = nullptr,
+	                          int *exp = nullptr);
 	// Write out to IREG file.
 	virtual void write_ireg(ODataSource *out) {
 		ignore_unused_variable_warning(out);
@@ -446,9 +461,17 @@ public:
 	// Return's the object's usecode for the shape number
 	virtual int get_usecode() const;
 	// Default:  Can't set it.
-	virtual bool set_usecode(int funid, const char *nm = 0);
+	virtual bool set_usecode(int ui, const char *nm = nullptr);
+	bool usecode_exists() const;
 };
 
+inline Game_object_weak weak_from_obj(Game_object *obj) {
+    return obj ? obj->weak_from_this() : Game_object_weak();
+}
+inline Game_object_shared shared_from_obj(Game_object *obj) {
+    return obj ? obj->shared_from_this() : Game_object_shared();
+}
+
 /*
  *  Object from U7chunks.
  */
@@ -460,15 +483,15 @@ public:
 		: Game_object(shapenum, framenum, tilex, tiley, lft),
 		  prev_flat(ShapeID(12, 0))
 	{  }
-	virtual ~Terrain_game_object() {  }
-	virtual Terrain_game_object *as_terrain() {
+	~Terrain_game_object() override = default;
+	Terrain_game_object *as_terrain() override {
 		return this;
 	}
 	// Move to new abs. location.
-	virtual void move(int newtx, int newty, int newlift, int newmap = -1);
+	void move(int newtx, int newty, int newlift, int newmap = -1) override;
 	// Remove/delete this object.
-	virtual void remove_this(int nodel = 0);
-	virtual void paint_terrain();
+	void remove_this(Game_object_shared *keep = nullptr) override;
+	void paint_terrain() override;
 };
 
 /*
@@ -480,13 +503,13 @@ public:
 	                 unsigned int tiley, unsigned int lft = 0)
 		: Game_object(shapenum, framenum, tilex, tiley, lft)
 	{  }
-	virtual ~Ifix_game_object() {  }
+	~Ifix_game_object() override = default;
 	// Move to new abs. location.
-	virtual void move(int newtx, int newty, int newlift, int newmap = -1);
+	void move(int newtx, int newty, int newlift, int newmap = -1) override;
 	// Remove/delete this object.
-	virtual void remove_this(int nodel = 0);
-	virtual void paint_terrain() {  }
-	virtual void write_ifix(ODataSource *ifix, bool v2);
+	void remove_this(Game_object_shared *keep = nullptr) override;
+	void paint_terrain() override {  }
+	void write_ifix(ODataSource *ifix, bool v2) override;
 };
 
 #endif
