@@ -3,7 +3,7 @@
  **
  ** Written: 4/8/02 - JSF
  **/
-/*  Copyright (C) 2000-2013  The Exult Team
+/*  Copyright (C) 2002-2022  The Exult Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -305,13 +305,15 @@ void Party_manager::move_followers(
 	ignore_unused_variable_warning(vindex);
 	int id = npc->get_party_id();   // (-1 if Avatar).
 	Tile_coord pos = npc->get_tile();
-	int lnum = followers[1 + id][0], rnum = followers[1 + id][1];
+	int lnum = followers[1 + id][0];
+	int rnum = followers[1 + id][1];
 	if (lnum == -1 && rnum == -1)
 		return;         // Nothing to do.
 	int dir4 = dir / 2;     // 0-3 now.
-	Actor *lnpc = (lnum == -1 || lnum >= validcnt) ? 0 : valid[lnum];
-	Actor *rnpc = (rnum == -1 || rnum >= validcnt) ? 0 : valid[rnum];
-	int ldir = -1, rdir = -1;
+	Actor *lnpc = (lnum == -1 || lnum >= validcnt) ? nullptr : valid[lnum];
+	Actor *rnpc = (rnum == -1 || rnum >= validcnt) ? nullptr : valid[rnum];
+	int ldir = -1;
+	int rdir = -1;
 	// Have each take a step.
 	if (lnpc)
 		ldir = step(lnpc, npc, dir, pos + Tile_coord(
@@ -336,7 +338,8 @@ inline Tile_coord Get_step_tile(
     int dir             // Dir. party is moving (0-7).
 ) {
 	ignore_unused_variable_warning(dir);
-	int dx = dest.tx - pos.tx, dy = dest.ty - pos.ty;
+	int dx = dest.tx - pos.tx;
+	int dy = dest.ty - pos.ty;
 	if (dx < -1)
 		dx = -1;        // Limit to 1 tile.
 	else if (dx > 1)
@@ -365,12 +368,13 @@ static Actor *Find_member_blocking(
 
 	for (int i = first; i < count; i++) {
 		Actor *npc = gwin->get_npc(pman->get_member(i));
+		assert(npc != nullptr);
 		pos.tz = npc->get_lift();// Use NPC's, since it might be up/dn
 		//   by a step.
 		if (npc->blocks(pos))
 			return npc; // Found.
 	}
-	return 0;
+	return nullptr;
 }
 
 /*
@@ -425,11 +429,11 @@ static int Get_cost(
     Actor *npc,         // NPC to take the step.
     Actor *leader,          // NPC he's following.
     Tile_coord to,          // Tile to step to.
-    Actor **find_blocking = 0   // Returns blocking party member.
+    Actor **find_blocking = nullptr   // Returns blocking party member.
 ) {
 	int cost = 0;
 	if (find_blocking)
-		*find_blocking = 0;
+		*find_blocking = nullptr;
 	if (npc->is_blocked(to)) {  // (To.tz is updated.)
 		// Can't go there.
 		if (find_blocking) {
@@ -444,9 +448,10 @@ static int Get_cost(
 			return max_cost;
 	}
 	Tile_coord lpos = leader->get_tile();
-	int difftz = to.tz - lpos.tz,   // Measure closeness.
-	    diffty = Tile_coord::delta(to.ty, lpos.ty),
-	    difftx = Tile_coord::delta(to.tx, lpos.tx);
+	int difftz = to.tz - lpos.tz;
+	int // Measure closeness.
+	    diffty = Tile_coord::delta(to.ty, lpos.ty);
+	int difftx = Tile_coord::delta(to.tx, lpos.tx);
 	// Get dist**2 in x-y plane.
 	int xydist2 = diffty * diffty + difftx * difftx;
 	cost += difftz * difftz + xydist2;
@@ -472,13 +477,12 @@ static bool Take_best_step(
     int dir             // Direction we want to go.
 ) {
 	static int deltadir[] = {0, 1, 7, 2, 6, 3, 5};
-	const int cnt = array_size(deltadir);
 
 	int best_cost = max_cost + 8;
 	Tile_coord best(-1, -1, -1);
-	Actor *best_in_way = 0;
-	for (int i = 0; i < cnt; i++) {
-		int diri = (dir + deltadir[i]) % 8;
+	Actor *best_in_way = nullptr;
+	for (int i : deltadir) {
+		int diri = (dir + i) % 8;
 		Tile_coord to = pos.get_neighbor(diri);
 		Actor *in_way;      // Fudge cost with diff. in dir.
 		int cost = Get_cost(npc, leader, to, &in_way);
@@ -491,11 +495,13 @@ static bool Take_best_step(
 	if (best_cost >= max_cost)
 		return false;
 	if (!best_in_way)       // Nobody in way?
-		return npc->step(best, frame) != 0;
+		return npc->step(best, frame);
 	best = best_in_way->get_tile(); // Swap positions.
-	npc->remove_this(true);
-	best_in_way->remove_this(true);
-	npc->set_frame(frame);      // Appear to take a step.
+    Game_object_shared npc_keep;
+    Game_object_shared best_keep;
+	npc->remove_this(&npc_keep);
+	best_in_way->remove_this(&best_keep);
+	npc->change_frame(frame);      // Appear to take a step.
 	npc->move(best);
 	best_in_way->move(pos);
 	return true;
@@ -520,10 +526,8 @@ inline bool Is_step_okay(
 	// How close in XY?
 	int dist = leader->distance(to);
 	if (dist == 1)
-		return (difftz <= 1);   // 1 tile away, so want dz <= 1.
-	if (!Clear_to_leader(npc, leader, to))
-		return false;       // Couldn't take a 2nd step.
-	return true;
+		return difftz <= 1;   // 1 tile away, so want dz <= 1.
+	return Clear_to_leader(npc, leader, to);       // Couldn't take a 2nd step.
 }
 
 /*
