@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2013  The Exult Team
+ *  Copyright (C) 2000-2022  The Exult Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -32,21 +32,17 @@ int Pathfinder_client::get_max_cost(
 ) {
 	int max_cost = 3 * cost_to_goal;
 	// (Raised from 64 on 9/4/2000).
-	return (max_cost < 74 ? 74 : max_cost);
+	return max_cost < 74 ? 74 : max_cost;
 }
 
 /*
  *  Is tile at goal?
  */
 
-int Pathfinder_client::at_goal(
+bool Pathfinder_client::at_goal(
     Tile_coord const &tile,
     Tile_coord const &goal
 ) {
-	return (tile.tx == goal.tx && tile.ty == goal.ty &&
-	        (goal.tz == -1 || tile.tz == goal.tz));
+	return tile.tx == goal.tx && tile.ty == goal.ty &&
+	        (goal.tz == -1 || tile.tz == goal.tz);
 }
-
-
-PathFinder::~PathFinder()
-{}
