@@ -1,7 +1,7 @@
 /*
  *  Schedule.h - Schedules for characters.
  *
- *  Copyright (C) 2000-2013  The Exult Team
+ *  Copyright (C) 2000-2022  The Exult Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,27 +23,24 @@
 
 #include "tiles.h"
 #include "singles.h"
-#include "objclient.h"
+#include <memory>
+#include <string>
 #include <vector>
 #include "ignore_unused_variable_warning.h"
 
-#ifdef WIN32
-#include <windows.h>
-#define Rectangle RECTX
-#endif
-
 class Game_object;
 class Actor;
-class Rectangle;
+class TileRect;
 class Actor_action;
 class Usecode_value;
 
-using std::vector;
+using Game_object_weak = std::weak_ptr<Game_object>;
+using Game_object_vector = std::vector<Game_object *>;
 
 /*
  *  A Schedule controls the NPC it is assigned to.
  */
-class Schedule : public Game_singletons, public Object_client {
+class Schedule : public Game_singletons {
 protected:
 	Actor *npc;         // Who this controls.
 	Tile_coord blocked;     // Tile where actor was blocked.
@@ -53,9 +50,7 @@ protected:
 	long street_maintenance_time;   // Time (msecs) when last tried.
 public:
 	Schedule(Actor *n);
-	virtual ~Schedule() {
-		remove_clients();
-	}
+	virtual ~Schedule() = default;
 	int get_prev_type() const {
 		return prev_type;
 	}
@@ -117,11 +112,6 @@ public:
 	virtual int get_actual_type(Actor *npc) const;
 	// Look for foes.
 	bool seek_foes();
-	/* For Object_client: +++++ Override in sub-classes. */
-	// Notify that schedule's obj. has been moved or deleted.
-	virtual void notify_object_gone(Game_object *obj) {
-		ignore_unused_variable_warning(obj);
-	}
 	bool try_proximity_usecode(int odds);
 };
 
@@ -129,21 +119,20 @@ public:
  *	A schedule that creates objects that need to be cleaned up after.
  */
 class Schedule_with_objects : public Schedule {
-	vector<Game_object *> created;	// Items we created.
+	std::vector<Game_object_weak> created;	// Items we created.
+	Game_object_weak current_item;		// One we're using/walking to.
 protected:
-	Game_object *current_item;		// One we're using/walking to.
-	int items_in_hand; 	  	// # NPC's desk items.
+    Game_object *get_current_item() {
+	  return current_item.lock().get();
+	}
+	void set_current_item(Game_object *obj);
+    int items_in_hand; 	  	// # NPC's desk items.
 	void cleanup();				// Remove items we created.
 public:
-	Schedule_with_objects(Actor *n) : Schedule(n), current_item(0),
-									items_in_hand(0) {
-	}
-	~Schedule_with_objects();
-	virtual void notify_object_gone(Game_object *obj);
-	void add_object(Game_object *obj) {
-		created.push_back(obj);
-		add_client(obj);
+	Schedule_with_objects(Actor *n) : Schedule(n), items_in_hand(0) {
 	}
+	~Schedule_with_objects() override;
+	void add_object(Game_object *obj);
 	// Find desk or waiter items.
 	virtual int find_items(Game_object_vector& vec, int dist) = 0;
 	bool walk_to_random_item(int dist = 16);
@@ -156,51 +145,45 @@ public:
 class Scripted_schedule : public Schedule {
 	Usecode_value *inst;        // Usecode schedule instance.
 	// Usecode function #'s:
-	int now_what_id, im_dormant_id, ending_id, set_weapon_id, set_bed_id,
-	    notify_object_gone_id;
+	int now_what_id, im_dormant_id, ending_id, set_weapon_id, set_bed_id;
 	void run(int id);
 public:
 	Scripted_schedule(Actor *n, int type);
-	virtual ~Scripted_schedule();
-	virtual void now_what() {
+	~Scripted_schedule() override;
+	void now_what() override {
 		run(now_what_id);
 	}
-	virtual void im_dormant() {
+	void im_dormant() override {
 		run(im_dormant_id);
 	}
-	virtual void ending(int newtype) {
+	void ending(int newtype) override {
 		ignore_unused_variable_warning(newtype);
 		run(ending_id);
 	}
-	virtual void set_weapon(bool removed = false) {
+	void set_weapon(bool removed = false) override {
 		ignore_unused_variable_warning(removed);
 		run(set_weapon_id);
 	}
-	virtual void set_bed(Game_object *b) {
+	void set_bed(Game_object *b) override {
 		ignore_unused_variable_warning(b);
 		run(set_bed_id);
 	}
-	virtual void notify_object_gone(Game_object *obj) {
-		ignore_unused_variable_warning(obj);
-		run(notify_object_gone_id);
-	}
 };
 
 /*
  *  Street maintenance (turn lamps on/off).
  */
 class Street_maintenance_schedule : public Schedule {
-	Game_object *obj;       // Lamp/shutters.
+	Game_object_weak obj;       // Lamp/shutters.
 	int shapenum, framenum;     // Save original shapenum.
 	Actor_action *paction;      // Path to follow to get there.
 	Tile_coord oldloc;
 public:
 	Street_maintenance_schedule(Actor *n, Actor_action *p, Game_object *o);
-	virtual void now_what();
+	void now_what() override;
 	// For Usecode intrinsic.
-	virtual int get_actual_type(Actor *npc) const;
-	virtual void notify_object_gone(Game_object *obj);
-	virtual void ending(int newtype);
+	int get_actual_type(Actor *npc) const override;
+	void ending(int newtype) override;
 };
 
 /*
@@ -212,7 +195,7 @@ class Follow_avatar_schedule : public Schedule {
 public:
 	Follow_avatar_schedule(Actor *n) : Schedule(n), next_path_time(0)
 	{  }
-	virtual void now_what();    // Now what should NPC do?
+	void now_what() override;    // Now what should NPC do?
 };
 
 /*
@@ -222,7 +205,7 @@ class Wait_schedule : public Schedule {
 public:
 	Wait_schedule(Actor *n) : Schedule(n)
 	{  }
-	virtual void now_what();    // Now what should NPC do?
+	void now_what() override;    // Now what should NPC do?
 };
 
 /*
@@ -240,7 +223,7 @@ public:
 	static Pace_schedule *create_horiz(Actor *n);
 	static Pace_schedule *create_vert(Actor *n);
 	static void pace(Actor *npc, char &which, int &phase, Tile_coord &blocked, int delay);
-	virtual void now_what();    // Now what should NPC do?
+	void now_what() override;    // Now what should NPC do?
 };
 
 /*
@@ -251,9 +234,9 @@ class Eat_at_inn_schedule : public Schedule {
 public:
 	Eat_at_inn_schedule(Actor *n) : Schedule(n), sitting_at_chair(false)
 	{  }
-	virtual void now_what();    // Now what should NPC do?
-	virtual void ending(int newtype); // Switching to another schedule
-	virtual void im_dormant();  // Just went dormant.
+	void now_what() override;    // Now what should NPC do?
+	void ending(int new_type) override; // Switching to another schedule
+	void im_dormant() override;  // Just went dormant.
 };
 
 /*
@@ -272,7 +255,7 @@ class Preach_schedule : public Schedule {
 public:
 	Preach_schedule(Actor *n) : Schedule(n), state(find_podium)
 	{  }
-	virtual void now_what();    // Now what should NPC do?
+	void now_what() override;    // Now what should NPC do?
 };
 
 /*
@@ -281,7 +264,7 @@ public:
 class Patrol_schedule : public Schedule {
 	enum {PATH_SHAPE = 607};
 	static int num_path_eggs;
-	vector<Game_object *> paths; // Each 'path' object.
+	std::vector<Game_object *> paths; // Each 'path' object.
 	int pathnum;            // # of next we're heading towards.
 	int dir;                // 1 or -1;
 	int state;              // The patrol state.
@@ -289,15 +272,14 @@ class Patrol_schedule : public Schedule {
 	char whichdir;          // For 'pace' path eggs.
 	int phase;              // For 'pace' path eggs.
 	int pace_count;         // For 'pace' path eggs.
-	Game_object *hammer;    // For 'hammer' path eggs.
-	Game_object *book;      // For 'read' path eggs.
+	Game_object_weak hammer;    // For 'hammer' path eggs.
+	Game_object_weak book;      // For 'read' path eggs.
 	bool seek_combat;       // The NPC should seek enemies while patrolling.
 	bool forever;           // If should keep executing last path egg.
 public:
 	Patrol_schedule(Actor *n);
-	virtual void now_what();    // Now what should NPC do?
-	virtual void ending(int newtype); // Switching to another schedule
-	virtual void notify_object_gone(Game_object *obj);
+	void now_what() override;    // Now what should NPC do?
+	void ending(int new_type) override; // Switching to another schedule
 };
 
 /*
@@ -311,7 +293,7 @@ protected:
 	Talk_schedule(Actor *n, int fb, int lb, int eid);
 public:
 	Talk_schedule(Actor *n);
-	virtual void now_what();    // Now what should NPC do?
+	void now_what() override;    // Now what should NPC do?
 };
 
 /*
@@ -321,8 +303,8 @@ class Arrest_avatar_schedule : public Talk_schedule {
 public:
 	Arrest_avatar_schedule(Actor *n);
 	// For Usecode intrinsic.
-	virtual int get_actual_type(Actor *npc) const;
-	virtual void ending(int newtype);// Switching to another schedule.
+	int get_actual_type(Actor *npc) const override;
+	void ending(int newtype) override;// Switching to another schedule.
 };
 
 /*
@@ -335,7 +317,7 @@ protected:
 	//   dir.
 public:
 	Loiter_schedule(Actor *n, int d = 12);
-	virtual void now_what();    // Now what should NPC do?
+	void now_what() override;    // Now what should NPC do?
 };
 
 /*
@@ -346,18 +328,18 @@ protected:
 	int phase;
 public:
 	Graze_schedule(Actor *n, int d = 12) : Loiter_schedule(n, d), phase(0) {  }
-	virtual void now_what();    // Now what should NPC do?
+	void now_what() override;    // Now what should NPC do?
 };
 
 /*
  *  Kid games.
  */
 class Kid_games_schedule : public Loiter_schedule {
-	vector<Actor *> kids;           // Other kids playing.
+	std::vector<Actor *> kids;           // Other kids playing.
 public:
 	Kid_games_schedule(Actor *n) : Loiter_schedule(n, 10)
 	{  }
-	virtual void now_what();    // Now what should NPC do?
+	void now_what() override;    // Now what should NPC do?
 };
 
 /*
@@ -367,7 +349,7 @@ class Dance_schedule : public Loiter_schedule {
 public:
 	Dance_schedule(Actor *n) : Loiter_schedule(n, 4)
 	{  }
-	virtual void now_what();    // Now what should NPC do?
+	void now_what() override;    // Now what should NPC do?
 };
 
 /*
@@ -376,22 +358,23 @@ public:
 class Tool_schedule : public Loiter_schedule {
 protected:
 	int toolshape;          // Pick/scythe shape.
-	Game_object *tool;
+	Game_object_weak tool;
 	void get_tool();
 public:
 	Tool_schedule(Actor *n, int shnum) : Loiter_schedule(n, 12),
-		toolshape(shnum), tool(0)
+		toolshape(shnum)
 	{  }
-	virtual void now_what() = 0;    // Now what should NPC do?
-	virtual void ending(int newtype);// Switching to another schedule.
+	void now_what() override = 0;    // Now what should NPC do?
+	void ending(int newtype) override;// Switching to another schedule.
 };
 
 /*
  *  Farmer.
  */
 class Farmer_schedule : public Tool_schedule {
-	Game_object *crop;
+	Game_object_weak crop;
 	int grow_cnt;
+  	int frame_group0;		// First in group of 3 that's being cut.
 	enum {
 	    start,
 	    find_crop,
@@ -401,16 +384,16 @@ class Farmer_schedule : public Tool_schedule {
 	} state;
 public:
 	Farmer_schedule(Actor *n) : Tool_schedule(n, 618),
-		crop(0), grow_cnt(0), state(start)
+		grow_cnt(0), frame_group0(-1), state(start)
 	{  }
-	virtual void now_what();    // Now what should NPC do?
+	void now_what() override;    // Now what should NPC do?
 };
 
 /*
  *  Miner.
  */
 class Miner_schedule : public Tool_schedule {
-	Game_object *ore;
+	Game_object_weak ore;
 	enum {
 	    find_ore,
 	    attack_ore,
@@ -418,10 +401,9 @@ class Miner_schedule : public Tool_schedule {
 	    wander
 	} state;
 public:
-	Miner_schedule(Actor *n) : Tool_schedule(n, 624),
-		ore(0), state(find_ore)
+	Miner_schedule(Actor *n) : Tool_schedule(n, 624), state(find_ore)
 	{  }
-	virtual void now_what();    // Now what should NPC do?
+	void now_what() override;    // Now what should NPC do?
 };
 
 /*
@@ -431,7 +413,7 @@ class Hound_schedule : public Schedule {
 public:
 	Hound_schedule(Actor *n) : Schedule(n)
 	{  }
-	virtual void now_what();    // Now what should NPC do?
+	void now_what() override;    // Now what should NPC do?
 };
 
 /*
@@ -441,7 +423,7 @@ class Wander_schedule : public Loiter_schedule {
 public:
 	Wander_schedule(Actor *n) : Loiter_schedule(n, 128)
 	{  }
-	virtual void now_what();    // Now what should NPC do?
+	void now_what() override;    // Now what should NPC do?
 };
 
 /*
@@ -449,45 +431,44 @@ public:
  */
 class Sleep_schedule : public Schedule {
 	Tile_coord floorloc;        // Where NPC was standing before.
-	Game_object *bed;       // Bed being slept on, or 0.
+	Game_object_weak bed;       // Bed being slept on, or 0.
 	int state;
 	int spread0, spread1;       // Range of bedspread frames.
 	bool for_nap_time;
 public:
 	Sleep_schedule(Actor *n);
-	virtual void now_what();    // Now what should NPC do?
-	virtual void ending(int newtype);// Switching to another schedule.
+	void now_what() override;    // Now what should NPC do?
+	void ending(int new_type) override;// Switching to another schedule.
 	// Set where to sleep.
-	virtual void set_bed(Game_object *b);
-	virtual void notify_object_gone(Game_object *obj);
-	virtual void im_dormant();  // Just went dormant.
+	void set_bed(Game_object *b) override;
+	void im_dormant() override;  // Just went dormant.
 	static bool is_bed_occupied(Game_object *bed, Actor *npc);
+	static bool sleep_interrupted;
 };
 
 /*
  *  Sit in a chair.
  */
 class Sit_schedule : public Schedule {
-	Game_object *chair;     // What to sit in.
+	Game_object_weak chair;     // What to sit in.
 	bool sat;           // True if we already sat down.
 	bool did_barge_usecode;     // So we only call it once.
 public:
-	Sit_schedule(Actor *n, Game_object *ch = 0);
-	virtual void now_what();    // Now what should NPC do?
-	virtual void notify_object_gone(Game_object *obj);
-	virtual void im_dormant();  // Just went dormant.
+	Sit_schedule(Actor *n, Game_object *ch = nullptr);
+	void now_what() override;    // Now what should NPC do?
+	void im_dormant() override;  // Just went dormant.
 	static bool is_occupied(Game_object *chairobj, Actor *actor);
-	static bool set_action(Actor *actor, Game_object *chairobj = 0,
-	                       int delay = 0, Game_object **chair_found = 0);
+	static bool set_action(Actor *actor, Game_object *chairobj = nullptr,
+	                       int delay = 0, Game_object **chair_found = nullptr);
 };
 
 /*
  *  Desk work - Just sit in front of desk.
  */
 class Desk_schedule : public Schedule_with_objects {
-	Game_object *chair;     // What to sit in.
-	Game_object *desk, *table;
-	vector<Game_object *> tables;	// Other tables to work at.
+	Game_object_weak chair;     // What to sit in.
+	Game_object_weak desk, table;
+	std::vector<Game_object_weak> tables;	// Other tables to work at.
 	enum {
 	    desk_setup,
 	    sit_at_desk,
@@ -495,15 +476,14 @@ class Desk_schedule : public Schedule_with_objects {
 	    picked_up_item,
 	    work_at_table
 	} state;
-	virtual int find_items(Game_object_vector& vec, int dist);
+	int find_items(Game_object_vector& vec, int dist) override;
 	void find_tables(int shapenum);
 	bool walk_to_table();
 public:
 	Desk_schedule(Actor *n);
-	virtual void now_what();    // Now what should NPC do?
-	virtual void ending(int newtype);// Switching to another schedule.
-	virtual void notify_object_gone(Game_object *obj);
-	virtual void im_dormant();  // Just went dormant.
+	void now_what() override;    // Now what should NPC do?
+	void ending(int new_type) override;// Switching to another schedule.
+	void im_dormant() override;  // Just went dormant.
 };
 
 /*
@@ -513,17 +493,17 @@ class Shy_schedule : public Schedule {
 public:
 	Shy_schedule(Actor *n) : Schedule(n)
 	{  }
-	virtual void now_what();    // Now what should NPC do?
+	void now_what() override;    // Now what should NPC do?
 };
 
 /*
  *  Lab work.
  */
 class Lab_schedule : public Schedule {
-	vector<Game_object *> tables;
-	Game_object *chair;     // Chair to sit in.
-	Game_object *book;      // Book to read.
-	Game_object *cauldron;
+	std::vector<Game_object_weak> tables;
+	Game_object_weak chair;     // Chair to sit in.
+	Game_object_weak book;      // Book to read.
+	Game_object_weak cauldron;
 	Tile_coord spot_on_table;
 	enum {
 	    start,
@@ -538,8 +518,7 @@ class Lab_schedule : public Schedule {
 	void init();
 public:
 	Lab_schedule(Actor *n);
-	virtual void now_what();    // Now what should NPC do?
-	virtual void notify_object_gone(Game_object *obj);
+	void now_what() override;    // Now what should NPC do?
 };
 
 /*
@@ -551,7 +530,7 @@ class Thief_schedule : public Schedule {
 public:
 	Thief_schedule(Actor *n) : Schedule(n), next_steal_time(0)
 	{  }
-	virtual void now_what();
+	void now_what() override;
 };
 
 /*
@@ -560,14 +539,14 @@ public:
 class Waiter_schedule : public Schedule_with_objects {
 	Tile_coord startpos;        // Starting position.
 	Actor *customer;        // Current customer.
-	Game_object *prep_table;    // Table we're working at.
+	Game_object_weak prep_table;    // Table we're working at.
 	bool cooking;
-	vector<Actor *> customers;  // List of customers.
-	vector<Actor *> customers_ordered;  // Taken orders from these.
-	vector<Game_object *> prep_tables; // Prep. tables.
-	vector<Game_object *> counters;    // Places to hang out.
-	vector<Game_object *> eating_tables; // Tables with chairs around them.
-	vector<Game_object *> unattended_plates;
+	std::vector<Actor *> customers;  // List of customers.
+	std::vector<Actor *> customers_ordered;  // Taken orders from these.
+	std::vector<Game_object_weak > prep_tables; // Prep. tables.
+	std::vector<Game_object_weak > counters;    // Places to hang out.
+	std::vector<Game_object_weak > eating_tables; // Tables with chairs around them.
+	std::vector<Game_object_weak > unattended_plates;
 	enum {
 	    waiter_setup,
 	    get_customer,
@@ -584,7 +563,7 @@ class Waiter_schedule : public Schedule_with_objects {
 	    walk_to_cleanup_food,
 	    cleanup_food
 	} state;
-	virtual int find_items(Game_object_vector& vec, int dist);
+	int find_items(Game_object_vector& vec, int dist) override;
 	bool find_unattended_plate();
 	bool find_customer();
 	void find_tables(int shapenum, int dist, bool is_prep = false);
@@ -601,22 +580,21 @@ class Waiter_schedule : public Schedule_with_objects {
 	Game_object *find_serving_spot(Tile_coord &spot);
 public:
 	Waiter_schedule(Actor *n);
-	virtual void now_what();    // Now what should NPC do?
-	virtual void ending(int newtype);// Switching to another schedule.
-	virtual void notify_object_gone(Game_object *obj);
+	void now_what() override;    // Now what should NPC do?
+	void ending(int new_type) override;// Switching to another schedule.
 };
 
 /*
  *  Sew/weave schedule.
  */
 class Sew_schedule : public Schedule {
-	Game_object *bale;      // Bale of wool.
-	Game_object *spinwheel;
-	Game_object *chair;     // In front of spinning wheel.
-	Game_object *spindle;       // Spindle of thread.
-	Game_object *loom;
-	Game_object *cloth;
-	Game_object *work_table, *wares_table;
+	Game_object_weak bale;      // Bale of wool.
+	Game_object_weak spinwheel;
+	Game_object_weak chair;     // In front of spinning wheel.
+	Game_object_weak spindle;       // Spindle of thread.
+	Game_object_weak loom;
+	Game_object_weak cloth;
+	Game_object_weak work_table, wares_table;
 	int sew_clothes_cnt;
 	enum {
 	    get_wool,
@@ -634,21 +612,20 @@ class Sew_schedule : public Schedule {
 	} state;
 public:
 	Sew_schedule(Actor *n);
-	virtual void now_what();    // Now what should NPC do?
-	virtual void ending(int newtype);// Switching to another schedule.
-	virtual void notify_object_gone(Game_object *obj);
+	void now_what() override;    // Now what should NPC do?
+	void ending(int new_type) override;// Switching to another schedule.
 };
 
 /*
  *  Bake schedule
  */
 class Bake_schedule : public Schedule {
-	Game_object *oven;
-	Game_object *worktable;
-	Game_object *displaytable;
-	Game_object *flourbag;
-	Game_object *dough;
-	Game_object *dough_in_oven;
+	Game_object_weak oven;
+	Game_object_weak worktable;
+	Game_object_weak displaytable;
+	Game_object_weak flourbag;
+	Game_object_weak dough;
+	Game_object_weak dough_in_oven;
 	bool clearing;
 	enum {
 	    find_leftovers,     // Look for misplaced dough already made by this schedule
@@ -666,22 +643,21 @@ class Bake_schedule : public Schedule {
 	} state;
 public:
 	Bake_schedule(Actor *n);
-	virtual void now_what();
-	virtual void ending(int newtype);
-	virtual void notify_object_gone(Game_object *obj);
+	void now_what() override;
+	void ending(int new_type) override;
 };
 
 /*
  *  Blacksmith schedule
  */
 class Forge_schedule : public Schedule {
-	Game_object *tongs;
-	Game_object *hammer;
-	Game_object *blank;
-	Game_object *firepit;
-	Game_object *anvil;
-	Game_object *trough;
-	Game_object *bellows;
+	Game_object_weak tongs;
+	Game_object_weak hammer;
+	Game_object_weak blank;
+	Game_object_weak firepit;
+	Game_object_weak anvil;
+	Game_object_weak trough;
+	Game_object_weak bellows;
 	enum {
 	    put_sword_on_firepit,
 	    use_bellows,
@@ -697,16 +673,15 @@ class Forge_schedule : public Schedule {
 	} state;
 public:
 	Forge_schedule(Actor *n);
-	virtual void now_what();    // Now what should NPC do?
-	virtual void ending(int newtype); // Switching to another schedule
-	virtual void notify_object_gone(Game_object *obj);
+	void now_what() override;    // Now what should NPC do?
+	void ending(int new_type) override; // Switching to another schedule
 };
 
 /*
  *  Eat without a server
  */
 class Eat_schedule : public Schedule {
-	Game_object *plate;
+	Game_object_weak plate;
 	enum {
 	    eat,        // eat food and say food barks
 	    find_plate, // make sure there is a plate, create one if not
@@ -714,10 +689,9 @@ class Eat_schedule : public Schedule {
 	} state;
 public:
 	Eat_schedule(Actor *n);
-	virtual void now_what();    // Now what should NPC do?
-	virtual void ending(int newtype); // Switching to another schedule
-	virtual void notify_object_gone(Game_object *obj);
-	virtual void im_dormant();  // Just went dormant.
+	void now_what() override;    // Now what should NPC do?
+	void ending(int new_type) override; // Switching to another schedule
+	void im_dormant() override;  // Just went dormant.
 };
 
 /*
@@ -730,31 +704,29 @@ class Walk_to_schedule : public Schedule {
 	int retries;            // # failures at finding path.
 	int legs;           // # times restarted walk.
 	// Set to walk off screen.
-	void walk_off_screen(Rectangle &screen, Tile_coord &goal);
+	void walk_off_screen(TileRect &screen, Tile_coord &goal);
 public:
 	Walk_to_schedule(Actor *n, Tile_coord const &d, int new_sched,
 	                 int delay = -1);
-	virtual void now_what();    // Now what should NPC do?
-	virtual void im_dormant();  // Just went dormant.
+	void now_what() override;    // Now what should NPC do?
+	void im_dormant() override;  // Just went dormant.
 	// For Usecode intrinsic.
-	virtual int get_actual_type(Actor *npc) const;
+	int get_actual_type(Actor *npc) const override;
 };
 
 /*
  *  An NPC schedule change:
  */
 class Schedule_change {
-	static vector<char *> script_names; // For Scripted_schedule's.
-	unsigned char time;     // Time*3hours when this takes effect.
-	unsigned char type;     // Schedule_type value.
-	unsigned char days;     // A bit for each day (0-6).  We don't
+	static std::vector<std::string> script_names; // For Scripted_schedule's.
+	unsigned char time = 0;     // Time*3hours when this takes effect.
+	unsigned char type = 0;     // Schedule_type value.
+	unsigned char days = 0x7f;  // A bit for each day (0-6).  We don't
 	//   yet use this.
 	Tile_coord pos;         // Location.
 public:
-	Schedule_change() : time(0), type(0), days(0x7f)
-	{  }
 	static void clear();
-	static vector<char *> &get_script_names() {
+	static std::vector<std::string> &get_script_names() {
 		return script_names;
 	}
 	void set4(const unsigned char *ent);  // Create from 4-byte entry.
@@ -771,9 +743,9 @@ public:
 	Tile_coord get_pos() const {
 		return pos;
 	}
-	static char *get_script_name(int ty) {
+	static const char *get_script_name(int ty) {
 		return ty >= Schedule::first_scripted_schedule ?
-		       script_names[ty - Schedule::first_scripted_schedule] : 0;
+		       script_names[ty - Schedule::first_scripted_schedule].c_str() : nullptr;
 	}
 };
 
