@@ -5,7 +5,7 @@
  **/
 
 /*
-Copyright (C) 2000 The Exult Team
+Copyright (C) 2001-2022 The Exult Team
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -27,7 +27,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #endif
 
 
-#include <stdio.h>
+#include <cstdio>
 #include "ucexpr.h"
 #include "ucsym.h"
 #include "utils.h"
@@ -39,6 +39,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "basic_block.h"
 #include "ignore_unused_variable_warning.h"
 
+using std::vector;
+
 /*
  *  Default.  Just push the one value.
  *
@@ -77,12 +79,20 @@ Uc_var_symbol *Uc_expression::need_var(
 	// Create a 'tmp' variable.
 	Uc_var_symbol *var = fun->add_symbol(buf);
 	if (!var)
-		return 0;       // Shouldn't happen.  Err. reported.
+		return nullptr;       // Shouldn't happen.  Err. reported.
 	gen_value(out);         // Want to assign this value to it.
 	var->gen_assign(out);
 	return var;
 }
 
+/*
+ *  Concatenate expression into an array.
+ */
+
+void Uc_expression::add_to(Uc_array_expression *arr) {
+	arr->add(this);
+}
+
 /*
  *  Evaluate constant.
  *
@@ -291,7 +301,7 @@ void Uc_flag_expression::gen_assign(
 }
 
 inline bool Uc_var_expression::is_struct() const {
-	return var->get_struct() != 0;
+	return var->get_struct() != nullptr;
 }
 
 inline Uc_struct_symbol *Uc_var_expression::get_struct() const {
@@ -331,7 +341,7 @@ void Uc_binary_expression::gen_value(
 ) {
 	int ival;
 	if (eval_const(ival)) {
-		Uc_int_expression *iexpr = new Uc_int_expression(ival, intop);
+		auto *iexpr = new Uc_int_expression(ival, intop);
 		iexpr->gen_value(out);
 		delete iexpr;
 	} else {
@@ -350,7 +360,8 @@ void Uc_binary_expression::gen_value(
 bool Uc_binary_expression::eval_const(
     int &val            // Value returned here.
 ) {
-	int val1, val2;         // Get each side.
+	int val1;
+	int val2;         // Get each side.
 	if (!left->eval_const(val1) || !right->eval_const(val2)) {
 		val = 0;
 		return false;
@@ -419,7 +430,7 @@ void Uc_unary_expression::gen_value(
 ) {
 	int ival;
 	if (eval_const(ival)) {
-		Uc_int_expression *iexpr = new Uc_int_expression(ival);
+		auto *iexpr = new Uc_int_expression(ival);
 		iexpr->gen_value(out);
 		delete iexpr;
 	} else {
@@ -618,9 +629,8 @@ int Uc_string_prefix_expression::get_string_offset(
 
 Uc_array_expression::~Uc_array_expression(
 ) {
-	for (std::vector<Uc_expression *>::iterator it = exprs.begin();
-	        it != exprs.end(); ++it)
-		delete(*it);
+	for (auto *expr : exprs)
+		delete expr;
 }
 
 /*
@@ -632,16 +642,18 @@ Uc_array_expression::~Uc_array_expression(
 void Uc_array_expression::concat(
     Uc_expression *e
 ) {
-	Uc_array_expression *arr = dynamic_cast<Uc_array_expression *>(e);
-	if (!arr)
-		add(e);         // Singleton?  Just add it.
-	else {
-		for (std::vector<Uc_expression *>::iterator it =
-		            arr->exprs.begin(); it != arr->exprs.end(); ++it)
-			add(*it);
-		arr->exprs.clear(); // Don't want to delete elements.
-		delete arr;     // But this array is history.
-	}
+	e->add_to(this);
+}
+
+/*
+ *  Concatenate array into another array, and destroys the current array
+ *  is destroyed and should not be used after this function is called.
+ */
+
+void Uc_array_expression::add_to(Uc_array_expression *arr) {
+	arr->add(exprs);
+	exprs.clear();   // Don't want to delete elements.
+	delete this;     // But this array is history.
 }
 
 /*
@@ -667,7 +679,7 @@ int Uc_array_expression::gen_values(
 ) {
 	int actual = 0;         // (Just to be safe.)
 	// Push backwards, so #0 pops first.
-	for (std::vector<Uc_expression *>::reverse_iterator it =
+	for (auto it =
 	            exprs.rbegin(); it != exprs.rend(); ++it) {
 		Uc_expression *expr = *it;
 		if (expr) {
@@ -679,7 +691,7 @@ int Uc_array_expression::gen_values(
 }
 
 inline bool Uc_call_expression::is_struct() const {
-	return sym->get_struct() != 0;
+	return sym->get_struct() != nullptr;
 }
 
 inline Uc_struct_symbol *Uc_call_expression::get_struct() const {
@@ -687,7 +699,7 @@ inline Uc_struct_symbol *Uc_call_expression::get_struct() const {
 }
 
 inline bool Uc_call_expression::is_class() const {
-	return sym->get_cls() != 0;
+	return sym->get_cls() != nullptr;
 }
 
 inline Uc_class *Uc_call_expression::get_cls() const {
@@ -700,7 +712,7 @@ inline Uc_class *Uc_call_expression::get_cls() const {
  */
 
 int Uc_call_expression::is_object_function(bool error) const {
-	Uc_intrinsic_symbol *fun = dynamic_cast<Uc_intrinsic_symbol *>(sym);
+	auto *fun = dynamic_cast<Uc_intrinsic_symbol *>(sym);
 	if (!fun)
 		return -1;  // Can't determine.
 
@@ -730,7 +742,7 @@ int Uc_call_expression::is_object_function(bool error) const {
  */
 
 void Uc_call_expression::check_params() {
-	Uc_function_symbol *fun = dynamic_cast<Uc_function_symbol *>(sym);
+	auto *fun = dynamic_cast<Uc_function_symbol *>(sym);
 	if (!fun) {
 		// Intrinsics; do nothing for now.
 		return;
@@ -751,7 +763,7 @@ void Uc_call_expression::check_params() {
 	for (unsigned long i = ignore_this; i < parmscnt; i++) {
 		Uc_expression *expr = callparms[i - ignore_this];
 		Uc_var_symbol *var = protoparms[i];
-		Uc_class_inst_symbol *cls =
+		auto *cls =
 		    dynamic_cast<Uc_class_inst_symbol *>(var);
 		char buf[180];
 		if (expr->is_class()) {
@@ -806,7 +818,7 @@ void Uc_call_expression::gen_value(
 		char buf[150];
 		sprintf(buf, "'%s' isn't a function or intrinsic",
 		        sym->get_name());
-		sym = 0;        // Avoid repeating error if in loop.
+		sym = nullptr;        // Avoid repeating error if in loop.
 		error(buf);
 	}
 }
