@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2001-2013  The Exult Team
+ *  Copyright (C) 2001-2022  The Exult Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -30,11 +30,11 @@
 #include "miscinf.h"
 #include "data/exult_bg_flx.h"
 #include "array_size.h"
+#include "touchui.h"
 
 using std::size_t;
 using std::strcpy;
 using std::string;
-using std::vector;
 
 //TODO: show_face & show_avatar_choices seem to share code?
 //TODO: show_avatar_choices shouldn't first convert to char**, probably
@@ -50,30 +50,23 @@ public:
 	//int frame;
 	bool text_pending;      // Text has been written, but user
 	//   has not yet been prompted.
-	Rectangle face_rect;    // Rectangle where face is shown.
-	Rectangle text_rect;    // Rectangle NPC statement is shown in.
+	TileRect face_rect;     // Rectangle where face is shown.
+	TileRect text_rect;     // Rectangle NPC statement is shown in.
 	bool large_face;        // Guardian, snake.
 	int last_text_height;   // Height of last text painted.
 	string cur_text;        // Current text being shown.
 	Npc_face_info(ShapeID &sid, int num)
 		: shape(sid), face_num(num),
-		  text_pending(0), large_face(false)
+		  text_pending(false), large_face(false)
 	{  }
 };
 
-Conversation::Conversation()
-	: num_faces(0), last_face_shown(0), avatar_face(0, 0, 0, 0), conv_choices(0) {
-	const int max_faces = array_size(face_info);
-	for (int i = 0; i < max_faces; i++)
-		face_info[i] = 0;
-}
-
 Conversation::~Conversation() {
 	delete [] conv_choices;
 }
 
 
-void Conversation::clear_answers(void) {
+void Conversation::clear_answers() {
 	answers.clear();
 }
 
@@ -93,18 +86,14 @@ void Conversation::add_answer(Usecode_value &val) {
 	if (size) {         // An array?
 		for (int i = 0; i < size; i++)
 			add_answer(val.get_elem(i));
-	} else if ((str = val.get_str_value()) != 0)
+	} else if ((str = val.get_str_value()) != nullptr)
 		add_answer(str);
 }
 
 void Conversation::remove_answer(const char *str) {
-	std::vector<string>::iterator it;
+	auto it = std::find(answers.cbegin(), answers.cend(), str);
 
-	for (it = answers.begin(); it != answers.end(); ++it)
-		if (*it == str)
-			break;
-
-	if (it != answers.end())
+	if (it != answers.cend())
 		answers.erase(it);
 }
 
@@ -135,7 +124,10 @@ void Conversation::init_faces() {
 	for (int i = 0; i < max_faces; i++) {
 		if (face_info[i])
 			delete face_info[i];
-		face_info[i] = 0;
+		face_info[i] = nullptr;
+		if (touchui != nullptr) {
+			touchui->showGameControls();
+		}
 	}
 	num_faces = 0;
 	last_face_shown = -1;
@@ -150,13 +142,15 @@ void Conversation::set_face_rect(
 	int text_height = sman->get_text_height(0);
 	// Figure starting y-coord.
 	// Get character's portrait.
-	Shape_frame *face = info->shape.get_shapenum() >= 0 ? info->shape.get_shape() : 0;
-	int face_w = 32, face_h = 32;
+	Shape_frame *face = info->shape.get_shapenum() >= 0 ? info->shape.get_shape() : nullptr;
+	int face_w = 32;
+	int face_h = 32;
 	if (face) {
 		face_w = face->get_width();
 		face_h = face->get_height();
 	}
-	int startx, extraw;
+	int startx;
+	int extraw;
 	if (face_w >= 119) {
 		startx = (screenw - face_w) / 2;
 		extraw = 0;
@@ -165,7 +159,8 @@ void Conversation::set_face_rect(
 		startx = 8;
 		extraw = 4;
 	}
-	int starty, extrah;
+	int starty;
+	int extrah;
 	if (face_h >= 142) {
 		starty = (screenh - face_h) / 2;
 		extrah = 0;
@@ -181,18 +176,19 @@ void Conversation::set_face_rect(
 		starty = 1;
 		extrah = 4;
 	}
-	info->face_rect = gwin->clip_to_win(Rectangle(startx, starty,
+	info->face_rect = gwin->clip_to_win(TileRect(startx, starty,
 	                                    face_w + extraw, face_h + extrah));
-	Rectangle &fbox = info->face_rect;
+	TileRect &fbox = info->face_rect;
 	// This is where NPC text will go.
-	info->text_rect = gwin->clip_to_win(Rectangle(
+	info->text_rect = gwin->clip_to_win(TileRect(
 	                                        fbox.x + fbox.w + 3, fbox.y + 3,
 	                                        screenw - fbox.x - fbox.w - 6, 4 * text_height));
 	// No room?  (Serpent?)
 	if (info->large_face) {
 		// Show in lower center.
-		int x = screenw / 5, y = 3 * (screenh / 4);
-		info->text_rect = Rectangle(x, y,
+		int x = screenw / 5;
+		int y = 3 * (screenh / 4);
+		info->text_rect = TileRect(x, y,
 		                            screenw - (2 * x), screenh - y - 4);
 	}
 	info->last_text_height = info->text_rect.h;
@@ -214,8 +210,9 @@ void Conversation::show_face(int shape, int frame, int slot) {
 		pal->set(-1, 100);
 
 	// Get screen dims.
-	int screenw = gwin->get_width(), screenh = gwin->get_height();
-	Npc_face_info *info = 0;
+	int screenw = gwin->get_width();
+	int screenh = gwin->get_height();
+	Npc_face_info *info = nullptr;
 	// See if already on screen.
 	for (int i = 0; i < max_faces; i++)
 		if (face_info[i] && face_info[i]->face_num == shape) {
@@ -231,7 +228,7 @@ void Conversation::show_face(int shape, int frame, int slot) {
 		if (slot == -1)     // Want next one?
 			slot = num_faces;
 		// Get last one shown.
-		Npc_face_info *prev = slot ? face_info[slot - 1] : 0;
+		Npc_face_info *prev = slot ? face_info[slot - 1] : nullptr;
 		last_face_shown = slot;
 		if (!face_info[slot])
 			num_faces++;    // We're adding one (not replacing).
@@ -242,6 +239,9 @@ void Conversation::show_face(int shape, int frame, int slot) {
 	}
 	gwin->get_win()->set_clip(0, 0, screenw, screenh);
 	paint_faces();          // Paint all faces.
+	if (touchui != nullptr) {
+		touchui->hideGameControls();
+	}
 	gwin->get_win()->clear_clip();
 }
 
@@ -271,8 +271,9 @@ void Conversation::change_face_frame(int frame, int slot) {
 
 	info->shape.set_frame(frame);
 	// Get screen dims.
-	int screenw = gwin->get_width(), screenh = gwin->get_height();
-	Npc_face_info *prev = slot ? face_info[slot - 1] : 0;
+	int screenw = gwin->get_width();
+	int screenh = gwin->get_height();
+	Npc_face_info *prev = slot ? face_info[slot - 1] : nullptr;
 	set_face_rect(info, prev, screenw, screenh);
 
 	gwin->get_win()->set_clip(0, 0, screenw, screenh);
@@ -315,7 +316,7 @@ void Conversation::remove_slot_face(
 		gwin->add_dirty(info->text_rect);
 	}
 	delete face_info[slot];
-	face_info[slot] = 0;
+	face_info[slot] = nullptr;
 	num_faces--;
 	if (last_face_shown == slot) {  // Just in case.
 		int j;
@@ -323,6 +324,9 @@ void Conversation::remove_slot_face(
 			if (face_info[j])
 				break;
 		last_face_shown = j;
+		if (touchui != nullptr && num_faces == 0) {
+			touchui->showGameControls();
+		}
 	}
 }
 
@@ -337,7 +341,7 @@ void Conversation::show_npc_message(const char *msg) {
 	Npc_face_info *info = face_info[last_face_shown];
 	int font = info->large_face ? 7 : 0;    // Use red for Guardian, snake.
 	info->cur_text = "";
-	Rectangle &box = info->text_rect;
+	TileRect &box = info->text_rect;
 //	gwin->paint(box);        // Clear what was there before.
 //	paint_faces();
 	gwin->paint();
@@ -348,7 +352,8 @@ void Conversation::show_npc_message(const char *msg) {
 	                                      gwin->get_text_bg())) < 0) {
 		// More to do?
 		info->cur_text = string(msg, -height);
-		int x, y;
+		int x;
+		int y;
 		char c;
 		gwin->paint();      // Paint scenery beneath
 		Get_click(x, y, Mouse::hand, &c, false, this, true);
@@ -358,7 +363,7 @@ void Conversation::show_npc_message(const char *msg) {
 	// All fit?  Store height painted.
 	info->last_text_height = height;
 	info->cur_text = msg;
-	info->text_pending = 1;
+	info->text_pending = true;
 	gwin->set_painted();
 //	gwin->show();
 }
@@ -384,7 +389,7 @@ void Conversation::clear_text_pending() {
 	const int max_faces = array_size(face_info);
 	for (int i = 0; i < max_faces; i++) // Clear 'pending' flags.
 		if (face_info[i])
-			face_info[i]->text_pending = 0;
+			face_info[i]->text_pending = false;
 }
 
 /*
@@ -396,8 +401,9 @@ void Conversation::show_avatar_choices(int num_choices, char **choices) {
 	Main_actor *main_actor = gwin->get_main_actor();
 	const int max_faces = array_size(face_info);
 	// Get screen rectangle.
-	Rectangle sbox = gwin->get_game_rect();
-	int x = 0, y = 0;       // Keep track of coords. in box.
+	TileRect sbox = gwin->get_game_rect();
+	int x = 0;
+	int y = 0;       // Keep track of coords. in box.
 	int height = sman->get_text_height(0);
 	int space_width = sman->get_text_width(0, " ");
 
@@ -423,7 +429,7 @@ void Conversation::show_avatar_choices(int num_choices, char **choices) {
 		if (!face_info[empty])
 			break;
 	// Get last one shown.
-	Npc_face_info *prev = empty ? face_info[empty - 1] : 0;
+	Npc_face_info *prev = empty ? face_info[empty - 1] : nullptr;
 	int fx = prev ? prev->face_rect.x + prev->face_rect.w + 4 : 16;
 	int fy;
 	if (SI) {
@@ -440,21 +446,19 @@ void Conversation::show_avatar_choices(int num_choices, char **choices) {
 			fy = prev->face_rect.y + prev->face_rect.h;
 		fy += height;
 	}
-	Rectangle mbox(fx, fy, face->get_width(), face->get_height());
+	TileRect mbox(fx, fy, face->get_width(), face->get_height());
 	mbox = mbox.intersect(sbox);
 	avatar_face = mbox;     // Repaint entire width.
 	// Set to where to draw sentences.
-	Rectangle tbox(mbox.x + mbox.w + 8, mbox.y + 4,
-	               sbox.w - mbox.x - mbox.w - 16,
-//				sbox.h - mbox.y - 16);
-	               5 * height); // Try 5 lines.
+	TileRect tbox(mbox.x + mbox.w + 8, mbox.y + 4,
+	              sbox.w - mbox.x - mbox.w - 16,
+	              5 * height); // Try 5 lines.
 	tbox = tbox.intersect(sbox);
-//	gwin->paint(tbox);              // Paint background.
 	// Draw portrait.
 	sman->paint_shape(mbox.x + face->get_xleft(),
 	                  mbox.y + face->get_yabove(), face);
 	delete [] conv_choices;     // Set up new list of choices.
-	conv_choices = new Rectangle[num_choices + 1];
+	conv_choices = new TileRect[num_choices + 1];
 	for (int i = 0; i < num_choices; i++) {
 		char text[256];
 		text[0] = 127;      // A circle.
@@ -466,7 +470,7 @@ void Conversation::show_avatar_choices(int num_choices, char **choices) {
 			y += height - 1;
 		}
 		// Store info.
-		conv_choices[i] = Rectangle(tbox.x + x, tbox.y + y,
+		conv_choices[i] = TileRect(tbox.x + x, tbox.y + y,
 		                            width, height);
 		conv_choices[i] = conv_choices[i].intersect(sbox);
 		avatar_face = avatar_face.add(conv_choices[i]);
@@ -478,7 +482,7 @@ void Conversation::show_avatar_choices(int num_choices, char **choices) {
 	avatar_face.enlarge((3 * c_tilesize) / 4);  // Encloses entire area.
 	avatar_face = avatar_face.intersect(sbox);
 	// Terminate the list.
-	conv_choices[num_choices] = Rectangle(0, 0, 0, 0);
+	conv_choices[num_choices] = TileRect(0, 0, 0, 0);
 	clear_text_pending();
 	gwin->set_painted();
 }
@@ -518,9 +522,9 @@ int Conversation::conversation_choice(int x, int y) {
 	        !conv_choices[i].has_point(x, y); i++)
 		;
 	if (conv_choices[i].w != 0) // Found one?
-		return (i);
+		return i;
 	else
-		return (-1);
+		return -1;
 }
 
 /*
@@ -549,27 +553,28 @@ void Conversation::paint_faces(
 		if (!finfo)
 			continue;
 		Shape_frame *face = finfo->face_num >= 0 ?
-		                    finfo->shape.get_shape() : 0;
-		int face_xleft = 0, face_yabove = 0;
+		                    finfo->shape.get_shape() : nullptr;
 		if (face) {
-			face_xleft = face->get_xleft();
-			face_yabove = face->get_yabove();
-			int fx = finfo->face_rect.x + face_xleft,
-			    fy = finfo->face_rect.y + face_yabove;
+			int face_xleft = face->get_xleft();
+			int face_yabove = face->get_yabove();
+			int fx = finfo->face_rect.x + face_xleft;
+			int fy = finfo->face_rect.y + face_yabove;
 			if (finfo->large_face) {
 				// Guardian, serpents: fill whole screen with the
 				// background pixel.
 				unsigned char px = face->get_topleft_pix();
 				const int xfstart = 0xff - sman->get_xforms_cnt();
-				int fw = finfo->face_rect.w,
-				    fh = finfo->face_rect.h;
+				int fw = finfo->face_rect.w;
+				int fh = finfo->face_rect.h;
 				Image_window8 *win = gwin->get_win();
-				int gw = win->get_game_width(), gh = win->get_game_height();
+				int gw = win->get_game_width();
+				int gh = win->get_game_height();
 				// Fill only if (a) not transparent, (b) is a translucent
 				// color and (c) the face is not covering the entire screen.
 				if (px >= xfstart && px <= 0xfe && (gw > fw || gh > fh)) {
 					Xform_palette &xform = sman->get_xform(px - xfstart);
-					int gx = win->get_start_x(), gy = win->get_start_y();
+					int gx = win->get_start_x();
+					int gy = win->get_start_y();
 					// Another option: 4 fills outside the face area.
 					win->fill_translucent8(0, gw, gh, gx, gy, xform);
 				}
@@ -578,7 +583,7 @@ void Conversation::paint_faces(
 			sman->paint_shape(fx, fy, face, true);
 		}
 		if (text) {     // Show text too?
-			Rectangle &box = finfo->text_rect;
+			TileRect &box = finfo->text_rect;
 			// Use red for Guardian, snake.
 			int font = finfo->large_face ? 7 : 0;
 			sman->paint_text_box(font, finfo->cur_text.c_str(),
@@ -594,11 +599,9 @@ void Conversation::paint_faces(
  */
 
 int Conversation::locate_answer(const char *str) {
-	int num;
-	std::vector<string>::iterator it;
-	num = 0;
-	for (it = answers.begin(); it != answers.end(); ++it) {
-		if (*it == str)
+	int num = 0;
+	for (auto& answer : answers) {
+		if (answer == str)
 			return num;
 		num++;
 	}
