@@ -1,7 +1,7 @@
 /*
  *  debugmsg.h: debugging messages
  *
- *  Copyright (C) 2002  The Exult Team
+ *  Copyright (C) 2002-2022  The Exult Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,8 +18,8 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#ifndef _DEBUGMSG_H
-#define _DEBUGMSG_H
+#ifndef DEBUGMSG_H
+#define DEBUGMSG_H
 
 namespace Exult_server {
 
