@@ -1,7 +1,7 @@
 /*
  *  debugserver.h: handle usecode debugging messages from client
  *
- *  Copyright (C) 2002  The Exult Team
+ *  Copyright (C) 2002-2022  The Exult Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,8 +18,8 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#ifndef _DEBUGSERVER_H
-#define _DEBUGSERVER_H
+#ifndef DEBUGSERVER_H
+#define DEBUGSERVER_H
 
 #if (defined(USE_EXULTSTUDIO) && defined(USECODE_DEBUGGER))
 
