@@ -1,7 +1,7 @@
 /*
  *  stackframe.cc - a usecode interpreter stack frame
  *
- *  Copyright (C) 2002  The Exult Team
+ *  Copyright (C) 2002-2022  The Exult Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -38,10 +38,10 @@ Stack_frame::Stack_frame(Usecode_function *fun,
                          int event,
                          Game_object *caller,
                          int chain, int depth)
-	: function(fun), ip(0), data(0), externs(0), code(0), endp(0),
+	: function(fun), ip(nullptr), data(nullptr), externs(nullptr), code(nullptr), endp(nullptr),
 	  line_number(-1), call_chain(chain), call_depth(depth),
-	  num_externs(0), num_args(0), num_vars(0), locals(0), eventid(event),
-	  caller_item(caller), save_sp(0) {
+	  num_externs(0), num_args(0), num_vars(0), locals(nullptr), eventid(event),
+	  caller_item(shared_from_obj(caller)), save_sp(nullptr) {
 	ip = function->code;
 	endp = ip + function->len;
 
@@ -85,7 +85,7 @@ std::ostream &operator<<(std::ostream &out, Stack_frame &frame) {
 	    << std::hex << std::setw(4) << std::setfill('0')
 	    << static_cast<int>(frame.ip - frame.code) << " in 0x"
 	    << std::setw(4) << frame.function->id
-	    << "(obj=" << std::setw(8) << frame.caller_item
+	    << "(obj=" << std::setw(8) << frame.caller_item.get()
 	    << ",ev=" << frame.eventid
 	    << std::setfill(' ') << std::dec;
 
