@@ -1,7 +1,7 @@
 /*
  *  ucdebugging.cc - Debugging-related functions for usecode
  *
- *  Copyright (C) 2002  The Exult Team
+ *  Copyright (C) 2002-2022  The Exult Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -68,10 +68,9 @@ StepoverBreakpoint::StepoverBreakpoint(Stack_frame *frame)
 }
 
 bool StepoverBreakpoint::check(Stack_frame *frame) const {
-	return ((frame->call_chain == call_chain &&
+	return (frame->call_chain == call_chain &&
 	         frame->call_depth <= call_depth) ||
-
-	        (frame->call_chain < call_chain));
+	        (frame->call_chain < call_chain);
 }
 
 FinishBreakpoint::FinishBreakpoint(Stack_frame *frame)
@@ -81,25 +80,19 @@ FinishBreakpoint::FinishBreakpoint(Stack_frame *frame)
 }
 
 bool FinishBreakpoint::check(Stack_frame *frame) const {
-	return ((frame->call_chain == call_chain &&
+	return (frame->call_chain == call_chain &&
 	         frame->call_depth < call_depth) ||
-
-	        (frame->call_chain < call_chain));
+	        (frame->call_chain < call_chain);
 }
 
 
 
 int Breakpoints::lastID = 0;
 
-Breakpoints::Breakpoints() {
-}
-
 Breakpoints::~Breakpoints() {
-	std::list<Breakpoint *>::iterator iter;
-
 	// clear queue
-	for (iter = breaks.begin(); iter != breaks.end(); ++iter) {
-		delete(*iter);
+	for (auto *brk : breaks) {
+		delete brk;
 	}
 }
 
@@ -119,19 +112,17 @@ int Breakpoints::check(Stack_frame *frame) {
 
 	int breakID = -1;
 
-	std::list<Breakpoint *>::iterator iter;
-
-	for (iter = breaks.begin(); iter != breaks.end(); ++iter) {
-		if ((*iter)->check(frame)) {
-			breakID = (*iter)->id;
-			if ((*iter)->once) {
-				delete(*iter);
-				(*iter) = 0;
+	for (auto *&brk : breaks) {
+		if (brk->check(frame)) {
+			breakID = brk->id;
+			if (brk->once) {
+				delete brk;
+				brk = nullptr;
 			}
 		}
 	}
 
-	breaks.remove(static_cast<Breakpoint *>(0)); // delete all NULLs from the list
+	breaks.remove(nullptr); // delete all nullptr from the list
 
 	return breakID;
 }
@@ -149,24 +140,20 @@ void Breakpoints::remove(Breakpoint *breakpoint) {
 bool Breakpoints::remove(int id) {
 	bool found = false;
 
-	std::list<Breakpoint *>::iterator iter;
-
-	for (iter = breaks.begin(); iter != breaks.end(); ++iter) {
-		if ((*iter)->id == id) {
+	for (auto *&brk : breaks) {
+		if (brk->id == id) {
 			found = true;
-			delete(*iter);
-			(*iter) = 0;
+			delete brk;
+			brk = nullptr;
 		}
 	}
 
-	breaks.remove(static_cast<Breakpoint *>(0)); // delete all NULLs from the list
+	breaks.remove(nullptr); // delete all nullptr from the list
 
 	return found;
 }
 
 void Breakpoints::transmit(int fd) {
-	std::list<Breakpoint *>::iterator iter;
-
-	for (iter = breaks.begin(); iter != breaks.end(); ++iter)
-		(*iter)->serialize(fd);
+	for (auto *brk : breaks)
+		brk->serialize(fd);
 }
