@@ -1,7 +1,7 @@
 /*
  *  ucfunction.h - Usecode function
  *
- *  Copyright (C) 2002  The Exult Team
+ *  Copyright (C) 2002-2022  The Exult Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,8 +18,8 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#ifndef _UCFUNCTION_H
-#define _UCFUNCTION_H
+#ifndef UCFUNCTION_H
+#define UCFUNCTION_H
 
 #include <iosfwd>
 #include <vector>
