@@ -2,7 +2,7 @@
  *  ucmachine.cc - Interpreter for usecode.
  *
  *  Copyright (C) 1999  Jeffrey S. Freedman
- *  Copyright (C) 2000-2013  The Exult Team
+ *  Copyright (C) 2000-2022  The Exult Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -42,12 +42,11 @@ class Tile_coord;
  *  Here's our virtual machine for running usecode.  The actual internals
  *  are in Usecode_internal.
  */
-class Usecode_machine : public Game_singletons {
-	UNREPLICATABLE_CLASS(Usecode_machine)
+class Usecode_machine : nonreplicatable, public Game_singletons {
 protected:
 	unsigned char gflags[c_last_gflag + 1]; // Global flags.
-	Keyring *keyring;
-	Conversation *conv;     // Handles conversations
+	Keyring *keyring = nullptr;
+	Conversation *conv = nullptr;     // Handles conversations
 public:
 	friend class Usecode_script;
 	// Create Usecode_internal.
@@ -112,6 +111,7 @@ public:
 	virtual void save_intercept(Game_object *&obj, Tile_coord *&t) = 0;
 	virtual void restore_intercept(Game_object *obj, Tile_coord *t) = 0;
 
+	virtual bool function_exists(int funcid) = 0;
 };
 
 #endif  /* INCL_USECODE */
