@@ -1,7 +1,7 @@
 /*
  *  ucserial.h - Serialization of usecode objects
  *
- *  Copyright (C) 2002  The Exult Team
+ *  Copyright (C) 2002-2022  The Exult Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,8 +18,8 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#ifndef _UCSERIAL_H
-#define _UCSERIAL_H
+#ifndef UCSERIAL_H
+#define UCSERIAL_H
 
 #include "objserial.h"
 
@@ -37,6 +37,7 @@ extern int Stack_frame_out(
     int num_vars,
     Usecode_value *locals
 );
+
 extern bool Stack_frame_in(
     const unsigned char *data,        // Data that was read.
     int datalen,            // Length of data.
@@ -51,5 +52,4 @@ extern bool Stack_frame_in(
     Usecode_value *&locals
 );
 
-
 #endif
