@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2001-2011  The Exult Team
+ *  Copyright (C) 2001-2022  The Exult Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -52,7 +52,7 @@ private:
 	std::string get_datadir(const Configuration &config, const UCOptions &options);
 	void misc();
 	void opcodes();
-	void intrinsics(const std::string &intrinsics_data, const std::string &intrinsics_root);
+	void intrinsics(const std::string &intrinsic_data, const std::string &intrinsic_root);
 
 	const std::string ucxtdata;
 	std::string datadir;
@@ -76,78 +76,63 @@ private:
 
 class UCOpcodeData {
 public:
-	UCOpcodeData() : opcode(0x00), num_bytes(0), num_pop(0),
-		num_push(0), flag_call_effect(false), flag_return(false),
-		flag_paren(false), flag_indent_inc(false),
-		flag_indent_dec(false), flag_indent_tmpinc(false),
-		flag_indent_tmpdec(false), flag_debug(false), flag_nosemicolon(false),
-		flag_abort(false), flag_staticref(false), flag_loop(false),
-		flag_new_effect(false), flag_method_effect(false), flag_function_effect(false),
-		flag_not_param(false)
-	{}
+	UCOpcodeData() = default;
 	UCOpcodeData(unsigned int op, const Configuration::KeyTypeList &ktl)
-		: opcode(op), num_bytes(0), num_pop(0),
-		  num_push(0), flag_call_effect(false), flag_return(false),
-		  flag_paren(false), flag_indent_inc(false),
-		  flag_indent_dec(false), flag_indent_tmpinc(false),
-		  flag_indent_tmpdec(false), flag_debug(false) ,flag_nosemicolon(false),
-		  flag_abort(false), flag_staticref(false), flag_loop(false),
-		  flag_new_effect(false), flag_method_effect(false), flag_function_effect(false),
-		  flag_not_param(false) {
-		for (Configuration::KeyTypeList::const_iterator k = ktl.begin(); k != ktl.end(); ++k) {
-			switch (k->first[0]) {
+		: opcode(op) {
+		for (const auto& k : ktl) {
+			switch (k.first[0]) {
 			case 'a':
-				if (k->first == "asm_nmo")             asm_nmo = strip_backticks(k->second);
-				else if (k->first == "asm_comment")    asm_comment = strip_backticks(k->second);
-				else if (k->first == "abort/")         flag_abort = true;
+				if (k.first == "asm_nmo")             asm_nmo = strip_backticks(k.second);
+				else if (k.first == "asm_comment")    asm_comment = strip_backticks(k.second);
+				else if (k.first == "abort/")         flag_abort = true;
 				break;
 			case 'c':
-				if (k->first == "call_effect/")        flag_call_effect = true;
+				if (k.first == "call_effect/")        flag_call_effect = true;
 				break;
 			case 'd':
-				if (k->first == "debug/")              flag_debug = true;
+				if (k.first == "debug/")              flag_debug = true;
 				break;
 			case 'f':
-				if (k->first == "function_effect/")    flag_function_effect = true;
+				if (k.first == "function_effect/")    flag_function_effect = true;
 				break;
 			case 'i':
-				if (k->first == "indent_inc/")         flag_indent_inc = true;
-				else if (k->first == "indent_dec/")    flag_indent_dec = true;
-				else if (k->first == "indent_tmpinc/") flag_indent_tmpinc = true;
-				else if (k->first == "indent_tmpdec/") flag_indent_tmpdec = true;
+				if (k.first == "indent_inc/")         flag_indent_inc = true;
+				else if (k.first == "indent_dec/")    flag_indent_dec = true;
+				else if (k.first == "indent_tmpinc/") flag_indent_tmpinc = true;
+				else if (k.first == "indent_tmpdec/") flag_indent_tmpdec = true;
 				break;
 			case 'l':
-				if (k->first == "loop/")               flag_loop = true;
+				if (k.first == "loop/")               flag_loop = true;
 				break;
 			case 'm':
-				if (k->first == "method_effect/")      flag_method_effect = true;
+				if (k.first == "method_effect/")      flag_method_effect = true;
 				break;
 			case 'n':
-				if (k->first == "name")                name = strip_backticks(k->second);
-				else if (k->first == "num_bytes")      num_bytes = static_cast<unsigned int>(strtol(k->second.c_str(), 0, 0));
-				else if (k->first == "num_pop")        num_pop = static_cast<unsigned int>(strtol(k->second.c_str(), 0, 0));
-				else if (k->first == "num_push")       num_push = static_cast<unsigned int>(strtol(k->second.c_str(), 0, 0));
-				else if (k->first == "nosemicolon/")   flag_nosemicolon = true;
-				else if (k->first == "new_effect/")    flag_new_effect = true;
-				else if (k->first == "not_param/")     flag_not_param = true;
+				if (k.first == "name")                name = strip_backticks(k.second);
+				else if (k.first == "num_bytes")      num_bytes = static_cast<unsigned int>(strtol(k.second.c_str(), nullptr, 0));
+				else if (k.first == "num_pop")        num_pop = static_cast<unsigned int>(strtol(k.second.c_str(), nullptr, 0));
+				else if (k.first == "num_push")       num_push = static_cast<unsigned int>(strtol(k.second.c_str(), nullptr, 0));
+				else if (k.first == "nosemicolon/")   flag_nosemicolon = true;
+				else if (k.first == "new_effect/")    flag_new_effect = true;
+				else if (k.first == "not_param/")     flag_not_param = true;
 				break;
 			case 'p':
-				if (k->first == "param_types")         param_types = qnd_ocsplit(k->second);
-				else if (k->first == "paren/")         flag_paren = true;
+				if (k.first == "param_types")         param_types = qnd_ocsplit(k.second);
+				else if (k.first == "paren/")         flag_paren = true;
 				break;
 			case 'r':
-				if (k->first == "return/")             flag_return = true;
+				if (k.first == "return/")             flag_return = true;
 				break;
 			case 's':
-				if (k->first == "staticref/")          flag_staticref = true;
+				if (k.first == "staticref/")          flag_staticref = true;
 				break;
 			case 'u':
-				if (k->first == "ucs_nmo")             ucs_nmo = strip_backticks(k->second);
+				if (k.first == "ucs_nmo")             ucs_nmo = strip_backticks(k.second);
 				break;
 			case '!': // ignore, it's a comment or something.
 				break;
 			default:
-				std::cerr << "invalid key `" << k->first << "` value `" << k->second << "`" << std::endl;
+				std::cerr << "invalid key `" << k.first << "` value `" << k.second << "`" << std::endl;
 			}
 		}
 		map_type_size(param_types, param_sizes);
@@ -161,8 +146,8 @@ public:
 		o << "ucs_nmo: " << ucs_nmo << std::endl;
 		o << "num_bytes: " << num_bytes << std::endl;
 		o << "param_types: ";
-		for (std::vector<std::string>::const_iterator i = param_types.begin(); i != param_types.end(); ++i)
-			o << *i << ',';
+		for (const auto& param_type : param_types)
+			o << param_type << ',';
 		o << std::endl;
 		o << "num_pop: " << num_pop << std::endl;
 		o << "num_push: " << num_push << std::endl;
@@ -184,31 +169,31 @@ public:
 		o << "flag_not_param: " << flag_not_param << std::endl;
 	}
 
-	unsigned int   opcode;
+	unsigned int   opcode = 0;
 	std::string    name;
 	std::string    asm_nmo;
 	std::string    asm_comment;
 	std::string    ucs_nmo;
-	unsigned int   num_bytes;
+	unsigned int   num_bytes = 0;
 
-	unsigned int   num_pop;
-	unsigned int   num_push;
-	bool           flag_call_effect;
-	bool           flag_return;
-	bool           flag_paren;
-	bool           flag_indent_inc;
-	bool           flag_indent_dec;
-	bool           flag_indent_tmpinc;
-	bool           flag_indent_tmpdec;
-	bool           flag_debug;
-	bool           flag_nosemicolon;
-	bool           flag_abort;
-	bool           flag_staticref;
-	bool           flag_loop;
-	bool           flag_new_effect;
-	bool           flag_method_effect;
-	bool           flag_function_effect;
-	bool           flag_not_param;
+	unsigned int   num_pop = 0;
+	unsigned int   num_push = 0;
+	bool           flag_call_effect = false;
+	bool           flag_return = false;
+	bool           flag_paren = false;
+	bool           flag_indent_inc = false;
+	bool           flag_indent_dec = false;
+	bool           flag_indent_tmpinc = false;
+	bool           flag_indent_tmpdec = false;
+	bool           flag_debug = false;
+	bool           flag_nosemicolon = false;
+	bool           flag_abort = false;
+	bool           flag_staticref = false;
+	bool           flag_loop = false;
+	bool           flag_new_effect = false;
+	bool           flag_method_effect = false;
+	bool           flag_function_effect = false;
+	bool           flag_not_param = false;
 
 	std::vector<std::string> param_types;
 	// values caluclated from param_types
