@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2001-2013  The Exult Team
+ *  Copyright (C) 2001-2022  The Exult Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,11 +23,7 @@
 #include "ucdata.h"
 #include "ucfunc.h"
 #include <set>
-#ifdef HAVE_SSTREAM
 #include <sstream>
-#else
-#include <strstream>
-#endif
 #include <algorithm>
 #include <iomanip>
 #include <cctype>
@@ -38,7 +34,7 @@
 
 #include "ops.h"
 
-#if 0
+#ifdef DEBUG_UCXT
 #define DEBUG_INDENT
 #define DEBUG_PARSE
 #define DEBUG_PARSE2
@@ -55,10 +51,6 @@
 #define DEBUG_READ_PAIR(X, Y)
 #endif
 
-//#define DEBUG_PARSE2
-//#define DEBUG_PARSE2a
-//#define DEBUG_PRINT
-
 using std::ostream;
 using std::ifstream;
 using std::string;
@@ -70,7 +62,6 @@ using std::ios;
 using std::streampos;
 using std::cout;
 using std::setw;
-using std::less;
 
 const string UCFunc::VARPREFIX = "var";
 const string UCFunc::CLASSVARPREFIX = "cvar";
@@ -147,9 +138,9 @@ bool UCFunc::output_ucs(ostream &o, const FuncMap &funcmap, const map<unsigned i
 
 	if (!_externs.empty()) tab_indent(indent, o) << "// externs" << endl;
 	// output the 'externs'
-	for (vector<unsigned int>::iterator e = _externs.begin(); e != _externs.end(); ++e) {
-		FuncMap::const_iterator fmp = funcmap.find(*e);
-		output_ucs_funcname(tab_indent(indent, o) << "extern ", funcmap, *e, fmp->second.num_args, symtbl) << ';' << endl;
+	for (unsigned int ext : _externs) {
+		auto fmp = funcmap.find(ext);
+		output_ucs_funcname(tab_indent(indent, o) << "extern ", funcmap, ext, fmp->second.num_args, symtbl) << ';' << endl;
 	}
 
 	if (!_externs.empty()) o << endl;
@@ -158,11 +149,10 @@ bool UCFunc::output_ucs(ostream &o, const FuncMap &funcmap, const map<unsigned i
 	output_ucs_funcname(tab_indent(indent, o), funcmap, _funcid, _num_args, symtbl) << endl;
 	// start of func
 	tab_indent(indent++, o) << '{' << endl;
-	std::map<unsigned int, std::string>::iterator it;
 
 	for (unsigned int i = _num_args; i < _num_args + _num_locals; i++) {
 		tab_indent(indent, o) << VARNAME << ' ';
-		it = _varmap.find(i);
+		auto it = _varmap.find(i);
 		if (it != _varmap.end())
 			o << it->second;
 		else
@@ -188,7 +178,7 @@ ostream &UCFunc::output_ucs_funcname(ostream &o, const FuncMap &funcmap,
                                      unsigned int funcid, unsigned int numargs,
                                      Usecode_symbol_table *symtbl) {
 	ignore_unused_variable_warning(symtbl);
-	FuncMap::const_iterator fmp = funcmap.find(funcid);
+	auto fmp = funcmap.find(funcid);
 	// do we return a variable
 	if (fmp->second.return_var) o << VARNAME << ' ';
 	else o << NORETURN << ' ';
@@ -215,11 +205,10 @@ ostream &UCFunc::output_ucs_funcname(ostream &o, const FuncMap &funcmap,
 		o << ")";
 	// output ObCurly braces
 	o << " (";
-	std::map<unsigned int, std::string>::iterator it;
 
 	for (unsigned int i = (fmp->second.class_fun ? 1 : 0); i < numargs; i++) {
 		o << VARNAME << ' ';
-		it = fmp->second.varmap.find(numargs - i - 1);
+		auto it = fmp->second.varmap.find(numargs - i - 1);
 		if (it != fmp->second.varmap.end())
 			o << it->second;
 		else
@@ -238,15 +227,15 @@ ostream &UCFunc::output_ucs_funcname(ostream &o, const FuncMap &funcmap, Usecode
 
 void UCFunc::output_ucs_data(ostream &o, const FuncMap &funcmap, const map<unsigned int, string> &intrinsics, const UCOptions &options, unsigned int indent, Usecode_symbol_table *symtbl) {
 	vector<unsigned int> labeltmp(1);
-	for (vector<GotoSet>::iterator i = gotoset.begin(); i != gotoset.end(); ++i) {
+	for (auto i = gotoset.begin(); i != gotoset.end(); ++i) {
 		// we don't want to output the first "jump" (the start of the function)
 		if (i != gotoset.begin()) {
 			labeltmp[0] = i->offset();
 			tab_indent(indent - 1, o) << demunge_ocstring(*this, funcmap, "label%f*_%1:", intrinsics, UCc(labeltmp), true, symtbl) << endl;
 		}
 
-		for (GotoSet::iterator j = (*i)().begin(); j != (*i)().end(); ++j) {
-			const UCc &ucc = *(j->first);
+		for (auto& j : (*i)()) {
+			const UCc &ucc = *(j.first);
 
 			if (options.uselesscomment)
 				tab_indent(indent, o) << "// Offset: " << std::setw(4) << ucc._offset << endl;
@@ -265,12 +254,12 @@ void UCFunc::output_ucs_opcode(ostream &o, const FuncMap &funcmap, const vector<
 	o << endl;
 
 #ifdef DEBUG_PRINT
-	for (vector<UCc *>::const_iterator i = op._popped.begin(); i != op._popped.end(); ++i) {
-		if ((*i)->_popped.size())
-			output_ucs_opcode(o, funcmap, opcode_table_data, **i, intrinsics, indent + 1, symtbl);
+	for (auto *i : op._popped) {
+		if (!i->_popped.empty())
+			output_ucs_opcode(o, funcmap, opcode_table_data, *i, intrinsics, indent + 1, symtbl);
 		else
 //			tab_indent(indent+1, o) << demunge_ocstring(*this, funcmap, optab[(*i)->_id].ucs_nmo, **i, symtbl) << endl;
-			tab_indent(indent + 1, o) << optab[(*i)->_id].ucs_nmo << endl;
+			tab_indent(indent + 1, o) << optab[i->_id].ucs_nmo << endl;
 	}
 #endif
 }
@@ -278,13 +267,13 @@ void UCFunc::output_ucs_opcode(ostream &o, const FuncMap &funcmap, const vector<
 void UCFunc::output_ucs_node(ostream &o, const FuncMap &funcmap, UCNode *ucn, const map<unsigned int, string> &intrinsics, unsigned int indent, const UCOptions &options, Usecode_symbol_table *symtbl) {
 	if (!ucn->nodelist.empty()) tab_indent(indent, o) << '{' << endl;
 
-	if (ucn->ucc != 0)
+	if (ucn->ucc != nullptr)
 		output_asm_opcode(tab_indent(indent, o), funcmap, opcode_table_data, intrinsics, *(ucn->ucc), options, symtbl);
 
 	if (!ucn->nodelist.empty())
-		for (vector<UCNode *>::iterator i = ucn->nodelist.begin(); i != ucn->nodelist.end(); ++i) {
+		for (auto *i : ucn->nodelist) {
 			//tab_indent(indent, o);
-			output_ucs_node(o, funcmap, *i, intrinsics, indent + 1, options, symtbl);
+			output_ucs_node(o, funcmap, i, intrinsics, indent + 1, options, symtbl);
 		}
 
 	// end of func
@@ -293,8 +282,8 @@ void UCFunc::output_ucs_node(ostream &o, const FuncMap &funcmap, UCNode *ucn, co
 
 /* Just a quick function to remove all the ucc structured flagged as removable */
 inline void gc_gotoset(vector<GotoSet> &gotoset) {
-	for (vector<GotoSet>::iterator i = gotoset.begin(); i != gotoset.end(); ++i) {
-		i->gc();
+	for (auto& i : gotoset) {
+		i.gc();
 #ifdef DEBUG_GOTOSET
 		cout << "----" << endl;
 #endif
@@ -302,11 +291,11 @@ inline void gc_gotoset(vector<GotoSet> &gotoset) {
 }
 
 void UCFunc::parse_ucs(const FuncMap &funcmap, const map<unsigned int, string> &intrinsics, const UCOptions &options, Usecode_symbol_table *symtbl) {
-	for (vector<UCc>::iterator i = _opcodes.begin(); i != _opcodes.end(); ++i)
-		node.nodelist.push_back(new UCNode(&(*i)));
+	for (auto& opcode : _opcodes)
+		node.nodelist.push_back(new UCNode(&opcode));
 
 	parse_ucs_pass1(node.nodelist);
-	parse_ucs_pass2(gotoset, funcmap, intrinsics, symtbl);
+	parse_ucs_pass2(gotoset, funcmap, intrinsics, options, symtbl);
 	gc_gotoset(gotoset);
 
 	if (!options.basic) {
@@ -314,11 +303,11 @@ void UCFunc::parse_ucs(const FuncMap &funcmap, const map<unsigned int, string> &
 	}
 
 #ifdef DEBUG_PARSE2
-	for (vector<GotoSet>::iterator i = gotoset.begin(); i != gotoset.end(); ++i) {
-		cout << std::setw(4) << i->offset() << endl;
+	for (auto& i : gotoset) {
+		cout << std::setw(4) << i.offset() << endl;
 
-		for (GotoSet::iterator j = (*i)().begin(); j != (*i)().end(); ++j) {
-			cout << '\t' << std::setw(4) << j->first->_offset << '\t' << j->first->_id << endl;
+		for (auto& j : i()) {
+			cout << '\t' << std::setw(4) << j.first->_offset << '\t' << j.first->_id << endl;
 		}
 	}
 #endif
@@ -330,30 +319,30 @@ void UCFunc::parse_ucs_pass1(vector<UCNode *> &nodes) {
 	vector<unsigned int> jumps;
 
 	// collect jump references
-	for (unsigned int i = 0; i < nodes.size(); i++) {
-		if (nodes[i]->ucc != 0) {
+	for (auto *node : nodes) {
+		if (node->ucc != nullptr) {
 			unsigned int isjump = 0;
-			for (vector<pair<unsigned int, unsigned int> >::iterator op = opcode_jumps.begin(); op != opcode_jumps.end(); ++op)
-				if (op->first == nodes[i]->ucc->_id) {
-					isjump = op->second;
+			for (auto& jump : opcode_jumps)
+				if (jump.first == node->ucc->_id) {
+					isjump = jump.second;
 					break;
 				}
 
 			if (isjump != 0) {
-				assert(nodes[i]->ucc->_params_parsed.size() >= isjump);
-				jumps.push_back(nodes[i]->ucc->_params_parsed[isjump - 1]);
+				assert(node->ucc->_params_parsed.size() >= isjump);
+				jumps.push_back(node->ucc->_params_parsed[isjump - 1]);
 			}
 		}
 	}
 
-	gotoset.push_back(GotoSet());
+	gotoset.emplace_back(GotoSet());
 
-	for (unsigned int i = 0; i < nodes.size(); i++) {
-		if (nodes[i]->ucc != 0) {
-			if (count(jumps.begin(), jumps.end(), nodes[i]->ucc->_offset)) {
-				gotoset.push_back(nodes[i]->ucc);
+	for (auto *node : nodes) {
+		if (node->ucc != nullptr) {
+			if (count(jumps.begin(), jumps.end(), node->ucc->_offset)) {
+				gotoset.emplace_back(node->ucc);
 			} else
-				gotoset.back().add(nodes[i]->ucc);
+				gotoset.back().add(node->ucc);
 		}
 	}
 }
@@ -362,33 +351,33 @@ void UCFunc::parse_ucs_pass1(vector<UCNode *> &nodes) {
    each UCc, having it's parameters sitting in it's UCc::_popped vector. Elements
    that are parameters are flagged for removal (Gotoset::()[i]->second=true) from
    the original GotoSet. */
-void UCFunc::parse_ucs_pass2(vector<GotoSet> &gotoset, const FuncMap &funcmap, const map<unsigned int, string> &intrinsics, Usecode_symbol_table *symtbl) {
-	for (vector<GotoSet>::iterator i = gotoset.begin(); i != gotoset.end(); ++i) {
-		parse_ucs_pass2a((*i)().rbegin(), (*i)(), 0, funcmap, intrinsics, symtbl);
+void UCFunc::parse_ucs_pass2(vector<GotoSet> &gotoset, const FuncMap &funcmap, const map<unsigned int, string> &intrinsics, const UCOptions &options, Usecode_symbol_table *symtbl) {
+	for (auto& i : gotoset) {
+		parse_ucs_pass2a(i().rbegin(), i(), 0, funcmap, intrinsics, options, symtbl);
 	}
 }
 
-vector<UCc *> UCFunc::parse_ucs_pass2a(vector<pair<UCc *, bool> >::reverse_iterator current, vector<pair<UCc *, bool> > &vec, int opsneeded, const FuncMap &funcmap, const map<unsigned int, string> &intrinsics, Usecode_symbol_table *symtbl) {
+vector<UCc *> UCFunc::parse_ucs_pass2a(vector<pair<UCc *, bool> >::reverse_iterator current, vector<pair<UCc *, bool> > &vec, int opsneeded, const FuncMap &funcmap, const map<unsigned int, string> &intrinsics, const UCOptions &options, Usecode_symbol_table *symtbl) {
 	vector<UCc *> vucc;
 	int opsfound = 0;
 
 #ifdef DEBUG_PARSE2
-	output_asm_opcode(tab_indent(4, cout), *this, funcmap, opcode_table_data, intrinsics, *(current->first));
+	output_asm_opcode(tab_indent(4, cout), funcmap, opcode_table_data, intrinsics, *(current->first), options, symtbl);
 #endif
 
 	for (; vec.rend() != current; current++) {
 #ifdef DEBUG_PARSE2
-		output_asm_opcode(tab_indent(3, cout), *this, funcmap, opcode_table_data, intrinsics, *(current->first));
+		output_asm_opcode(tab_indent(3, cout), funcmap, opcode_table_data, intrinsics, *(current->first), options, symtbl);
 #endif
 
-		if (current->second == false) {
+		if (!current->second) {
 			if ((opcode_table_data[current->first->_id].num_pop != 0) || (opcode_table_data[current->first->_id].flag_call_effect) ||
 			        (opcode_table_data[current->first->_id].flag_new_effect) || (opcode_table_data[current->first->_id].flag_method_effect) ||
 			        (opcode_table_data[current->first->_id].flag_function_effect)) {
 				//if(opcode_table_data[current->first->_id].num_pop<0x7F)
 				{
 #ifdef DEBUG_PARSE2
-					output_asm_opcode(tab_indent(3, cout << "0x" << std::setw(2) << current->first->_id << "-"), *this, funcmap, opcode_table_data, intrinsics, *(current->first));
+					output_asm_opcode(tab_indent(3, cout << "0x" << std::setw(2) << current->first->_id << "-"), funcmap, opcode_table_data, intrinsics, *(current->first), options, symtbl);
 					tab_indent(3, cout << "0x" << std::setw(2) << current->first->_id << "-") << opcode_table_data[current->first->_id].num_pop << endl;
 #endif
 
@@ -402,8 +391,11 @@ vector<UCc *> UCFunc::parse_ucs_pass2a(vector<pair<UCc *, bool> >::reverse_itera
 
 						unsigned int offset = current->first->_params_parsed[0];
 						Usecode_class_symbol *cls = symtbl->get_class(offset);
-						assert(cls);
-						num_args = cls->get_num_vars();
+						if (cls != nullptr) {
+							num_args = cls->get_num_vars();
+						} else {
+							std::cerr << "Class not found: " << offset << '\n';
+						}
 
 #ifdef DEBUG_PARSE2a
 						cout << num_args << endl;
@@ -431,7 +423,7 @@ vector<UCc *> UCFunc::parse_ucs_pass2a(vector<pair<UCc *, bool> >::reverse_itera
 							assert(cls);
 							int method = cls->get_method_id(current->first->_params_parsed[0]);
 							assert(method >= 0);
-							FuncMap::const_iterator fmp = funcmap.find(method);
+							auto fmp = funcmap.find(method);
 							assert(fmp != funcmap.end());
 							num_args = fmp->second.num_args;
 
@@ -456,7 +448,7 @@ vector<UCc *> UCFunc::parse_ucs_pass2a(vector<pair<UCc *, bool> >::reverse_itera
 
 					} else if (opcode_table_data[current->first->_id].flag_function_effect) {
 						assert(!current->first->_params_parsed.empty());
-						FuncMap::const_iterator fmp = funcmap.find(current->first->_params_parsed[0]);
+						auto fmp = funcmap.find(current->first->_params_parsed[0]);
 						assert(fmp != funcmap.end());
 #ifdef DEBUG_PARSE2
 						cout << "CALL:     " << fmp->second.funcid << '\t' << fmp->second.num_args << endl;
@@ -466,7 +458,7 @@ vector<UCc *> UCFunc::parse_ucs_pass2a(vector<pair<UCc *, bool> >::reverse_itera
 					} else if (opcode_table_data[current->first->_id].flag_call_effect) {
 						assert(!current->first->_params_parsed.empty());
 						assert(_externs.size() >= current->first->_params_parsed[0]);
-						FuncMap::const_iterator fmp = funcmap.find(_externs[current->first->_params_parsed[0]]);
+						auto fmp = funcmap.find(_externs[current->first->_params_parsed[0]]);
 						assert(fmp != funcmap.end());
 #ifdef DEBUG_PARSE2
 						cout << "CALL EXTERN:" << fmp->second.funcid << '\t' << fmp->second.num_args << endl;
@@ -484,10 +476,10 @@ vector<UCc *> UCFunc::parse_ucs_pass2a(vector<pair<UCc *, bool> >::reverse_itera
 					if (num_args != 0) {
 						/* save the 'current' value as the return value and increment it so it's
 						   pointing at the 'next' current value */
-						vector<pair<UCc *, bool> >::reverse_iterator ret(current);
+						auto ret(current);
 
-						ret->first->_popped = parse_ucs_pass2a(++current, vec, num_args, funcmap, intrinsics, symtbl);
-						unsigned nargs = static_cast<unsigned>(num_args);
+						ret->first->_popped = parse_ucs_pass2a(++current, vec, num_args, funcmap, intrinsics, options, symtbl);
+						auto nargs = static_cast<unsigned>(num_args);
 						if (num_args < 0 || nargs <= ret->first->_popped.size()) {
 							ret->first->_missing_args = 0;
 						} else {
@@ -500,22 +492,22 @@ vector<UCc *> UCFunc::parse_ucs_pass2a(vector<pair<UCc *, bool> >::reverse_itera
 
 						assert(current == ret);
 #ifdef DEBUG_PARSE2a
-						output_asm_opcode(tab_indent(1, cout), *this, funcmap, opcode_table_data, intrinsics, *(ret->first), symtbl);
+						output_asm_opcode(tab_indent(1, cout), funcmap, opcode_table_data, intrinsics, *(ret->first), options, symtbl);
 
-						for (vector<UCc *>::iterator i = ret->first->_popped.begin(); i != ret->first->_popped.end(); ++i)
-							output_asm_opcode(tab_indent(2, cout), *this, funcmap, opcode_table_data, intrinsics, **i, symtbl);
+						for (auto *i : ret->first->_popped)
+							output_asm_opcode(tab_indent(2, cout), funcmap, opcode_table_data, intrinsics, *i, options, symtbl);
 #endif
 					}
 				}
 			}
-			if ((opsneeded != 0) && (current->second == false)) {
+			if ((opsneeded != 0) && (!current->second)) {
 				if (opcode_table_data[current->first->_id].flag_not_param) {
 					continue;
 				}
 				// if it's a 'push' opcode and we need items to return that we've popped off the stack...
 				else if (opcode_table_data[current->first->_id].num_push != 0) {
 #ifdef DEBUG_PARSE2
-					output_asm_opcode(tab_indent(4, cout << "P-"), *this, funcmap, opcode_table_data, intrinsics, *(current->first));
+					output_asm_opcode(tab_indent(4, cout << "P-"), funcmap, opcode_table_data, intrinsics, *(current->first), options, symtbl);
 #endif
 
 					opsfound += opcode_table_data[current->first->_id].num_push;
@@ -535,11 +527,11 @@ vector<UCc *> UCFunc::parse_ucs_pass2a(vector<pair<UCc *, bool> >::reverse_itera
 						unsigned int offset = current->first->_params_parsed[1];
 						Usecode_class_symbol *cls = symtbl->get_class(offset);
 						int method = cls->get_method_id(current->first->_params_parsed[0]);
-						FuncMap::const_iterator fmp = funcmap.find(method);
+						auto fmp = funcmap.find(method);
 						assert(fmp != funcmap.end());
 						if (fmp->second.return_var) {
 #ifdef DEBUG_PARSE2
-							output_asm_opcode(tab_indent(4, cout << "C-"), *this, funcmap, opcode_table_data, intrinsics, *(current->first));
+							output_asm_opcode(tab_indent(4, cout << "C-"), funcmap, opcode_table_data, intrinsics, *(current->first), options, symtbl);
 #endif
 
 							opsfound += 1;
@@ -550,12 +542,12 @@ vector<UCc *> UCFunc::parse_ucs_pass2a(vector<pair<UCc *, bool> >::reverse_itera
 				}
 				// if it's a call to a function that returns a variable...
 				else if (opcode_table_data[current->first->_id].flag_function_effect) {
-					FuncMap::const_iterator fmp = funcmap.find(current->first->_params_parsed[0]);
+					auto fmp = funcmap.find(current->first->_params_parsed[0]);
 					assert(fmp != funcmap.end());
 
 					if (fmp->second.return_var) {
 #ifdef DEBUG_PARSE2
-						output_asm_opcode(tab_indent(4, cout << "C-"), *this, funcmap, opcode_table_data, intrinsics, *(current->first));
+						output_asm_opcode(tab_indent(4, cout << "C-"), funcmap, opcode_table_data, intrinsics, *(current->first), options, symtbl);
 #endif
 
 						opsfound += 1;
@@ -565,12 +557,12 @@ vector<UCc *> UCFunc::parse_ucs_pass2a(vector<pair<UCc *, bool> >::reverse_itera
 				}
 				// if it's a call to an extern function that returns a variable...
 				else if (opcode_table_data[current->first->_id].flag_call_effect) {
-					FuncMap::const_iterator fmp = funcmap.find(_externs[current->first->_params_parsed[0]]);
+					auto fmp = funcmap.find(_externs[current->first->_params_parsed[0]]);
 					assert(fmp != funcmap.end());
 
 					if (fmp->second.return_var) {
 #ifdef DEBUG_PARSE2
-						output_asm_opcode(tab_indent(4, cout << "C-"), *this, funcmap, opcode_table_data, intrinsics, *(current->first));
+						output_asm_opcode(tab_indent(4, cout << "C-"), funcmap, opcode_table_data, intrinsics, *(current->first), options, symtbl);
 #endif
 
 						opsfound += 1;
@@ -580,11 +572,11 @@ vector<UCc *> UCFunc::parse_ucs_pass2a(vector<pair<UCc *, bool> >::reverse_itera
 				}
 
 				// if we've found all the ops we were searching for, return them
-				if (opsneeded != 0 && current->second == false) {
+				if (opsneeded != 0 && !current->second) {
 					if (opsneeded >= 0 && opsfound < opsneeded) {
 						cout << "DID NOT FIND ALL OPCODE PARAMETERS." << endl;
 					}
-					while (++current != vec.rend() && current->second == true) {
+					while (++current != vec.rend() && current->second) {
 						current->second = false;
 					}
 					return vucc;
@@ -607,9 +599,9 @@ void UCFunc::parse_ucs_pass3(vector<GotoSet> &gotoset, const map<unsigned int, s
 bool UCFunc::output_tt(std::ostream &o) {
 	o << "\t<0x" << setw(4) << _funcid << ">" << endl;
 
-	for (map<unsigned int, string, less<unsigned int> >::iterator i = _data.begin(); i != _data.end(); ++i) {
-		o << "\t\t<0x" << setw(4) << i->first << ">" << endl
-		  << "\t\t`" << i->second << "`" << endl
+	for (auto& i : _data) {
+		o << "\t\t<0x" << setw(4) << i.first << ">" << endl
+		  << "\t\t`" << i.second << "`" << endl
 		  << "\t\t</>" << endl;
 	}
 	o << "\t</>" << endl;
@@ -630,13 +622,13 @@ inline int calc32reloffset(const UCc &op, unsigned int param) {
 void ucc_parse_parambytes(UCc &ucop, const UCOpcodeData &otd) {
 	unsigned int first = 0;
 
-	for (vector<pair<unsigned int, bool> >::const_iterator s = otd.param_sizes.begin(); s != otd.param_sizes.end(); ++s) {
+	for (const auto& param_size : otd.param_sizes) {
 		//cout << ucop._id << '\t' << ucop._params.size() << endl;
 
 		assert(first < ucop._params.size());
 
-		unsigned int ssize = s->first;
-		bool offset_munge = s->second;
+		unsigned int ssize = param_size.first;
+		bool offset_munge = param_size.second;
 
 		assert(ssize != 0);
 
@@ -644,19 +636,19 @@ void ucc_parse_parambytes(UCc &ucop, const UCOpcodeData &otd) {
 			ucop._params_parsed.push_back(static_cast<unsigned int>(ucop._params[first++]));
 		else if (ssize == 2)
 			if (offset_munge) {
-				unsigned int calcvar = static_cast<unsigned int>(ucop._params[first++]);
+				auto calcvar = static_cast<unsigned int>(ucop._params[first++]);
 				calcvar += ((static_cast<unsigned int>(ucop._params[first++])) << 8);
 				unsigned int reloffset = calc16reloffset(ucop, calcvar);
 				ucop._params_parsed.push_back(reloffset);
 				ucop._jump_offsets.push_back(reloffset);
 			} else {
-				unsigned int calcvar = static_cast<unsigned int>(ucop._params[first++]);
+				auto calcvar = static_cast<unsigned int>(ucop._params[first++]);
 				calcvar += ((static_cast<unsigned int>(ucop._params[first++])) << 8);
 				ucop._params_parsed.push_back(calcvar);
 			}
 		else if (ssize == 4)
 			if (offset_munge) {
-				unsigned int calcvar = static_cast<unsigned int>(ucop._params[first++]);
+				auto calcvar = static_cast<unsigned int>(ucop._params[first++]);
 				calcvar += ((static_cast<unsigned int>(ucop._params[first++])) << 8);
 				calcvar += ((static_cast<unsigned int>(ucop._params[first++])) << 16);
 				calcvar += ((static_cast<unsigned int>(ucop._params[first++])) << 24);
@@ -664,7 +656,7 @@ void ucc_parse_parambytes(UCc &ucop, const UCOpcodeData &otd) {
 				ucop._params_parsed.push_back(reloffset);
 				ucop._jump_offsets.push_back(reloffset);
 			} else {
-				unsigned int calcvar = static_cast<unsigned int>(ucop._params[first++]);
+				auto calcvar = static_cast<unsigned int>(ucop._params[first++]);
 				calcvar += ((static_cast<unsigned int>(ucop._params[first++])) << 8);
 				calcvar += ((static_cast<unsigned int>(ucop._params[first++])) << 16);
 				calcvar += ((static_cast<unsigned int>(ucop._params[first++])) << 24);
@@ -698,11 +690,11 @@ bool UCFunc::output_asm(ostream &o, const FuncMap &funcmap, const map<unsigned i
 	o << "\t.localc      " << std::setw(4) << _num_locals << "H" << endl;
 	o << "\t.externsize  " << std::setw(4) << _externs.size() << "H" << endl;
 
-	for (std::vector<unsigned int>::iterator i = _externs.begin(); i != _externs.end(); ++i)
-		o << '\t' << "  .extern    " << std::setw(4) << *i << "H" << endl;
+	for (unsigned int ext : _externs)
+		o << '\t' << "  .extern    " << std::setw(4) << ext << "H" << endl;
 
-	for (vector<UCc>::iterator op = _opcodes.begin(); op != _opcodes.end(); ++op)
-		output_asm_opcode(o, funcmap, opcode_table_data, intrinsics, *op, options, symtbl);
+	for (auto& opcode : _opcodes)
+		output_asm_opcode(o, funcmap, opcode_table_data, intrinsics, opcode, options, symtbl);
 
 	return true;
 }
@@ -710,16 +702,16 @@ bool UCFunc::output_asm(ostream &o, const FuncMap &funcmap, const map<unsigned i
 void UCFunc::output_asm_data(ostream &o) {
 	static const unsigned int nochars = 60;
 	// limit of about 60 chars to a line, wrap to the next line if longer then this...
-	for (map<unsigned int, string, less<unsigned int> >::iterator i = _data.begin(); i != _data.end(); ++i) {
-		for (unsigned int j = 0; j < i->second.size(); j++) {
+	for (auto& i : _data) {
+		for (unsigned int j = 0; j < i.second.size(); j++) {
 			if (j == 0)
-				o << "L" << setw(4) << i->first << ":";
+				o << "L" << setw(4) << i.first << ":";
 			if ((j != 0) && !(j % nochars))
 				o << "'" << endl;
 			if (!(j % nochars))
 				o << "\tdb\t'";
 
-			o << i->second[j];
+			o << i.second[j];
 		}
 		o << "'" << endl;
 		o << "\tdb\t00" << endl;
@@ -804,11 +796,7 @@ static void print_number(char c, std::stringstream &str, const vector<unsigned i
 }
 
 string demunge_ocstring(UCFunc &ucf, const FuncMap &funcmap, const string &asmstr, const map<unsigned int, string> &intrinsics, const UCc &op, bool ucs_output, Usecode_symbol_table *symtbl) {
-#ifdef HAVE_SSTREAM
 	std::stringstream str;
-#else
-	std::strstream str;
-#endif
 	str << std::setfill('0') << std::setbase(16);
 	str.setf(ios::uppercase);
 	size_t  len = asmstr.length();
@@ -955,11 +943,11 @@ string demunge_ocstring(UCFunc &ucf, const FuncMap &funcmap, const string &asmst
 					assert(params.size() >= t);
 					assert(t != 0);
 
-					FuncMap::const_iterator fmp = funcmap.find(params[t - 1]);
+					auto fmp = funcmap.find(params[t - 1]);
 					funcname = fmp->second.funcname;
 					funcid = params[t - 1];
 				} else if (c == 'm') {
-					assert(symtbl != 0);
+					assert(symtbl != nullptr);
 					i++;
 					c = asmstr[i];
 					unsigned int t = charnum2uint(c);
@@ -979,7 +967,7 @@ string demunge_ocstring(UCFunc &ucf, const FuncMap &funcmap, const string &asmst
 
 					funcid = cls->get_method_id(params[t - 1]);
 					assert(funcid >= 0);
-					FuncMap::const_iterator fmp = funcmap.find(funcid);
+					auto fmp = funcmap.find(funcid);
 					assert(fmp != funcmap.end());
 					funcname = cls->get_name();
 					funcname += "::";
@@ -991,7 +979,7 @@ string demunge_ocstring(UCFunc &ucf, const FuncMap &funcmap, const string &asmst
 					assert(t != 0);
 					assert(!params.empty());
 
-					FuncMap::const_iterator fmp = funcmap.find(ucf._externs[params[t - 1]]);
+					auto fmp = funcmap.find(ucf._externs[params[t - 1]]);
 					funcname = fmp->second.funcname;
 					funcid = ucf._externs[params[t - 1]];
 				}
@@ -1014,7 +1002,7 @@ string demunge_ocstring(UCFunc &ucf, const FuncMap &funcmap, const string &asmst
 				// followed by plus sign, start on the parameter specified by the
 				// following digit.
 				if ((t == 0) && (c == ',' || c == '+')) {
-					vector<UCc *>::const_iterator it = op._popped.begin();
+					auto it = op._popped.begin();
 					if (c == '+') {
 						i++;
 						c = asmstr[i];
@@ -1080,8 +1068,7 @@ string demunge_ocstring(UCFunc &ucf, const FuncMap &funcmap, const string &asmst
 					if (paramval < ucf._num_args)
 						paramval = ucf._num_args - paramval - 1;
 					if (paramval || !ucf._cls) {
-						std::map<unsigned int, std::string>::iterator it;
-						it = ucf._varmap.find(params[t - 1]);
+						auto it = ucf._varmap.find(params[t - 1]);
 						if (it != ucf._varmap.end())
 							str << it->second;
 						else
@@ -1112,8 +1099,7 @@ string demunge_ocstring(UCFunc &ucf, const FuncMap &funcmap, const string &asmst
 				unsigned int t = charnum2uint(c);
 				assert(t != 0);
 				assert(params.size() >= t);
-				std::map<unsigned int, std::string>::iterator it;
-				it = UCFunc::FlagMap.find(params[t - 1]);
+				auto it = UCFunc::FlagMap.find(params[t - 1]);
 				if (it != UCFunc::FlagMap.end()) {
 					str << it->second;
 				} else {
@@ -1195,12 +1181,12 @@ void readbin_U7UCFunc(
 
 	if (symtbl) {
 		Usecode_symbol *sym = (*symtbl)[ucf._funcid];
-		Usecode_class_symbol *class_sym = 0;
+		Usecode_class_symbol *class_sym = nullptr;
 		// No symbol? Try looking inside class definitions.
 		if (!sym) {
 			for (int ii = 0; ii < symtbl->get_num_classes(); ii++) {
 				Usecode_class_symbol *cls = symtbl->get_class(ii);
-				assert(cls != 0);
+				assert(cls != nullptr);
 				sym = (*cls)[ucf._funcid];
 				if (sym) {
 					class_sym = cls;
@@ -1237,12 +1223,6 @@ void readbin_U7UCFunc(
 		f.seekg(ucf._datasize, ios::cur); // paranoia
 	}
 
-#if 0
-	if (ucf._funcid == _search_func)
-		for (map<unsigned int, string>::iterator i = ucf._data.begin(); i != ucf._data.end(); ++i)
-			cout << i->first << "\t" << i->second << endl;
-#endif
-
 	// process code segment
 	{
 		//streampos start_of_code_seg = f.tellg(); // what's this used for?
@@ -1272,7 +1252,7 @@ void readbin_U7UCFunc(
 		    which are stored in the file as 16bit shorts, with the exception of
 		    an ext32 function header, where the _datasize is a 32bit structure */
 		unsigned int code_size = ucf._funcsize - ucf._datasize - ((4 + ucf._num_externs) * SIZEOF_USHORT);
-		if (ucf.ext32 == true) code_size -= 2;
+		if (ucf.ext32) code_size -= 2;
 
 		DEBUG_READ_PAIR("Code Size: ", code_size);
 
@@ -1303,12 +1283,12 @@ void readbin_U7UCFunc(
 
 			/* if we're an opcode that sets a return value, we need to mark the
 			    function as one that returns a value */
-			if (otd.flag_return == true)
+			if (otd.flag_return)
 				ucf.return_var = true;
 
 			/* if we're an opcode that causes the function to abort, mark it as
 			    such */
-			if (otd.flag_abort == true)
+			if (otd.flag_abort)
 				ucf.aborts = true;
 
 			/* if we're a function debugging opcode, set the debuging flag, and
@@ -1325,17 +1305,17 @@ void readbin_U7UCFunc(
 					str.setf(ios::uppercase);
 					str << std::setw(4) << ucf._funcid;
 					ucf.funcname = str.str();
-					std::map<unsigned int, std::string,
-					         std::less<unsigned int> >::iterator it;
-					it = ucf._data.find(ucf.debugging_offset);
+					auto it = ucf._data.find(ucf.debugging_offset);
 					for (unsigned int i = 0;
 					     i < ucf._num_args + ucf._num_locals && it != ucf._data.end();
 					     ++it, i++) {
 						std::string varname = it->second;
 						if (!varname.empty()) {
-							if (std::isdigit(varname[0]))
-								varname = UCFunc::VARPREFIX + varname;
-							else if (varname == "item")
+							if (std::isdigit(static_cast<unsigned char>(varname[0]))) {
+								string temp = UCFunc::VARPREFIX;
+								temp += varname;
+								varname.swap(temp);
+							} else if (varname == "item")
 								varname = "_item";
 							ucf._varmap[i] = varname;
 						}
@@ -1360,9 +1340,9 @@ void readbin_U7UCFunc(
 			ucf._opcodes.push_back(ucop);
 
 #ifdef DEBUG_READ
-			cout << std::setw(4) << code_size << "\t" << std::setw(4) << code_offset << "\t" << std::setw(4) << (unsigned int)ucop._offset << "\t" << std::setw(2) << (unsigned int)ucop._id << "\t";
-			for (unsigned int i = 0; i < ucop._params.size(); i++)
-				cout << std::setw(2) << (unsigned int)ucop._params[i] << ',';
+			cout << std::setw(4) << code_size << "\t" << std::setw(4) << code_offset << "\t" << std::setw(4) << ucop._offset << "\t" << std::setw(2) << ucop._id << "\t";
+			for (unsigned char _param : ucop._params)
+				cout << std::setw(2) << static_cast<unsigned int>(_param) << ',';
 			cout << endl;
 #endif
 		}
